/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 275                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:47:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file howto.c
 *   @brief This file tells you how to mess with modules.
 *
 *   It contains all functions which are neccessary to write<br/>
 *   and handle a Services module. (IT'S NOT RUNABLE MODULE IN ANY WAY!)
 */
/*  ************************************************************************************************  */

/** Module HEAD **/

/* For Each module you need something like this: */
#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

/*  ************************************************************************************************  */

/* Things you should know about from modules.h */

/* These are all available module types 
 * (specifying where they are used
 * and what for modules are needed - ENCRYPT is UNIQUE!) */
typedef enum { CORE, THIRD, EVENT, ENCRYPT } 						MODType;

/* These are all available position to what a module can be added 
 * KNOWN_POSITIONS is used for <KNOWN_POSITIONS operations.
 * (makes changing very easy) */
typedef enum { MOD_UNIQUE, MOD_HEAD, MOD_TAIL, KNOWN_POSITIONS } 	MODposition;

/* These are all available access types 
 * (specifying who has access to what module
 * and what level is needed) */
typedef enum { USER, ADMIN, BOTH, NOONE } 							MODACCESS;

/* These are all available languages 
 * LASTLANG is used for <LASTLANG operations.
 * (makes changing very easy) */
typedef enum { EN, DE, FR, ES, IT, PL, LASTLANG } 					Languages;

/* These are to tell orion if his modules
 * are unloaded or only reloaded.
 * (Makes us able to reload modules without a single QUIT) */
typedef enum { DO_UNLOAD, DO_RELOAD } 								DO_ACTION;

/* These are all available module handling operations 
 * They control running/stopping
 * if MOD_STOP is returned, we stop processing 
 * (and skip other modules).
 * if MOD_CONT is returned, we continue processing
 * other modules */
typedef enum { MOD_STOP, MOD_CONT }									MOD_BOOL;

/* List of Commands */
typedef struct commands_ {
	
	/* The the call the same that is given to 
	 * "addCommand" -> addCommand(theads,module,"call",rootlevel,userlevel,minparams,maxparams);
	 
	 * Example:
	 *  (Read from Socket: :shock PRIVMSG ChanServ :register ..)
	 * On that Command input orion searches for a modulename named "register" (which must belong to servicetag "cs")
	 * in the "List of Modules".
	 *
	 * He finds it and grabs itself the "prototype" from that module - he splits it on " "
	 * According to the found parameters he now builds a string with the parameters we sent with the command
	 * That sting now should match "call"
	 *  TRUE: The command is executed.
	 *  FALSE: We skip it and proceed searching */
	char call[MAX_COMMAND_CALL_LEN];
	
	/* The pointer to the function we want to execute if "call" matched (Callback) - pointer to our modulefunction */
	MOD_BOOL (*func) (Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[]);
	
	/* The required admin level to access that command */
	int root_level;
	
	/* The required user level to access that command */
 	int user_level;
 	
 	/* The minimum of required parameters */
	size_t low_param;
	
 	/* The maximum of wanted parameters (if we don't know - reasons, descriptions - we add it with "0") */
	size_t high_param;
	
	struct commands_ *next, *prev;
}CMD;

/* List of Modules */
typedef struct modules_ {
	/* Here we save the author of our module */
	char Author[MAX_MODUL_AUTHOR_LEN];
	
	/* Here we save the versionstring of our module */
	char Version[MAX_MODUL_VERSION_LEN];
	
	/* Moduletype. (enum)
		Available types are: [typedef enum { CORE, THIRD, EVENT, ENCRYPT } MODType;] (modules.h)
		CORE: You shouldn't use it (it's only designed for CORE modules)
		THIRD: You are able to use that flag to tell us it's a third party module
		EVENT: Tells us that you add an Eventmodule (you will use that if you make a new client)
			look to "ns_events", "bs_events" and all other "*_events" for more details (each client MUST have an *_events file)
		ENCRYPT: It's an encryption module (you won't build a new one? =P) */
	MODType type;
	
	/* Moduleaccess options. (enum)
			Available options are: USER,ADMIN,BOTH,NONE
	 *	USER: Only users are able to execute that module.
	 *	ADMIN: Only admins are able to execute that module.
	 *	BOTH: Both are able to execute it.
	 *	NONE: Event modules. */
	MODACCESS access;
	
	/* Moduleoptions. (bitmask)
	 *	Available options are: LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY

	 * LOGINONLY: Module will only be executed it the user is logged in (identified with nickserv)
	 * OPER_IF_DEFCON: Module will only be executed if the user is an IRCOp (if defcon is >=3)
						If Defcon is <3 users will also be able to execute it.
	 * OPERONLY: Module can ONLY be executed by IRCOps
	 * READONLY: The module won't be executed if "readonly" is on (global variable)*/
	int options;

	/* The Servicetag (TAKE CARE: the servicetag is not the Nick!!
	 * known Servicetags are: Q,O,sockscontrol,ns,cs,ms,hs,bs,global
	 * Module files MUST EVER have the tag in front of the modulename !!! */
	char service[MAX_MODULENAME_LEN];

	/* Command to call in IRC, example: /cs register has "register" here */
	char modulename[MAX_COMMANDNAME_LEN];

	/* Filename, example: q_chanlev */
	char filename[MAX_FILEPATH_LEN];
	
	/* Dynamic Library Handle - used to handle (close/symbol search) opened librarys */
	void *dl_handle;
	
	/* Command Prototype - makes orion know what of the input parameters is part of the Command */
	char prototype[MAX_COMMANDNAME_LEN*4];
	
	/* HelpCommand Prototype - makes orion know what of the input parameters is part of the help Command */
	char helpprototype[MAX_COMMANDNAME_LEN*4];
	
	/* Finish function - is called before unloading the module */
	MOD_BOOL (*func_finish) (Threads *thread, DO_ACTION Action);
	
	/* VerifyLevels(); finishes initializing and sets the values for them (DON'T EVER FORGET IT) */
	int LeastUserLevel;							//Least user level to execute something in that module
	int LeastRootLevel;							//Least admin level to execute something in that module
	
	/* Holds specific langstrings for that module (only for third party modules)
	 * core modules use lang/*.c files to send with language dependent replies */
	ModuleLang lang[NUM_LANGS];
	
	/* List of Commands for that module */
	CMD *commands;
	
	struct modules_ *next, *prev;
}ModCo;

/* The ModInfo struct */
typedef struct ModInfo_ {
	ModCo *modul;			/* - holds a pointer to the active module */
	MOD_ERROR ModErr;		/* - holds the last occured error (used to grab errors on loading) */
}ModInfo;

/****/

struct threads_ {
/* ........
  .........
  ......... */
	struct SQL_ {

		/* The account information of the users who executed the command
		 * Information comes from `orion_ns_core`
		 */
		struct access_ {
			/* The authnick (display nick from NickServ) */
			char authnick[MAX_NICK_LEN];
			
			/* The users nspassword */
			char password[ENC_PASSMAX];
			
			/* The reason (if the nick is suspended) */
			char reason[MAX_REASON_LEN];
			
			/* The users global Qlevel (for Userscommands necessary are only 0-1)
			 * admins will have a higher level >100 */
			int qlevel;

			/* The users global Olevel - OperServ - (for Users ever 0)
			 * admins will have a higher level >=1 */
			int olevel;

			/* The users global sockscontrol control level (for Users ever 0)
			 * admins will have a higher level >=1 */
			int slevel;

			/* Not implemented now */
			int llevel;

			/* His email adress */
			char email[MAX_EMAIL_LEN];

			/* His greet message */
			char greet[MAX_GREET_LEN];

			/* His URL */
			char url[MAX_URL_LEN];

			/* His ICQ UIN */
			long icq;

			/* His yahoo address */
			char yahoo[MAX_YAHOO_LEN];

			/* His location */
			char location[MAX_LOCATION_LEN];

			/* His birthday */
			char birthday[MAX_BIRTHDAY_LEN];

			/* His realname (NOT the one on IRC) */
			char realname[MAX_REALNAME_LEN];

			/* His gender */
			bool gender;

			/* His interests (sports or whatever) */
			char interests[MAX_INTERESTS_LEN];

			/* His profession (programmer? ^^) */
			char profession[MAX_PROFESSION_LEN];

			/* His slogan */
			char slogan[MAX_SLOGAN_LEN];

			/* His language (modules.h)
			 * typedef enum { EN, DE, FR, ES, IT, PL, LASTLANG } Languages; */
			int language;

			/* His global rights - for NS,BS,CS,HS,MS,Global (services.h)
			 * #define is_user 				0
			 * #define is_services_user 	1
			 * #define is_services_oper 	2
			 * #define is_services_admin 	3
			 * #define is_services_root 	4 */
			int rights;

			/* His account settings (datatypes.h)
			 * #define NS_SET_KILLPROTECT			0x00000001  // Kill others who take this nick
			 * #define NS_SET_KILL_QUICK			0x00000002  // Kill in 20 seconds instead of 60
			 * #define NS_SET_KILL_IMMED			0x00000004  // Kill immediately instead of in 60 sec
			 * #define NS_SET_MSG					0x00000008  // Use PRIVMSGs instead of NOTICEs
			 * #define NS_SET_SECURE				0x00000010  // NOT IMPLEMENTED - No Usage by now!
			 * #define NS_SET_PRIVATE				0x00000020  // Don't show in LIST to non-servadmins
			 * #define NS_SET_HIDE_EMAIL			0x00000040  // Don't show E-mail in INFO
			 * #define NS_SET_HIDE_MASK				0x00000080  // Don't show last seen address in INFO
			 * #define NS_SET_HIDE_QUIT				0x00000100  // Don't show last quit message in INFO
			 * #define NS_SET_HIDE_STATUS			0x00000200  // Don't show services access status
			 * #define NS_SUSPENDED					0x00000400  // NOT IMPLEMENTED - No Usage by now */
			int settings;

			/* His maximum memo receive setting
			 * could be changed with /ms limit newlimit */
			int maxmemos;

			/* Here we save the last timestamp at which he failed logging in */
			time_t last_failed_login_time;

			/* Here we save the password with which he failed logging in
			 * used to created that percentage output */
			char last_failed_login_password[MAX_PASSWORD_LEN];

			/* This is the number of already failed logins
			 * since the last valid login */
			int failed_logins;

			/* Selected fields from MySQL */
			int fields;
		}access;


		struct aliases_ {
			/* His alias nickname */
			char nick[MAX_NICK_LEN];

			/* His displaynick
			 * (must be identical to access.authnick) */
			char authnick[MAX_NICK_LEN];

			/* His last VirtualHost */
			char last_vhost[MAX_HOST_LEN];

			/* His last realname (on IRC) */
			char last_realname[MAX_REALNAME_LEN];

			/* His last quit message (on IRC) */
			char last_quit[MAX_QUITMSG_LEN];

			/* His alias nickname link status
			 * 0: linked permanent
			 * 1: linked temporarily */
			time_t linked;

			/* His aliasnick linktime
			 * This is the time the alias was registered */
			time_t time_linked;

			/* His Last login with that alias */
			time_t last_login;

			/* Last used (changed any settings, identified, ... ) */
			time_t last_used;

			/* Expires or not!?
			 * 1: Nick will expire
			 * 0: Nick is no-expire */
			bool expire;

			/* Active or not (means identified)!?
			 * 1: yes
			 * 0: no */
			bool active;

			/* Nick forbidden!?
			 * 1: yes
			 * 0: no */
			bool forbid;

			/* Selected fields by MySQL */
			int fields;
		}aliases;

		struct condata_ {
			/* Target users nickname */
			char nick[MAX_NICK_LEN];
			
			/* The servertime on which target user connected (at the time he connects) */
			time_t time;
			
			/* ServicesServertime when the user connected */
			time_t localtime;
			
			/* The real ident (user) of that user */
			char rident[MAX_IDENT_LEN];
			
			/* The current ident (user) - could be a virtual ident - of that user */
			char ident[MAX_IDENT_LEN];
			
			/* The real host of target user */
			char rhost[MAX_HOST_LEN];
			
			/* The server where the user connected */
			char server[MAX_SERVER_LEN];
			
			/* The modes target user currently has */
			char modes[MAX_USER_MODES];
			
			/* Target users virtual host */
			char vhost[MAX_HOST_LEN];
			
			/* Target users realname (on IRC - the one specified in mIRC) */
			char realname[MAX_REALNAME_LEN];
			
			/* The swhois of target user */
			char swhois[MAX_SWHOIS_LEN];
			
			/* The subnet from which target user is currently connecting */
			char subnet[MAX_SUBNET_LEN];
			
			/* The ip target user is using */
			char ip[MAX_IP_LEN];
			
			/* The nickserv-warntime, if !=0 he has to identify */
			time_t ns_warntime;
			
			/* Target users faketype (0 if it's a user, higher if it's a known service) */
			int faketype;
			
			/* number of fields selected from MySQL */
			int fields;
		}Cdata;
	}sql;
	
	/* DB4 c - The channel struct from the Threads */
	struct db4_ {			/* That's the channel struct included in the Threads struct */

		/* The access data for target user (if given to UpdateChannelInformation*();) */
		struct access_ {
			/* The authnick is the NickServ groups mainnick
			 * the display. (changeable with /ns set DISPLAY) */
			char authnick[MAX_NICK_LEN];
			
			/* The users password (if encryption is off, unencrypted) */
			char password[ENC_PASSMAX];
			
			/* The reason.
			 * (in most cases empty, only "full" if the group is suspended) */
			char reason[MAX_REASON_LEN];
			
			/* The users global Qlevel (for Userscommands necessary are only 0-1)
			 * admins will have a higher level >100 */
			int qlevel;
			
			/* The users global Olevel - OperServ - (for Users ever 0)
			 * admins will have a higher level >=1 */
			int olevel;
			
			/* The users global sockscontrol control level (for Users ever 0)
			 * admins will have a higher level >=1 */
			int slevel;
			
			/* Not implemented now */
			int llevel;
			
			/* Target users email adress */
			char email[MAX_EMAIL_LEN];
			
			/* Target users greet message */
			char greet[MAX_GREET_LEN];
			
			/* Target users URL */
			char url[MAX_URL_LEN];
			
			/* Target users ICQ UIN */
			long icq;
			
			/* Target users yahoo adress */
			char yahoo[MAX_YAHOO_LEN];
			
			/* Target users location */
			char location[MAX_LOCATION_LEN];
			
			/* Target users birthday */
			char birthday[MAX_BIRTHDAY_LEN];
			
			/* Target users realname (NOT the one on IRC) */
			char realname[MAX_REALNAME_LEN];
			
			/* Target users gender */
			bool gender;
			
			/* Target users interests (sports or whatever) */
			char interests[MAX_INTERESTS_LEN];
			
			/* Target users profession (programmer? ^^) */
			char profession[MAX_PROFESSION_LEN];
			
			/* Target users slogan */
			char slogan[MAX_SLOGAN_LEN];
			
			/* Target users language (modules.h)
			 * typedef enum { EN, DE, FR, ES, IT, PL, LASTLANG } Languages; */
			int language;
			
			/* Target users global rights (services.h)
			 * #define is_user 				0
			 * #define is_services_user 	1
			 * #define is_services_oper 	2
			 * #define is_services_admin 	3
			 * #define is_services_root 	4 */
			int rights;
			
			/* Target users settings (datatypes.h)
			 * #define NS_SET_KILLPROTECT			0x00000001  // Kill others who take this nick
			 * #define NS_SET_KILL_QUICK			0x00000002  // Kill in 20 seconds instead of 60
			 * #define NS_SET_KILL_IMMED			0x00000004  // Kill immediately instead of in 60 sec
			 * #define NS_SET_MSG					0x00000008  // Use PRIVMSGs instead of NOTICEs
			 * #define NS_SET_SECURE				0x00000010  // NOT IMPLEMENTED - No Usage by now!
			 * #define NS_SET_PRIVATE				0x00000020  // Don't show in LIST to non-servadmins
			 * #define NS_SET_HIDE_EMAIL			0x00000040  // Don't show E-mail in INFO
			 * #define NS_SET_HIDE_MASK				0x00000080  // Don't show last seen address in INFO
			 * #define NS_SET_HIDE_QUIT				0x00000100  // Don't show last quit message in INFO
			 * #define NS_SET_HIDE_STATUS			0x00000200  // Don't show services access status
			 * #define NS_SUSPENDED					0x00000400  // NOT IMPLEMENTED - No Usage by now */
			int settings;
			
			/* Target users maximum memo receive setting
			 * could be changed by the user with /ms limit newlimit */
			int maxmemos;

			/* Here we save the last timestamp at which he failed logging in */
			time_t last_failed_login_time;
			
			/* Here we save the password with which he failed logging in
			 * used to created that percentage output */
			char last_failed_login_password[MAX_PASSWORD_LEN];
			
			/* This is the number of already failed logins
			 * since the last valid login */
			int failed_logins;

			/* Selected fields from MySQL */
			int fields;
		}access;

		/* Infos about target users aliases nick */
		struct aliases_ {
			/* The alias nickname of target user */
			char nick[MAX_NICK_LEN];
			
			/* The displaynick of target user
			 * (must be identical to access.authnick) */
			char authnick[MAX_NICK_LEN];
			
			/* Target users last VirtualHost */
			char last_vhost[MAX_HOST_LEN];
			
			/* Target users last realname (on IRC) */
			char last_realname[MAX_REALNAME_LEN];
			
			/* Target users last quit message (on IRC) */
			char last_quit[MAX_QUITMSG_LEN];
			
			/* Target users alias nickname link status
			 * 0: linked permanent
			 * 1: linked temporarily */
			time_t linked;
			
			/* Target users linktime
			 * This is the time the alias was registered */
			time_t time_linked;
			
			/* Last login with that alias */
			time_t last_login;
			
			/* Last used (changed any settings, identified, ... ) */
			time_t last_used;
			
			/* Expires or not!?
			 * 1: Nick will expire
			 * 0: Nick is no-expire */
			bool expire;
			
			/* Active or not (means identified)!?
			 * 1: yes
			 * 0: no */
			bool active;
			
			/* Nick forbidden!?
			 * 1: yes
			 * 0: no */
			bool forbid;
			
			/* Selected fields by MySQL */
			int fields;
		}aliases;

		struct condata_ {
			/* His nickname */
			char nick[MAX_NICK_LEN];

			/* The servertime of the Server on which he connected (at the time he connects) */
			time_t time;

			/* ServicesServertime when he connected */
			time_t localtime;

			/* His real ident (user) */
			char rident[MAX_IDENT_LEN];

			/* His current ident (user) - could be a virtual ident */
			char ident[MAX_IDENT_LEN];

			/* His real host */
			char rhost[MAX_HOST_LEN];

			/* The server where he connected */
			char server[MAX_SERVER_LEN];

			/* His current global usermodes */
			char modes[MAX_USER_MODES];

			/* His virtual host */
			char vhost[MAX_HOST_LEN];

			/* His realname (on IRC - the one specified in mIRC) */
			char realname[MAX_REALNAME_LEN];

			/* His swhois */
			char swhois[MAX_SWHOIS_LEN];

			/* The subnet from which he is currently connecting */
			char subnet[MAX_SUBNET_LEN];

			/* His ip-address */
			char ip[MAX_IP_LEN];

			/* The nickserv-warntime, if !=0 he has to identify */
			time_t ns_warntime;

			/* His faketype (0 if it's a user, higher if it's a known service) */
			int faketype;

			/* number of fields selected from MySQL */
			int fields;
		}Cdata;

		struct channel3_ {
			/* Here you are able to find all settings for the channel */
			char name[MAX_CHANNELNAME_LEN];
			
			/* The Botname assigned for that channel */
			char bot[MAX_NICK_LEN];
			
			/* If the Channel is registered serv[0]!='\0' and serv is equal ChanServ or Q
			 * depends for which service the channel was registered */
			char serv[MAX_NICK_LEN];
			
			/* The channels founder (if registered) */
			char founder[MAX_NICK_LEN];
			
			/* The channels successor (if registered) */
			char successor[MAX_NICK_LEN];
			
			/* The timestamp when the channel was created (first join of a user) */
			time_t datecreate;
			
			/* The timestamp when the channel was registered */
			time_t regdate;
			
			/* The channels description */
			char description[MAX_CHANNELDESCRIPTION_LEN];
			
			/* The current channel modes */
			char mode[MAX_CHAN_MODES];
			
			/* The positive mlockmodes for a channel (only +) */
			char mlock_positive[MAX_CHAN_MODES];
			
			/* The negative mlockmodes for a channel (only -) */
			char mlock_negative[MAX_CHAN_MODES];
			
			/* The mlock controlled channel limit */
			long mlock_limit;
			
			/* The mlock controlled channel key */
			char mlock_key[MAX_KEY_LEN];
			
			/* The mlock's floodmode */
			char mlock_flood[MAX_COMMON_LEN];
			
			/* The mlock's redirect channel */
			char mlock_redirect[MAX_CHANNELNAME_LEN];
			
			/* The normal redirect channel (if it's not mlocked) */
			char link[MAX_CHANNELNAME_LEN];
			
			/* The current channel limit */
			long limit;
			
			/* The current channel key */
			char key[MAX_KEY_LEN];
			
			/* The channels floodmode setting */
			char flood[MAX_COMMON_LEN];
			
			/* The last channel topic */
			char last_topic[MAX_TOPIC_LEN];
			
			/* The one who set the topic at last */
			char last_topic_setter[MAX_NICK_LEN+MAX_IDENT_LEN+MAX_HOST_LEN+3]; //19
			
			/* The last time the topic was changed */
			time_t last_topic_time;
			
			/* The channels URL */
			char url[MAX_URL_LEN];
			
			/* The channels autolimit (atm only used by Q) */
			long alimit;
			
			/* The channels flags (for Q) */
			char flags[MAX_CHAN_MODES];
			
			/* The channels welcome message (greet message) */
			char welcome[MAX_WELCOME_LEN];
			
			/* Timestamp when the channel was last used */
			time_t last_used;
			
			/* Reason - if the channel is forbidden or suspended */
			char reason[MAX_REASON_LEN];
			
			/* The channels email address - to possibly contact the channels owners */
			char email[MAX_EMAIL_LEN];
			
			/* This are the Channels settings: (datatypes.h)
			 * #define CS_XOP_BIT				0x00000001
			 * #define CS_TOPICLOCK_BIT			0x00000002
			 * #define CS_KEEPTOPIC_BIT			0x00000004
			 * #define CS_SECUREFOUNDER_BIT		0x00000008
			 * #define CS_SECUREOPS_BIT			0x00000010
			 * #define CS_SECURE_BIT			0x00000020
			 * #define CS_RESTRICTED_BIT		0x00000040
			 * #define CS_PRIVATE_BIT			0x00000080
			 * #define CS_PEACE_BIT				0x00000100
			 * #define CS_OPNOTICE_BIT			0x00000200
			 * #define CS_SIGNKICK_1_BIT		0x00000400
			 * #define CS_SIGNKICK_2_BIT		0x00000800
			 * #define CS_BANTYPE_1_BIT			0x00001000
			 * #define CS_BANTYPE_2_BIT			0x00002000
			 * #define CS_FORBID_BIT			0x00004000
			 * #define CS_SUSPEND_BIT			0x00008000
			 * #define BS_SET_NOBOT_BIT			0x00010000 */
			long settings;
			
			/* Expire? (0: no, 1: yes) */
			bool expire;
			int maxmemos;

			/* How many users are in that channel !? */
			int countusers;
			
			/* bot is in channel? (0: no, 1: yes) */
			bool botinchan;

			/* Here you find the LEVEL settings for that channel */
			struct chanlevels_ {
				char channel[MAX_CHANNELNAME_LEN]; //1
				short int autodeop;
				short int autohalfop;
				short int autoop;
				short int autoprotect;
				short int autovoice;
				short int nojoin;
				short int signkick;
				short int acclist;
				short int accchange;
				short int akick;
				short int set;
				short int ban;
				short int banme;
				short int clear;
				short int getkey;
				short int halfop;
				short int halfopme;
				short int info;
				short int kick;
				short int kickme;
				short int invite;
				short int opdeop;
				short int opdeopme;
				short int protect;
				short int protectme;
				short int topic;
				short int unban;
				short int voice;
				short int voiceme;
				short int memo;
				short int assign;
				short int badwords;
				short int fantasia;
				short int greet;
				short int nokick;
				short int say;
				int fields;
			}levels;

			/* Here are all BotServ settings for that channel */
			struct botserv_ {
				int 		set;
				int			caps_ttb;
				short int	caps_min;//
				short int	caps_per;//
				int			flood_ttb;
				short int	flood_ln;
				time_t		flood_secs;
				short int	repeat_ttb;
				int			repeat;
				int			colors;//
				int			underlines;//
				int 		reverses;
				int			badwords;//
				int			bolds;//

				int fields;
			}botserv;

			struct chanaccess2_ {
				/* This is the channel access level and
				 * the display (from nickserv) of the user who
				 * executed the command (i.e. my level in that channel) */
				char ownlevel[MAX_CHAN_ACCLEVEL_LEN];
				char ownauthnick[MAX_NICK_LEN];

				/* This is very useful if we adapt channel access for a user
				 * So we need to know all about ourselves and him too
				 * and this is where you are able to find his
				 * 1) level in that target channel (local channel level, not the global rights level)
				 * 2) display-nick (from nickserv)*/
				char targetlevel[MAX_CHAN_ACCLEVEL_LEN];
				char targetauthnick[MAX_NICK_LEN];
				int fields;
			}access;

			int fields; //31
		}chan;

		int fields;
	}c;
	
	/* To get actual channel data use (the one which fit's your needs):
		void UpdateChannelInformation(Threads *thread, char *channel, char *target);
		void UpdateChannelInformation2(Threads *thread, char *channel, char *target);
		void UpdateChannelInformation3(Threads *thread, char *channel, char *target);
		
		Lookup the ChanServ modules to get more information about how to use that command or just read on.. ;)
		You find these functions in services.c
	*/
/* ........
  .........
  ......... */
	bool Webform;

	struct threads_ *next, *prev;
};

/* So you see, you should be able to find nearly ALL informations in thread->sql.* and thread->c.* */
/* If you need more structs or defines look @ datatypes.h (most defines are in datatypes.h, a few in modules.h and services.h) */

/*  ************************************************************************************************  */

/* Defines to be used for a few module internal commands, just use it like we did */
#define SERVICE "cs"						// The Servicetag
#define MODUL "info"						// The IRCkeyword for which we create this module
#define MODUL_FILE "cs_info"				// The Filename of the module - so we can be sure it's unique (used to identify modules)
#define MODUL_FILE_CS_GETKEY "cs_getkey" 	// NOT EVER NEEDED: useable to call levels from other modules
#define MODUL_VERSION "1.0"					// The Moduleversion

/* The functions you want to program, in most cases one command function, the help to that command and a output for the listing (/cs help) */
MOD_BOOL cs_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

/* We use this function as finish function, it's automatically called on UNLOADING - specify that with addFinish(); */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/* The Init function - it should EVER look like this (copy & paste, don't change anything */
ModInfo orion_Init(Threads *thread, void *dl_handle){
/* Functions you may need in the Init */
	
	/* This is to initialize our module to the "List of Modules"
	 *----------------------------------------------------------
	 * if you write a complete new module - that shouldn't react at the start/end of others
	 *	use that Initialization with MODposition "MOD_UNIQUE"
	 
	 * The only things which you have to change DIRECTLY here are "MODposition"
	 * "Command-prototype" and "HelpCommand-prototype" 
	 *
	 * More details?: Look to cs_info_after.c (detailled information)
	 *   src/core/ns_set.c, src/core/ns_saset.c or src/core/cs_register.c
	 */
	ModInfo ModuleInit(Threads *thread,void *dl_handle,/* MODposition */,SERVICE,MODUL_FILE,MODUL,/* Command-prototype */,/* HelpCommand-prototype */);

	/* This is to add an Author (to our module in the "List of Modules")
	 *------------------------------------------------------------------
	 * The value we enter here will be saved to "Author" to the "List of Modules"
	 * Example: addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	 */
	MOD_ERROR addAuthor(Threads *thread, ModInfo *insert, const char *Author, ...);

	/* This is to add a Version (to our module in the "List of Modules")
	 *------------------------------------------------------------------
	 * The value we enter here will be saved to "Version" to the "List of Modules"
	 * Example: addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	 */
	MOD_ERROR addVersion(Threads *thread, ModInfo *insert, const char *Version, ...);

	/* This is to add options to care of (to our module in the "List of Modules")
	 *------------------------------------------------------------------
	 * The values we enter here will merged and saved to "options" to the "List of Modules"
	 * Example: addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	 
	 * More details?: Look to cs_info_after.c
	 */
	MOD_ERROR addOptions(Threads *thread, ModInfo *insert, int parameters, ...);

	/* This is to add the Accesstype (to our module in the "List of Modules")
	 *------------------------------------------------------------------
	 * The value we enter here will be saved to "access" to the "List of Modules"
	 * Example: addAccess (thread,&Module,BOTH);
	 
	 * More details?: Look to cs_info_after.c
	 */
	MOD_ERROR addAccess (Threads *thread, ModInfo *insert, MODACCESS access);
	
	/* This is to add the Moduletype (to our module in the "List of Modules")
	 *------------------------------------------------------------------
	 * The value we enter here will be saved to "type" to the "List of Modules"
	 * Example: addAccess (thread,&Module,BOTH);
	 
	 * More details?: Look to cs_info_after.c
	 */
	MOD_ERROR addMODType(Threads *thread, ModInfo *insert, MODType itype);

	/* This is to add a Command to our module in the "List of Modules"
	 *----------------------------------------------------------
	 * It adds a Command to the internal commandlist ("List of Commands") of the module
	 * It is dependent from Command-prototype" and "HelpCommand-prototype"
	 * Examples:
	 *   addCommand(thread,&Module,"info",cs_info,is_services_admin,is_services_user,4,4);
	 *   addCommand(thread,&Module,"help info",cs_info_help,is_services_user,is_services_user,4,4);
	 *   addCommand(thread,&Module,"help",cs_info_help_listing,is_services_user,is_services_user,3,3);
	 
	 * More details?: Look to cs_info_after.c (detailled description)
	 *   src/core/ns_set.c, src/core/ns_saset.c or src/core/cs_register.c
	 */
	MOD_ERROR addCommand(
		/* The thread struct (to handle/queue threads) */
		Threads *thread,
		
		/* Our struct - holds the pointer to the current processing module and the last occured Error */
		ModInfo *insert,
		
		/* The call - string to what the (IRCkeywords) command would be recreated */
		const char *call,
		
		/* Function Callback - pointer to our module function */
		MOD_BOOL (*func) (Threads *thread, void* language, bool howtosend, char *snick, char *nick, char *param[]),
		
		/* required admin level */
		int root_level,
		
		/* required user level */
		int user_level,
		
		/* at least required parameters */
		size_t low_param,
		
		/* maximum parameter count ("0" if we don't know the max.) */
		size_t high_param
	);

	/* This is to add a Finishfunction to our module in the "List of Modules"
	 *----------------------------------------------------------
	 * It adds a Finishroutine to the module
	 * This function is then called when the module is unloaded.
	 * Examples:
	 *   addFinish(thread,&Module,orion_Fini);
	 
	 * More details?: Look to cs_info_after.c (detailled description)
	 */
	MOD_ERROR addFinish(Threads *thread, ModInfo *insert,MOD_BOOL (*Finish_Func) (Threads *thread, DO_ACTION Action));
	
	/* This is to verify the levels for our module in the "List of Modules"
	 *----------------------------------------------------------
	 * It looks up the levels and saves the at least required levels
	 * to access this module.
	 * Examples:
	 *   VerifyLevels(thread,&Module);
	 
	 * More details?: Look to cs_info_after.c (detailled description)
	 */
	MOD_ERROR VerifyLevels(Threads *thread, ModInfo *module);
	
/* Return of orion_init(); MUST EVER BE: */
	return Module;
}

/** END OF MODULEHEAD **/

/*  ************************************************************************************************  */

/** MODULEBODY **/

MOD_BOOL cs_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
/* Functions you may need in this part of the module */

	/* These select a User or a Root level (Dynamic decides itself with the help of modul->access)
	 * by searching for the IRCKeyword to match module->modulname.
	 * NOTE: Take in mind that there could match many various modules ..
	 *   because you are able to add modules with the same specification before the core modul
	 *   -> BUT you will only receive the level of the FIRST time "char *modul" matches "module->modulname"
	 *   Take care using these three - use them only if you really need it.
	 */
	int User_Command_Level   			(Threads *thread, char *service, char *modul, char *command);
	int Root_Command_Level   			(Threads *thread, char *service, char *modul, char *command);
	int Dynamic_Command_Level			(Threads *thread, char *service, char *modul, char *command);

	/* These select a User or a Root level (Dynamic decides itself with the help of modul->access)
	 * by searching for the modul's FILENAME to match module->filename.
	 * NOTE: Filename (MUST) (don't know how not..) be/ARE UNIQUE so
	 *   it's impossible that you receive various levels
	 *   (possibly you have the opportunity to receive the level twice...)
	 *   -> BUT you will ever get the commands REAL level!
	 */
	int ModuleFile_UserCommandLevel		(Threads *thread, char *modul_file, char *command);
	int ModuleFile_RootCommandLevel		(Threads *thread, char *modul_file, char *command);
	int ModuleFile_DynamicCommandLevel	(Threads *thread, char *modul_file, char *command);

	/* You can use this to translate a module errornumber to a readable error string
	 * (don't know what for you could use it here - but now you know it ^^) */
	char *ModuleError(MOD_ERROR ModErr);

/* MySQL functions that may be useful for you (OVERVIEW) - services.h for prototypes */

	/* For connecting the Database (don't use it unless you need it - and you shouldn't ! :>) */
	void mysql_connect(Threads *thread);
	
	/* For disconnecting the Database (don't use it unless you need it - and you shouldn't ! :>) */
	void mysql_disconnect(Threads *thread);
	
	/* Free the MySQL result - NEVER FORGET THAT (i.e. after doSelectMySQL();) */
	void mysql_free_result_myr(SQLres *res);

	/* This is to escape strings (to keep MySQL happy ^^) */
	char* mysql_quote(Threads *thread, char *sql);
	
	/* You can use that at the end of your command, but it's not really necessary
	 * TAKE CARE if you use large loops - free it after a few loops (or better after each)
	 * to ensure it doesn't delete information (because it has to) if you still need it!! */
	void mysql_free_quotes(Threads *thread);
	
	/* To Execute Queries (deprecated) */
	void ExecuteQuery(Threads *thread, char *query, size_t len);
	
	/* To Execute Queries ThreadSafe - you should ever program threadsafe, since the program is threaded */
	void ExecuteQuery_r(Threads *thread, char *query, size_t len);
	
	/* To Execute InlineQueries ThreadSafe (means the return value is the MySQL error (if it occured)
	 * - you should ever program threadsafe, since the program is threaded */
	char *InlineExecuteQuery_r(Threads *thread, char *query, size_t len);

	/* Send a SELECT with a single result and retrieve it (i.e. SELECT `bla` FROM `..` WHERE `hm`='yes' - this gets you `bla`) */
	char* mysql_sel_var(Threads *thread, const char *text, ...);
	
	/* Send INSERT, UPDATE, DELETE, TRUNCATE Queries (all queries for which you don't need any return value */
	void domysql(Threads *thread, const char *text, ...);

	/* Free mysql buffers (you shouldn't use that - you don't need it) */
	int mysql_free(void);
	
	/* Deprecated - forget that */
	void mysql_sel(Threads *thread, char *nick, char *text);

	/* These are for BotServ (i think you won't need them, and takes too long to describe)
	 * you find them in src/mysql.c (read it, study it) and to understand it easier
	 * read in src/core/bs_events the function bs_on_channelmessage(); */
	void sql_BsChan(Threads *thread, char *nick, char *channel, BsChan *bsChan);
	char* sql_BsChan2(Threads *thread, char *nick, char *target, char *channel, BsChan *bsChan);

	/* Selects
	 * 1) Channelstuff to typedef struct "Channel" (datatypes.h) - it will be filled afterwards
	 * 2) Channellevels to typedef struct "ChanLevels" (datatypes.h) - it will be filled afterwards */
	void sql_chanmodes(Threads *thread, char *channel, Channel *c, ChanLevels *cLev);
	
	/* Another way to select channel data (used from Q in some parts) */
	ChanCA sql_chandata(Threads *thread, char *channel, char *authnick, char *target);
	
	/* Selects from orion_cs_core and fills typedef struct "ChannelDB" (datatypes.h) */
	ChannelDB sql_chan_db(Threads *thread, char *channel);

	/* Selects from orion_cs_core and fills typedef struct "DB3" (datatypes.h) */
	DB3 sql_chan_data(Threads *thread, char *starttype, int parameters, ...);
	
	/* Selects from orion_cs_core and fills typedef struct "DB4" (datatypes.h)
	 * This is the same channel structure as the one used in the threads struct */
	DB4 sql_thread_data(Threads *thread, char *starttype, int parameters, ...);
	
	/* Selects only the most necessary Channel information (very minimalistic) */
	Channel sql_chan(Threads *thread, char *channel);
	
	/* Selects only the channellevels for target channel */
	ChanLevels sql_chanlevels(Threads *thread, char *channel);

	/* Selects all important information about target user. (`orion_core_condata`, `orion_ns_aliases`, `orion_ns_core`) */
	void sql_mainAAC(Threads *thread, char *nick); //Aliases, Authdata, Condata
	
	/* Selects the channelaccess information for given authnick - `orion_cs_access` */
	Access sql_access(Threads *thread, char *authnick);
	
	/* Selects the alias information for target alias - all fields from `orion_ns_aliases` */
	Aliases sql_aliases(Threads *thread, char *aliasnick);
	
	/* Selects information about a botserv channel
	 * (if a bot is holding it) from table `orion_cs_core` (mysql.c - function and datatypes.h - struct) */
	Bassign sql_assign(Threads *thread, char *channel);
	
	/* Selects information about a botserv channel,
	 * and it's kick settings `orion_cs_core` (mysql.c - function and datatypes.h - struct) */
	Bkick sql_bkick(Threads *thread, char *channel);
	
	/* Selects information about the
	 * channelaccess of a single user from table `orion_cs_access` */
	Acclist sql_acclist(Threads *thread, const char *serv, char *channel, char *authnick);
	
	/* Selects connection-information for a single user */
	Condata sql_condata(Threads *thread, char *nick);

	/* Counts to how many channels a user has access (orion_cs_access)
	 * and returns the count value */
	int count(Threads *thread, const char *serv, char *authnick);
	
	/* Returns the Users current usermodes */
	char* currmodes(Threads *thread, char *nick);
	
	/* Looks if the given user is in the given channel
	 * if it's true the nick is returned, otherwise a string with string[0]=='\0' is returned */
	char *sql_users(Threads *thread, char *chan, char *nick);
	
	/* Selects the channel accesslevel as a char* to be able
	 * to select Qchannellevels and ChanServ channellevels (orion_cs_access) */
	char* acclevel(Threads *thread, char *serv, char *channel, char *authnick);
	
	/* Select a specific field (const char *field) from the `orion_ns_core` table */
	char* accessselect(Threads *thread, const char *field, char *authnick);
	
	/* Select a specific field (const char *field) from the `orion_ns_aliases` table */
	char* selectaliases(Threads *thread, const char *field, char *nnick);
	
	/* Used to select the value for a channellevel (ChanServ)
	 * Avoid using it - using it to often may stress the Database */
	int SingleLevel(Threads *thread, const char *field, char *channel);
	
	/* Selects all Session informations for target host */
	Trust O_sql_trust(Threads *thread, char *host);
	
	/* deprecated - don't use it */
	QUserlist *Q_userlist(Threads *thread, char *pattern_in);
	
	/* Use this to send Select Queries - (i will give a short introduction beneath) */
	SQLres *doSelectMySQL(Threads *thread, const char *text, ...);

	/* Used in cs_akick.c look for details there or in src/mysql.c .... */
	SQLres *AkickEnforce(Threads *thread, char *channel);
	SQLres *AkickOnJoin(Threads *thread, char *channel, char *nick);
	SQLres *AkickOnSJoin(Threads *thread, char *channel, char *nicks);

	/* I think you shouldn't need that - it queue's queries to be sent compressed on flushing */
	size_t doBufferedInsert(Threads *thread, bool duplicate_update, size_t directtable, const char *input, ...);
/**********************************************************************************/

/* The most important MySQL functions: (OVERVIEW) */

	/* This is to escape strings (to keep MySQL happy ^^) */
	char* mysql_quote(Threads *thread, char *sql);

	/* Send a SELECT with a single result and retrieve it (i.e. SELECT `bla` FROM `..` WHERE `hm`='yes' - this gets you `bla`) */
	char* mysql_sel_var(Threads *thread, const char *text, ...);

	/* Send INSERT, UPDATE, DELETE, TRUNCATE Queries (all queries for which you don't need any return value */
	void domysql(Threads *thread, const char *text, ...);

	/* Use this to send Select Queries - (i will give a short introduction beneath) */
	SQLres *doSelectMySQL(Threads *thread, const char *text, ...);
	
/* Short Usage Information: (USAGE) */
	
	/* PART OF src/core/o_chan.c - O_chan_globmode();
	 * Useage of doSelectMySQL(); */
		modes.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`channel`,`orion_cs_core`.`datecreate` FROM `orion_cs_core`");
		modes.fields = mysql_num_rows(modes.res);
		for(i=0;i<modes.fields;i++){
			modes.row = mysql_fetch_row(modes.res);

			do_mode(thread,snick,modes.row[0],(time_t)stol(modes.row[1]),_dstrjoin(' ',param,5));
		}
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");

		/* mysql_free_result_myr(); - modes  - DON'T EVER FORGET THIS !! (if you forget it you'll have memleaks) */
		mysql_free_result_myr(modes.res);
	
	/* PART OF src/core/cs_akick.c - cs_akick_stick();
	 * Useage of mysql_quote();, mysql_sel_var(); and domysql(); */
        if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_cs_akick` WHERE `channel`='%s' AND `mask`='%s'",mysql_quote(thread,param[4]),mysql_quote(thread,param[6])))==1){
			domysql(thread,"UPDATE `orion_cs_akick` LEFT OUTER JOIN `orion_cs_core` ON `orion_cs_core`.`channel`=`orion_cs_akick`.`channel` SET `orion_cs_akick`.``flags`='1', `orion_cs_core`.`last_used`='%ld' WHERE `orion_cs_akick`.`channel`='%s' AND `mask`='%s'",time(NULL),mysql_quote(thread,param[4]),mysql_quote(thread,param[6]));
            send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_STICK_MASK",param[6],param[4]);
            do_mode(thread,CBot,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",param[6]);

		}else send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_MASK_NOT_IN_AKICK",param[6],param[4]);
	
/**********************************************************************************/

/* Thread/Channel Information management */

	/* To manage channel settings and most users/target user settings use:  (services.c) */
	void UpdateChannelInformation(Threads *thread, char *channel, char *target);
	void UpdateChannelInformation2(Threads *thread, char *channel, char *target);
	void UpdateChannelInformation3(Threads *thread, char *channel, char *target);
	
	/* Most needed will be: */
	void UpdateChannelInformation(Threads *thread, char *channel, char *target);
	/* It retrieves all important information about target channel ...
	 * you can pass *target as NULL to get only the channels information
	 * or you pass a nickname as *target to get information about that nick in *channel
	 *  in detail his accesslevel in that channel and some other stuff */
	 
	 /* IMPORTANT: We are programming Threadsafe so EVER take in mind:
	  * - If you use it you get mysql-information passed to our thread
	  *   if you now do i.e.: "UPDATE `orion_cs_core` SET `channel`='' WHERE `channel`='#bla'"
	  *   and use ANY of orion's functions
	  *   (for example do_mode() - here we set/delete modes and save new modes to Database IMMEDIATELY;
	  *    but you deleted the channel before so we will get a thread->c.chan.name[0]=='\0' from the SELECT in do_mode();)
	  *   it selects all again and after mode you are again up to the newest date!!
	  
	  * - This means: doing raw Queries in modules and i.e. a do_mode(); afterwards; lets do_mode collect the new channel data
	  *   before continuing with setting modes - so when do_mode(); returns to your module ..
	  *
	  *		YOUR CHANNEL'S INFORMATION MIGHT HAVE CHANGED THEN!!!!
	  *
	  * 1) Look to "src/core/bs_set.c" in bs_set_nobot(); for an example of how to handle that
	  * 2) Look to "src/core/cs_drop.c" in cs_drop(); for an example of how to handle that*/

/* Other important functions you may need */

	/* Used to send modes depneding on the loaded IRC Protocol - you shouldn't have to think about anything
	 * IMPORTANT: Modes sent with do_mode(); will immediately effect the Database ..
	 *  if you don't want that (of whatever reason) take rawmodes - src/modes.c */
	PROTOCOL_BOOL do_mode(Threads *thread, char *nick, char *target, time_t date, const char *buf, ...);
	
	/* For more commands have a look at include/osocket.h and in src/ at the specific files */
	
/* Send stuff to the user who messaged us */
	
	/* This is to send a notice/privmsg to the user executing the command
	 * howtosend == 0 (notice), howtosend == 1 (privmsg)
	 * This is also VERY useable to handle language support in your module */
	PROTOCOL_BOOL moduleSendLang(Threads *thread, int language, bool howtosend, char *nick, char *target, char *message, ...);
	
	/* This is to send a notice/privmsg to the user executing the command
	 * howtosend == 0 (notice), howtosend == 1 (privmsg)
	 * This could be used to send ALREADY known (core) language strings
	 *
	 * NOTE: But it's ever better for third party modules to use msend(); from a const char * array
	 * this ensures that your langstrings can't ever be deleted!! (your module should ever run */
	PROTOCOL_BOOL send_lang(Threads *thread, void *lang, bool howtosend, char *nick, char *target, char *message, ...);
	
/* Where do you find what ? */
	/* mail.c - All stuff for sending emails */
	/* services.c - various stuff (look to include/services.h)
	 * there you have a kinda overview about what is in services.c */
	 
	/* string.c - various string messing stuff (look to include/services.h)
	 * there you have a kinda overview about what is in string.c */
	 
	/* dynamic.c - various string messing stuff (look to include/services.h)
	 * there you have a kinda overview about what is in dynamic.c
	 * DON'T USE dynunshift(); */
	 
 /* Yeah that's it - and now have some fun ^^ */

	/* If we want to keep on processing other modules we use MOD_CONT, otherwise MOD_STOP */
	return MOD_CONT;
}

MOD_BOOL cs_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	
	/* If we want to keep on processing other modules we use MOD_CONT, otherwise MOD_STOP */
	return MOD_CONT;
}

MOD_BOOL cs_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	
	/* If we want to keep on processing other modules we use MOD_CONT, otherwise MOD_STOP */
	return MOD_CONT;
}

/** END OF MODULEBODY **/

/*  ************************************************************************************************  */

/** MODULETAIL **/

/* The finish function, called before the module is unloaded */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	
	/* If we want to keep on processing other modules we use MOD_CONT, otherwise MOD_STOP */
	return MOD_CONT;
}

/** END OF MODULETAIL **/

/*  ************************************************************************************************  */
