/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 282                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-17 17:12:10 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file services.h
 *   @brief This is the main header.
 *
 *   It contains all functions which are used in the main moduls.
 */
/*  ************************************************************************************************  */

#ifndef SERVICES_H
#define SERVICES_H

/* Define our macro's */
#define sfree(x)	if ((x) != NULL) free(x)


/* Some Linux boxes (or maybe glibc includes) require this for the
 * prototype of strsignal(). */
#ifndef _GNU_SOURCE
  #define _GNU_SOURCE
#endif

/*
 * Includes
 ************************
 * Windows does not have:
 * unistd.h, grp.h,
 * netdb.h, netinet/in.h,
 * sys/socket.h, sys/time.h
 *
 * Windows requires:
 * winsock.h
 ************************/
 
#if INTTYPE_WORKAROUND
# undef int16
# undef int32
#endif
 
#include "sysconf.h"
#include "global.h"
#include "datatypes.h"

#include <sys/un.h>
#include <stdio.h>
#include <signal.h>					/* signal Handler */
#include <time.h>
#include <errno.h>
#include <limits.h>
#include <stdarg.h>
#include <ctype.h>

#ifdef HAVE_SYS_TYPES_H
  #include <sys/types.h>
#endif

#ifdef HAVE_SYS_STAT_H
  #include <sys/stat.h>	/* for umask() on some systems */
#endif

#ifdef HAVE_STDLIB_H
  #include <stdlib.h>
#endif

#ifdef HAVE_STRING_H
  #include <string.h>
#endif

#ifdef HAVE_MALLOC_H
  #include <malloc.h>
#endif

#ifdef USE_THREADS
  #include <pthread.h>				/* Thread Library (Maybe should be passed with Win32 conform Paths) */
#endif

#ifndef _WIN32
  #include <netdb.h>
  #include <netinet/in.h>
  #include <arpa/inet.h>
  #include <sys/socket.h>
  #include <sys/time.h>
  #include <grp.h>
  #include <dirent.h>
  
  #ifdef HAVE_UNISTD_H
    #include <unistd.h>
  #endif
  
  #ifdef HAVE_BACKTRACE
    #include <execinfo.h>
  #endif
#else
  #include <winsock.h>
  #include <winsock2.h>
  #include <afxwin.h>
  #include <windows.h>
  #include <sys/timeb.h>
  #include <direct.h>
  #include <io.h>
  
  #define vsnprintf               _vsnprintf
#endif

#ifdef USE_MYSQL
  #ifdef MYSQL_HEADER_PREFIX
    #include <mysql/mysql.h>
  #else
    #include <mysql.h>
  #endif
#endif

#ifdef USE_THREADS
  #define THREAD_SAFE mysql_thread_safe()
  #define MYSQL_CONNECTION_HANDLE &mysql
#else
  #define THREAD_SAFE 0
  #define MYSQL_CONNECTION_HANDLE &thread->mysql
#endif

#ifndef HAVE_BZERO
  #define bzero(x,y) memset(x,0,y)
#endif

/* Some SUN fixs */
#ifdef __sun
/* Solaris specific code, types that do not exist in Solaris'
 * sys/types.h
 **/
  #undef u_int8_t
  #undef u_int16_t
  #undef u_int32_t
  #undef u_int64_t
  #define u_int8_t uint8_t
  #define u_int16_t uint16_t
  #define u_int32_t uint32_t
  #define u_int64_t uint64_t

  #ifndef INADDR_NONE
    #define INADDR_NONE (-1)
  #endif
#endif

#ifdef __WINS__
#ifndef BKCHECK
#define BKCHECK
  extern "C" void __pfnBkCheck() {}
#endif
#endif

#define is_user 			0
#define is_services_user 	1
#define is_services_oper 	2
#define is_services_admin 	3
#define is_services_root 	4

#define VERSION "orion2.9-stable.rc1.2"
#define VERSION_NUMBER "2.9-stable.rc1.2"
#define PROJECTNAME "orion"
#define MAINPROJECT "deepServices"
#define YEAR "2005-2009"

#define SQLrow MYSQL_ROW
#define SQLres MYSQL_RES

/************************ STRUCTS FOR TRUST-SESSIONSYSTEM******************************/
/* This is caused by MySQL - since MySQL isn't fast enough to run so many SELECTS due to
 * many users are connecting very fast (in case of ServerLinks for example)
 * So we had to manage some Lists to dump Information into on starting orion.
 * The dumps for this Lists come in a period of ~5minutes, or by updating through the
 * trust command @ O
 */

void add_Trust(char *host, char* description, char* limit, Trust_art art, int fields);
void add_Trustgroup(char *group, long limit, int fields);
bool del_Trust(Trusts *trust);
int del_Trust_by_Group(char *group, Trust_art T_art);
int edit_Trust_by_Group(char *group, char *newgroup, Trust_art T_art);
bool del_Trustgroup(Trustgroups *trustgroup);
void DelTrustgroups(void);
void DelTrusts(void);
void GetTrustsSql(Threads *thread);
void GetTrustgroupsSql(Threads *thread);
Trusts *getTrust(char *host, Trust_art T_art);
Trustgroups *getTrustgroup(char *group);
long getHostLimit(char *host);

/**************************************************************************************/



void EndThread(void *in_thread);
bool KillThread(Threads *thread);
void NewThread(char *whatfor, void *(*func) (void *thread_in));
void NewTData(char *whatfor);
Threads *SeekThread(char *index);
#ifdef USE_THREADS  
  int my_pthread_mutex_unlock(pthread_mutex_t *mutex);
  int my_pthread_mutex_lock(pthread_mutex_t *mutex);
#endif

/**************************************************************************************/

/* Server data */

Servers *findserver(Threads *thread, Servers * s, char *name);
Servers *findserver_uid(Threads *thread, Servers * s, char *name);
Servers *findserver_next(Threads *thread, Servers * s, char *name);
Servers *new_server(Threads *thread, Servers * uplink, char *name, char *desc, short int flags, char *suid);
void server(Threads *thread, char *server, char *uplink, short int hopcount);
Servers* addServer(Threads *thread, char *source, char *servername, char *hops, char *descript, char *numeric);
void ts6_uid_init(void);
void ts6_uid_increment(unsigned int slot);
char *ts6_uid_retrieve(void);

/*************************************************************************/

Uchannels *GetUserChannel(Users *user, char *channel);
Users *GetUser(char *nick);
Users *GetUserByUID(char *uid);
int GetUsersInChannel(char *channel);
char *GetNewID(char *id);
char *GetUserLastUID(char *sid);
int GetUserIntID(char *id);

bool DelUser(char *channel, char *nick);
bool DelUserNick(char *nick);
bool DelUserServer(char *server);
bool DelChannelsForUser(char *nick);

Users *ChangeUser(char *oldnick, char *newnick);
Users *addUser(char *nick, char *server);
Uchannels *addUserChannel(Users *user, char *channel);

void PartAllChannels(Users *user);
Uchannels *PartChannel(Users *user, char *channel);
Uchannels *PartChannelNick(char *nick, char *channel);
//Uchannels *addUserChannelMore(Users *user, char *channel, char *lastmsg);

/**************************************************************************************/

Ulines *addUline(Threads *thread, char *in_server);
Ulines *getUline(Threads *thread, char *in_server);
bool delUline(Threads *thread, char *in_server);
bool delAllUlines(Threads *thread);

/**************************************************************************************/

int getModeNick(UserChanModes *ucmodes, size_t ucmodes_count, char *nick);
bool multiply_occurance(int nr, unsigned int max, char *param[]);

/**************************************************************************************/

LevelChanging GetLevelvalueByIndex(LevelChanging *leveltable, char *index);
void FillLevel(LevelChanging *level, char *index, int type, char *CSlevel, int XOPlevel, char *Qlevel);

/**************************************************************************************/

void addLimit(Threads *thread, char *channel, time_t date, bool flag, char* serv, long alimit);
bool delLimit(Threads *thread, Limits *limit);
void PerformLimits(Threads *thread);
void delLimits(Threads *thread);

/**************************************************************************************/

//CServe.c
void do_backtrace(Threads *thread, bool show_segheader, int steps);
void _orion_exit(void);
void orion_exit(Threads *thread);
void remove_pidfile(void);
extern int serverid;

//movedb.c
void anope_to_orion(Threads *thread, bool blob);

//mail.c
MailInfo *o_email(Threads *thread, char *sender, char *recipient, char *email_address, char *subject);
void c_email(Threads *thread, MailInfo *mail);
void s_email(Threads *thread, void *language, FILE *pipe, char *message, ...);
int MailValidate(const char *email);

//Stay.c
/**************************************************************************************/

void addStayChannel(Threads *thread, char *channel, char *botnick, int bot_joined, int cs_had_to_join);
bool delStayChannel(Threads *thread, char *channel);
bool delStayNick(Threads *thread, char *nick);
Stays *getStayChannel(Threads *thread, char *channel);
void PerformStay(Threads *thread);
void DeleteStay(Stays *stay);

/**************************************************************************************/

//Base64.c
const char *inttobase64(char *buf, unsigned int v, unsigned int count);
unsigned int base64toIP(char *s);

//services.c
bool check_expire(char *expirestring);
time_t expire(char *time_in);
void randomstring(char *array, size_t stellen);
void randomnumber(char *array, size_t stellen);
char *get_localtime(time_t time_in);

bool deopall(Threads *thread, char *service, char *channel);
bool unbanall(Threads *thread, char *service, char *channel);
bool devoiceall(Threads *thread, char *service, char *channel);
bool dehopall(Threads *thread, char *service, char *channel);
void defconlev(Threads *thread, int defconlevel);
bool do_mlock(Threads *thread, char *snick, char *channel, time_t date, char *modes, char *buf);

void get_last_function(char *ret, int steps);
int NSCheckSettings(void);
int BSCheckSettings(void);
void CSCheckBot(char *newbot, char *bot, long set, size_t size);
void QCheckBot(char *newbot, char *bot, long set, size_t size);
char *CSCheckBotInPlace(char *bot, long set);
char *QCheckBotInPlace(char *bot, long set);
int CSCheckSettings(void);
int CSAddBanType(int settings);
int CSAnalyseBantype(int settings);
int CSAnalyseSignKick(int settings);
char *BuildBanMask(Threads *thread, Condata *Cdata, int type);
void unknown_access(char *access, size_t len);
void UpdateChannelInformation(Threads *thread,char *channel,char *target);
void UpdateChannelInformation2(Threads *thread,char *channel,char *target);
void UpdateChannelInformation3(Threads *thread,char *channel,char *target);

//log.c
int get_logdate(char *date, int32 count, struct tm *tm);
bool open_logs(Threads *thread);
void close_logs(Threads *thread, char *lastmsg);
void remove_logs(Threads *thread);

bool open_all_logs(void);
void close_all_logs(char *message, ...);
void remove_all_logs(void);

void alog(Threads *thread, const char *message, ...);
void sqllog(Threads *thread, const char *message, ...);
void dlog(Threads *thread, int priority, const char *message, ...);

//encrypt.c
int orion_encrypt(const char *src, size_t len, char *dest, size_t size);
int orion_encrypt_in_place(char *buf, size_t size);
int orion_encrypt_check_len(size_t passlen, size_t bufsize);
int orion_decrypt(const char *src, char *dest, size_t size);
int orion_check_password(const char *plaintext, const char *password);
char* orion_check_encryption(void);
void binary_to_hex(unsigned char *bin, char *hex, size_t length);

//string.c
void CleanSocketString(char *string, size_t strsize);
void strclean(char *receive);
void c_lowercase(char *string);

#ifdef USE_THREADS
  #define _lc(x) 		lc(thread,x)
  #define _uc(x) 		uc(thread,x)

  char* lc(Threads *thread, char *str);
  char* uc(Threads *thread, char *str);
#else
  #define _lc(x) 		lc(x)
  #define _uc(x) 		uc(x)

  char* lc(char *str);
  char* uc(char *str);
#endif

void strunshift(char *addchar, char *string, size_t strsize);

/* string manipulations */


void strrepc(char *search_string, char nower, char newchar); //Replaces nower(char) with newchar(char) in the *search_string

void strrepcof(char *search_string, char nower, char newchar); //Replaces nower(char) with newchar(char) in the *search_string but it replaces only the first found nower.

size_t strsplit(char *search_string, char *param[], char repchar); //splits search_string to a *param[] array by the occurance of repchar (it's deleted), returns int (number of array items)


char *sstrdup(const char *s); //duplicates a string
void sort(char *string); //Sorts the string and overwrites it with the sorted result!
/************************/


/* Delete from a string */
void substr(char *string, size_t position);
void strdelc(char *search_string, char delchar);
void strdelcof(char *search_string, char delchar);
void strdels(char *search_string, char search, char *delchar);
/************************/

/* Add to a string */
void straddc(char *string, char addchar); //Adds addchar to the end of the already existing *string
void stradds(char *string, char *addstring); //Adds *addstring to the already existing *string (make sure there's enough space)
void straddsni(char *string, char *addstring); //Adds *addstring to the already existing *string (make sure there's enough space)
/************************/

/* Add to a string (SAFE LIBRARY) */
void straddnc(char *string, char addchar, size_t strsize); //Adds addchar to the end of the already existing *string
void straddns(char *string, char *addstring, size_t strsize); //Adds *addstring to the already existing *string (make sure there's enough space)
void straddnsni(char *string, char *addstring, size_t strsize); //Adds *addstring to the already existing *string (make sure there's enough space)
/************************/

/* Equalize strings */
bool streq(char *str1, char *str2); //case equal
bool streqnc(char *str1, char *str2); //no case equal
/********************/

/* Search in a String */
bool strcor(char *string, char *chr); //case - strcor("abcdefghi","hko"); - return 1 cause of  "h" isn *string
bool stricor(char *string, char *chr); //no case - strcor("abcdefghi","HKO"); - return 1 cause of  "H/h" isn *string

bool strcin(char chr, char *str); //case - chr is in *str
bool stricin(char chr, char *str); //no case - chr is in *str

bool strins(char *string_to_search, char *search_string); //case - *string_to_search is in *search_string
void strmatch(char *string_to_search, char *search_string, char *stringmatch, size_t strmatchsize);
bool strinsnc(char *string_to_search, char *search_string); //no case - *string_to_search is in *search_string

bool str_start(char *instring, char *parameter);
bool str_end(char *instring, char *parameter);
uint32 str_is_ip(char *str);

bool strcsea(char *string, char *search_in); //case - strcsea("abcdefghi","a|b|c&d|e|f"); if (a b or c) and (d e or f) is in *string

int stoi(char* convert);
long stol(char* convert);

char *myStrGetToken(const char *str, const char dilim, int token_number);
char *myStrGetOnlyToken(const char *str, const char dilim, int token_number);
char *myStrGetTokenRemainder(const char *str, const char dilim, int token_number);
char *myStrSubString(const char *src, int start, int end);
char *str_signed(unsigned char *str);
/**********************/
int maxValue(int a, ...);
int findchar(char *mode_string, char findchar);
int getoccurance_x(char *mode_string, char findchar, int x);
int getposition(char *mode_string, char findchar);

/* string proofings and char manipultions */
char lower(char value);
bool digitcol(char *string);
bool alphacol(char *string);
bool is_valid_chars(char *text, char *allowed);
/******************************************/

//dynamic.c
void dynsplice(char *value[], size_t x, size_t y);
bool dynunshift(char *value[], char *string, size_t x, size_t size);
void reverse(char *target, char *source);
void join(char *receive, char *param[], char inputchar); /* You shouldn't use that function - it's unsafe */
#ifdef USE_THREADS
  #define _djoin(w,x) 				djoin(thread,w,x)
  #define _dstrjoin(w,x,y) 			dstrjoin(thread,w,x,y)
  #define _dstrjoin_det(w,x,y,z) 	dstrjoin_det(thread,w,x,y,z)
  #define _dynadd(...) 				dynadd(thread, __VA_ARGS__)

  char* djoin(Threads *thread, char inputchar, char *param[]);
  char* dstrjoin(Threads *thread, char inputchar, char *param[], size_t begin);
  char* dstrjoin_det(Threads *thread, char inputchar, char *param[], size_t begin, size_t end);
  char* dynadd(Threads *thread, const char *text, ...);
#else
  #define _djoin(w,x) 				djoin(w,x)
  #define _dstrjoin(w,x,y) 			dstrjoin(w,x,y)
  #define _dstrjoin_det(w,x,y,z) 	dstrjoin_det(w,x,y,z)
  #define _dynadd(...) 				dynadd(__VA_ARGS__)

  char* djoin(char inputchar, char *param[]);
  char* dstrjoin(char inputchar, char *param[], size_t begin);
  char* dstrjoin_det(char inputchar, char *param[], size_t begin, size_t end);
  char* dynadd(const char *text, ...);
#endif
size_t dynlen(char *string);
void dynncat(char *source, char *data, size_t len);
void dyncpy(char *target, char *source);
void dynncpy(char *target, char *source, size_t length);
char* dyndup(char *source);
size_t arraysize(char *param[]);

//mysql.c
void mysql_connect(Threads *thread);
void mysql_disconnect(Threads *thread);
void mysql_free_result_myr(SQLres *res);

char* mysql_quote(Threads *thread, char *sql);
void mysql_free_quotes(Threads *thread);
void ExecuteQuery(Threads *thread, char *query, size_t len);
void ExecuteQuery_r(Threads *thread, char *query, size_t len);
char *InlineExecuteQuery_r(Threads *thread, char *query, size_t len);

char* mysql_sel_var(Threads *thread, const char *text, ...);
void domysql(Threads *thread, const char *text, ...);
int mysql_free(void);
void mysql_sel(Threads *thread, char *nick, char *text);

void sql_BsChan(Threads *thread, char *nick, char *channel, BsChan *bsChan);
char* sql_BsChan2(Threads *thread, char *nick, char *target, char *channel, BsChan *bsChan);

void sql_chanmodes(Threads *thread, char *channel, Channel *c, ChanLevels *cLev);
ChanCA sql_chandata(Threads *thread, char *channel,char *authnick,char *target);
ChannelDB sql_chan_db(Threads *thread, char *channel);
DB3 sql_chan_data(Threads *thread, char *starttype, int parameters, ...);
DB4 sql_thread_data(Threads *thread, char *starttype, int parameters, ...);
Channel sql_chan(Threads *thread, char *channel);
ChanLevels sql_chanlevels(Threads *thread, char *channel);

void sql_mainAAC(Threads *thread, char *nick);
Access sql_access(Threads *thread, char *authnick);
Aliases sql_aliases(Threads *thread, char *aliasnick);
Bassign sql_assign(Threads *thread, char *channel);
Bkick sql_bkick(Threads *thread, char *channel);
Acclist sql_acclist(Threads *thread, const char *serv, char *channel, char *authnick);
Condata sql_condata(Threads *thread, char *nick);

int count(Threads *thread, const char *serv, char *authnick);
char* currmodes(Threads *thread, char *nick);
char *sql_users(Threads *thread, char *chan, char *nick);
char* acclevel(Threads *thread, char *serv, char *channel, char *authnick);
char* accessselect(Threads *thread, const char *field, char *authnick);
char* selectaliases(Threads *thread, const char *field, char *nnick);
int SingleLevel(Threads *thread, const char *field, char *channel);
Trust O_sql_trust(Threads *thread, char *host);
QUserlist *Q_userlist(Threads *thread, char *pattern_in);
SQLres *doSelectMySQL(Threads *thread, const char *text, ...);

SQLres *AkickEnforce(Threads *thread, char *channel);
SQLres *AkickOnJoin(Threads *thread, char *channel, char *nick);
SQLres *AkickOnSJoin(Threads *thread, char *channel, char *nicks);

size_t doBufferedInsert(Threads *thread, bool duplicate_update, size_t directtable, const char *input, ...);
void MySQL_FlushInserts(Threads *thread, size_t begin, size_t end);
void MySQL_CleanInserts(size_t begin, size_t end);

void DropAuthnick(Threads *thread, char *authnick);

//modules.c
size_t getmodules(Threads *thread);

//config.c
void loadconf(Threads *thread, char *modul);
void moduleLoadconfig(Threads *thread, ModCo *module, char *configfile);
void dumpconf_mysql(Threads *thread);
bool proof_config(Threads *thread);
void zero_config_values(Threads *thread);
void main_build(Threads *thread, char *desc,char *value);

//protocol.c & ptcsupport.c
bool LoadProtocol(Threads *thread, Protocol *protocol, char *IRCd);
bool on_protocol (Threads *thread, Protocol *protocol, size_t ac, char *param[]);

void CreateProtocol(Threads *thread, Protocol *protocol, void *dl_handle);
void event_init    (Threads *thread, Protocol *protocol, void *dl_handle, char *Author, char *Version);

void createEvent(Threads *thread, Protocol *protocol,
	const char *call,
	MOD_BOOL (*func) (Threads *thread, size_t ac, char **av),
	size_t check_param
	);
	
//sharedlanguage.c
void InitializeSharedLanguages(void);
bool addModuleLanguage(Threads *thread, ModInfo *ModHandle, Languages lang, char *item, char *value);
char *GetSharedLangstring(Threads *thread, int language, char *message);
void delSharedLanguage(Threads *thread, char *modulefile);
#endif
