/*  ************************************************************************************************
 * 
 *	Copyright (C) 2004-2009
 * 		Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *  	Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 * 
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 * 
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 * 
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 * 
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file bs_act.c
 *   @brief This is the Module file for BotServ ACT Command.
 * 
 * 	 It contains all functions which are neccessary to initialize and use<br/>
 *   the BotServ ACT command.
 *  
 *   The BotServ ACT Command allows the user to let his assigned Channel Bot<br/>
 *   (if there is a bot assigned) act with something in his channel.
 * 
 * 	 Syntax is: /msg BotServ ACT #channel Act-Message
 */
/*  ************************************************************************************************  */ 

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "bs"
#define MODUL "act"
#define MODUL_FILE "bs_act"
#define MODUL_VERSION "1.0"

MOD_BOOL bs_act(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_act_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_act_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"act","act");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"act",bs_act,is_services_oper,is_services_user,5,5);
	addCommand(thread,&Module,"help act",bs_act_help,is_services_oper,is_services_user,4,4);
	addCommand(thread,&Module,"help",bs_act_help_listing,is_services_oper,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the BotServ ACT Command.
 * 
 * The BotServ ACT Command allows the user to let his assigned Channel Bot (if there is a bot<br/>
 * assigned) say something in his channel.
 *  
 * Syntax is: /msg BotServ ACT #channel Act-Message
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_act(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Act Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"act")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.bot[0]=='\0')									send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",thread->c.chan.name);
		
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
			if(thread->c.chan.levels.say<=stoi(thread->c.chan.access.ownlevel) || strcor(thread->c.chan.access.ownlevel,"mn") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"act")){
				main_raw(thread,":%s PRIVMSG %s :\1ACTION %s\1",thread->c.chan.bot,param[4],_dstrjoin(' ',param,5));
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the BotServ ACT Command (/msg BotServ HELP ACT)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP ACT,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP ACT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_act_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Act Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help act"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_ACT");
	else
		send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
		
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BotServ Help List Command - ACT part (/msg BotServ help)
 * 
 * When the user tries to get general BotServ help, using /msg BotServ HELP, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the ACT command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_act_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_BS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Act (Help Listing)
	 */
	if(Users_BS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_CMD_ACT");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
