/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_drop.c
 *   @brief This is the Module file for ChanServ DROP Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ DROP command.
 *
 *   The ChanServ DROP Command allows the user to drop target #channel<br/>
 *   (He has to to be the REAL ChannelOwner - MultiownerSystem - or a ServicesAdmin)
 *
 *   Syntax is: /msg ChanServ DROP #channel
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "drop"
#define MODUL_FILE "cs_drop"
#define MODUL_VERSION "1.0"

MOD_BOOL cs_drop(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_drop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_drop_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"drop","drop");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"drop",cs_drop,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help drop",cs_drop_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_drop_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ DROP Command.
 * 
 * The ChanServ DROP Command allows the user to drop target #channel<br/>
 *   (He has to to be the REAL ChannelOwner - MultiownerSystem - or a ServicesAdmin)
 *  
 * Syntax is: /msg ChanServ DROP #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_drop(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
#ifdef USE_THREADS
	char founder[MAX_NICK_LEN];
#endif
	
	/* Initialise Variables to (NULL) */
#ifdef USE_THREADS
	bzero(&founder,sizeof(founder));
#endif
	
	/*
	 * Drop Command
	 */
 	// :shock PRIVMSG chanserv :aop #channel add nick
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"drop")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv") && (~thread->c.chan.settings & CS_FORBID_BIT)){
			if(streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) || (stoi(thread->c.chan.access.ownlevel)>=ACCESS_FOUNDER && ~thread->c.chan.settings&CS_SECUREFOUNDER_BIT) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"drop")){
        		
        		if(!streqnc(thread->sql.aliases.authnick,thread->c.chan.founder) && stoi(thread->c.chan.access.ownlevel)<=ACCESS_FOUNDER && thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"drop")){
					if(m->WallDrop)
						do_global(thread,snick,"%s used drop for channel %s",nick,thread->c.chan.name);
				}
        		
        		if(thread->c.chan.bot[0]!='\0'){
	        		do_privmsg(thread,thread->c.chan.bot,param[4],"I was unassigned from that channel (caused by dropping the channel), bye ladies and gentlemen");
					do_part(thread,thread->c.chan.bot,param[4],NULL);
					thread->c.chan.botinchan = 0;
				}
				
				if(thread->c.chan.settings&CS_INHABIT_BIT){
                    do_part(thread,cs->nick,param[4],NULL);
				}

#ifdef USE_THREADS
				dynncpy(founder,thread->c.chan.founder,sizeof(thread->c.chan.founder));
#endif

				if(thread->c.chan.countusers){
					domysql(thread,
						"UPDATE `orion_cs_core` SET "
						
							"`botnick`=NULL,`serv`='',`founder`=NULL,`successor`=NULL,`mode`=REPLACE(`mode`,'r',''),`email`='',`settings`='',`regdate`='',`description`='',`mlock_positive`='',"
							"`mlock_negative`='',`mlock_limit`='',`mlock_key`='',`mlock_flood`='',`mlock_redirect`='',`url`='',`alimit`='',`flags`='',"
							"`welcome`='',`last_used`='',`reason`='',`expire`='',`maxmemos`='',"
							"`cs_set`='',`cs_autodeop`='',`cs_autohalfop`='',`cs_autoop`='',`cs_autoprotect`='',`cs_autovoice`='',"
							"`cs_nojoin`='',`cs_signkick`='',`cs_acclist`='',`cs_accchange`='',`cs_akick`='',`cs_ban`='',`cs_banme`='',"
							"`cs_cclear`='',`cs_getkey`='',`cs_halfop`='',`cs_halfopme`='',`cs_info`='',`cs_kick`='',`cs_kickme`='',`cs_invite`='',"
							"`cs_opdeop`='',`cs_opdeopme`='',`cs_protect`='',`cs_protectme`='',`cs_topic`='',`cs_unban`='',`cs_voice`='',"
							"`cs_voiceme`='',`cs_memo`='',`cs_botassign`='',`cs_badwords`='',`cs_fantasia`='',`cs_greet`='',`cs_nokick`='',"
							"`cs_say`='',`bs_set`='-1',`bs_caps_ttb`='-1',`bs_caps_min`='-1',`bs_caps_per`='-1',`bs_flood_ttb`='-1',`bs_flood_ln`='-1',`bs_flood_secs`='-1',`bs_repeat_ttb`='-1',`bs_repeat`='-1',"
							"`bs_colors`='-1',`bs_underlines`='-1',`bs_reverses`='-1',`bs_badwords`='-1',`bs_bolds`='-1'"			
	
						" WHERE `channel`='%s' AND `serv`='ChanServ'",mysql_quote(thread,param[4])
					);
					
					domysql(thread,"DELETE `orion_cs_access`,`orion_cs_akick` FROM `orion_cs_access` LEFT OUTER JOIN `orion_cs_akick` ON (`orion_cs_akick`.`channel`=`orion_cs_access`.`channel`) WHERE `orion_cs_access`.`channel`='%s'",mysql_quote(thread,param[4]));
				}
				else
					domysql(thread,"DELETE `orion_cs_core`,`orion_ms_core` FROM `orion_cs_core` LEFT OUTER JOIN `orion_ms_core` ON (`orion_ms_core`.`receiver`=`orion_cs_core`.`channel`) WHERE `orion_cs_core`.`channel`='%s'",param[4]);
				
				if(IRC.ChanRegMode && thread->c.chan.countusers)
				/* if we are threadsafe we retrieve channel information
				 * with "UpdateChannelInformation3" in do_mode() again (we deleted the botnick via update before)
				 * means that we don't have "thread->c.chan.bot" filled properly in and after do_mode.
				 * (so we have to save it before here!) */
    				do_mode(thread,snick,param[4],thread->c.chan.datecreate,IRC.ModeOnChannelDrop);
    				
    			send_lang(thread,language,howtosend,snick,nick,"CS_DROP_DONE",param[4]);
    			
#ifdef USE_THREADS
				do_logchan(thread,snick,NULL,NULL,NULL,"Channel %s dropped by %s!%s@%s (founder: %s)",thread->c.chan.name,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,founder);
#else
                do_logchan(thread,snick,NULL,NULL,NULL,"Channel %s dropped by %s!%s@%s (founder: %s)",thread->c.chan.name,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,thread->sql.aliases.authnick);
#endif
				bzero(&thread->c,sizeof(thread->c));
            }else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
		}
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv") && (thread->c.chan.settings & CS_FORBID_BIT)){
			if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"drop")){
				if(m->WallDrop)
					do_global(thread,snick,"%s used drop for channel %s",nick,thread->c.chan.name);
					
        		if(thread->c.chan.bot[0]!='\0'){
	        		do_privmsg(thread,thread->c.chan.bot,param[4],"I was unassigned from that channel (caused by dropping the channel), bye ladies and gentlemen");
					do_part(thread,thread->c.chan.bot,param[4],NULL);
					thread->c.chan.botinchan = 0;
				}
				
				if(thread->c.chan.settings&CS_INHABIT_BIT){
                    do_part(thread,cs->nick,param[4],NULL);
				}

				if(thread->c.chan.countusers){
					domysql(thread,
						"UPDATE `orion_cs_core` SET "
						
							"`botnick`=NULL,`serv`='',`founder`=NULL,`successor`=NULL,`mode`=REPLACE(`mode`,'r',''),`email`='',`settings`='',`regdate`='',`description`='',`mlock_positive`='',"
							"`mlock_negative`='',`mlock_limit`='',`mlock_key`='',`mlock_flood`='',`mlock_redirect`='',`url`='',`alimit`='',`flags`='',"
							"`welcome`='',`last_used`='',`reason`='',`expire`='',`maxmemos`='',"
							"`cs_set`='',`cs_autodeop`='',`cs_autohalfop`='',`cs_autoop`='',`cs_autoprotect`='',`cs_autovoice`='',"
							"`cs_nojoin`='',`cs_signkick`='',`cs_acclist`='',`cs_accchange`='',`cs_akick`='',`cs_ban`='',`cs_banme`='',"
							"`cs_cclear`='',`cs_getkey`='',`cs_halfop`='',`cs_halfopme`='',`cs_info`='',`cs_kick`='',`cs_kickme`='',`cs_invite`='',"
							"`cs_opdeop`='',`cs_opdeopme`='',`cs_protect`='',`cs_protectme`='',`cs_topic`='',`cs_unban`='',`cs_voice`='',"
							"`cs_voiceme`='',`cs_memo`='',`cs_botassign`='',`cs_badwords`='',`cs_fantasia`='',`cs_greet`='',`cs_nokick`='',"
							"`cs_say`='',`bs_set`='-1',`bs_caps_ttb`='-1',`bs_caps_min`='-1',`bs_caps_per`='-1',`bs_flood_ttb`='-1',`bs_flood_ln`='-1',`bs_flood_secs`='-1',`bs_repeat_ttb`='-1',`bs_repeat`='-1',"
							"`bs_colors`='-1',`bs_underlines`='-1',`bs_reverses`='-1',`bs_badwords`='-1',`bs_bolds`='-1'"			
	
						" WHERE `channel`='%s' AND `serv`='ChanServ'",mysql_quote(thread,param[4])
					);
				
    				domysql(thread,"DELETE `orion_cs_access`,`orion_cs_akick` FROM `orion_cs_access` LEFT OUTER JOIN `orion_cs_akick` ON (`orion_cs_akick`.`channel`=`orion_cs_access`.`channel`) WHERE `orion_cs_access`.`channel`='%s' AND `orion_cs_access`.`serv`='ChanServ'",mysql_quote(thread,param[4]));
				}
				else
					domysql(thread,"DELETE `orion_cs_core`,`orion_ms_core` FROM `orion_cs_core` LEFT OUTER JOIN `orion_ms_core` ON (`orion_ms_core`.`receiver`=`orion_cs_core`.`channel`) WHERE `orion_cs_core`.`channel`='%s' AND `orion_cs_core`.`serv`='ChanServ'",mysql_quote(thread,param[4]));

				if(IRC.ChanRegMode && thread->c.chan.countusers)
					do_mode(thread,snick,param[4],thread->c.chan.datecreate,IRC.ModeOnChannelDrop);
					
    			send_lang(thread,language,howtosend,snick,nick,"CS_DROP_DONE",param[4]);
    			
    			do_logchan(thread,snick,NULL,NULL,NULL,"Channel %s dropped (from forbid) by %s!%s@%s (admin: %s)",thread->c.chan.name,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,thread->sql.aliases.authnick);

				bzero(&thread->c,sizeof(thread->c));
			}else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
		}
        else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ DROP Command (/msg ChanServ HELP DROP)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP DROP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP DROP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_drop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Drop Help
	 */
	if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"help drop"))
		send_lang(thread,language,howtosend,snick,nick,"CS_SERVADMIN_HELP_DROP");
	else if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help drop"))
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_DROP");

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - DROP part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the DROP command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_drop_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Drop (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_DROP");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
