/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Chrisostomos Adamos (alias: Ally), Email: Ally@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 203                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 23:10:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file hs_add.c
 *   @brief This is the Module file for HostServ ADD Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the HostServ ADD command.
 *
 *   The HostServ ADD Command allows the user to activate<br/>
 *   a requested vHost for the given nick.
 *
 *   Syntax is: /msg HostServ ADD < nick >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "hs"
#define MODUL "add"
#define MODUL_FILE "hs_add"
#define MODUL_VERSION "1.1"

MOD_BOOL hs_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_add_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	if(!IRC.VHost){
		Module.ModErr  = MOD_ERR_CANCELLED;
		Module.module  = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"add","add");

	addAuthor (thread,&Module,"Ally, cLiEbiG, (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"add",hs_add,is_services_oper,INFINIT,4,4);
	addCommand(thread,&Module,"help add",hs_add_help,is_services_oper,INFINIT,4,4);
	addCommand(thread,&Module,"help",hs_add_help_listing,is_services_oper,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the HostServ ADD Command.
 * 
 * The HostServ ADD Command allows the user to activate<br/>
 *   a requested vHost for the given nick.
 *
 * Syntax is: /msg HostServ ADD < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Aliases aliases;
	deepSQL checkvhost;
	char virthost[MAX_IDENT_LEN + MAX_HOST_LEN +2];
    char* HostParts[2];
    char* vIdent = NULL;
    char* vHost  = NULL;
    size_t parts = 0;
    
	/* Initialise Variables to (NULL) */
	bzero(&aliases,sizeof(aliases));
	bzero(&checkvhost,sizeof(checkvhost));
	bzero(&virthost,sizeof(virthost));
	bzero(HostParts,sizeof(HostParts));

	/*
	 * Add Command
	 */
   	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"add")){

		checkvhost.res    = doSelectMySQL(thread,"SELECT `nick`, `active`, `vhost`, `authorized` FROM `orion_hs_core` WHERE `nick`='%s'",mysql_quote(thread,param[4]));
		checkvhost.fields = mysql_num_rows(checkvhost.res);
		
		if(checkvhost.fields){
			checkvhost.row = mysql_fetch_row(checkvhost.res);
			
			if(strstr(checkvhost.row[3],"") && stoi(checkvhost.row[1]) == 0){
				domysql(thread,"UPDATE `orion_hs_core` SET `authorized` = '%s',`changetime` = '%ld', `active` = '1' WHERE `nick`='%s'",mysql_quote(thread,thread->sql.aliases.authnick),time(NULL),mysql_quote(thread,param[4]));
				dynncpy(virthost,checkvhost.row[2],sizeof(virthost));

				aliases	= sql_aliases(thread,param[4]);
				parts	= strsplit(virthost,HostParts,'@');

				if(parts==2){ /* vIdent + vHost */
					vIdent = HostParts[0];
					vHost = HostParts[1];
					do_chgident(thread,snick,param[4],vIdent);
					do_chghost(thread,snick,param[4],vHost);
					send_lang(thread,language,howtosend,snick,nick,"HS_ADD_SUCCESS",_dynadd("%s@%s",vIdent,vHost),nick);
					
					if(aliases.authnick[0]!='\0'){
						domysql(thread,"INSERT INTO `orion_ms_core` (`sender`,`receiver`,`priority`,`rsend`,`mode`,`time`,`memo`) VALUES ('HostServ','%s','%d','0','0','%ld','%s')",
							mysql_quote(thread,aliases.authnick),0,time(NULL),
		    				mysql_quote(thread,_dynadd((const char*)dynsym(thread,language,"HS_ADD_ACTIVATED_MEMO"),checkvhost.row[2],param[4],m->network))
						);
					}
				}else if(parts==1){  /* nur vHost */
					vHost = HostParts[0];
					do_chghost(thread,snick,param[4],vHost);
					send_lang(thread,language,howtosend,snick,nick,"HS_ADD_SUCCESS",vHost,param[4]);
					
					if(aliases.authnick[0]!='\0'){
						domysql(thread,"INSERT INTO `orion_ms_core` (`sender`,`receiver`,`priority`,`rsend`,`mode`,`time`,`memo`) VALUES ('HostServ','%s','%d','0','0','%ld','%s')",
							mysql_quote(thread,aliases.authnick),0,time(NULL),
		    				mysql_quote(thread,_dynadd((const char*)dynsym(thread,language,"HS_ADD_ACTIVATED_MEMO"),checkvhost.row[2],param[4],m->network))
						);
					}
				}
				
				do_logchan(thread,snick,NULL,NULL,NULL,"vHost Added: (%s) by %s",checkvhost.row[2],nick,hs->nick,nick);
				
				if(GetUser(param[4])!=NULL)
					send_lang(thread,language,howtosend,snick,param[4],"HS_ADD_ACTIVATED",nick);
					
			}else send_lang(thread,language,howtosend,snick,nick,"HS_ADD_IS_ACTIVE",checkvhost.row[2],param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"HS_DEL_NO_VHOST",param[4]);
		
		/* mysql_free_result_myr(); - checkvhost */
		mysql_free_result_myr(checkvhost.res);
    }else{
	/*
	 * Error Messages
	 */
 		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
    }
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the HostServ ADD Command (/msg HostServ HELP ADD)
 * 
 * When the user tries to get help for this command, using /msg HostServ HELP ADD,<br/>
 * this function is executed.
 * 
 * Syntax: /msg HostServ HELP ADD
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Add Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help add"))
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_ADD",snick);
	else
	    send_lang(thread,language,howtosend,snick,nick,"HS_NO_HELP_AVAIL",snick);

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief HostServ Help List Command - ADD part (/msg HostServ help)
 * 
 * When the user tries to get general HostServ help, using /msg HostServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the ADD command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_add_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_HS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Add (Help Listing)
	 */
	if(Users_HS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_CMD_ADD");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
