/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Chrisostomos Adamos (alias: Ally), Email: Ally@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 203                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 23:10:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file hs_delall.c
 *   @brief This is the Module file for HostServ DELALL Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the HostServ DELALL command.
 *
 *   The HostServ DELALL Command allows the user to delete the vHost for all nick's<br/>
 *   in the same group as that of the given nick.
 *
 *   Syntax is: /msg HostServ DELALL < nick >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "hs"
#define MODUL "delall"
#define MODUL_FILE "hs_delall"
#define MODUL_VERSION "1.1"

MOD_BOOL hs_delall(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_delall_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_delall_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	
	if(!IRC.VHost){
		Module.ModErr = MOD_ERR_CANCELLED;
		Module.module = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"delall","delall");

	addAuthor (thread,&Module,"Ally, cLiEbiG, (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"delall",hs_delall,is_services_oper,INFINIT,4,4);
	addCommand(thread,&Module,"help delall",hs_delall_help,is_services_oper,INFINIT,4,4);
	addCommand(thread,&Module,"help",hs_delall_help_listing,is_services_oper,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the HostServ DELALL Command.
 * 
 * The HostServ DELALL Command allows the user to delete the vHost for all nick's<br/>
 *   in the same group as that of the given nick.
 *
 * Syntax is: /msg HostServ DELALL < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_delall(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL checkvhost;
    Aliases aliases5;
    
	/* Initialise Variables to (NULL) */
	bzero(&checkvhost,sizeof(checkvhost));
	bzero(&aliases5,sizeof(aliases5));
	
	/*
	 * Delall Command
	 */
   	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"delall")){

    	aliases5          = sql_aliases(thread,param[4]);
		checkvhost.res    = doSelectMySQL(thread,"SELECT `nick`, `display` FROM `orion_hs_core` WHERE `display`='%s' LIMIT 1",mysql_quote(thread,aliases5.authnick));
		checkvhost.fields = mysql_num_rows(checkvhost.res);
		
		if(checkvhost.fields){
			checkvhost.row = mysql_fetch_row(checkvhost.res);
			
			if(checkvhost.row[1]!=NULL){
                                  
				domysql(thread,"DELETE FROM `orion_hs_core` WHERE `display`='%s' AND `active` = '1'",mysql_quote(thread,checkvhost.row[1]));
				send_lang(thread,language,howtosend,snick,nick,"HS_DELALL_DELETED",checkvhost.row[1]);
				
			}else send_lang(thread,language,howtosend,snick,nick,"HS_DELALL_NOT_ACTIVE",checkvhost.row[1],snick,param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"HS_DEL_NO_VHOST",param[4]);

		/* mysql_free_result_myr(); - checkvhost */
		mysql_free_result_myr(checkvhost.res);
    }else{
	/*
	 * Error Messages
	 */
 		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
    }
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the HostServ DELALL Command (/msg HostServ HELP DELALL)
 * 
 * When the user tries to get help for this command, using /msg HostServ HELP DELALL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg HostServ HELP DELALL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_delall_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Delall Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help delall"))
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_DELALL",snick);
	else
	    send_lang(thread,language,howtosend,snick,nick,"HS_NO_HELP_AVAIL",snick);
	    
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief HostServ Help List Command - DELALL part (/msg HostServ help)
 * 
 * When the user tries to get general HostServ help, using /msg HostServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the DELALL command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_delall_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_HS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Delall (Help Listing)
	 */
	if(Users_HS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_CMD_DELALL");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
