/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Chrisostomos Adamos (alias: Ally), Email: Ally@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 203                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 23:10:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file hs_group.c
 *   @brief This is the Module file for HostServ GROUP Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/> 
 *   the HostServ GROUP command.
 *
 *   The HostServ GROUP Command allows the user to set the vHost of their<br/>
 *   CURRENT nick to be the vHost for all nicks in the same (their) group.
 *
 *   Syntax is: /msg HostServ GROUP
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "hs"
#define MODUL "group"
#define MODUL_FILE "hs_group"
#define MODUL_VERSION "1.1"

MOD_BOOL hs_group(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_group_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL hs_group_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	
	if(!IRC.VHost){
		Module.ModErr = MOD_ERR_CANCELLED;
		Module.module = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"group","group");

	addAuthor (thread,&Module,"Ally, cLiEbiG (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"group",hs_group,is_services_oper,is_services_user,3,3);
	addCommand(thread,&Module,"help group",hs_group_help,is_services_oper,is_services_user,4,4);
	addCommand(thread,&Module,"help",hs_group_help_listing,is_services_oper,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the HostServ GROUP Command.
 * 
 * The HostServ GROUP Command allows the user to set the vHost of their<br/>
 *   CURRENT nick to be the vHost for all nicks in the same (their) group.
 *
 * Syntax is: /msg HostServ GROUP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_group(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
    deepSQL getaliases;
    deepSQL vhostgroup;
    deepSQL checkvhost;
    int i = 0;
    
	/* Initialise Variables to (NULL) */
	bzero(&getaliases,sizeof(getaliases));
	bzero(&vhostgroup,sizeof(vhostgroup));
	bzero(&checkvhost,sizeof(checkvhost));

	/*
	 * Group Command
	 */
	 //:shock PRIVMSG HostServ :help
   	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"group")){

		getaliases.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s' ORDER BY `nick`",mysql_quote(thread,thread->sql.aliases.authnick));
		getaliases.fields = mysql_num_rows(getaliases.res);
		
		vhostgroup.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_hs_core` WHERE `display`='%s' ORDER BY `nick`",mysql_quote(thread,thread->sql.aliases.authnick));
		vhostgroup.fields = mysql_num_rows(vhostgroup.res);
		
        checkvhost.res    = doSelectMySQL(thread,"SELECT `nick`, `active`, `vhost`, `authorized` FROM `orion_hs_core` WHERE `nick`='%s'",mysql_quote(thread,nick));
		checkvhost.fields = mysql_num_rows(checkvhost.res);
		
		if(checkvhost.fields){
			checkvhost.row = mysql_fetch_row(checkvhost.res);
			if(checkvhost.row[3][0]!='\0'){
			
        		if(getaliases.fields == 1) domysql(thread,"UPDATE `orion_hs_core` SET `vhost` = '%s', `changetime`='%ld', `authorized`='%s', `active`='1' WHERE `display`='%s'",mysql_quote(thread,checkvhost.row[2]),time(NULL),mysql_quote(thread,checkvhost.row[3]),mysql_quote(thread,thread->sql.aliases.authnick));
        		else{
    				for(i = 0; i < getaliases.fields && vhostgroup.fields; i++){
    					getaliases.row = mysql_fetch_row(getaliases.res);
    					vhostgroup.row = mysql_fetch_row(vhostgroup.res);
    					
                        if(getaliases.row[0] || vhostgroup.row[0]){
        					if(streqnc(getaliases.row[0],vhostgroup.row[0]))
        						domysql(thread,"UPDATE `orion_hs_core` SET `vhost` = '%s', `changetime`='%ld', `authorized`='%s', `active`='1' WHERE `display`='%s'",mysql_quote(thread,checkvhost.row[2]),time(NULL),mysql_quote(thread,checkvhost.row[3]),mysql_quote(thread,thread->sql.aliases.authnick));
        					
                            else
        						domysql(thread,"INSERT INTO `orion_hs_core` (`nick`,`display`,`nick_type`,`vhost`,`authorized`,`changetime`,`active`) VALUES ('%s','%s','%d','%s','%s','%d','%d')",mysql_quote(thread,getaliases.row[i]),mysql_quote(thread,thread->sql.aliases.authnick),0,mysql_quote(thread,checkvhost.row[2]),mysql_quote(thread,checkvhost.row[3]),time(NULL),1);
        				}
                        else break;
                    }
                }
    			send_lang(thread,language,howtosend,snick,nick,"HS_SETALL_SUCCESS",checkvhost.row[2],thread->sql.aliases.authnick);
    		}else send_lang(thread,language,howtosend,snick,nick,"HS_DEL_NO_VHOST",nick);
		}else send_lang(thread,language,howtosend,snick,nick,"HS_DEL_NO_VHOST",nick);
		
		/* mysql_free_result_myr(); - getaliases, vhostgroup, checkvhost */
		mysql_free_result_myr(getaliases.res);
		mysql_free_result_myr(vhostgroup.res);
		mysql_free_result_myr(checkvhost.res);
    }else{
	/*
	 * Error Messages
	 */
 		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the HostServ GROUP Command (/msg HostServ HELP GROUP)
 * 
 * When the user tries to get help for this command, using /msg HostServ HELP GROUP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg HostServ HELP GROUP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_group_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Group Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help group"))
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_GROUP",snick);
	else
	    send_lang(thread,language,howtosend,snick,nick,"HS_NO_HELP_GROUP",snick);
		
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief HostServ Help List Command - GROUP part (/msg HostServ help)
 * 
 * When the user tries to get general HostServ help, using /msg HostServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the GROUP command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL hs_group_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_HS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Group (Help Listing)
	 */
	if(Users_HS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"HS_HELP_CMD_GROUP");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
