/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_group.c
 *   @brief This is the Module file for NickServ GROUP Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ GROUP command.
 *
 *   The NickServ GROUP Command allows the user to create<br/>
 *   a new user in a group - or changs the group of the target one.
 *
 *   Syntax is: /msg NickServ GROUP < target-nick > < group-password >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "group"
#define MODUL_FILE "ns_group"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_group(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_group_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_group_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"group","group");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"group",ns_group,is_user,is_user,5,5);
	addCommand(thread,&Module,"help group",ns_group_help,is_user,is_user,4,4);
	addCommand(thread,&Module,"help",ns_group_help_listing,is_user,is_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ GROUP Command.
 * 
 * The NickServ GROUP Command allows the user to create<br/>
 *   a new user in a group - or changs the group of the target one.
 *
 * Syntax is: /msg NickServ GROUP < target-nick > < group-password >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_group(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	char email[100], *checker[2], regdate[200];
	char tmppasswd[MAX_PASSWORD_LEN + 1];
	char From[80];
	char Subject[80];
	Access access5;
	Aliases aliases5;
	int AliasesEntries = 0;
	//int lastcheck=0, i=0;
	
	/* Initialise Variables to (NULL) */
	bzero(&email,sizeof(email));
	bzero(checker,sizeof(checker));
	bzero(&regdate,sizeof(regdate));
	bzero(&tmppasswd,sizeof(tmppasswd));
	bzero(&From,sizeof(From));
	bzero(&Subject,sizeof(Subject));
	bzero(&access5,sizeof(access5));
	bzero(&aliases5,sizeof(aliases5));
	
	/*
	 * Group Command
	 */
	 //   0     1     2    3           4                  5
	 //:shock PRIVMSG Q :hello Manuel-Moser@aon.at Manuel-Moser@aon.at
	 //   0       1       2         3        4           5
	 //:shock PRIVMSG NickServ :group      shock      34154424
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"group") && dynlen(param[5])<=MAX_PASSWORD_LEN){ //cmd group
		if(param[4]){
			if((!m->StrictPasswords && dynlen(param[5])>=ns->min_passwordlen) || (m->StrictPasswords && !strins(nick,param[5]) && dynlen(param[5])>=ns->min_passwordlen)){  //password longer than 5 charakters
				aliases5 = sql_aliases(thread,param[4]);
				access5  = sql_access(thread,aliases5.authnick);
			
				/*msend(thread,1,snick,nick,"mein nick = %s",nick);
				msend(thread,1,snick,nick,"der nick lt. SQL = %s",thread->sql.aliases.nick);
				msend(thread,1,snick,nick,"ist der nick aktiv? = %d",thread->sql.aliases.active);
				msend(thread,1,snick,nick,"m->NSNoGroupChange = %d",m->NSNoGroupChange);*/
			
				if(thread->sql.access.settings&NS_SUSPENDED){
					send_lang(thread,language,howtosend,snick,nick,"LOGIN_SUSPENDED");
					send_lang(thread,language,howtosend,snick,nick,"LOGIN_SUSPEND_REASON",thread->sql.access.reason);
				}
                else if(thread->sql.aliases.forbid) send_lang(thread,language,howtosend,snick,nick,"NS_GROUP_NICK_FORBIDDEN_WITH_REASON",thread->sql.aliases.last_quit);
				
/*registered*/  else if(streqnc(thread->sql.aliases.nick,nick) && thread->sql.aliases.active && (!m->NSNoGroupChange || thread->sql.aliases.linked)){
					//msend(thread,1,snick,nick,"if hamma mal");
					if(orion_check_password(param[5],access5.password)>0){
						AliasesEntries = stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_aliases` WHERE `display`='%s'",mysql_quote(thread,aliases5.authnick)));
						//msend(thread,1,snick,nick,"passwort stimmt auch");
						if(AliasesEntries<m->NSMaxAliases){
							//msend(thread,1,snick,nick,"aliases erlauben es");
							
							/* ADDED */
							char newdisplay[MAX_NICK_LEN*2];
							bzero(&newdisplay,sizeof(newdisplay));
							dynncpy(newdisplay,mysql_sel_var(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s' AND `nick`!='%s' LIMIT 0,1",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,thread->sql.aliases.authnick)),sizeof(newdisplay));
							/*********/
							
							if(newdisplay[0]=='\0'){ //ADDED
								/* With that setting all inforamtion about your old group will be deleted,
								   Anope uses it that way, but we want to avoid that this happens !!
								   So with orion you are forced to DROP the nick and reregister it with GROUP,
								   for your choosen Group!! (since this is the same like drop) */
								//DropAuthnick(thread, thread->sql.aliases.authnick); // - Read Information above
								//do_logchan(thread,snick,NULL,NULL,NULL,"deleting nickname group %s",thread->sql.aliases.authnick); // - Read Information above
	
								/* And here comes the security change ... */
								send_lang(thread,language,howtosend,snick,nick,"NS_GROUP_SEETINGS_MAY_BE_LOST");
								return MOD_CONT;
								
							}
							
							domysql(thread,"UPDATE `orion_ns_aliases` SET `display`='%s',`linked`='0' WHERE `nick`='%s'",mysql_quote(thread,aliases5.authnick),mysql_quote(thread,nick));
							
							if(newdisplay[0]!='\0' && streqnc(thread->sql.aliases.authnick,thread->sql.aliases.nick)){ //GROUP Display HAS TO BE MOVED! (need to change display) - ADDED
								domysql(thread,"UPDATE `orion_ns_core` SET `authnick`='%s' WHERE `authnick`='%s'",mysql_quote(thread,newdisplay),mysql_quote(thread,thread->sql.aliases.authnick));
								do_logchan(thread,snick,NULL,NULL,NULL,"%s changed %s group (display) to %s (e-mail: %s)",nick,thread->sql.aliases.authnick,newdisplay,thread->sql.access.email);
							}
							
							do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"makes %s join group of %s (%s) (e-mail: %s)",thread->sql.aliases.authnick,param[4],aliases5.authnick,access5.email);
							send_lang(thread,language,howtosend,snick,nick,"NS_GROUP_YOU_ARE_NOW_IN_GROUP_X",aliases5.authnick);
						}else send_lang(thread,language,howtosend,snick,nick,"NS_GROUP_TOO_MANY_ALIASES_FOR_X",aliases5.authnick);
					}else send_lang(thread,language,howtosend,snick,nick,"NO_LOGIN_OR_WRONG_PASSWORD");
/*not regged */ }else if(!streqnc(thread->sql.aliases.nick,nick) && !streqnc(thread->sql.aliases.authnick,nick) && aliases5.authnick[0]!='\0'){
					//msend(thread,1,snick,nick,"im not regged if");
					//dynncpy(tmppasswd,param[5],sizeof(tmppasswd));
					//Crypt ON/OFF
					if(orion_check_password(param[5],access5.password)>0){
						AliasesEntries = stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_aliases` WHERE `display`='%s'",mysql_quote(thread,aliases5.authnick)));
						
						if(AliasesEntries<m->NSMaxAliases){
							domysql(thread,"INSERT INTO `orion_ns_aliases` (`nick`,`display`,`last_usermask`,`last_realname`,"
													"`time_linked`,`last_login`,`last_used`,"
													"`expire`,`linked`,`active`)"
													"VALUES ('%s','%s','%s','%s','%ld','%ld','%ld','%d','0','1')",
										mysql_quote(thread,nick),mysql_quote(thread,aliases5.authnick),mysql_quote(thread,thread->sql.Cdata.vhost),mysql_quote(thread,thread->sql.Cdata.realname),time(NULL),time(NULL),time(NULL),aliases5.expire);
		

							if(IRC.NickRegMode){
								do_svs2mode(thread,ns->nick,nick,IRC.ModeOnNickReg);
							}
								
							if(IRC.p10protocol){
								do_add_account(thread,GetUser(nick)->uid,aliases5.authnick);
							}
								
							do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"makes %s join group of %s (%s) (e-mail: %s)",nick,param[4],aliases5.authnick,access5.email);
							send_lang(thread,language,howtosend,snick,nick,"NS_GROUP_YOU_ARE_NOW_IN_GROUP_X",aliases5.authnick);
						}else send_lang(thread,language,howtosend,snick,nick,"NS_GROUP_TOO_MANY_ALIASES_FOR_X",aliases5.authnick);
					}else send_lang(thread,language,howtosend,snick,nick,"NO_LOGIN_OR_WRONG_PASSWORD");
				}else{
					//msend(thread,1,snick,nick,"garkein if");
					send_lang(thread,language,howtosend,snick,nick,"NICK_REGISTERED_BY_AN_OTHER_USER");
				}
			}else send_lang(thread,language,howtosend,snick,nick,"PASSWORD_TOO_SHORT",ns->min_passwordlen);
		}else send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_SYNTAX",ns->nick);
	}else{
	/*
	 *  Error Messages
	 */
	 	if(dynlen(param[4])>MAX_PASSWORD_LEN)	send_lang(thread,language,howtosend,snick,nick,"PASSWORD_TOO_LONG",MAX_PASSWORD_LEN);
		else									send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ GROUP Command (/msg NickServ HELP GROUP)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP GROUP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP GROUP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_group_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Group Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help group"))
		send_lang(thread,language,howtosend,ns->nick,nick,"NS_HELP_GROUP",snick,snick,snick);

	else do_notice_lang(thread,language,ns->nick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - GROUP part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the GROUP command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_group_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Group (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_GROUP");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
