/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_set.c
 *   @brief This is the Module file for NickServ SET Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ SET command.
 *
 *   The NickServ SET Command allows the user to set various SET-Options<br/>
 *   to his profile Information. (Controls DISPLAY/PASSWORD etc.)
 *
 *   Syntax is: /msg NickServ SET < option > < value ... >
 *
 *<table border="0">
 *    <tr><td> DISPLAY	</td><td> Syntax is: /msg NickServ SET DISPLAY	</td><td> < new-display >									</td></tr>
 *    <tr><td> PASSWORD	</td><td> Syntax is: /msg NickServ SET PASSWORD	</td><td> < new-password >									</td></tr>
 *    <tr><td> LANGUAGE	</td><td> Syntax is: /msg NickServ SET LANGUAGE	</td><td> { 1-5 }											</td></tr>
 *    <tr><td> URL		</td><td> Syntax is: /msg NickServ SET URL		</td><td> < new-Url >										</td></tr>
 *    <tr><td> EMAIL	</td><td> Syntax is: /msg NickServ SET EMAIL	</td><td> < new-Email >										</td></tr>
 *    <tr><td> ICQ		</td><td> Syntax is: /msg NickServ SET ICQ		</td><td> < IcqUIN >										</td></tr>
 *    <tr><td> GREET	</td><td> Syntax is: /msg NickServ SET GREET	</td><td> < New-GreetMessage >								</td></tr>
 *    <tr><td> KILL		</td><td> Syntax is: /msg NickServ SET KILL		</td><td> { ON | QUICK | IMMED | OFF }						</td></tr>
 *    <tr><td> SECURE	</td><td> Syntax is: /msg NickServ SET SECURE	</td><td> { ON | OFF }										</td></tr>
 *    <tr><td> PRIVATE	</td><td> Syntax is: /msg NickServ SET PRIVATE	</td><td> { ON | OFF }										</td></tr>
 *    <tr><td> HIDE		</td><td> Syntax is: /msg NickServ SET HIDE		</td><td> { EMAIL | USERMASK | STATUS | QUIT } { ON | OFF }	</td></tr>
 *    <tr><td> MSG		</td><td> Syntax is: /msg NickServ SET MSG		</td><td> { ON | OFF }										</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "set"
#define MODUL_FILE "ns_set"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_set_display(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_password(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_language(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_url(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_email(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_greet(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_icq(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_kill(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
//MOD_BOOL ns_set_secure(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_private(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_hide(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_msg(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
//--------------------------------------------------------------------------------------

MOD_BOOL ns_set_display_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_password_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_language_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_url_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_email_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_greet_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_icq_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_kill_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
//MOD_BOOL ns_set_secure_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_private_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_hide_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_msg_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL ns_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"set option[1]","set option[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"set",is_services_user,is_services_user); /* just to add a displaylevel for /msg ns help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"set display",ns_set_display,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"set password",ns_set_password,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"set language",ns_set_language,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"set url",ns_set_url,is_services_user,is_services_user,4,5);
	addCommand(thread,&Module,"set email",ns_set_email,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"set greet",ns_set_greet,is_services_user,is_services_user,4,0);
	addCommand(thread,&Module,"set icq",ns_set_icq,is_services_user,is_services_user,4,5);
	addCommand(thread,&Module,"set kill",ns_set_kill,is_services_user,is_services_user,5,5);
	//addCommand(thread,&Module,"set secure",ns_set_secure,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"set private",ns_set_private,is_services_user,is_services_user,5,6);
	addCommand(thread,&Module,"set hide",ns_set_hide,is_services_user,is_services_user,6,6);
	addCommand(thread,&Module,"set msg",ns_set_msg,is_services_user,is_services_user,5,5);
	
	addCommand(thread,&Module,"help set",ns_set_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help set display",ns_set_display_help,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"help set password",ns_set_password_help,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"help set language",ns_set_language_help,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"help set url",ns_set_url_help,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"help set email",ns_set_email_help,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"help set greet",ns_set_greet_help,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"help set icq",ns_set_icq_help,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"help set kill",ns_set_kill_help,is_services_user,is_services_user,5,5);
	//addCommand(thread,&Module,"help set secure",ns_set_secure_help,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"help set private",ns_set_private_help,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"help set hide",ns_set_hide_help,is_services_user,is_services_user,5,5);
	addCommand(thread,&Module,"help set msg",ns_set_msg_help,is_services_user,is_services_user,5,5);

	addCommand(thread,&Module,"help",ns_set_help_listing,is_services_user,is_services_user,3,3);
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Display function for the NickServ SET Command.
 * 
 * The NickServ SET Display Command allows a user to change it's<br/>
 *   Main-GroupNick to another Nick in the users NickGroup.
 *  
 * Syntax is: /msg NickServ SET DISPLAY < newdisplay >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_display(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /*
     * Set Display Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set display")){
		if(!streqnc(thread->sql.access.authnick,param[5])){
			if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_ns_aliases` WHERE `nick`='%s' AND `display`='%s'",mysql_quote(thread,param[5]),mysql_quote(thread,thread->sql.access.authnick)))){
				domysql(thread,"UPDATE `orion_ns_core` LEFT OUTER JOIN `orion_ms_core` ON (`orion_ms_core`.`receiver`=`orion_ns_core`.`authnick`) SET `orion_ns_core`.`authnick`='%s',`orion_ms_core`.`receiver`='%s' WHERE `orion_ns_core`.`authnick`='%s'",mysql_quote(thread,param[5]),mysql_quote(thread,param[5]),mysql_quote(thread,thread->sql.aliases.authnick));
				//domysql(thread,"UPDATE `orion_cs_access` SET `authnick`='%s' WHERE `authnick`='%s'",mysql_quote(thread,param[5]),mysql_quote(thread,thread->sql.aliases.authnick));
				
				//domysql(thread,"UPDATE `orion_cs_core` SET `founder`='%s' WHERE `founder`='%s'",mysql_quote(thread,param[5]),mysql_quote(thread,thread->sql.aliases.authnick));
				//domysql(thread,"UPDATE `orion_cs_core` SET `successor`='%s' WHERE `successor`='%s'",mysql_quote(thread,param[5]),mysql_quote(thread,thread->sql.aliases.authnick));
				/* 
				InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
				domysql(thread,"UPDATE `orion_ns_aliases` SET `display`='%s' WHERE `display`='%s'",mysql_quote(thread,param[5]),mysql_quote(thread,thread->sql.aliases.authnick));
				*/
				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_DISPLAY",param[5]);
				do_logchan(thread,snick,NULL,NULL,NULL,"changing %s nickname group display to %s",thread->sql.access.authnick,param[5]);
				
			}else send_lang(thread,language,howtosend,snick,nick,"NS_SET_DISPLAYNICK_IS_NOT_IN_YOUR_GROUP");
		}else send_lang(thread,language,howtosend,snick,nick,"NS_SET_DISPLAYNICK_X_ALREADY_USED",param[5]);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Password function for the NickServ SET Command.
 * 
 * The NickServ SET Password Command allows a user to change it's<br/>
 *   Password!
 *  
 * Syntax is: /msg NickServ SET PASSWORD < new-Password >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_password(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
    char tmppasswd[MAX_PASSWORD_LEN + 1];
	
	/* Initialise Variables to (NULL) */
	bzero(&tmppasswd,sizeof(tmppasswd));
    
    /*
     * Set Password Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set password") && dynlen(param[5])<=MAX_PASSWORD_LEN){
			if(dynlen(param[5])>=ns->min_passwordlen){
				if(is_valid_chars(param[5],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-.:<>|^!\"$%&/()=?*'#,;~{[]}\\`")){
					send_lang(thread,language,howtosend,snick,nick,"INVALID_PASSWORD_CHARS","0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-.:<>|^!\"$%&/()=?*'#,;~{[]}\\`");
					return MOD_CONT;
				}

    			char enc_password[ENC_PASSMAX+1];
    			dynncpy(tmppasswd,param[5],sizeof(tmppasswd));
				bzero(&enc_password,sizeof(enc_password));
				
    			if (orion_encrypt(tmppasswd, strlen(tmppasswd), enc_password, ENC_PASSMAX) < 0) {
    				//memset(tmppasswd, 0, PASSWORD_LEN);
    				alog(thread,"%s: Failed to encrypt password for %s (register)",
    					ns->nick, nick);
    				send_lang(thread,language,howtosend,snick,nick,"REGISTRATION_FAILED");
    				return MOD_CONT;
    			}
    			domysql(thread,"UPDATE `orion_ns_core` SET `password`='%s' WHERE `authnick`='%s'",mysql_quote(thread,enc_password),mysql_quote(thread,thread->sql.aliases.authnick));

				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_PASSWORD",param[5]);
				do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"(e-mail: %s) changed its password.",thread->sql.access.email);
            }else send_lang(thread,language,howtosend,snick,nick,"PASSWORD_TOO_SHORT",ns->min_passwordlen);
	}else{
	/*
	 *  Error Messages
	 */
		if(dynlen(param[5])>MAX_PASSWORD_LEN)	send_lang(thread,language,howtosend,snick,nick,"PASSWORD_TOO_LONG",MAX_PASSWORD_LEN);
		else									send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Language function for the NickServ SET Command.
 * 
 * The NickServ SET Language Command allows a user to change it's<br/>
 *   Language Package!
 *  
 * Syntax is: /msg NickServ SET LANGUAGE < new-Language (1-5) >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_language(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	void *define = NULL;
    
    /*
     * Set Language Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set language")){
		if(digitcol(param[5]) && (stoi(param[5])==1 || stoi(param[5])==2)){
			domysql(thread,"UPDATE `orion_ns_core` SET `language`='%d' WHERE `authnick`='%s'",stoi(param[5])-1,mysql_quote(thread,thread->sql.aliases.authnick));
			define=dynsym(thread,languages[stoi(param[5])-1],"LANGUAGE");
			send_lang(thread,languages[stoi(param[5])-1],howtosend,snick,nick,"NS_SET_CHANGED_LANGUAGE",define);
        }else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[5]);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Url function for the NickServ SET Command.
 * 
 * The NickServ SET Url Command allows a user to change it's<br/>
 *   Url (Homepage)!
 *  
 * Syntax is: /msg NickServ SET URL < new-Url >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_url(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /*
     * Set Url Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set url")){
        if(!param[5]){
			domysql(thread,"UPDATE `orion_ns_core` SET `url`='' WHERE `authnick`='%s'",mysql_quote(thread,thread->sql.aliases.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_URL_NONE");
		}else if(!is_valid_chars(param[5],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-_.:/%")){
			domysql(thread,"UPDATE `orion_ns_core` SET `url`='%s' WHERE `authnick`='%s'",mysql_quote(thread,param[5]),mysql_quote(thread,thread->sql.aliases.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_URL",param[5]);
		}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[5]);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Email function for the NickServ SET Command.
 * 
 * The NickServ SET Email Command allows a user to change it's<br/>
 *   Email-Address!
 *  
 * Syntax is: /msg NickServ SET EMAIL < new-EmailAddress >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_email(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	char From[80];
	char Subject[80];
	char passcode[PASSWORD_LEN];
	
	/* Initialise Variables to (NULL) */
	bzero(&From,sizeof(From));
	bzero(&Subject,sizeof(Subject));
	bzero(&passcode,sizeof(passcode));
	
    /*
     * Set Email Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set email")){
		/*dynncpy(email,param[5],sizeof(email));
		
		if(!strcsea(email,"@&.")){
			send_lang(thread,language,howtosend,snick,nick,"NS_SET_EMAIL_INVALID_EMAIL");
			return MOD_CONT;
		}
		
		strsplit(email,domain,'@');
		dlog(thread,3,"domain[0] = %s\n",domain[0]);
		dlog(thread,3,"domain[1] = %s\n",domain[1]);
		strsplit(domain[1],parts,'.');
		
		if(domain[0]=='\0' || parts[0]=='\0' || parts[1]=='\0' || is_valid_chars(parts[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_.") || is_valid_chars(parts[1],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_.") || is_valid_chars(domain[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_.")){
			dlog(thread,3,"parts[0] validation = %d\n",is_valid_chars(parts[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_."));
			dlog(thread,3,"parts[1] validation = %d\n",is_valid_chars(parts[1],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_."));
			dlog(thread,3,"domain[0] validation = %d\n",is_valid_chars(domain[0],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_."));

			dlog(thread,3,"parts[0] = %s\n",parts[0]);
			dlog(thread,3,"parts[1] = %s\n",parts[1]);
			dlog(thread,3,"domain[0] = %s\n",domain[0]);

			send_lang(thread,language,howtosend,snick,nick,"NS_SET_EMAIL_INVALID_EMAIL");
			return MOD_CONT;
		}*/
		
		/*else */
		if(MailValidate(param[5])/*dynlen(parts[0])>=ns->min_emaildomainlen && dynlen(parts[1])>=ns->min_emaildomainlen*/){ //email ok and accepted
			if(m->NSEmailReg){
				randomstring(passcode,sizeof(passcode)-1);
				
				if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_request` WHERE `display`='%s'",mysql_quote(thread,thread->sql.access.authnick)))==0){
					domysql(thread,"INSERT INTO `orion_ns_request` (`display`,`passcode`,"
											  	"`email`,`reqtime`,`reqsent`,`emailchange`)"
										"VALUES ('%s','%s','%s','%ld','%ld','1')",
							mysql_quote(thread,thread->sql.access.authnick),mysql_quote(thread,passcode),mysql_quote(thread,param[5]),time(NULL),time(NULL));

					if(thread->sql.access.email[0]!='\0' && thread->sql.access.authnick[0]!='\0' && m->UseMail){
						dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ns->nick,m->SendFrom),sizeof(From));
						dynncpy(Subject,_dynadd("Your %s account (Your requested email change)",snick),sizeof(Subject));

						MailInfo *mail = o_email(thread,From,nick,param[5],Subject);
						//Email					
						s_email(thread,language,mail->pipe,"NS_CHANGE_EMAIL_REQUEST",thread->sql.access.authnick,passcode,ns->nick,0xA0,snick,ns->host,passcode,m->NSExpire,m->homepage);
						c_email(thread,mail);
					}

					do_logchan(thread,snick,NULL,NULL,NULL,"Nick %s has requested an email change (%s -> %s) - awaiting validation",thread->sql.aliases.nick,thread->sql.access.email,param[5]);
					send_lang(thread,language,howtosend,snick,nick,"NS_CHANGED_EMAIL_VALIDATION",thread->sql.access.authnick,param[5],(m->NSRExpire / 3600) % 24);
				}else send_lang(thread,language,howtosend,snick,nick,"NS_EMAIL_CHANGE_ALREADY_IN_PROGRESS");
			}else{
				domysql(thread,"UPDATE `orion_ns_core` SET `email`='%s' WHERE `authnick`='%s'",mysql_quote(thread,param[5]),mysql_quote(thread,thread->sql.aliases.authnick));
				do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"(e-mail: %s) changed its e-mail to %s",thread->sql.access.email,param[5]);
				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_EMAIL",param[5]);
			}
		}
		else send_lang(thread,language,howtosend,snick,nick,"NS_SET_EMAIL_INVALID_EMAIL");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Icq function for the NickServ SET Command.
 * 
 * The NickServ SET Icq Command allows a user to change it's<br/>
 *   ICQ-UIN!
 *  
 * Syntax is: /msg NickServ SET ICQ < icq-uin >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_icq(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){	
    /*
     * Set ICQ Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set icq")){
		if(!param[5]){
			domysql(thread,"UPDATE `orion_ns_core` SET `icq`='' WHERE `authnick`='%s'",mysql_quote(thread,thread->sql.aliases.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_ICQ_NONE");
		}else if(digitcol(param[5]) && !is_valid_chars(param[5],"0123456789")){
			domysql(thread,"UPDATE `orion_ns_core` SET `icq`='%ld' WHERE `authnick`='%s'",stol(param[5]),mysql_quote(thread,thread->sql.aliases.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_ICQ",stol(param[5]));
		}else send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[5]);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Greet function for the NickServ SET Command.
 * 
 * The NickServ SET Greet Command allows a user to change it's<br/>
 *   GreetMessage!
 *  
 * Syntax is: /msg NickServ SET GREET < new-GreetMessage >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_greet(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	char *greet = NULL;
    /*
     * Set Greet Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set greet")){
        if(!param[5]){
    		domysql(thread,"UPDATE `orion_ns_core` SET `greet`='' WHERE `authnick`='%s'",mysql_quote(thread,thread->sql.aliases.authnick));
    		send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_GREET_NONE");
		}else{
			greet = _dstrjoin(' ',param,5);
    		domysql(thread,"UPDATE `orion_ns_core` SET `greet`='%s' WHERE `authnick`='%s'",mysql_quote(thread,greet),mysql_quote(thread,thread->sql.aliases.authnick));
    		send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_GREET",greet);
        }
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Kill function for the NickServ SET Command.
 * 
 * The NickServ SET Kill Command allows a user to change it's<br/>
 *   Kill-Settings!
 *  
 * Syntax is: /msg NickServ SET KILL { ON | QUICK | IMMED | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_kill(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /*
     * Set Kill Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set kill")){
		if(streqnc(param[5],"immed")){
			thread->sql.access.settings |= NS_SET_KILL_IMMED;
			thread->sql.access.settings &= ~NS_SET_KILL_QUICK;
			thread->sql.access.settings &= ~NS_SET_KILLPROTECT;
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",thread->sql.access.settings,mysql_quote(thread,thread->sql.aliases.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_KILL",_uc(param[5]));
			return MOD_CONT;
		}else if(streqnc(param[5],"quick")){
			thread->sql.access.settings |= NS_SET_KILL_QUICK;
			thread->sql.access.settings &= ~NS_SET_KILLPROTECT;
			thread->sql.access.settings &= ~NS_SET_KILL_IMMED;
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",thread->sql.access.settings,mysql_quote(thread,thread->sql.aliases.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_KILL",_uc(param[5]));
			return MOD_CONT;
		}else if(streqnc(param[5],"on")){
			thread->sql.access.settings |= NS_SET_KILLPROTECT;
			thread->sql.access.settings &= ~NS_SET_KILL_QUICK;
			thread->sql.access.settings &= ~NS_SET_KILL_IMMED;
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",thread->sql.access.settings,mysql_quote(thread,thread->sql.aliases.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_KILL",_uc(param[5]));
			return MOD_CONT;
		}else if(streqnc(param[5],"off")){
			thread->sql.access.settings &= ~NS_SET_KILLPROTECT;
			thread->sql.access.settings &= ~NS_SET_KILL_QUICK;
			thread->sql.access.settings &= ~NS_SET_KILL_IMMED;
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",thread->sql.access.settings,mysql_quote(thread,thread->sql.aliases.authnick));
			send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_KILL",_uc(param[5]));
			return MOD_CONT;
		}else send_lang(thread,language,howtosend,snick,nick,"NS_SET_KILL_NO_SUCH_FUNCTION");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Secure function for the NickServ SET Command.
 * 
 * The NickServ SET Secure Command allows a user to change it's<br/>
 *   Secure State!
 *  
 * Syntax is: /msg NickServ SET SECURE { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_secure(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	
    /*
     * Set Secure Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set secure")){
		if(streqnc(param[5],"on"))
			thread->sql.access.settings |= NS_SET_SECURE;
			
		else if(streqnc(param[5],"off")) 
			thread->sql.access.settings &= ~NS_SET_SECURE;
			
		else return MOD_CONT;
		
		domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",thread->sql.access.settings,mysql_quote(thread,thread->sql.aliases.authnick));
		send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_SECURE",_uc(param[5]));
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Private function for the NickServ SET Command.
 * 
 * The NickServ SET Private Command allows a user to change it's<br/>
 *   Private State!
 *  
 * Syntax is: /msg NickServ SET PRIVATE { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_private(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	
    /*
     * Set Private Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set private")){
		if(streqnc(param[5],"on"))
			thread->sql.access.settings |= NS_SET_PRIVATE;
			
		else if(streqnc(param[5],"off")) 
			thread->sql.access.settings &= ~NS_SET_PRIVATE;
			
		else return MOD_CONT;
		
		domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",thread->sql.access.settings,mysql_quote(thread,thread->sql.aliases.authnick));
		send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_PRIVATE",_uc(param[5]));
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Hide function for the NickServ SET Command.
 * 
 * The NickServ SET Hide Command allows a user to change it's<br/>
 *   Hide State!
 *  
 * Syntax is: /msg NickServ SET HIDE { EMAIL | USERMASK | STATUS | QUIT } { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_hide(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /*
     * Set Hide Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set hide")){
	//0001 0x01 - QUIT		NS_SET_HIDE_QUIT
	//0010 0x02 - USERMASK	NS_SET_HIDE_USERMASK
	//0100 0x04 - EMAIL		NS_SET_HIDE_EMAIL
		if(streqnc(param[6],"on")){
			if(streqnc(param[5],"email")){
				thread->sql.access.settings |= NS_SET_HIDE_EMAIL;
				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_HIDE_EMAIL_ON",snick);
			}else if(streqnc(param[5],"usermask")){
				thread->sql.access.settings |= NS_SET_HIDE_MASK;
				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_HIDE_USERMASK_ON",snick);
			}else if(streqnc(param[5],"quit")){
				thread->sql.access.settings |= NS_SET_HIDE_QUIT;
				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_HIDE_QUIT_ON",snick);
			}else if(streqnc(param[5],"status")){
				thread->sql.access.settings |= NS_SET_HIDE_STATUS;
				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_HIDE_STATUS_ON",snick);
			}else{
                send_lang(thread,language,howtosend,ns->nick,nick,"NS_SET_HIDE_NO_SUCH_FUNCTION_OR_ALREADY_SET");
                return MOD_CONT;
            }
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",thread->sql.access.settings,mysql_quote(thread,thread->sql.aliases.authnick));
			//send_lang(thread,language,howtosend,snick,nick,"NS_COMMAND_DONE");
		}else if(streqnc(param[6],"off")){
			if(streqnc(param[5],"email")){
				thread->sql.access.settings &= ~NS_SET_HIDE_EMAIL;
				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_HIDE_EMAIL_OFF",snick);
			}else if(streqnc(param[5],"usermask")){
				thread->sql.access.settings &= ~NS_SET_HIDE_MASK;
				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_HIDE_USERMASK_OFF",snick);
			}else if(streqnc(param[5],"quit")){
				thread->sql.access.settings &= ~NS_SET_HIDE_QUIT;
				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_HIDE_QUIT_OFF",snick);
			}else if(streqnc(param[5],"status")){
				thread->sql.access.settings &= ~NS_SET_HIDE_STATUS;
				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_HIDE_STATUS_OFF",snick);
			}else{
                send_lang(thread,language,howtosend,ns->nick,nick,"NS_SET_HIDE_NO_SUCH_FUNCTION_OR_A_NOT_SET_HIDE_OPTION");
                return MOD_CONT;
            }
			domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",thread->sql.access.settings,mysql_quote(thread,thread->sql.aliases.authnick));
			//send_lang(thread,language,howtosend,snick,nick,"NS_COMMAND_DONE");
		}else send_lang(thread,language,howtosend,snick,nick,"NS_SET_HIDE_NO_SUCH_FUNCTION",param[6]);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}


/*  ************************************************************************************************  */

/**
 * @brief NS SET Msg function for the NickServ SET Command.
 * 
 * The NickServ SET Msg Command allows a users to change it's<br/>
 *   Msg Options (Decides weather to PRIVMSG or NOTICE that User)!<br/>
 *   If it's set to "off" NickServ will NOTICE you, otherwise he uses PRIVMSG
 *  
 * Syntax is: /msg NickServ SET MSG { ON | OFF }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_msg(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	
    /*
     * Set Msg Command
     */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set msg")){
		if(streqnc(param[5],"on"))
			thread->sql.access.settings |= NS_SET_MSG;
			
		else if(streqnc(param[5],"off")) 
			thread->sql.access.settings &= ~NS_SET_MSG;
			
		else return MOD_CONT;
		
		domysql(thread,"UPDATE `orion_ns_core` SET `settings`='%d' WHERE `authnick`='%s'",thread->sql.access.settings,mysql_quote(thread,thread->sql.aliases.authnick));
		send_lang(thread,language,thread->sql.access.settings&NS_SET_MSG ? 1 : 0,snick,nick,"NS_SET_CHANGED_MSG",_uc(param[5]));
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ SET Command (/msg NickServ HELP SET)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET",snick,snick,snick);

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Display Help function for the NickServ SET Command (/msg NickServ HELP SET DISPLAY)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET DISPLAY,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET DISPLAY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_display_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Display Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set display")){
			send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_DISPLAY");

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Password Help function for the NickServ SET Command (/msg NickServ HELP SET PASSWORD)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET PASSWORD,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET PASSWORD
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_password_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Password Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set password")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_PASSWORD");

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Language Help function for the NickServ SET Command (/msg NickServ HELP SET LANGUAGE)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET LANGUAGE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET LANGUAGE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_language_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Language Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set language")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_LANGUAGE");

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Url Help function for the NickServ SET Command (/msg NickServ HELP SET URL)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET URL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET URL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_url_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Url Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set url")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_URL");

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Email Help function for the NickServ SET Command (/msg NickServ HELP SET EMAIL)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET EMAIL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET EMAIL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_email_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Email Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set email")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_EMAIL");

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Icq Help function for the NickServ SET Command (/msg NickServ HELP SET ICQ)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET ICQ,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET ICQ
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_icq_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Icq Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set icq")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_ICQ");

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Greet Help function for the NickServ SET Command (/msg NickServ HELP SET GREET)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET GREET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET GREET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_greet_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Greet Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set greet")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_GREET");

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Kill Help function for the NickServ SET Command (/msg NickServ HELP SET KILL)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET KILL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET KILL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_kill_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Kill Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set kill")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_KILL",snick);

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Secure Help function for the NickServ SET Command (/msg NickServ HELP SET SECURE)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET SECURE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET SECURE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_secure_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Secure Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set secure")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_SECURE",snick,snick);

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Private Help function for the NickServ SET Command (/msg NickServ HELP SET PRIVATE)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET PRIVATE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET PRIVATE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_private_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Private Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set private")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_PRIVATE",snick,snick);

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Hide Help function for the NickServ SET Command (/msg NickServ HELP SET HIDE)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET HIDE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET HIDE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_hide_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Hide Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set hide")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_HIDE",snick);

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief NS SET Msg Help function for the NickServ SET Command (/msg NickServ HELP SET MSG)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SET MSG,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SET MSG
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_msg_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Msg Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set msg")){
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SET_MSG",snick);

	}else send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - SET part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SET command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Set (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_SET");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
