/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_countusers.c
 *   @brief This is the Module file for O COUNTUSERS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O COUNTUSERS command.
 *
 *   The O COUNTUSERS Command allows an admin to count and list all<br/>
 *   users matching a *nick-pattern*
 *
 *   Syntax is: /msg O COUNTUSERS *nick-pattern*
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "countusers"
#define MODUL_FILE "o_countusers"
#define MODUL_VERSION "1.0"

MOD_BOOL O_countusers(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_countusers_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_countusers_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"countusers","countusers");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"countusers",O_countusers,820,INFINIT,4,4);
	addCommand(thread,&Module,"help countusers",O_countusers_help,820,INFINIT,4,4);
	addCommand(thread,&Module,"help",O_countusers_help_listing,820,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the O COUNTUSERS Command.
 * 
 * The O COUNTUSERS Command allows an admin to count and list all<br/>
 *   users matching a *nick-pattern*
 *
 * Syntax is: /msg O COUNTUSERS *nick-pattern*
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_countusers(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL UserList;
	Condata Cdata;
	char pattern[BUFSIZE];
	int counter=0, i=0;

	/* Initialise Variables to (NULL) */
	bzero(&UserList,sizeof(UserList));
	bzero(&Cdata,sizeof(Cdata));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Countusers Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"countusers") && strcor(param[4],"*")){
		dynncpy(pattern,param[4],sizeof(pattern));
		
		strrepc(pattern,'*','%');

		UserList.res	= doSelectMySQL(thread,"SELECT `nick`,`ident`,`vhost` FROM `orion_core_condata` WHERE `nick` LIKE '%s'",mysql_quote(thread,pattern));
		UserList.fields	= mysql_num_rows(UserList.res);

		for (i=0;i<UserList.fields;i++) {
			UserList.row = mysql_fetch_row(UserList.res);
			send_lang(thread,language,howtosend,snick,nick,"O_COUNTUSERS_LIST_FOUND",++counter,UserList.row[0],UserList.row[1],UserList.row[2]);
		}

		if(counter>0)	send_lang(thread,language,howtosend,snick,nick,"O_COUNTUSERS_FOUND_NICKS_WITH_PATTERN",counter,param[4]);
		else			send_lang(thread,language,howtosend,snick,nick,"O_COUNTUSERS_FOUND_NOTHING_WITH_THAT_PATTERN",param[4]);

		/* mysql_free_result_myr(); - UserList */
		mysql_free_result_myr(UserList.res);
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[4],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[4]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O COUNTUSERS Command (/msg O HELP COUNTUSERS)
 * 
 * When the user tries to get help for this command, using /msg O HELP COUNTUSERS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP COUNTUSERS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_countusers_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Countusers Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help countusers")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"countusers"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_COUNTUSERS");
	}else send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - COUNTUSERS part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the COUNTUSERS command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_countusers_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Countusers (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_COUNTUSERS",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"countusers"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
