/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_list.c
 *   @brief This is the Module file for O LIST Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O LIST command.
 *
 *   The O LIST Command allows an admin to dump various<br/>
 *   information.
 *
 *   Syntax is: /msg O LIST { IDENTIFIED | VIDENTS | RIDENTS | REALNAMES | SUBNET | SUSPENDED | SCL } ...
 *
 *<table border="0">
 *    <tr><td> IDENTIFIED	</td><td> Syntax is: /msg O LIST IDENTIFIED	</td><td>						</td><td> (Dumps all identified users)									</td></tr>
 *    <tr><td> VIDENTS		</td><td> Syntax is: /msg O LIST VIDENTS	</td><td>						</td><td> (Shows the 20(default) most common vidents on the network)	</td></tr>
 *    <tr><td> RIDENTS		</td><td> Syntax is: /msg O LIST RIDENTS	</td><td>						</td><td> (Shows the 20(default) most common ridents on the network)	</td></tr>
 *    <tr><td> REALNAMES	</td><td> Syntax is: /msg O LIST REALNAMES	</td><td>						</td><td> (Shows the 20(default) most common realnames on the network)	</td></tr>
 *    <tr><td> SUBNET		</td><td> Syntax is: /msg O LIST SUBNET		</td><td> < *subnet-pattern* >	</td><td> (Dumps all users with a subnet that matches the pattern)		</td></tr>
 *    <tr><td> SUSPENDED	</td><td> Syntax is: /msg O LIST SUSPENDED	</td><td> { NICKS | CHANNELS }	</td><td> (Dumps all suspended nicks or channels)						</td></tr>
 *    <tr><td> SCL			</td><td> Syntax is: /msg O LIST SCL		</td><td> < limit >				</td><td> (Dumps all +p(private) and +s(secret) channels)				</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "list"
#define MODUL_FILE "o_list"
#define MODUL_VERSION "1.0"

MOD_BOOL O_list_identified(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_vidents(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_ridents(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_realnames(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_subnet(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_suspended(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_scl(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_list_identified_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_vidents_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_ridents_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_realnames_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_subnet_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_suspended_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_scl_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_list_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"list option[1]","list option[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"list",820,INFINIT); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"list identified",O_list_identified,870,INFINIT,4,4);
	addCommand(thread,&Module,"list vidents",O_list_vidents,850,INFINIT,4,5);
	addCommand(thread,&Module,"list ridents",O_list_ridents,850,INFINIT,4,5);
	addCommand(thread,&Module,"list realnames",O_list_realnames,850,INFINIT,4,0);
	addCommand(thread,&Module,"list subnet",O_list_subnet,820,INFINIT,5,5);
	addCommand(thread,&Module,"list suspended",O_list_suspended,850,INFINIT,5,5);
	addCommand(thread,&Module,"list scl",O_list_scl,870,INFINIT,4,5);

	addCommand(thread,&Module,"help list",O_list_help,820,INFINIT,4,4);
	addCommand(thread,&Module,"help list identified",O_list_identified_help,870,INFINIT,5,5);
	addCommand(thread,&Module,"help list vidents",O_list_vidents_help,850,INFINIT,5,5);
	addCommand(thread,&Module,"help list ridents",O_list_ridents_help,850,INFINIT,5,5);
	addCommand(thread,&Module,"help list realnames",O_list_realnames_help,850,INFINIT,5,5);
	addCommand(thread,&Module,"help list subnet",O_list_subnet_help,820,INFINIT,5,5);
	addCommand(thread,&Module,"help list suspended",O_list_suspended_help,850,INFINIT,5,5);
	addCommand(thread,&Module,"help list scl",O_list_scl_help,870,INFINIT,5,5);
	
	addCommand(thread,&Module,"help",O_list_help_listing,820,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST Identified function for the O LIST Command.
 * 
 * The O LIST Identified Command allows an admin to List all Identified users on the Network
 *  
 * Syntax is: /msg O LIST IDENTIFIED
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_identified(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL loginslist;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&loginslist,sizeof(loginslist));

	/*
	 * List Identified Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list identified")){
		loginslist.res    = doSelectMySQL(thread,"SELECT `nick`,`display` FROM `orion_ns_aliases` WHERE `active`='1' ORDER BY `nick` ASC LIMIT 80000");
		loginslist.fields = mysql_num_rows(loginslist.res);
		
		send_lang(thread,language,howtosend,snick,nick,"O_LIST_IDENTIFIED_LIST_HEADER");
		for(i=0;i<loginslist.fields;i++){
			loginslist.row = mysql_fetch_row(loginslist.res);
			
			send_lang(thread,language,howtosend,snick,nick,"O_LIST_IDENTIFIED_LIST",loginslist.row[0],loginslist.row[1]);
		}

		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");

		mysql_free_result_myr(loginslist.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST VIdents function for the O LIST Command.
 * 
 * The O LIST VIdents Command allows an admin to seek the DataBase for<br/>
 *   the most common VIDENTS and creates an Output
 *  
 * Syntax is: /msg O LIST VIDENTS [*vident*]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_vidents(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL ic;
	char ident[MAX_IDENT_LEN];
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&ic,sizeof(ic));
	bzero(&ident,sizeof(ident));

	/*
	 * List VIdents Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list vidents")){
		if(!param[5]){
			send_lang(thread,language,howtosend,snick,nick,"O_LIST_IDENTS_RANK_IDENT_TIMES");

			ic.res	   = doSelectMySQL(thread,"SELECT count( ident ) as countreal, `ident` FROM `orion_core_condata` GROUP BY `ident` ORDER BY countreal DESC,`ident` ASC LIMIT 20");
			ic.fields = mysql_num_rows(ic.res);
			
			for(i=0;i<ic.fields;i++){
				ic.row = mysql_fetch_row(ic.res);
				
				msend(thread,howtosend,snick,nick,"%d %s [%s]",i+1,ic.row[1],ic.row[0]);
			}

			/* mysql_free_result_myr(); - ic */
			mysql_free_result_myr(ic.res);

			send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		}else if(param[5]){
		// > :shock PRIVMSG O :ic *final*
			//strrepcof(param[0],'*','%');
			dyncpy (ident,param[5]);
			strrepc(ident,'*','%');

			ic.res    = doSelectMySQL(thread,"SELECT count( ident ) as countreal, `ident` FROM `orion_core_condata` WHERE `ident` LIKE '%s' GROUP BY `ident` ORDER BY countreal DESC, `ident` ASC LIMIT 80",ident);
			ic.fields = mysql_num_rows(ic.res);
			
			for(i=0;i<ic.fields;i++){
				ic.row = mysql_fetch_row(ic.res);
				send_lang(thread,language,howtosend,snick,nick,"O_LIST_IDENTS_IDENT_X_YTIMES_USED",ic.row[1],ic.row[0]);
			}
			
			strrepc(ident,'%','*');

			/* mysql_free_result_myr(); - ic */
			mysql_free_result_myr(ic.res);

			if(i==0)	send_lang(thread,language,howtosend,snick,nick,"O_LIST_IDENTS_FOUND_NOTHING_BY_SEARCHING_FOR_IDENT",ident);
			else 		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}


/*  ************************************************************************************************  */

/**
 * @brief O LIST RIdents function for the O LIST Command.
 * 
 * The O LIST RIdents Command allows an admin to seek the DataBase for<br/>
 *   the most common RIDENTS and creates an Output
 *  
 * Syntax is: /msg O LIST RIDENTS [*rident*]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_ridents(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL ic;
	char ident[MAX_IDENT_LEN];
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&ic,sizeof(ic));
	bzero(&ident,sizeof(ident));

	/*
	 * List RIdents Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list ridents")){
		if(!param[5]){
			send_lang(thread,language,howtosend,snick,nick,"O_LIST_IDENTS_RANK_IDENT_TIMES");

			ic.res	   = doSelectMySQL(thread,"SELECT count( rident ) as countreal, `rident` FROM `orion_core_condata` GROUP BY `rident` ORDER BY countreal DESC,`rident` ASC LIMIT 20");
			ic.fields = mysql_num_rows(ic.res);
			
			for(i=0;i<ic.fields;i++){
				ic.row = mysql_fetch_row(ic.res);
				
				msend(thread,howtosend,snick,nick,"%d %s [%s]",i+1,ic.row[1],ic.row[0]);
			}

			/* mysql_free_result_myr(); - ic */
			mysql_free_result_myr(ic.res);

			send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		}else if(param[5]){
		// > :shock PRIVMSG O :ic *final*
			//strrepcof(param[0],'*','%');
			dyncpy (ident,param[5]);
			strrepc(ident,'*','%');

			ic.res    = doSelectMySQL(thread,"SELECT count( rident ) as countreal, `rident` FROM `orion_core_condata` WHERE `rident` LIKE '%s' GROUP BY `rident` ORDER BY countreal DESC, `rident` ASC LIMIT 80",ident);
			ic.fields = mysql_num_rows(ic.res);
			
			for(i=0;i<ic.fields;i++){
				ic.row = mysql_fetch_row(ic.res);
				send_lang(thread,language,howtosend,snick,nick,"O_LIST_IDENTS_IDENT_X_YTIMES_USED",ic.row[1],ic.row[0]);
			}
			
			strrepc(ident,'%','*');

			/* mysql_free_result_myr(); - ic */
			mysql_free_result_myr(ic.res);

			if(i==0)	send_lang(thread,language,howtosend,snick,nick,"O_LIST_IDENTS_FOUND_NOTHING_BY_SEARCHING_FOR_IDENT",ident);
			else 		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST Realnames function for the O LIST Command.
 * 
 * The O LIST Realnames Command allows an admin to seek the DataBase for<br/>
 *   the most common REALNAMES and creates an Output
 *  
 * Syntax is: /msg O LIST REALNAMES [*realname*]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_realnames(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL rnc;
	char realname[300];
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&rnc,sizeof(rnc));
	bzero(&realname,sizeof(realname));

	/*
	 * List Realnames Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list realnames")){
		if(!param[5]){
			send_lang(thread,language,howtosend,snick,nick,"O_LIST_REALNAMES_RANK_REALNAME_TIMES");

			rnc.res	   = doSelectMySQL(thread,"SELECT count( realname ) as countreal, `realname` FROM `orion_core_condata` GROUP BY `realname` ORDER BY countreal DESC,`realname` ASC LIMIT 20");
			rnc.fields = mysql_num_rows(rnc.res);
			
			for(i=0;i<rnc.fields;i++){
				rnc.row = mysql_fetch_row(rnc.res);
				
				msend(thread,howtosend,snick,nick,"%d %s [%s]",i+1,rnc.row[1],rnc.row[0]);
			}

			/* mysql_free_result_myr(); - rnc */
			mysql_free_result_myr(rnc.res);

			send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		}else if(param[5]){
		// > :shock PRIVMSG O :rnc *final*
			dyncpy (realname,_dstrjoin(' ',param,5));
			strrepc(realname,'*','%');

			rnc.res    = doSelectMySQL(thread,"SELECT count( realname ) as countreal, `realname` FROM `orion_core_condata` WHERE `realname` LIKE '%s' GROUP BY `realname` ORDER BY countreal DESC, `realname` ASC LIMIT 80",realname);
			rnc.fields = mysql_num_rows(rnc.res);
			
			for(i=0;i<rnc.fields;i++){
				rnc.row = mysql_fetch_row(rnc.res);
				send_lang(thread,language,howtosend,snick,nick,"O_LIST_REALNAMES_REALNAME_X_YTIMES_USED",rnc.row[1],rnc.row[0]);
			}
			
			//strrepc(param[4],'%','*');
			//bzero(&realname,sizeof(realname));
			dynncpy(realname,_dstrjoin(' ',param,5),sizeof(realname));

			/* mysql_free_result_myr(); - rnc */
			mysql_free_result_myr(rnc.res);

			if(i==0)	send_lang(thread,language,howtosend,snick,nick,"O_LIST_REALNAMES_FOUND_NOTHING_BY_SEARCHING_FOR_REALNAME",realname);
			else 		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST Subnet function for the O LIST Command.
 * 
 * The O LIST Subnet Command allows an admin to seek the DataBase for<br/>
 *   Subnets matching a given pattern!
 *  
 * Syntax is: /msg O LIST SUBNET *subnet*
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_subnet(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL subnetlist;
	char pattern[BUFSIZE];
	int i=0;

	/* Initialise Variables to (NULL) */
	bzero(&subnetlist,sizeof(subnetlist));
	bzero(&pattern,sizeof(pattern));

	/*
	 * List Subnet Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list subnet") && strcor(param[5],"*")){
		dyncpy (pattern,param[5]);
		strrepc(pattern,'*','%');

		subnetlist.res    = doSelectMySQL(thread,"SELECT `nick`,`rhost`,`vhost`,`subnet` FROM `orion_core_condata` WHERE `subnet` LIKE '%s' LIMIT 100",mysql_quote(thread,pattern));
		subnetlist.fields = mysql_num_rows(subnetlist.res);

		if(subnetlist.fields>0){
			for(i=0;i<subnetlist.fields;i++){
				subnetlist.row = mysql_fetch_row(subnetlist.res);
				
				send_lang(thread,language,howtosend,snick,nick,"O_LIST_SUBNET_LIST_MATCHING_USERS",i+1,subnetlist.row[0],subnetlist.row[1],subnetlist.row[2],subnetlist.row[3]);
			}
		}
		//strrepc(param[4],'%','*');
		if(i != 0)
			send_lang(thread,language,howtosend,snick,nick,"O_LIST_SUBNET_FOUND_X_MATCHING_REALHOSTS_BY_SEARCHING_WITH_PATTERN",i,param[5]);
		else
			send_lang(thread,language,howtosend,snick,nick,"O_LIST_SUBNET_FOUND_NOTHING_BY_SEARCHING_WITH_PATTERN",param[5]);

		/* mysql_free_result_myr(); - subnetlist */
		mysql_free_result_myr(subnetlist.res);
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[5],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[4]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST Suspended function for the O LIST Command.
 * 
 * The O LIST Suspended Command allows an admin to seek the DataBase for<br/>
 *   Suspended Channels (Q,L,ChanServ)
 *  
 * Syntax is: /msg O LIST SUSPENDED { NICKS | CHANNELS }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_suspended(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL suspended;
	int count1=0, count2=0;

	/* Initialise Variables to (NULL) */
	bzero(&suspended,sizeof(suspended));

	/*
	 * List Suspended Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list suspended")){
		if(streqnc(param[5],"nicks")){
			count1=0;
			count2=-1;

			suspended.res	 = doSelectMySQL(thread,"SELECT `authnick` FROM `orion_ns_core` WHERE `settings`&0x00000400"); // NS_SUSPENDED
			suspended.fields = mysql_num_rows(suspended.res);

			if(suspended.fields) send_lang(thread,language,howtosend,snick,nick,"O_LIST_SUSPENDED_NICKS_HEADER"); //Single suspended nicks
			while(count1<suspended.fields){
				suspended.row = mysql_fetch_row(suspended.res);

				send_lang(thread,language,howtosend,snick,nick,"O_LIST_SUSPENDED_NICKS",count1,suspended.row[0]);
				count1++;
			}

			send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");

			mysql_free_result_myr(suspended.res); //FREE
		}else if(streqnc(param[5],"channels")){
			count1=0;
			suspended.res	 = doSelectMySQL(thread,"SELECT `serv`,`channel`,`url` FROM `orion_cs_core` WHERE `serv` IN ('Q','L','ChanServ') AND `settings`&0x00008000"); //CS_SUSPEND_BIT
			suspended.fields = mysql_num_rows(suspended.res);

			while(count1<suspended.fields){
				suspended.row = mysql_fetch_row(suspended.res);
				
				count1++;
				send_lang(thread,language,howtosend,snick,nick,"O_LIST_SUSPENDED_CHANNELS",count1,suspended.row[0],suspended.row[1],suspended.row[2]);
			}
			send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");

			mysql_free_result_myr(suspended.res); //FREE
		}else send_lang(thread,language,howtosend,snick,nick,"O_LIST_SUSPENDED_WRONG_SYNTAX");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST Scl function for the O LIST Command.
 * 
 * The O LIST Scl Command allows an admin to seek the DataBase for<br/>
 *   Private, or Secret Channels
 *  
 * Syntax is: /msg O LIST SCL [limit]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_scl(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL scl;
	int counter = 0;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&scl,sizeof(scl));

	/*
	 * List Scl Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list scl")){
		counter = 1;
		if(!param[5])
			scl.res = doSelectMySQL(thread,"SELECT `channel`,`mode`,`serv` FROM `orion_cs_core` WHERE `mode` REGEXP '(s|c)' ORDER BY `channel` ASC");
		else if(param[5] && digitcol(param[5]))
			scl.res = doSelectMySQL(thread,"SELECT `channel`,`mode`,`serv` FROM `orion_cs_core` WHERE `mode` REGEXP '(s|c)' ORDER BY `channel` ASC LIMIT %d",stoi(param[5]));
		
		scl.fields = mysql_num_rows(scl.res);

		if(scl.fields>0){
			for(i=0;i<scl.fields;i++){
				scl.row = mysql_fetch_row(scl.res);
				
				if(scl.row[1][0]!='\0'){
					if(scl.row[2][0]!='\0')
						send_lang(thread,language,howtosend,snick,nick,"O_LIST_SCL_LIST_CHANNEL_REGISTERED",counter,scl.row[0],scl.row[1]);
					else
						send_lang(thread,language,howtosend,snick,nick,"O_LIST_SCL_LIST_DEFAULT_RETURN_MESSAGE",counter,scl.row[0],scl.row[1]);
					counter++;
				}
			}
		}

		mysql_free_result_myr(scl.res); //free MySQL result
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O LIST Command (/msg O HELP LIST)
 * 
 * When the user tries to get help for this command, using /msg O HELP LIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list")) {
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_LIST");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST Identified Help function for the O LIST Command (/msg O HELP LIST IDENTIFIED)
 * 
 * When the user tries to get help for this command, using /msg O HELP LIST IDENTIFIED,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP LIST IDENTIFIED
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_identified_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List Identified Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list identified")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list identified"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_LIST_IDENTIFIED");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST VIdents Help function for the O LIST Command (/msg O HELP LIST VIDENTS)
 * 
 * When the user tries to get help for this command, using /msg O HELP LIST VIDENTS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP LIST VIDENTS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_vidents_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List VIdents Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list vidents")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list vidents"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_LIST_VIDENTS");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST RIdents Help function for the O LIST Command (/msg O HELP LIST RIDENTS)
 * 
 * When the user tries to get help for this command, using /msg O HELP LIST RIDENTS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP LIST RIDENTS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_ridents_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List RIdents Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list ridents")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list ridents"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_LIST_RIDENTS");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST Scl Help function for the O LIST Command (/msg O HELP LIST REALNAMES)
 * 
 * When the user tries to get help for this command, using /msg O HELP LIST REALNAMES,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP LIST REALNAMES
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_realnames_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List Realnames Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list realnames")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list realnames"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_LIST_REALNAMES");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST Subnet Help function for the O LIST Command (/msg O HELP LIST SUBNET)
 * 
 * When the user tries to get help for this command, using /msg O HELP LIST SUBNET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP LIST SUBNET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_subnet_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List Subnet Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list subnet")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list subnet"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_LIST_SUBNET");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST Suspended Help function for the O LIST Command (/msg O HELP LIST SUSPENDED)
 * 
 * When the user tries to get help for this command, using /msg O HELP LIST SUSPENDED,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP LIST SUSPENDED
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_suspended_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List Suspended Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list suspended")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list suspended"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_LIST_SUSPENDED");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O LIST Scl Help function for the O LIST Command (/msg O HELP LIST SCL)
 * 
 * When the user tries to get help for this command, using /msg O HELP LIST SCL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP LIST SCL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_scl_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List Scl Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list scl")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list scl"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_LIST_SCL");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - LIST part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the LIST command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_list_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * List (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_LIST",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
