/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 275                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:47:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_regexpgline.c
 *   @brief This is the Module file for O REGEXPGLINE Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O REGEXPGLINE command.
 *
 *   The O REGEXPGLINE Command allows an admin to manage the Realname-Glines.<br/>
 *   (these Glines will be set normal for nick!ident@realhost#ip=realname,<br/>
 *    but they are completely processed and controlled by orion!)
 *
 *   Syntax is: /msg O REGEXPGLINE { ADD | DEL | LIST | TEST } [< OPTION >] [< regexp[,regexp] > [:< duration(i.e. 2d) > < reason >]]
 *
 *<table border="0">
 *    <tr><td> ADD	</td><td> Syntax is: /msg O REGEXPGLINE ADD	</td><td> { G | K } < regexp > :< duration(i.e. 2d) > < reason ></td></tr>
 *    <tr><td> DEL	</td><td> Syntax is: /msg O REGEXPGLINE DEL	</td><td> { G | K } < regexp >									</td></tr>
 *    <tr><td> TEST	</td><td> Syntax is: /msg O REGEXPGLINE TEST</td><td> < regexp >											</td></tr>
 *    <tr><td> LIST	</td><td> Syntax is: /msg O REGEXPGLINE LIST</td><td> 														</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "regexpgline"
#define MODUL_FILE "o_regexpgline"
#define MODUL_VERSION "1.0"

MOD_BOOL O_regexpgline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_regexpgline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_regexpgline_test(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_regexpgline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_regexpgline_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_regexpgline_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_regexpgline_test_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_regexpgline_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_regexpgline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_regexpgline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"regexpgline option[1]","regexpgline helpoption[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);
	
	addCommand(thread,&Module,"regexpgline",NULL,870,INFINIT,0,0); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"regexpgline add",O_regexpgline_add,920,INFINIT,8,0);
	addCommand(thread,&Module,"regexpgline del",O_regexpgline_del,920,INFINIT,6,0);
	addCommand(thread,&Module,"regexpgline test",O_regexpgline_test,920,INFINIT,5,5);
	addCommand(thread,&Module,"regexpgline list",O_regexpgline_list,920,INFINIT,4,4);
	
	addCommand(thread,&Module,"help regexpgline",O_regexpgline_help,920,INFINIT,4,4);
	addCommand(thread,&Module,"help regexpgline add",O_regexpgline_add_help,920,INFINIT,5,5);
	addCommand(thread,&Module,"help regexpgline del",O_regexpgline_del_help,920,INFINIT,5,5);
	addCommand(thread,&Module,"help regexpgline test",O_regexpgline_test_help,920,INFINIT,5,5);
	addCommand(thread,&Module,"help regexpgline list",O_regexpgline_list_help,920,INFINIT,5,5);
		
	addCommand(thread,&Module,"help",O_regexpgline_help_listing,870,INFINIT,3,3); //100 - each Admin is able to see it.
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O REGEXPGLINE Add function for the O REGEXPGLINE Command.
 * 
 * The O REGEXPGLINE Add Command allows an Admin to add a REGEXP Gline/Kill for the given mask (nick!ident@realhost#ip=realname)
 *  
 * Syntax is: /msg O REGEXPGLINE ADD { G | K }[,{ G | K }] regexp[,regexp] :< duration > < reason >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_regexpgline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL condata;
	int expirestring	= -1;
	
	time_t  expiretime	= 0;
	time_t  date		= 0;
	int i 				= 0;
	
	char regexp[500];
	char reason[500];
	char query[MYSQL_BUF];
	char flag;
	char *QueryError = NULL;

	/* Initialise Variables to (NULL) */
	bzero(&condata,sizeof(condata));
	bzero(&regexp,sizeof(regexp));
	bzero(&reason,sizeof(reason));
	bzero(&query,sizeof(query));
	
	/*
	 * Regexpgline Add Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"regexpgline add")){
		//:shock PRIVMSG O :regexpgline add g ident :expire reason
		for(i=6;param[i]!=NULL;i++){
			if(param[i][0]==':'){
				if(check_expire(&param[i][1])){
					expirestring = i;
					break;
				}
			}else{
				straddns(regexp,param[i],sizeof(regexp));
				straddnc(regexp,' ',sizeof(regexp));
			}
		}

		if(expirestring<0){
			send_lang(thread,language,howtosend,snick,nick,"O_REGEXPGLINE_PARSER_ERROR_MISSING_CORRECT_EXPIRE");
			return MOD_CONT;
		}
		
		if(dynlen(param[5])>1){
			send_lang(thread,language,howtosend,snick,nick,"O_REGEXPGLINE_TOO_MANY_FLAGS");
			return MOD_CONT;
		}
		
		/* Clean our regular expression string */
		regexp[dynlen(regexp)-1]='\0';
		
		/* Evaluate Expiretime (duration) */
		expiretime=expire(&param[expirestring][1])+time(NULL);
		
		/* Evaluate Reasonstring */
		dynncpy(reason,_dstrjoin(' ',param,expirestring+1),sizeof(reason));

		date=time(NULL);
		if(stricor(param[5],"gk")){
			flag = param[5][0];

			// nick!ident@realhost#ip=realname
			//(.*)!shock@(.*)

			sprintf(query,"SELECT CONCAT('*@',`rhost`) AS `realhost`,`nick` FROM `orion_core_condata` WHERE CONCAT(`orion_core_condata`.`nick`,'!',`orion_core_condata`.`rident`,'@',`orion_core_condata`.`rhost`,'#',`orion_core_condata`.`ip`,'=',`orion_core_condata`.`realname`) REGEXP '%s' AND `faketype`='0'",mysql_quote(thread,regexp));
			QueryError = InlineExecuteQuery_r(thread,query,dynlen(query));
			
			if(QueryError != NULL){
				msend(thread,howtosend,snick,nick,QueryError);
			}else{
#ifdef USE_THREADS
				my_pthread_mutex_lock(&mysql_lock);
#endif
				condata.res		= mysql_store_result(MYSQL_CONNECTION_HANDLE);
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&mysql_lock);
#endif
				condata.fields 	= mysql_num_rows(condata.res);

				if(condata.fields){
					for(i=0;i<condata.fields;i++){
						condata.row = mysql_fetch_row(condata.res);
						
						if(lower(flag)=='g' && condata.row[0])
							do_gline(thread,m->servername,condata.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),expiretime,date,reason);
						
						else if(lower(flag)=='k' && condata.row[1])
							do_kill(thread,snick,condata.row[1],reason);
					}
				}
				
				if(lower(flag)=='g'){
					domysql(thread,"INSERT INTO `orion_core_glines` (`rng`,`line`,`gline`,`duration`,`addedby`,`reason`) VALUES ('5','G','%s','%ld','%s','%s') ON DUPLICATE KEY UPDATE `duration`=VALUES(`duration`),`addedby`=VALUES(`addedby`),`reason`=VALUES(`reason`)",mysql_quote(thread,regexp),expiretime,mysql_quote(thread,nick),mysql_quote(thread,reason));
				}else if(lower(flag)=='k'){
					domysql(thread,"INSERT INTO `orion_core_glines` (`rng`,`line`,`gline`,`duration`,`addedby`,`reason`) VALUES ('6','G','%s','%ld','%s','%s') ON DUPLICATE KEY UPDATE `duration`=VALUES(`duration`),`addedby`=VALUES(`addedby`),`reason`=VALUES(`reason`)",mysql_quote(thread,regexp),expiretime,mysql_quote(thread,nick),mysql_quote(thread,reason));
				}
			}

			mysql_free_result_myr(condata.res); //free MySQL result
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
		}else send_lang(thread,language,howtosend,snick,nick,"O_WRONG_PARAMETERS");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O REGEXPGLINE Del function for the O REGEXPGLINE Command.
 * 
 * The O REGEXPGLINE Del Command allows an Admin to delete a REGEXP Gline/Kill for the given mask (nick!ident@realhost#ip=realname)
 *  
 * Syntax is: /msg O REGEXPGLINE DEL { G | K }[{ G | K }..] < mask >[,< mask >]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_regexpgline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL rhost;
	char regexp[500];

	/* Initialise Variables to (NULL) */
	bzero(&rhost,sizeof(rhost));
	bzero(&regexp,sizeof(regexp));

	/*
	 * Regexpgline Del Command
	 */
	 //:shock PRIVMSG operserv@services.deepirc.net :regexpgline del K (.*)r(.*)!rofl(.*)@(.*)\.telekom\.at#(.*)=(.*)
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"regexpgline del")){
		//:shock PRIVMSG O :regexpgline del g ident
		if(dynlen(param[5])>1){
			send_lang(thread,language,howtosend,snick,nick,"O_REGEXPGLINE_TOO_MANY_FLAGS");
			return MOD_CONT;
		}
		
		/* Evaluate RegEXP */
		dynncpy(regexp,_dstrjoin(' ',param,6),sizeof(regexp));

		if(lower(param[5][0])=='g'){
			domysql(thread,"DELETE FROM `orion_core_glines` WHERE `gline`='%s' AND `rng`='5'",mysql_quote(thread,regexp));

			/*rhost.res    = doSelectMySQL(thread,"SELECT `rhost`,`nick` FROM `orion_core_condata` WHERE CONCAT(`orion_core_condata`.`nick`,'!',`orion_core_condata`.`rident`,'@',`orion_core_condata`.`rhost`,'#',`orion_core_condata`.`ip`,'=',`orion_core_condata`.`realname`) REGEXP '%s' AND faketype = '0'",mysql_quote(thread,regexp));
			rhost.fields = mysql_num_rows(rhost.res);

			if(rhost.fields){
				for(i=0;i<rhost.fields;i++){
					rhost.row = mysql_fetch_row(rhost.res);
						
					do_ungline(thread,m->servername,rhost.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),time(NULL));
				}
			}

			mysql_free_result_myr(rhost.res);*/
		}
		else if(lower(param[5][0])=='k')
			domysql(thread,"DELETE FROM `orion_core_glines` WHERE `gline`='%s' AND `rng`='6'",mysql_quote(thread,regexp));
		
		send_lang(thread,language,howtosend,snick,nick,"O_REGEXPGLINE_DEL_REMOVED_REGEXPGLINE",regexp);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O REGEXPGLINE Test function for the O REGEXPGLINE Command.
 * 
 * The O REGEXPGLINE Test Command allows an Admin to test a REGEXP (and reply with it's targets)
 *  
 * Syntax is: /msg O REGEXPGLINE TEST < mask >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_regexpgline_test(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL rhost;
	char query[MYSQL_BUF];
	char *QueryError = NULL;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&rhost,sizeof(rhost));
	bzero(&query,sizeof(query));

	/*
	 * Regexpgline Test Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"regexpgline test")){
		sprintf(query,"SELECT `nick`,`rident`,`rhost`,`ip`,`realname` FROM `orion_core_condata` WHERE CONCAT(`orion_core_condata`.`nick`,'!',`orion_core_condata`.`rident`,'@',`orion_core_condata`.`rhost`,'#',`orion_core_condata`.`ip`,'=',`orion_core_condata`.`realname`) REGEXP '%s' AND faketype = '0' LIMIT 50",mysql_quote(thread,param[5]));

		QueryError = InlineExecuteQuery_r(thread,query,dynlen(query)) ;
		if(QueryError != NULL){
			msend(thread,howtosend,snick,nick,QueryError);
		}else{
#ifdef USE_THREADS
			my_pthread_mutex_lock(&mysql_lock);
#endif
			rhost.res		= mysql_store_result(MYSQL_CONNECTION_HANDLE);
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&mysql_lock);
#endif
			rhost.fields 	= mysql_num_rows(rhost.res);

			if(rhost.fields){
				for(i=0;i<rhost.fields;i++){
					rhost.row = mysql_fetch_row(rhost.res);
				
					if(i==0) 	send_lang(thread,language,howtosend,snick,nick,"O_REGEXPGLINES_TEST_HEADER",param[5]);
								send_lang(thread,language,howtosend,snick,nick,"O_REGEXPGLINES_TEST_LIST_MATCHES",i+1,rhost.row[0],rhost.row[0],rhost.row[1],rhost.row[2],rhost.row[3],rhost.row[4]);
				}
			}

			mysql_free_result_myr(rhost.res); //free MySQL result

			send_lang(thread,language,howtosend,snick,nick,"O_REGEXPGLINES_TEST_FOOTER",rhost.fields<50 ? rhost.fields : 50,rhost.fields<50 ? rhost.fields : rhost.fields);
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O REGEXPGLINE List function for the O REGEXPGLINE Command.
 * 
 * The O REGEXPGLINE List Command allows an Admin to dump the list of REGEXP Glines/Kills
 *  
 * Syntax is: /msg O REGEXPGLINE LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_regexpgline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
    deepSQL MySQL;
	int i=0;
	char time_return[150];
	time_t gettime;
	
	/* Initialise Variables to (NULL) */
	bzero(&MySQL,sizeof(MySQL));
	bzero(&time_return,sizeof(time_return));
	
	/*
	 * Regexpgline List Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"regexpgline list")){
		MySQL.res    = doSelectMySQL(thread,"SELECT `gline`,`duration`,`addedby`,`reason`,`rng` FROM `orion_core_glines` WHERE `rng` IN (5,6) ORDER BY `rng` ASC");
		MySQL.fields = mysql_num_rows(MySQL.res);
		
		if(MySQL.fields>0){
			for(i=0;i<MySQL.fields;i++){
				MySQL.row = mysql_fetch_row(MySQL.res);
				
				if(stoi(MySQL.row[1])==0) bzero(&time_return,sizeof(time_return));
				else{
					gettime=(time_t)stol(MySQL.row[1]);
					//bzero(&time_return,sizeof(time_return));
					dynncpy(time_return,get_localtime(gettime),sizeof(time_return));
				}
			
				if(i==0) 						send_lang(thread,language,howtosend,snick,nick,"O_REGEXPGLINES_LIST_HEADER_TIME_NOW",get_localtime(time(NULL)));
				if(stoi(MySQL.row[4])==5) 		send_lang(thread,language,howtosend,snick,nick,"O_REGEXPGLINES_LIST_LIST_RG_LINES",i+1,MySQL.row[0],MySQL.row[2],time_return,MySQL.row[3]);
				else if(stoi(MySQL.row[4])==6)	send_lang(thread,language,howtosend,snick,nick,"O_REGEXPGLINES_LIST_LIST_RK_LINES",i+1,MySQL.row[0],MySQL.row[2],time_return,MySQL.row[3]);
			}
		}
		
        mysql_free_result_myr(MySQL.res); //free MySQL result
		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O REGEXPGLINE Command (/msg O HELP REGEXPGLINE)
 * 
 * When the user tries to get help for this command, using /msg O HELP REGEXPGLINE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP REGEXPGLINE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_regexpgline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Regexpgline Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help regexpgline")){
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_REGEXPGLINE");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O REGEXPGLINE Add Help function for the O REGEXPGLINE Command (/msg O HELP REGEXPGLINE ADD)
 * 
 * When the user tries to get help for this command, using /msg O HELP REGEXPGLINE ADD,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP REGEXPGLINE ADD
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_regexpgline_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Regexpgline Add Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help regexpgline add")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"regexpgline add"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_REGEXPGLINE_ADD",snick,snick);
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O REGEXPGLINE Del Help function for the O REGEXPGLINE Command (/msg O HELP REGEXPGLINE DEL)
 * 
 * When the user tries to get help for this command, using /msg O HELP REGEXPGLINE DEL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP REGEXPGLINE DEL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_regexpgline_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Regexpgline Del Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help regexpgline del")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"regexpgline del"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_REGEXPGLINE_DEL",snick);
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O REGEXPGLINE Test Help function for the O REGEXPGLINE Command (/msg O HELP REGEXPGLINE TEST)
 * 
 * When the user tries to get help for this command, using /msg O HELP REGEXPGLINE TEST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP REGEXPGLINE TEST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_regexpgline_test_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Regexpgline Test Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help regexpgline test")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"regexpgline test"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_REGEXPGLINE_TEST",snick);
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O REGEXPGLINE List Help function for the O RNLINE Command (/msg O HELP REGEXPGLINE LIST)
 * 
 * When the user tries to get help for this command, using /msg O HELP REGEXPGLINE LIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP REGEXPGLINE LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_regexpgline_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Regexpgline List Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help regexpgline list")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"regexpgline list"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_REGEXPGLINE_LIST");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - REGEXPGLINE part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the REGEXPGLINE command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_regexpgline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Regexpgline (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_REGEXPGLINE",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"regexpgline"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
