/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_status.c
 *   @brief This is the Module file for O STATUS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O STATUS command.
 *
 *   The O STATUS Command allows an admin to view<br/>
 *   some common Information about the Network Traffic etc.
 *
 *   Syntax is: /msg O STATUS
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "status"
#define MODUL_FILE "o_status"
#define MODUL_VERSION "1.0"

MOD_BOOL O_status(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_status_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_status_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	/*Module.ModErr = MOD_ERR_CANCELLED;
	Module.modul  = NULL;
	return Module;*/
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"status","status");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"status",O_status,700,INFINIT,3,3);
	addCommand(thread,&Module,"help status",O_status_help,700,INFINIT,4,4);
	addCommand(thread,&Module,"help",O_status_help_listing,700,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the O STATUS Command.
 * 
 * The O STATUS Command allows an admin to view<br/>
 *   some common Information about the Network Traffic etc.
 *
 * Syntax is: /msg O STATUS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_status(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL acclevels;

	/* Initialise Variables to (NULL) */
	bzero(&acclevels,sizeof(acclevels));

	/*
	 * Status Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"status")){
		acclevels.res = doSelectMySQL(thread,"SELECT SUM( `olevel` !=0 ) AS `olevel` , SUM( `qlevel` !=0 ) AS `qlevel` , SUM( `rights` !=0 ) AS `rights` , SUM( `slevel` >=700 ) FROM `orion_ns_core`");
		acclevels.row = mysql_fetch_row(acclevels.res);

#ifdef USE_THREADS
		my_pthread_mutex_lock(&globvar_lock);
#endif
		send_lang(thread,language,howtosend,snick,nick,
				"O_STATUS",
				    get_localtime(time(NULL)),
				    time(NULL),
					serv_uplink->server,
					m->servername,
					stoi(acclevels.row[0]),
					stoi(acclevels.row[1]),
					stoi(acclevels.row[3]),
					stoi(acclevels.row[2]),
					(m->TrafficIn),
					(m->TrafficOut),
					(m->TrafficIn2_bps/1000),
					(m->TrafficOut2_bps/1000),
					((m->TrafficOut2_bps+m->TrafficIn2_bps)/1000)
				);
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&globvar_lock);
#endif

		/* mysql_free_result_myr(); - acclevels */
		mysql_free_result_myr(acclevels.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O STATUS Command (/msg O HELP STATUS)
 * 
 * When the user tries to get help for this command, using /msg O HELP STATUS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP STATUS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_status_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Status Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help status")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"status"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_STATUS");
	}else send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - STATUS part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the STATUS command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_status_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Status (Overview Help)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_STATUS",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"status"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
