/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 206                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-04 17:15:02 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_trust.c
 *   @brief This is the Module file for O TRUST Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O TRUST command.
 *
 *   The O TRUST Command allows an admin to manipulate the SessionLimits<br/>
 *   for specific ips / hosts. (Managed with normal sessionlimits AND sessiongroups!!)
 *
 *   Syntax is: /msg O TRUST { ADD | DEL | MODIFY | LIST | SPEW } ...
 *
 *<table border="0">
 *    <tr><td> ADD		</td><td> Syntax is: /msg O TRUST ADD		</td><td> { < #trustgroup > | < limit > } { < host > | < ip > } < description >		</td></tr>
 *    <tr><td> DEL		</td><td> Syntax is: /msg O TRUST DEL		</td><td> { < host > | < ip > }														</td></tr>
 *    <tr><td> MODIFY	</td><td> Syntax is: /msg O TRUST MODIFY	</td><td> { < host > | < ip > } { DESCRIPTION | LIMIT | GROUP } < new-parameter >	</td></tr>
 *    <tr><td> LIST		</td><td> Syntax is: /msg O TRUST LIST		</td><td> [ *host-pattern* ]														</td></tr>
 *    <tr><td> SPEW		</td><td> Syntax is: /msg O TRUST SPEW		</td><td> < *host-pattern* >														</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "trust"
#define MODUL_FILE "o_trust"
#define MODUL_VERSION "1.0"

MOD_BOOL O_trust_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trust_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trust_modify(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trust_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trust_spew(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_trust_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trust_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trust_modify_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trust_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trust_spew_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_trust(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trust_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_trust_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"trust option[1]","trust helpoption[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"trust",NULL,840,INFINIT,0,0); /*just to add a displaylevel for /msg O help
															  this should always be the lowest used Module level! */

	addCommand(thread,&Module,"trust add",O_trust_add,840,INFINIT,7,0);
	addCommand(thread,&Module,"trust del",O_trust_del,840,INFINIT,5,5);
	addCommand(thread,&Module,"trust modify",O_trust_modify,840,INFINIT,7,0);
	addCommand(thread,&Module,"trust list",O_trust_list,840,INFINIT,5,6);
	addCommand(thread,&Module,"trust spew",O_trust_spew,840,INFINIT,5,5);

	addCommand(thread,&Module,"help trust",O_trust_help,840,INFINIT,4,4);
	addCommand(thread,&Module,"help trust add",O_trust_add_help,840,INFINIT,5,5);
	addCommand(thread,&Module,"help trust del",O_trust_del_help,840,INFINIT,5,5);
	addCommand(thread,&Module,"help trust modify",O_trust_modify_help,840,INFINIT,5,5);
	addCommand(thread,&Module,"help trust list",O_trust_list_help,840,INFINIT,5,5);
	addCommand(thread,&Module,"help trust spew",O_trust_spew_help,840,INFINIT,5,5);
	
	addCommand(thread,&Module,"help",O_trust_help_listing,840,INFINIT,3,3); //100 - each Admin is able to see it.
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUST Add Command. (To add trusts)
 * 
 * The O TRUST Add Command allows the user to add a trust for target host
 *  
 * Syntax is: /msg O TRUST ADD { < #trustgroup > | < limit > } { < host > | < ip >} < description >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL trustgroup;
	deepSQL trustdeny;

	/* Initialise Variables to (NULL) */
	bzero(&trustgroup,sizeof(trustgroup));
	bzero(&trustdeny,sizeof(trustdeny));

	/*
	 * Trust Add Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trust add")){
		//> :shockzZ PRIVMSG O :trust add 
		// 5 6 7   -   #services 62.128.92.197 shock
		if(CheckOption(thread,snick,nick,READONLY)){
			bzero(&trustdeny,sizeof(trustdeny));
			trustdeny.res    = doSelectMySQL(thread,"SELECT `rhost`,`description` FROM `orion_os_trusts` WHERE `rhost`='%s' AND `art`='0'",mysql_quote(thread,param[6]));
			trustdeny.row    = mysql_fetch_row(trustdeny.res);
			trustdeny.fields = mysql_num_rows(trustdeny.res);

			if(!trustdeny.fields && stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_os_trusts` WHERE `rhost`='%s'",mysql_quote(thread,param[6])))==0){
				if(param[5][0]=='#'){
					bzero(&trustgroup,sizeof(trustgroup));
					trustgroup.res    = doSelectMySQL(thread,"SELECT `group`,`limit` FROM `orion_os_trustgroups` WHERE `group`='%s'",mysql_quote(thread,param[5]));
					trustgroup.row    = mysql_fetch_row(trustgroup.res);
					trustgroup.fields = mysql_num_rows(trustgroup.res);

					if(trustgroup.fields){
						domysql(thread,"INSERT INTO `orion_os_trusts` (`art`,`rhost`,`limit`,`description`) VALUES ('1','%s','%s','%s')",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]),mysql_quote(thread,_dstrjoin(' ',param,7)));
						add_Trust(param[6],_dstrjoin(' ',param,7),param[5],(Trust_art)1,1);
						send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
					}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_INVALID_GROUP");

					/* mysql_free_result_myr(); - trustgroup */
					mysql_free_result_myr(trustgroup.res);
				}else if(digitcol(param[5])){
					domysql(thread,"INSERT INTO `orion_os_trusts` (`art`,`rhost`,`limit`,`description`) VALUES ('1','%s','%s','%s')",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]),mysql_quote(thread,_dstrjoin(' ',param,7)));
					send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
				}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_INVALID_LIMIT_OR_GROUP");
			}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_ADD_CAN_NOT_ADD_THIS_TRUST");
				
			/* mysql_free_result_myr(); - trustdeny */
			mysql_free_result_myr(trustdeny.res);
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUST Del Command. (To delete trusts)
 * 
 * The O TRUST Del Command allows the user to delete a HOST/IP from the trust-list
 *  
 * Syntax is: /msg O TRUST DEL { < host > | < ip > }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL killing;
	Trust O_trust;
	int i = 0;
	int trustconnections = 0;

	/* Initialise Variables to (NULL) */
	bzero(&killing,sizeof(killing));
	bzero(&O_trust,sizeof(O_trust));

	/*
	 * Trust Del Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trust del")){
		//> :shockzZ PRIVMSG O :trust add 
		// 5 6 7   -   #services 62.128.92.197 shock
		if(CheckOption(thread,snick,nick,READONLY)){
			bzero(&O_trust,sizeof(O_trust));
			O_trust = O_sql_trust(thread,param[5]);
			if(O_trust.rhost[0]!='\0'){
				/***************************************TRUST EXTENSION (ADVANCED)*************************************/
				if(m->OSForceTrusts){
					trustconnections = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_condata` WHERE `rhost`='%s'",mysql_quote(thread,param[5])));
					killing.res      = doSelectMySQL(thread,"SELECT `nick` FROM `orion_core_condata` WHERE `rhost`='%s' ORDER BY `time` DESC LIMIT 0,%d",mysql_quote(thread,param[5]),abs(trustconnections-m->maxconnections));
					killing.fields   = mysql_num_rows(killing.res);
						
					if(killing.fields<m->MaxSessionKill){
						for(i=0;i<killing.fields;i++){
							killing.row = mysql_fetch_row(killing.res);
								
							if(m->SessionLimitExceeded[0]!='\0')
								do_kill(thread,m->servername,killing.row[0],_dynadd(m->SessionLimitExceeded,param[5])); //# die letzten killen
							else
								do_kill(thread,m->servername,killing.row[0],"Session limit exceeded"); //# die letzten killen
						}
					}
					else{
						char rhost[MAX_HOST_LEN];
						bzero(&rhost,sizeof(rhost));
						
						straddns(rhost,"*@",sizeof(rhost));
						straddns(rhost,param[5],sizeof(rhost));
						
						if(m->SessionLimitExceeded[0]!='\0')
							do_gline(thread,m->servername,rhost,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),_dynadd(m->SessionLimitExceeded,param[5]));
						else
							do_gline(thread,m->servername,rhost,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),"Session limit exceeded");
					}
					/* mysql_free_result_myr(); - killing */
					mysql_free_result_myr(killing.res);
				}
				/******************************************************************************************************/
				domysql(thread,"DELETE FROM `orion_os_trusts` WHERE `rhost`='%s' AND `art`='1'",mysql_quote(thread,param[5]));
				del_Trust(getTrust(param[5],1));
				send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
			}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_HOST_NOT_FOUND");
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}
/*  ************************************************************************************************  */

/**
 * @brief O TRUST Modify Command. (To modify the trusts LIMIT/GROUP)
 * 
 * The O TRUST Modify Command allows the user to modify the SESSION-LIMIT<br/>
 *   by modifying LIMIT or GROUP of the trust
 *  
 * Syntax is: /msg O TRUST MODIFY { < host > | < ip > } { DESCRIPTION | LIMIT | GROUP } < new-parameter >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_modify(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL trustgroup;
	deepSQL killing;
	Trust O_trust;
	int i = 0;
	int trustconnections = 0;

	/* Initialise Variables to (NULL) */
	bzero(&trustgroup,sizeof(trustgroup));
	bzero(&killing,sizeof(killing));
	bzero(&O_trust,sizeof(O_trust));

	/*
	 * Trust Modify Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trust modify")){
		if(CheckOption(thread,snick,nick,READONLY)){
			
			bzero(&O_trust,sizeof(O_trust));
			O_trust=O_sql_trust(thread,param[5]);
			if(O_trust.rhost[0]!='\0'){
				if(streqnc(param[6],"description")){
					domysql(thread,"UPDATE `orion_os_trusts` SET `description`='%s' WHERE `rhost`='%s' AND `art`='1'",mysql_quote(thread,_dstrjoin(' ',param,7)),mysql_quote(thread,param[5]));
					send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
				}else if(streqnc(param[6],"limit")){
					if(digitcol(param[7])){
						Trusts *trust=getTrust(param[5],1);
						dynncpy(trust->limit,param[7],sizeof(trust->limit));
						domysql(thread,"UPDATE `orion_os_trusts` SET `limit`='%s' WHERE `rhost`='%s' AND `art`='1'",mysql_quote(thread,param[7]),mysql_quote(thread,param[5]));
						/***************************************TRUST EXTENSION (ADVANCED)*************************************/
						if(m->OSForceTrusts){
							trustconnections = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_condata` WHERE `rhost`='%s'",mysql_quote(thread,param[5])));
							if(stoi(param[7])<trustconnections){
								killing.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_core_condata` WHERE `rhost`='%s' ORDER BY `time` DESC LIMIT 0,%d",mysql_quote(thread,param[5]),abs(trustconnections-stoi(param[7])));
								killing.fields = mysql_num_rows(killing.res);
				
								if(killing.fields<m->MaxSessionKill){
									for(i=0;i<killing.fields;i++){
										killing.row = mysql_fetch_row(killing.res);
											
										if(m->SessionLimitExceeded[0]!='\0')
											do_kill(thread,m->servername,killing.row[0],_dynadd(m->SessionLimitExceeded,param[5])); //# die letzten killen
										else
											do_kill(thread,m->servername,killing.row[0],"Session limit exceeded"); //# die letzten killen
									}
								}
								else{
									char rhost[MAX_HOST_LEN];
									bzero(&rhost,sizeof(rhost));

									straddns(rhost,"*@",sizeof(rhost));
									straddns(rhost,param[5],sizeof(rhost));

									if(m->SessionLimitExceeded[0]!='\0')
										do_gline(thread,m->servername,rhost,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),_dynadd(m->SessionLimitExceeded,param[5]));
									else
										do_gline(thread,m->servername,rhost,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),"Session limit exceeded");
								}
	
								/* mysql_free_result_myr(); - killing */
								mysql_free_result_myr(killing.res);
							}
						}
						/******************************************************************************************************/
						send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
					}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_INVALID_LIMIT");
				}else if(streqnc(param[6],"group")){
					//> :shock PRIVMSG ircdos :trust modify 127.0.192.1 group #blub
					bzero(&trustgroup,sizeof(trustgroup));
					trustgroup.res    = doSelectMySQL(thread,"SELECT `group`,`limit` FROM `orion_os_trustgroups` WHERE `group`='%s'",mysql_quote(thread,param[7]));
					trustgroup.row    = mysql_fetch_row(trustgroup.res);
					trustgroup.fields = mysql_num_rows(trustgroup.res);

					if(trustgroup.fields){
						Trusts *trust=getTrust(param[5],1);
						dynncpy(trust->limit,param[7],sizeof(trust->limit));
						domysql(thread,"UPDATE `orion_os_trusts` SET `limit`='%s' WHERE `rhost`='%s' AND `art`='1'",mysql_quote(thread,param[7]),mysql_quote(thread,param[5]));
						/***************************************TRUST EXTENSION (ADVANCED)*************************************/
						if(m->OSForceTrusts){
							trustconnections = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_condata` WHERE `rhost`='%s'",mysql_quote(thread,param[5])));
							if(stoi(trustgroup.row[1])<trustconnections){  
								killing.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_core_condata` WHERE `rhost`='%s' ORDER BY `time` DESC LIMIT 0,%d",mysql_quote(thread,param[5]),abs(trustconnections-stoi(trustgroup.row[1])));
								killing.fields = mysql_num_rows(killing.res);
				
								if(killing.fields<m->MaxSessionKill){
									for(i=0;i<killing.fields;i++){
										killing.row = mysql_fetch_row(killing.res);
								
										if(m->SessionLimitExceeded[0]!='\0')
											do_kill(thread,m->servername,killing.row[0],_dynadd(m->SessionLimitExceeded,param[5])); //# die letzten killen
										else
											do_kill(thread,m->servername,killing.row[0],"Session limit exceeded"); //# die letzten killen
									}
								}
								else{
									char rhost[MAX_HOST_LEN];
									bzero(&rhost,sizeof(rhost));

									straddns(rhost,"*@",sizeof(rhost));
									straddns(rhost,param[5],sizeof(rhost));

									if(m->SessionLimitExceeded[0]!='\0')
										do_gline(thread,m->servername,rhost,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),_dynadd(m->SessionLimitExceeded,param[5]));
									else
										do_gline(thread,m->servername,rhost,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),"Session limit exceeded");
								}
	
								/* mysql_free_result_myr(); - killing */
								mysql_free_result_myr(killing.res);	
							}
						}
						/******************************************************************************************************/
						send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
					}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_INVALID_SYNTAX_FOR_GROUP");
						
					/* mysql_free_result_myr(); - trustgroup */
					mysql_free_result_myr(trustgroup.res);
				}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_INVALID_OPTION");
			}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_NO_TRUST_FOR_HOST",param[5]);
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUST List Command. (To list trusts in a trustgroup or in #maingroup)
 * 
 * The O TRUST List Command allows the user to dump the list of trusted hosts<br/>
 *   in any given #trustgroup or in the #maingroup
 *  
 * Syntax is: /msg O TRUST LIST [*host-pattern*]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL trustgroup;
	deepSQL vlimit;
	char pattern[BUFSIZE];
	int counter = 0;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&trustgroup,sizeof(trustgroup));
	bzero(&vlimit,sizeof(vlimit));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Trust List Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trust list")){
		//> :shockzZ PRIVMSG O :trust add 
		// 5 6 7   -   #services 62.128.92.197 shock
		bzero(&trustgroup,sizeof(trustgroup));
		trustgroup.res    = doSelectMySQL(thread,"SELECT `group`,`limit` FROM `orion_os_trustgroups` WHERE `group`='%s'",mysql_quote(thread,param[5]));
		trustgroup.row    = mysql_fetch_row(trustgroup.res);
		trustgroup.fields = mysql_num_rows(trustgroup.res);

		if(trustgroup.fields){
			counter=0;
			if(param[6]){
				dyncpy (pattern,param[6]);
				strrepc(pattern,'*','%');
				vlimit.res = doSelectMySQL(thread,"SELECT `rhost`,`limit`,`description` FROM `orion_os_trusts` WHERE `limit`='%s' AND `rhost` LIKE '%s' AND `art`='1' ORDER BY `rhost` ASC",mysql_quote(thread,param[5]),pattern);
			}else
				vlimit.res = doSelectMySQL(thread,"SELECT `rhost`,`limit`,`description` FROM `orion_os_trusts` WHERE `limit`='%s' AND `art`='1' ORDER BY `rhost` ASC",mysql_quote(thread,param[5]));
				
			vlimit.fields = mysql_num_rows(vlimit.res);

			send_lang(thread,language,howtosend,snick,nick,"O_TRUST_LIST_GROUP_XYS_VIRTUAL_LIMIT",param[5],trustgroup.row[1]);
			for(i=0;i<vlimit.fields;i++){
				vlimit.row = mysql_fetch_row(vlimit.res);
					
				send_lang(thread,language,howtosend,snick,nick,"O_TRUST_LIST_LIST_IF_DESCRIPTION",vlimit.row[0],vlimit.row[2]);
			}
			mysql_free_result_myr(vlimit.res); //free MySQL result

			if(i != 0){
				//strrepc(param[6],'%','*');
				if(param[6]) send_lang(thread,language,howtosend,snick,nick,"O_TRUST_LIST_RESULT_FOUND_X_MATCHING_WITH_PATTERN",i,param[6],param[5]);
				else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_LIST_RESULT_FOUND_X_MATCHING_IN_GROUP",i,param[5]);
			}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_LIST_RESULT_FOUND_NO_TRUSTS_IN_GROUP",param[5]);
		}else if(streqnc(param[5],"#main")){
			counter=0;
			if(param[6]){
				dyncpy (pattern,param[6]);
				strrepc(pattern,'*','%');
					
				vlimit.res = doSelectMySQL(thread,"SELECT `rhost`,`limit`,`description` FROM `orion_os_trusts` WHERE `rhost` LIKE '%s' AND `art`='1' ORDER BY `rhost` ASC",pattern);
			}else
				vlimit.res = doSelectMySQL(thread,"SELECT `rhost`,`limit`,`description` FROM `orion_os_trusts` WHERE `art`='1' ORDER BY `rhost` ASC");

			vlimit.fields = mysql_num_rows(vlimit.res);

			for(i=0;i<vlimit.fields;i++){
				vlimit.row = mysql_fetch_row(vlimit.res);
					
				if(!strcor(vlimit.row[1],"#")){
					send_lang(thread,language,howtosend,snick,nick,"O_TRUST_LIST_LIST_HOST_WITH_VIRTUAL_LIMIT",vlimit.row[0],vlimit.row[1],vlimit.row[2]);
					counter++;
				}
			}
			mysql_free_result_myr(vlimit.res); //free MySQL result

			if(counter != 0){
				//strrepc(param[6],'%','*');
				if(param[6]) send_lang(thread,language,howtosend,snick,nick,"O_TRUST_LIST_RESULT_FOUND_X_MATCHING_WITH_PATTERN",counter,param[6],param[5]);
				else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_LIST_RESULT_FOUND_X_MATCHING_IN_GROUP",counter,param[5]);
			}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_LIST_RESULT_FOUND_NO_TRUSTS_IN_GROUP",param[5]);
		}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_INVALID_GROUP");

		/* Free MySQL Result */
		mysql_free_result_myr(trustgroup.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O TRUST SPEW Command. (To search for trusts matching a pattern)
 * 
 * The O TRUST SPEW Command allows the user to search for trusts<br/>
 *   matching any given *pattern*
 *  
 * Syntax is: /msg O TRUST SPEW *host-pattern*
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_spew(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL trustgroup;
	deepSQL select;
	char pattern[BUFSIZE];
	int i=0;

	/* Initialise Variables to (NULL) */
	bzero(&trustgroup,sizeof(trustgroup));
	bzero(&select,sizeof(select));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Trust Spew Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trust spew") && strcor(param[5],"*")){
		dyncpy (pattern,param[5]);
		strrepc(pattern,'*','%');

		send_lang(thread,language,howtosend,snick,nick,"O_TRUST_SPEW_LIST_HEADER");

		select.res    = doSelectMySQL(thread,"SELECT `rhost`,`limit`,`description`,`art` FROM `orion_os_trusts` WHERE `rhost` LIKE '%s' ORDER BY `rhost` ASC,`art` DESC",mysql_quote(thread,pattern));
		select.fields = mysql_num_rows(select.res);

		if(select.fields){
			for(i=0;i<select.fields;i++){
				select.row = mysql_fetch_row(select.res);
				
				if(stoi(select.row[3])==1){
					if(select.row[0][0]=='#'){
						trustgroup.res = doSelectMySQL(thread,"SELECT `group`,`limit` FROM `orion_os_trustgroups` WHERE `group`='%s'",mysql_quote(thread,select.row[1]));
						trustgroup.row = mysql_fetch_row(trustgroup.res);

						send_lang(thread,language,howtosend,snick,nick,"O_TRUST_SPEW_LIST_WITHIN_GROUP",i+1,"T",select.row[0],trustgroup.row[1],select.row[1],select.row[2]);
						mysql_free_result_myr(trustgroup.res); //free MySQL result
					}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_SPEW_LIST_WITHOUT_TRUST_BEING_GROUPED",i+1,"T",select.row[0],select.row[1],select.row[2]);
				}else if(stoi(select.row[3])==0){
					if(select.row[0][0]=='#'){
						trustgroup.res = doSelectMySQL(thread,"SELECT `group`,`limit` FROM `orion_os_trustgroups` WHERE `group`='%s'",mysql_quote(thread,select.row[1]));
						trustgroup.row = mysql_fetch_row(trustgroup.res);

						send_lang(thread,language,howtosend,snick,nick,"O_TRUST_SPEW_LIST_WITHIN_GROUP",i+1,"D",select.row[0],trustgroup.row[1],select.row[1],select.row[2]);
						mysql_free_result_myr(trustgroup.res); //free MySQL result
					}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_SPEW_LIST_WITHOUT_TRUST_BEING_GROUPED",i+1,"D",select.row[0],select.row[1],select.row[2]);
				}
			}
			//strrepc(param[4],'%','*');
			if(i != 0) send_lang(thread,language,howtosend,snick,nick,"O_TRUST_SPEW_FOUND_X_MATCHING_BY_SEARCHING_WITH_PATTERN",i,param[5]);
			else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_SPEW_FOUND_NOTHING_BY_SEARCHING_WITH_PATTERN",param[5]);
		}else send_lang(thread,language,howtosend,snick,nick,"O_TRUST_SPEW_FOUND_NOTHING_BY_SEARCHING_WITH_PATTERN",param[5]);

		mysql_free_result_myr(select.res); //free MySQL result
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[5],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[5]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O TRUST Command (/msg O HELP TRUST)
 * 
 * When the user tries to get help for this command, using /msg O HELP TRUST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP TRUST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trust Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help trust")){
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_TRUST",snick,snick);

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Help Trust Add function for the O TRUST Command (/msg O HELP TRUST ADD)
 * 
 * When the user tries to get help for command (trust add), using /msg O HELP TRUST ADD,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP TRUST ADD
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trust Help Add
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help trust add")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trust add"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_TRUST_ADD");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Help Trust Del function for the O TRUST Command (/msg O HELP TRUST DEL)
 * 
 * When the user tries to get help for command (trust del), using /msg O HELP TRUST DEL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP TRUST DEL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trust Help Del
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help trust del")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trust del"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_TRUST_DEL");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Help Trust Modify function for the O TRUST Command (/msg O HELP TRUST MODIFY)
 * 
 * When the user tries to get help for command (trust modify), using /msg O HELP TRUST MODIFY,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP TRUST MODIFY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_modify_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trust Help Modify
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help trust modify")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trust modify"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_TRUST_MODIFY");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Help Trust List function for the O TRUST Command (/msg O HELP TRUST LIST)
 * 
 * When the user tries to get help for command (trust list), using /msg O HELP TRUST LIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP TRUST LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trust Help List
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help trust list")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trust list"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_TRUST_LIST");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Help Trust Spew function for the O TRUST Command (/msg O HELP TRUST SPEW)
 * 
 * When the user tries to get help for command (trust spew), using /msg O HELP TRUST SPEW,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP TRUST SPEW
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_spew_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Trust Help Spew
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help trust spew")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trust spew"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_TRUST_SPEW");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - TRUST part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the TRUST command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_trust_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Trust (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_TRUST",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"trust"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
