/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_changelev.c
 *   @brief This is the Module file for Q CHANGELEV Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q CHANGELEV command.
 *
 *   The Q CHANGELEV Command allows an admin to change the Q Global-Auth-level for target < nick >
 *
 *   Syntax is: /msg Q CHANGELEV < nick >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "Q"
#define MODUL "changelev"
#define MODUL_FILE "q_changelev"
#define MODUL_VERSION "1.0"

MOD_BOOL Q_changelev(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_changelev_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_changelev_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"changelev","changelev");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"changelev",Q_changelev,700,INFINIT,4,5);
	addCommand(thread,&Module,"help changelev",Q_changelev_help,700,INFINIT,4,4);
	addCommand(thread,&Module,"showcommands",Q_changelev_showcommands,700,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q CHANGELEV Command.
 * 
 * The Q CHANGELEV Command allows an admin to change the Q Global-Auth-level for target < nick >
 *
 * Syntax is: /msg Q CHANGELEV < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_changelev(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */
	Aliases aliases5;
	Access access5;
	deepSQL whois;
	deepSQL SendChanaccess;
	char *authnick = NULL;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	bzero(&whois,sizeof(whois));
	bzero(&SendChanaccess,sizeof(SendChanaccess));

	/*
	 * Changelev Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"changelev")){
		//Initialise Access and Logins
		aliases5 = sql_aliases(thread,param[4]);
		if(aliases5.fields && aliases5.authnick[0]!='\0')
			access5 = sql_access(thread,aliases5.authnick);
		//End of Initialisation
		
		if(param[5]!=NULL){
			if(thread->sql.access.fields && access5.authnick[0]!='\0' && stoi(param[5])<=thread->sql.access.qlevel){
				domysql(thread,"UPDATE `orion_ns_core` SET `qlevel`='%d' WHERE `authnick`='%s'",stoi(param[5]),mysql_quote(thread,aliases5.authnick));
				send_lang(thread,language,howtosend,snick,nick,"Q_CHANGELEV_CHANGED_AUTHLEVEL",aliases5.authnick,access5.qlevel,stoi(param[5]));
				send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
			}else if(thread->sql.access.fields && !param[5] && access5.qlevel<=thread->sql.access.qlevel){
				send_lang(thread,language,howtosend,snick,nick,"Q_CHANGELEV_RETURN_AUTHLEVEL",aliases5.authnick,access5.qlevel);
				send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
			}else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
		/* Added the part of Q Whois Command after here */
		}else if(strcor(param[4],"#") && param[4][0]=='#'){
			authnick = &param[4][1];
			
			whois.res = doSelectMySQL(thread,"SELECT `orion_ns_core`.`authnick`,`orion_ns_aliases`.`time_linked` AS `regdate`,`orion_ns_core`.`email`,`orion_ns_core`.`qlevel`,`orion_ns_aliases`.`nick`,`orion_ns_aliases`.`last_login`,`orion_ns_core`.`settings`,`orion_ns_core`.`reason` FROM `orion_ns_core` LEFT OUTER JOIN `orion_ns_aliases` ON `orion_ns_core`.`authnick`=`orion_ns_aliases`.`display` AND `orion_ns_aliases`.`active`=1 WHERE `authnick`='%s' LIMIT 0,1",mysql_quote(thread,authnick));
			if(mysql_num_rows(whois.res)){
				whois.row = mysql_fetch_row(whois.res);
				if(stoi(whois.row[6])&NS_SUSPENDED){
					send_lang(thread,language,howtosend,snick,nick,"LOGIN_SUSPENDED");
					send_lang(thread,language,howtosend,snick,nick,"LOGIN_SUSPEND_GROUP_REASON",whois.row[0],whois.row[7]);
				}else{
					if(whois.row[4]!=NULL){
						send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_USER_ONLINE_NOW",authnick);
						send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_USER_IS_AUTHED_AS_SINCE",whois.row[4],whois.row[0],get_localtime((time_t)stol(whois.row[5])));
					}else send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_USER_IS_NOT_AUTHED",authnick);

					if(thread->sql.access.qlevel>=100 || thread->sql.access.olevel>=100 || thread->sql.access.slevel>=100 || thread->sql.access.llevel>=100) send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_EMAIL",whois.row[2]);
					send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_GLOBAL_LEVEL",authnick,stoi(whois.row[3]));
					//send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_REGISTERED_SINCE",get_localtime((time_t)stol(whois.row[1])));

					SendChanaccess.res = doSelectMySQL(thread,"SELECT `authnick`,`channel`,`level` FROM `orion_cs_access` WHERE `authnick`='%s' AND `serv`='Q'",mysql_quote(thread,authnick));
					while((SendChanaccess.row = mysql_fetch_row(SendChanaccess.res))){
						send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_ACCESS_LISTINGS",SendChanaccess.row[2],SendChanaccess.row[1]);
					}
					send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
					mysql_free_result_myr(SendChanaccess.res);
				}
			}else send_lang(thread,language,howtosend,snick,nick,"Q_WHO_ON_EARTH");
			mysql_free_result_myr(whois.res);
		}else{
			whois.res = doSelectMySQL(thread,"SELECT `orion_ns_core`.`authnick`,`orion_ns_aliases`.`time_linked` AS `regdate`,`orion_ns_core`.`email`,`orion_ns_core`.`qlevel`,`orion_ns_aliases`.`nick`,`orion_ns_aliases`.`last_login`,`orion_ns_aliases`.`active`,`orion_ns_core`.`settings`,`orion_ns_aliases`.`last_quit`,`orion_ns_core`.`reason` FROM `orion_ns_aliases` LEFT OUTER JOIN `orion_ns_core` ON `orion_ns_core`.`authnick`=`orion_ns_aliases`.`display` WHERE `orion_ns_aliases`.`nick`='%s' LIMIT 0,1",mysql_quote(thread,param[4]));
			if(mysql_num_rows(whois.res)>0){
				whois.row = mysql_fetch_row(whois.res);
				if(stoi(whois.row[7])&NS_SUSPENDED){
					send_lang(thread,language,howtosend,snick,nick,"LOGIN_SUSPENDED");
					send_lang(thread,language,howtosend,snick,nick,"LOGIN_SUSPEND_GROUP_REASON",whois.row[0],whois.row[9]);
				}

				if(whois.row[0]!=NULL && whois.row[4]!=NULL && stoi(whois.row[6])==1){
					send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_USER_ONLINE_NOW",param[4]);
					send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_USER_IS_AUTHED_AS_SINCE",whois.row[4],whois.row[0],get_localtime((time_t)stol(whois.row[5])));
				}
				else send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_USER_IS_NOT_AUTHED",param[4]);

				if(thread->sql.access.qlevel>=100 || thread->sql.access.olevel>=100 || thread->sql.access.slevel>=100 || thread->sql.access.llevel>=100) send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_EMAIL",whois.row[2]);
				send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_GLOBAL_LEVEL",param[4],stoi(whois.row[3]));
				send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_REGISTERED_SINCE",get_localtime((time_t)stol(whois.row[1])));
				send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_LAST_AUTH",get_localtime((time_t)stol(whois.row[5])));

				SendChanaccess.res = doSelectMySQL(thread,"SELECT `authnick`,`channel`,`level` FROM `orion_cs_access` WHERE `authnick`='%s'  AND `serv`='Q'",mysql_quote(thread,whois.row[0]));
				while((SendChanaccess.row = mysql_fetch_row(SendChanaccess.res))){ //fetch res to row's
					send_lang(thread,language,howtosend,snick,nick,"Q_WHOIS_ACCESS_LISTINGS",SendChanaccess.row[2],SendChanaccess.row[1]);
				}
				send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
				/* mysql_free_result_myr(); - SendChanaccess */
				mysql_free_result_myr(SendChanaccess.res);
			}else send_lang(thread,language,howtosend,snick,nick,"Q_WHO_ON_EARTH");
			mysql_free_result_myr(whois.res);
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q CHANGELEV Command (/msg Q HELP CHANGELEV)
 * 
 * When the user tries to get help for this command, using /msg Q HELP CHANGELEV,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP CHANGELEV
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_changelev_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Changelev Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help changelev")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"changelev"));
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_CHANGELEV",snick,snick,snick,snick);
	}else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - CHANGELEV part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the CHANGELEV command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_changelev_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=0, Users_Q_Level=0;
	command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Changelev (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_CHANGELEV",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"changelev"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
