/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_delchan.c
 *   @brief This is the Module file for Q DELCHAN Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q DELCHAN command.
 *
 *   The Q DELCHAN Command allows the user to delete<br/>
 *   a channel he registered before.<br/>
 *   (Only the RealFounder is permitted to use this)
 *
 *   Syntax is: /msg Q DELCHAN #channel
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "Q"
#define MODUL "delchan"
#define MODUL_FILE "q_delchan"
#define MODUL_VERSION "1.0"

MOD_BOOL Q_delchan(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_delchan_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_delchan_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"delchan","delchan");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"delchan",Q_delchan,550,1,4,4);
	addCommand(thread,&Module,"help delchan",Q_delchan_help,100,1,4,4);
	addCommand(thread,&Module,"showcommands",Q_delchan_showcommands,100,1,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q DELCHAN Command.
 * 
 * The Q DELCHAN Command allows the user to delete<br/>
 *   a channel he registered before.<br/>
 *   (Only the RealFounder is permitted to use this)
 *
 * Syntax is: /msg Q DELCHAN #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_delchan(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */

	/* Initialise Variables to (NULL) */

	/*
	 * Delchan Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"delchan")){
		/* Updates the Threads Channel-Management */
		UpdateChannelInformation(thread,param[4],NULL);

		if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q")){
			if((thread->c.chan.settings & CS_FORBID_BIT))		send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
			else if((thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
			else if(thread->sql.access.qlevel>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"delchan") || /*strcor(thread->c.chan.access.ownlevel,"n")*/streqnc(thread->c.chan.founder,thread->sql.access.authnick)){

				/* Update SQL as Live Update */
        		if(thread->c.chan.bot[0]!='\0'){
	        		do_privmsg(thread,thread->c.chan.bot,param[4],"I was unassigned from that channel (caused by dropping the channel), bye ladies and gentlemen");
					do_part(thread,thread->c.chan.bot,param[4],NULL);
					thread->c.chan.botinchan = 0;
				}

				if(IRC.ChanRegMode)
					do_mode(thread,snick,param[4],thread->c.chan.datecreate,IRC.ModeOnChannelDrop);

                /* Update SQL as Live Update */
				domysql(thread,"UPDATE `orion_cs_core` SET `serv`='',`founder`=NULL,`successor`=NULL,`email`='',`settings`='',`regdate`='',`description`='',`mlock_positive`='',`mlock_negative`='',`mlock_limit`='',`mlock_key`='',"
					"`mlock_flood`='',`mlock_redirect`='',`url`='',`alimit`='',`flags`='',`welcome`='',`reason`='',`last_used`='',`expire`='' WHERE `channel`='%s' AND `serv`='Q'",mysql_quote(thread,param[4]));
				domysql(thread,"DELETE `orion_cs_access`,`orion_cs_akick` FROM `orion_cs_access` LEFT OUTER JOIN `orion_cs_akick` ON (`orion_cs_akick`.`channel`=`orion_cs_access`.`channel`) LEFT OUTER JOIN `orion_ms_core` ON (`orion_ms_core`.`receiver`=`orion_cs_access`.`channel`) WHERE `orion_cs_access`.`channel`='%s' AND `orion_cs_access`.`serv`='Q'",mysql_quote(thread,param[4]));
				
				do_part(thread,snick,param[4],"channel has been deleted");
				send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
				bzero(&thread->c,sizeof(thread->c));
			}else send_lang(thread,language,howtosend,snick,nick,"Q_RIGHT_OWNER_TO_DELETE_CHANNEL");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q DELCHAN Command (/msg Q HELP DELCHAN)
 * 
 * When the user tries to get help for this command, using /msg Q HELP DELCHAN,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP DELCHAN
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_delchan_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Delchan Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help delchan")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"delchan"));
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_DELCHAN");
	}else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - DELCHAN part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the DELCHAN command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_delchan_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=0, Users_Q_Level=0;
	command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Delchan (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_DELCHAN",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"delchan"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
