/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 207                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:32:05 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_events.c
 *   @brief This is the Module file for Q EVENTS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q EVENTS command.
 *
 *   Main functions - EVENTS ONLY
 *
 *   Syntax is: /msg Q EVENTS
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "Q"
#define MODUL "events"
#define MODUL_FILE "q_events"
#define MODUL_VERSION "1.0"

/*  ******************************************* Event List *****************************************  */

MOD_BOOL q_on_kill(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_kick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_link(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_linkdone(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_netinfo(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_nick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_part(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_quit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_protoctl(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_sethost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_setident(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_chghost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_chgident(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_version(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_topic(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_join(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL q_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL q_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

/*  **************************************** End of Event List **************************************  */

MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,EVENT_PROTOTYPE,EVENT_PROTOTYPE);

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addAccess (thread,&Module,NOONE);
	addMODType(thread,&Module,EVENT);

	addEventHook(thread,&Module,"on_join"		,q_on_join);
	addEventHook(thread,&Module,"on_part"		,q_on_part);
	
	addEventHook(thread,&Module,"on_kill"		,q_on_kill);
	addEventHook(thread,&Module,"on_kick"		,q_on_kick);
	addEventHook(thread,&Module,"on_link"		,q_on_link);
	addEventHook(thread,&Module,"on_linkdone"	,q_on_linkdone);
	addEventHook(thread,&Module,"on_netinfo"	,q_on_netinfo);

	addEventHook(thread,&Module,"on_nick"		,q_on_nick);
	addEventHook(thread,&Module,"on_quit"		,q_on_quit);
	addEventHook(thread,&Module,"on_protoctl"	,q_on_protoctl);
	addEventHook(thread,&Module,"on_sethost"	,q_on_sethost);
	addEventHook(thread,&Module,"on_setident"	,q_on_setident);
	addEventHook(thread,&Module,"on_chghost"	,q_on_chghost);
	addEventHook(thread,&Module,"on_chgident"	,q_on_chgident);
	addEventHook(thread,&Module,"on_version"	,q_on_version);
	addEventHook(thread,&Module,"on_topic"		,q_on_topic);

	addEventHook(thread,&Module,"on_message"	,q_on_message);

	addEventHook(thread,&Module,"on_connect_primary"	,q_primary);
	addEventHook(thread,&Module,"on_connect_secondary"	,q_secondary);
	addEventHook(thread,&Module,"on_squit"				,q_squit);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);

/* Things we want to process "on load" */
	Q->loaded = 1;

	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q PRIMARY Command.
 * 
 * The Q PRIMARY Command allows the user....
 *  
 * Syntax is: /msg Q PRIMARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL q_primary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    do_kill(thread,m->servername,Q->nick,"nick used by orion");
	do_sqline(thread,m->servername,Q->nick,0,time(NULL),"nick used by orion");
	createuser(thread,2,Q->nick,Q->ident,Q->host,IRC.QMode,Q->realname);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q SECONDARY Command.
 * 
 * The Q SECONDARY Command allows the user....
 *  
 * Syntax is: /msg Q SECONDARY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL q_secondary(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	if(m->logchan[0] != '\0'){
		do_services_join(thread,Q->nick,m->logchan,time(NULL),2);
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q SQUIT Command.
 * 
 * The Q SQUIT Command allows the user....
 *  
 * Syntax is: /msg Q SQUIT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL q_squit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_kill: Q Events: On Kill Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_kill(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_kick: Q Events: On Kick Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_kick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	//@param - [0](sender), [1](channel), [2](nick)
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,param[1],NULL);
	
	addLimit(thread,thread->c.chan.name,thread->c.chan.datecreate,strcor("c",thread->c.chan.flags),thread->c.chan.serv,thread->c.chan.alimit);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_link: Q Events: On Link Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_link(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_linkdone: Q Events: On Linkdone Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_linkdone(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_netinfo: Q Events: On Netinfo Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_netinfo(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	//@param - [0](NULL), [1](NULL), ...
	//@nick  - NULL
    deepSQL channels;
    deepSQL nicks;
    int i = 0;
    
    bzero(&channels,sizeof(channels));
	channels.res = doSelectMySQL(thread,"SELECT `channel`,`datecreate`,`flags`,`last_topic`,`last_topic_setter`,`last_topic_time`,`settings`,`reason` FROM `orion_cs_core` WHERE `serv`='Q'");
	while((channels.row = mysql_fetch_row(channels.res))){
		do_join(thread,2,Q->nick,channels.row[0],stol(channels.row[1]) == 0 ? time(NULL) : (time_t)stol(channels.row[1]));
		do_mode(thread,m->servername,channels.row[0],stol(channels.row[1]) == 0 ? time(NULL) : (time_t)stol(channels.row[1]),"+oaq %s %s %s",Q->nick,Q->nick,Q->nick);
		
		if(IRC.ChanRegMode) 
			do_mode(thread,m->servername,channels.row[0],stol(channels.row[1]) == 0 ? time(NULL) : (time_t)stol(channels.row[1]),IRC.ModeOnChannelReg);
		
		if(stol(channels.row[6])&CS_SUSPEND_BIT){
			bzero(&nicks,sizeof(nicks));
			
			nicks.res    = doSelectMySQL(thread,"SELECT `orion_chan_users`.`nick`,`orion_core_condata`.`vhost` FROM `orion_chan_users` LEFT JOIN `orion_core_condata` ON `orion_chan_users`.`nick`=`orion_core_condata`.`nick` WHERE `channel`='%s' AND `orion_core_condata`.`faketype`='0'",mysql_quote(thread,channels.row[0]));
			nicks.fields = mysql_num_rows(nicks.res);
		
			do_mode(thread,Q->nick,channels.row[0],thread->c.chan.datecreate,IRC.OnSuspend);
				
			for(i=0;i<nicks.fields;i++){
				nicks.row = mysql_fetch_row(nicks.res);
			
				do_mode(thread,Q->nick,channels.row[0],thread->c.chan.datecreate,"+b %s",nicks.row[1]);
				do_kick(thread,Q->nick,channels.row[0],nicks.row[0],"channel is suspended reason: %s",channels.row[7]);
			}
			/* mysql_free_result_myr(); - Chanaccess */
			mysql_free_result_myr(nicks.res);
		
		}
		else if(strcor(channels.row[2],"t") && channels.row[3][0]!='\0')
			do_topic(thread, Q->nick, channels.row[4][0]=='\0' ? Q->nick : channels.row[4], channels.row[0], time(NULL), channels.row[3]);
    }
    mysql_free_result_myr(channels.res);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_nick: Q Events: On Nick Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_nick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_part: Q Events: On Part Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_part(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	//@param - [0](user), [1](channel)
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,param[1],NULL);
	addLimit(thread,thread->c.chan.name,thread->c.chan.datecreate,strcor("c",thread->c.chan.flags),thread->c.chan.serv,thread->c.chan.alimit);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_quit: Q Events: On Quit Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_quit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_protoctl: Q Events: On Protoctl Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_protoctl(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_sethost: Q Events: On Sethost Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_sethost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_setident: Q Events: On Setident Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_setident(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_chghost: Q Events: On Chghost Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_chghost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_chgident: Q Events: On Chgident Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_chgident(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_version: Q Events: On Version Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_version(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_topic: Q Events: On Topic Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_topic(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	//@param - [0](setter), [1](channel), [2](date), [3](topic)
	//@nick  - setter
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,param[1],NULL);
	
	if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q")){
		if(strcor("f",thread->c.chan.flags))
			//do_topic(thread,Q->nick,param[1],thread->c.chan.datecreate,thread->c.chan.last_topic);
			//else   We Will leave the Topic then to cache and SQL... (change: shock)
			do_rawtopic(thread, Q->nick, param[1], Q->nick, thread->c.chan.last_topic, time(NULL));
		else{
			/* 	Update our Cache	*/
			//bzero(&thread->c.chan.last_topic,sizeof(thread->c.chan.last_topic));
			//bzero(&thread->c.chan.last_topic_setter,sizeof(thread->c.chan.last_topic_setter));
		
			dynncpy(thread->c.chan.last_topic,param[3],sizeof(thread->c.chan.last_topic));
			dynncpy(thread->c.chan.last_topic_setter,param[0],sizeof(thread->c.chan.last_topic_setter));
			thread->c.chan.last_topic_time = (time_t)stol(param[2]);
			/**			***			**/
		
			domysql(thread,"UPDATE `orion_cs_core` SET `last_topic`='%s',`last_topic_setter`='%s',`last_topic_time`='%ld' WHERE `channel`='%s'",mysql_quote(thread,thread->c.chan.last_topic),mysql_quote(thread,thread->c.chan.last_topic_setter),thread->c.chan.last_topic_time,mysql_quote(thread,param[1]));
		}
    }
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_join: Q Events: On Join Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_join(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	//@param - [0](nick), [1](channel), [2](time)
	/* Deklarate Variables */
	DB3 c;
	deepSQL Akick;
	//int botjoin  = -1;
	//bool csjoined = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&c,sizeof(c));
	bzero(&Akick,sizeof(Akick));
	//int8 botjoin  = -1;
	//bool csjoined = 0;
	
	/*
	 * If it's a Q channel
	 */
	if(GetUser(nick)==NULL)
	    return MOD_CONT;
	    
	c = sql_chan_data(thread,param[1],3,CHANNEL,AAC,COUNTUSERS,nick);
	
	if(c.fields && streqnc(c.chan.serv,"q") && c.chan.bot[0]=='\0'){
		
		//if(c.chan.countusers > m->BSMinUsers)
		//	delStayChannel(thread,c.chan.name);

		/* ******************************* AKick Check **************************************** */
		Akick.res    = AkickOnJoin(thread,param[1],nick);
		Akick.fields = mysql_num_rows(Akick.res);
				
		if(Akick.fields){
			while((Akick.row = mysql_fetch_row(Akick.res))){
				if(Akick.row[1]!=NULL){
					do_mode(thread,c.chan.bot,c.chan.name,c.chan.datecreate,"+b %s",Akick.row[1]);
					do_kick_nopart(thread,c.chan.bot,c.chan.name,nick,"Akicked: %s",Akick.row[2]);
					c.chan.countusers--;
				}
	        }
		}
		mysql_free_result_myr(Akick.res);
		/* ************************************************************************************ */

		if(c.chan.settings & CS_SUSPEND_BIT || c.chan.settings & CS_FORBID_BIT){
			if(c.access.qlevel<=100){
				do_mode(thread,Q->nick,c.chan.name,c.chan.datecreate,IRC.OnSuspend);
				do_mode(thread,Q->nick,c.chan.name,c.chan.datecreate,"+b %s",c.Cdata.vhost);
				
				if(c.chan.settings & CS_SUSPEND_BIT)	 do_kick_nopart(thread,Q->nick,c.chan.name,nick,"suspended [%s]",c.chan.reason);
				else if(c.chan.settings & CS_FORBID_BIT) do_kick_nopart(thread,Q->nick,c.chan.name,nick,c.chan.reason[0]=='\0' ? "Forbidden" : c.chan.reason);
				c.chan.countusers--;
			}
		}else if(!streqnc(nick,Q->nick)){
			if(c.chan.access.ownauthnick[0]!='\0' && c.chan.access.ownlevel[0]!='\0'){
				if(strcor("b",c.chan.access.ownlevel)){
					do_mode(thread,Q->nick,c.chan.name,c.chan.datecreate,"+b %s",c.Cdata.vhost);
					do_kick_nopart(thread,Q->nick,c.chan.name,nick,"%s-banned",Q->nick);
					c.chan.countusers--;
					
				}else if(strcor("a",c.chan.access.ownlevel)){
					if(strcor("o",c.chan.access.ownlevel)){
						if(strcor("p",c.chan.access.ownlevel)) do_mode(thread,Q->nick,c.chan.name,c.chan.datecreate,"+oa %s %s",nick,nick);
						else do_mode(thread,Q->nick,c.chan.name,c.chan.datecreate,"+o %s",nick);
					}else if(strcor("h",c.chan.access.ownlevel)){
						if(strcor("p",c.chan.access.ownlevel)) do_mode(thread,Q->nick,c.chan.name,c.chan.datecreate,"+ha %s %s",nick,nick);
						else do_mode(thread,Q->nick,c.chan.name,c.chan.datecreate,"+h %s",nick);
					}else if(strcor("v",c.chan.access.ownlevel)) do_mode(thread,Q->nick,c.chan.name,c.chan.datecreate,"+v %s",nick);
				}
			}
			if(strcor("w",c.chan.flags) && c.chan.welcome[0]!='\0')
				msend(thread,0,Q->nick,nick,"[%s] %s",c.chan.name,c.chan.welcome);
				
			if(strcor("c",c.chan.flags) && c.chan.alimit!=0)
				addLimit(thread,c.chan.name,c.chan.datecreate,strcor("c",c.chan.flags),c.chan.serv,c.chan.alimit);
			
			if(strcor("o",c.chan.flags))
				do_mode(thread,Q->nick,c.chan.name,c.chan.datecreate,"+o %s",nick);
				
			else if(strcor("v",c.chan.flags))
				do_mode(thread,Q->nick,c.chan.name,c.chan.datecreate,"+v %s",nick);
				
			if(strcor(c.chan.flags,"t") && c.chan.last_topic[0]!='\0' && ~c.chan.settings&CS_SUSPEND_BIT && c.chan.countusers==1)
				do_topic(thread, Q->nick, c.chan.last_topic_setter[0]=='\0' ? Q->nick : c.chan.last_topic_setter, c.chan.name, time(NULL), c.chan.last_topic);
		}
		
		/* Manage our Bot */
		//if((botjoin || csjoined) && c.chan.countusers < m->BSMinUsers){
		//	addStayChannel(thread,c.chan.name,c.chan.bot,botjoin,2); //3 tells us that it's a Q channel!
		//}
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * q_on_message: Q Events: On Message Event
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL q_on_message(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* (language == NULL, howtosend = 0, snick == NULL) */

	if((m->UseStrictPrivMsg && streqnc(param[2],_dynadd("%s@%s",Q->ident,Q->host))) || (!m->UseStrictPrivMsg && (streqnc(param[2],Q->nick) || streqnc(param[2],_dynadd("%s@%s",Q->ident,Q->host))))){
		if(GetUser(nick)==NULL)
	    	return MOD_CONT;

		ModCheck modcheck;
    	ModCo *modul = NULL;
    	char *cmd    = &param[3][1];
    	
    	bzero(&modcheck,sizeof(modcheck));
    	
    	bzero(&thread->sql,sizeof(SQL));
		sql_mainAAC(thread,nick);
		
    	/* Since language == NULL, howtosend = 0, snick == NULL - we wake them up now ;) */
    	snick		= Q->nick;
    	howtosend	= thread->sql.access.settings&NS_SET_MSG ? 1 : 0;
    	language	= languages[thread->sql.access.language];
    	/*********************************************************************************/
    	
	/* Q Commands */
	
		/* If help for a specific Command was requested */
    	if(streqnc(cmd,"help") && param[4]){
			
			/* Help with wildmasks */
    		if(strcor(param[4],"*"))
				call_wildview(thread, snick, nick, SERVICE, "showcommands", param);
    		
			/* Normal help for a specific command (no wildmasks) */
			else{
    			modul 		= findModule(thread, SERVICE, param[4]);
  				modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
  				
				if(!ModReply(thread, &modcheck, snick, nick, param))
					send_lang(thread, language, howtosend, snick, nick, "Q_NO_HELP_AVAIL", snick);
    		}
    	}
		
		/* If Command was only "help" */
		else if(streqnc(cmd,"showcommands") && !param[4]){
    		modul 		= findModule(thread, SERVICE, cmd);
    		modcheck 	= call_helpcommand(thread, modul, cmd, snick, nick, param);
    		
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "Q_NO_HELP_AVAIL", snick);
    	}
    	
		/* All other commands which aren't "help" */
		else if(!streqnc(cmd,"help")){
    		modul 		= findModule(thread, SERVICE, cmd);
  			modcheck 	= call_command(thread, modul, cmd, snick, nick, param);
			  	
			if(!ModReply(thread, &modcheck, snick, nick, param))
    			send_lang(thread, language, howtosend, snick, nick, "Q_NO_HELP_AVAIL", snick);
    	}
		else send_lang(thread, language, howtosend, snick, nick, "Q_NO_HELP_AVAIL", snick);
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return MOD_BOOL
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
    if(Action == DO_UNLOAD){
		Q->loaded = 0;
		do_quit(thread,Q->nick,"module unloaded");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */
