/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_showcommands.c
 *   @brief This is the Module file for Q SHOWCOMMANDS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q SHOWCOMMANDS command.
 *
 *   Showcommands function - /msg Q showcommands on IRC. - Q shows all commands that are available for your userlevel.
 *
 *   Syntax is: /msg Q SHOWCOMMANDS
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "Q"
#define MODUL "showcommands"
#define MODUL_FILE "q_showcommands"
#define MODUL_VERSION "1.0"

MOD_BOOL Q_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"showcommands","showcommands");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,1,OPER_IF_DEFCON_BIT);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"showcommands",Q_showcommands,0,0,3,6);
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q SHOWCOMMANDS Command.
 * 
 * The Q SHOWCOMMANDS Command allows the user....
 *  
 * Syntax is: /msg Q SHOWCOMMANDS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Header */
	if(thread->sql.access.fields && thread->sql.access.authnick)
		msend(thread,howtosend,snick,nick,"=======================> Your Level: [%d] <=======================",thread->sql.access.qlevel);
	else
 		msend(thread,howtosend,snick,nick,"=======================> Your Level: [%d] <=======================",0);

	send_lang(thread,language,howtosend,snick,nick,"Q_HELP_HEADER",snick,snick,snick,snick);
	send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_SHOWCOMMANDS");
	send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_HELP");

	/* Help */
	call_overviewhelp(thread, language, howtosend, snick, SERVICE, "showcommands", nick, param);

	/* Footer */
	send_lang(thread,language,howtosend,snick,nick,"Q_HELP_FOOTER",m->CSExpire);
	
    return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
