/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 204                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-03 20:33:44 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file IRCu.c
 *   @brief This is the Module file for IRCu (Server to Server) protocol.
 *
 *   It contains all functions which are neccessary to process
 *   Inputs from from IRCu and outputs to IRCu!
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "osocket.h"
#include "services.h"
#include "modules.h"

char *p10id = "AA";

/* Outbound */
void orion_set_umode(Threads *thread, size_t ac, char **av);
PROTOCOL_BOOL orion_cmd_initialize(Threads *thread);
PROTOCOL_BOOL orion_cmd_add_account(Threads *thread, char *uid, char *authnick);
PROTOCOL_BOOL orion_cmd_del_account(Threads *thread, char *uid);

PROTOCOL_BOOL orion_cmd_topic(Threads *thread, char *whosets, char *chan, char *whosetit, char *topic, time_t when);

PROTOCOL_BOOL orion_cmd_gline(Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason);
PROTOCOL_BOOL orion_cmd_ungline(Threads *thread, char *unsetter, char *host, char *user, time_t time_in);

PROTOCOL_BOOL orion_cmd_svskill(Threads *thread, char *source, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_kill(Threads *thread, char *source, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_rawsvsmode(Threads *thread, char *source, char *nick, char *modes);
PROTOCOL_BOOL orion_cmd_svsmode(Threads *thread, char *nick, char *target, char *modes);
PROTOCOL_BOOL orion_cmd_svs2mode(Threads *thread, char *nick, char *target, char *modes);
PROTOCOL_BOOL orion_cmd_nick(Threads *thread, char *nick, char *ident, char *host, char *name, char *modes);
PROTOCOL_BOOL orion_cmd_mode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);
PROTOCOL_BOOL orion_cmd_rawmode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);

PROTOCOL_BOOL orion_cmd_kick(Threads *thread, char *source, char *chan, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_notice(Threads *thread, char *source, char *dest, char *buf);
PROTOCOL_BOOL orion_cmd_privmsg(Threads *thread, char *source, char *dest, char *buf);
PROTOCOL_BOOL orion_cmd_serv_notice(Threads *thread, char *source, char *dest, char *msg);
PROTOCOL_BOOL orion_cmd_serv_privmsg(Threads *thread, char *source, char *dest, char *msg);

PROTOCOL_BOOL orion_cmd_quit(Threads *thread, char *source, char *buf);
PROTOCOL_BOOL orion_cmd_protoctl(Threads *thread);
PROTOCOL_BOOL orion_cmd_pass(Threads *thread, char *pass);
PROTOCOL_BOOL orion_cmd_server(Threads *thread, char *servname, int hop, char *descript);
PROTOCOL_BOOL orion_cmd_pong(Threads *thread, char *servname, char *who);
PROTOCOL_BOOL orion_cmd_ping(Threads *thread, char *servname, char *who);
PROTOCOL_BOOL orion_cmd_join(Threads *thread, char *user, char *channel, time_t chantime);
PROTOCOL_BOOL orion_cmd_services_join(Threads *thread, char *user, char *channel, time_t chantime, int fake);

PROTOCOL_BOOL orion_cmd_chghost(Threads *thread, char *source, char *nick, char *vhost);
PROTOCOL_BOOL orion_cmd_chgident(Threads *thread, char *source, char *nick, char *vIdent);
PROTOCOL_BOOL orion_cmd_invite(Threads *thread, char *source, char *chan, char *nick);
PROTOCOL_BOOL orion_cmd_part(Threads *thread, char *nick, char *chan, char *buf);

PROTOCOL_BOOL orion_cmd_wallops(Threads *thread, char *source, char *fmt);
PROTOCOL_BOOL orion_cmd_global(Threads *thread, char *source, char *buf);

PROTOCOL_BOOL orion_cmd_sqline(Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason);
PROTOCOL_BOOL orion_cmd_unsqline(Threads *thread, char *servername, char *nick, char *snick);

PROTOCOL_BOOL orion_cmd_squit(Threads *thread, char *servname, char *message);
PROTOCOL_BOOL orion_cmd_svso(Threads *thread, char *source, char *nick, char *flag);
PROTOCOL_BOOL orion_cmd_chg_nick(Threads *thread, char *oldnick, char *newnick);
PROTOCOL_BOOL orion_cmd_svsnick(Threads *thread, char *source, char *oldnick, char *newnick, time_t when);
PROTOCOL_BOOL orion_cmd_connect(Threads *thread, bool how);
PROTOCOL_BOOL orion_cmd_svshold(Threads *thread, char *nick);
PROTOCOL_BOOL orion_cmd_release_svshold(Threads *thread, char *nick);

PROTOCOL_BOOL orion_cmd_szline(Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason);
PROTOCOL_BOOL orion_cmd_unszline(Threads *thread, char *servername, char *mask, char *snick, time_t in_time);

PROTOCOL_BOOL orion_cmd_unban(Threads *thread, char *name, char *nick);
PROTOCOL_BOOL orion_cmd_svsmode_chan(Threads *thread, char *name, char *mode, char *nick);
PROTOCOL_BOOL orion_cmd_svsjoin(Threads *thread, char *source, char *nick, char *chan, time_t date);
PROTOCOL_BOOL orion_cmd_svspart(Threads *thread, char *source, char *nick, char *chan);
PROTOCOL_BOOL orion_cmd_sapart(Threads *thread, char *source, char *nick, char *chan);
PROTOCOL_BOOL orion_cmd_sajoin(Threads *thread, char *source, char *nick, char *chan, time_t date);

PROTOCOL_BOOL orion_cmd_swhois(Threads *thread, char *source, char *who, char *mask);
PROTOCOL_BOOL orion_cmd_eob(Threads *thread);
PROTOCOL_BOOL orion_cmd_jupe(Threads *thread, char *jserver, char *who, char *reason);
PROTOCOL_BOOL orion_cmd_global_legacy(Threads *thread, char *source, char *fmt);

PROTOCOL_BOOL orion_cmd_raw(Threads *thread, char *param[]);
bool orion_valid_nick(Threads *thread, char *nick);
bool orion_valid_chan(Threads *thread, char *chan);
bool orion_flood_mode_check(Threads *thread, char *value);

/* Inbound */
MOD_BOOL orion_event_gline(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_clearmode(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_sjoin(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_account(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_nick(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_topic(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_ping(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_kill(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_join(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_create(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_part(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_privmsg(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_quit(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_mode(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_error(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_squit(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_services_link_done(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_pong(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_server(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_eob(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_fakehost(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_kick(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_version(Threads *thread, size_t ac, char **av);

void CreateProtocol(Threads *thread, Protocol *protocol, void *dl_handle) {
    
	//printf("Creating Protocol ...\n");
    event_init(thread, protocol, dl_handle, "shock", "UndernetIRCu 2.10.12.12+ [version 0.1]");

    createEvent(thread,protocol, "J",      	  orion_event_join,     1);
    createEvent(thread,protocol, "L",         orion_event_part,     1);

    createEvent(thread,protocol, "M",         orion_event_mode,     1);
    createEvent(thread,protocol, "OM",        orion_event_mode,     1);
    createEvent(thread,protocol, "CM",        orion_event_clearmode,1);
    createEvent(thread,protocol, "B",         orion_event_sjoin,    1);

    createEvent(thread,protocol, "K",      	  orion_event_kick,     1);
    createEvent(thread,protocol, "N",      	  orion_event_nick,     1);
	
    createEvent(thread,protocol, "T",     	  orion_event_topic,  1);
    createEvent(thread,protocol, "C",     	  orion_event_create,  1);
    
    createEvent(thread,protocol, "G",      	  orion_event_ping,     1);
    createEvent(thread,protocol, "Z",         orion_event_pong,     1);
    createEvent(thread,protocol, "P",   	  orion_event_privmsg,  1);
    createEvent(thread,protocol, "Q",      	  orion_event_quit,     1);
    
    createEvent(thread,protocol, "GL",        orion_event_gline,      1);

    createEvent(thread,protocol, "V",   	  orion_event_version,  1);
    
    /** Server Modes **/
    createEvent(thread,protocol, "S",    	  orion_event_server,					1);
    createEvent(thread,protocol, "SERVER",    orion_event_server,					0);
    createEvent(thread,protocol, "SQ",     	  orion_event_squit,					1);

    createEvent(thread,protocol, "EB",  	  orion_event_eob,					1);
    
   protocol->NSMode						= "+ok";
   protocol->CSMode						= "+ok";
   protocol->MSMode						= "+ok";
   protocol->HSMode						= "+ok";
   protocol->OSMode						= "+iok";
   protocol->BSMode						= "+ok";
   protocol->HeServMode					= "+ok";
   protocol->GlobMode					= "+iok";
   protocol->QMode						= "+ok";
   protocol->SCMode						= "+iok";
   protocol->BSBotsMode					= "+k";
   protocol->ChannelModes				= "itmnopstrDdR";					/* Avail Channel Modes */
   protocol->ParametricChannelModes		= "klAU";							/* Channel Modes with an allowed Parameter (all possible) */
   protocol->MLockAbleModes				= "kl";								/* Channel Modes provided by MLOCK */
   protocol->OnSuspend					= "+pmntil 1";						/* Channel Modes on adding a suspend (forbid) */
   protocol->OnUnSuspend				= "-pmntil";						/* Channel Modes on removing a suspend (forbid) */
   protocol->SupportSVS					= false;							/* Supports SVS-COMMANDS */
   protocol->SupportSVSJoin				= false;							/* Supports SVSJOIN */
   protocol->SvsNick					= false;							/* Supports SVSNICK */
   protocol->VHost						= false;							/* Supports VHosts */
   protocol->HasOwner					= false;							/* Supports ChannelOwnerMode (+q) */
   protocol->HasProtect					= false;							/* Supports ChannelProtectMode (+a) */
   protocol->OwnerMode					= ' ';								/* UserChannelMode for Owner */
   protocol->ProtectMode				= ' ';								/* UserChannelMode for Protected */
   protocol->OpMode						= 'o';								/* UserChannelMode for Op */
   protocol->HOpMode					= ' ';								/* UserChannelMode for HalfOp */
   protocol->VMode						= 'v';								/* UserChannelMode for Voice */
   protocol->BanMode					= 'b';								/* UserChannelMode for Ban */
   protocol->ExceptMode					= ' ';								/* UserChannelMode for BanException */
   protocol->SupportedUserModes			= "iowgxkdO";						/* Known UserModes */
   protocol->SupportedUChanModes		= "ovb";							/* All UserChannelModes */
   protocol->SupportedU2ChanModes		= "ov";								/* All UserChannelModes except "Except (+e)" and Ban (+b) */
   protocol->ModeOnNickReg				= "";								/* Mode for Registered / Identified Nicks */
   protocol->ModeOnNickDrop				= "";								/* Mode for Dropping a registered Nick */
   protocol->ModeOnChannelReg			= "+R";								/* Mode for Registered Channels */
   protocol->ModeOnChannelDrop			= "-R";								/* Mode for Dropping a registered Channel */
   protocol->RegMode					= 'R';								/* Mode for Registration */
   protocol->ChanRegMode				= true;								/* Supports a Channel Registration Mode */
   protocol->NickRegMode				= false;							/* Supports a Nick Registration Mode */
   protocol->SupportSGlines				= false;							/* Supports Glines */
   protocol->SupportSQlines				= false;							/* Supports Qlines */
   protocol->SupportSZlines				= false;							/* Supports Zlines */
   protocol->SupportHalfOp				= false;							/* Supports HalfOpMode */
   protocol->SupportBanMode				= true;								/* Supports BanMode */
   protocol->SupportExceptMode			= false;							/* Supports ExceptMode */
   protocol->SupportProtectUmode		= true;								/* Supports a Protection UMode */
   protocol->ProtectUmode				= "k";								/* Mode for Protecting a Nickname */
   protocol->SupportInvisibleUMode		= false;							/* Supports InvisibleUMode */
   protocol->InvisibleUMode				= "";								/* Mode for Invisibility */
   protocol->SupportChanSQlines			= false;							/* Supports Chan SQlines */
   protocol->SupportVIdents				= false;							/* Supports VIdent's */
   protocol->VHostOnNickChange			= false;							/* Supports setting VHosts on NickChanges */
   protocol->SupportRealnameChanges		= false;							/* Supports RealnameChanges */
   protocol->SupportAdminOnlyChanMode	= false;							/* Supports an AdminOnly ChannelMode */
   protocol->SupportOperOnlyChanMode	= false;							/* Supports an OperOnly ChannelMode */
   
   protocol->DoesGlobalChangeAfterSVS	= false;							/*  */
   protocol->DoesGlobalChangeAfterSA	= false;							/*  */
   protocol->DoesQuitAfterKill			= true;								/* Sends a Quit-Msg on Kill Execution */
   
   protocol->AdminOnlyCMode				= " ";								/* AdminOnly ChannelMode */
   protocol->OperOnlyCMode				= " ";								/* OperOnly ChannelMode */
   protocol->DefaultMLOCK				= "+nt";							/* Default MLOCK Modes */
   protocol->VhostMode					= ' ';								/* Vhost Mode (is set on identify) */
   protocol->SupportFloodMode			= false;							/* Supports a FloodMode */
   protocol->SupportLinkMode			= false;							/* Supports a ChannelLinkMode */
   protocol->FloodCMode					= ' ';								/* ChannelFloodMode setting */
   protocol->LinkCMode					= ' ';								/* ChannelMode for linking Channels */
   protocol->KeyCMode					= 'k';								/* ChannelKeyMode setting */
   protocol->LimitCMode					= 'l';								/* ChannelMode for limit */
   protocol->InviteCMode				= 'i';								/* ChannelMode for inviteonly */
   protocol->SupportTokens				= false;							/* Supports Tokens */
   protocol->SupportSjoin				= true;								/* Supports SJOIN */
   protocol->SupportSjoinBanprefix		= true;								/* Supports a BanPrefix for SJOIN */
   protocol->SupportSjoinExceptprefix	= false;							/* Supports an ExceptPrefix for SJOIN */
   protocol->SjoinOwnerPrefix			= ' ';								/* OwnerPrefix for SJOIN */
   protocol->SjoinProtectPrefix			= ' ';								/* ProtectionPrefix for SJOIN */
   protocol->SjoinOpPrefix				= '@';								/* OpPrefix for SJOIN */
   protocol->SjoinHOpPrefix				= ' ';								/* HalfOpPrefix for SJOIN */
   protocol->SjoinVoicePrefix			= '+';								/* VoicePrefix for SJOIN */
   protocol->SjoinBanPrefix				= '%';								/* BanPrefix for SJOIN */
   protocol->SjoinExceptPrefix			= ' ';								/* ExceptPrefix for SJOIN */
   protocol->SjoinHasIgnoreChar			= false;							/* SJOIN has an ignored Mode (it will be skipped) */
   protocol->SjoinIgnoredChar			= ',';								/* Mode to be ignored from SJOIN */
   protocol->SupportHelperUMode			= false;							/* Supports Helper UMODE */
   protocol->SupportSyncState			= true;								/* Supports SyncState */
   protocol->p10protocol				= true;								/* Is p10 protocol */
   protocol->p10new						= false;							/* Is newp10 protocol */
   
   
	protocol->orion_cmd_initialize		= orion_cmd_initialize;
	protocol->orion_cmd_add_account		= orion_cmd_add_account;
	protocol->orion_cmd_del_account		= orion_cmd_del_account;
	protocol->orion_cmd_topic			= orion_cmd_topic;
	protocol->orion_cmd_kill			= orion_cmd_kill;
	protocol->orion_cmd_svskill			= orion_cmd_svskill;
	protocol->orion_cmd_rawsvsmode		= orion_cmd_rawsvsmode;
	protocol->orion_cmd_svsmode			= orion_cmd_svsmode;
	protocol->orion_cmd_svs2mode		= orion_cmd_svs2mode;
	protocol->orion_cmd_nick			= orion_cmd_nick;
	protocol->orion_cmd_mode			= orion_cmd_mode;
	protocol->orion_cmd_rawmode			= orion_cmd_rawmode;
	protocol->orion_cmd_kick			= orion_cmd_kick;
	protocol->orion_cmd_notice			= orion_cmd_notice;
	protocol->orion_cmd_privmsg			= orion_cmd_privmsg;
	protocol->orion_cmd_serv_notice		= orion_cmd_serv_notice;
	protocol->orion_cmd_serv_privmsg	= orion_cmd_serv_privmsg;
	
	protocol->orion_cmd_quit			= orion_cmd_quit;
	protocol->orion_cmd_protoctl		= orion_cmd_protoctl;
	protocol->orion_cmd_pass			= orion_cmd_pass;
	protocol->orion_cmd_server			= orion_cmd_server;
	protocol->orion_cmd_pong			= orion_cmd_pong;
	protocol->orion_cmd_ping			= orion_cmd_ping;
	protocol->orion_cmd_join			= orion_cmd_join;
	protocol->orion_cmd_services_join	= orion_cmd_services_join;
	protocol->orion_cmd_chghost			= orion_cmd_chghost;
	protocol->orion_cmd_chgident		= orion_cmd_chgident;
	protocol->orion_cmd_invite			= orion_cmd_invite;
	protocol->orion_cmd_part			= orion_cmd_part;
	
	protocol->orion_cmd_wallops			= orion_cmd_wallops;
	protocol->orion_cmd_global			= orion_cmd_global;
	
	protocol->orion_cmd_squit			= orion_cmd_squit;
	//protocol->orion_cmd_svso =orion_cmd_svso
	protocol->orion_cmd_chg_nick		= orion_cmd_chg_nick;
	protocol->orion_cmd_svsnick			= orion_cmd_svsnick;
	protocol->orion_cmd_connect			= orion_cmd_connect;
	/*protocol->orion_cmd_svshold =orion_cmd_svshold
	protocol->orion_cmd_release_svshold =orion_cmd_release_svshold*/
	
	protocol->orion_cmd_gline			= orion_cmd_gline;
	protocol->orion_cmd_ungline			= orion_cmd_ungline;
	
	protocol->orion_cmd_szline			= orion_cmd_szline;
	protocol->orion_cmd_unszline		= orion_cmd_unszline;
	
	protocol->orion_cmd_sqline			= orion_cmd_sqline;
	protocol->orion_cmd_unsqline		= orion_cmd_unsqline;
	
	protocol->orion_cmd_unban			= orion_cmd_unban;
	protocol->orion_cmd_svsmode_chan	= orion_cmd_svsmode_chan;
	protocol->orion_cmd_svsjoin			= orion_cmd_svsjoin;
	protocol->orion_cmd_svspart			= orion_cmd_svspart;
	
	protocol->orion_cmd_sajoin			= orion_cmd_sajoin;
	protocol->orion_cmd_sapart			= orion_cmd_sapart;
	
	protocol->orion_cmd_swhois			= orion_cmd_swhois;
	protocol->orion_cmd_eob				= orion_cmd_eob;
	protocol->orion_cmd_jupe			= orion_cmd_jupe;
	protocol->orion_cmd_global_legacy	= orion_cmd_global_legacy;
	protocol->orion_cmd_raw				= orion_cmd_raw;
	protocol->orion_valid_nick			= orion_valid_nick;
	protocol->orion_valid_chan			= orion_valid_chan;
	protocol->orion_flood_mode_check	= orion_flood_mode_check;
	
	dynncpy(global_target,m->host,sizeof(global_target));
	unload_modules(thread,"hs");
	printf(" [*] Created UndernetIRCu Protocol (version: 0.1)...\n");
	return;
}

#ifdef _WIN32
void DeleteProtocol(Threads *thread, size_t ac, char **av)
{
    return MOD_CONT;
}
#endif

int p10nickcnt = 1;
int p10servcnt = 1;
int p10inuse = 0;

char *militime_float(char *start)
{
	static char timebuf[18];
#if HAVE_GETTIMEOFDAY
	struct timeval tv;
	char *p;
	gettimeofday(&tv, NULL);
	
	if (start) {
		if ((p = strchr(start, '.'))) {
			p++;
			sprintf(timebuf, "%lu",
					(tv.tv_sec - stoi(start)) * 1000 + (tv.tv_usec -
													stoi(p)) / 1000);
		} else
			strcpy(timebuf, "0");
	} else
		sprintf(timebuf, "%lu.%lu", tv.tv_sec, tv.tv_usec);
		return timebuf;
#else
	sprintf(timebuf, "%lu", time(NULL));
	return sstrdup(timebuf);
#endif
}

PROTOCOL_BOOL orion_cmd_add_account(Threads *thread, char *uid, char *authnick)
{
	Users *user = GetUserByUID(uid);
    send_cmd(thread,p10id, "AC %s %s", uid, authnick);
    domysql(thread,"UPDATE `orion_core_condata` SET `modes`=CONCAT(REPLACE(`modes`,'r',''),'r') WHERE `nick`='%s'",mysql_quote(thread,user->nick));
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_del_account(Threads *thread, char *uid)
{
    return CMD_NOT_SUPPORTED;
}

/* ABAAA M +i */
/* ABAAA M +ih lol@roflman.de */
void orion_set_umode(Threads *thread, size_t ac, char **av)
{
    //int add = 1;                /* 1 if adding modes, 0 if deleting */
    int paramstart = 0;
    Users *u = NULL;
    deepSQL umode;
    char *modes = NULL;
    char *nick = NULL;

    bzero(&umode,sizeof(umode));
    
    //> ABAAX M shock -h
    if(av[2][0]=='+' || av[2][0]=='-'){
		modes = av[2];
		nick = av[0];
		paramstart = 3;
	}else if(av[3][0]=='+' || av[3][0]=='-'){
		modes = av[3];
		nick = av[2];
		paramstart = 4;
	}
	
	if(ac-1<paramstart)
		paramstart = 0;
	
	u = GetUserByUID(nick);
	if(!u)
		u = GetUser(nick);
		
	//printf("No User!?\n");
	if(!u) return;
	//printf("we found the User =D\n");
	
    //ac--;

    dlog(thread,2,"debug: Changing mode for %s to %s\n", u->nick, modes);
        
	char addmodes[70]; //Modes for Usermode add
	char delmodes[70]; //Modes for Usermode del
	char allmode[80]; //DelModes and AddModes
	
    // < :O SVSMODE shock -w
	
	allmode[0]='\0'; //set allmode NULL
	addmodes[0]='\0'; //set addmodes NULL
	delmodes[0]='\0'; //set delmodes NULL
	
	umode.res		= doSelectMySQL(thread,"SELECT `orion_core_condata`.`modes`, COUNT(`orion_ns_aliases`.`nick`), `orion_ns_aliases`.`nick`, `orion_ns_aliases`.`display`,`orion_ns_aliases`.`last_usermask`,`orion_ns_aliases`.`last_realname`,`orion_ns_aliases`.`last_quit`,`orion_ns_aliases`.`time_linked`,`orion_ns_aliases`.`last_login`,`orion_ns_aliases`.`last_used`,`orion_ns_aliases`.`expire`,`orion_ns_aliases`.`linked`,`orion_ns_aliases`.`active`,`orion_ns_aliases`.`forbid` FROM `orion_core_condata` LEFT OUTER JOIN `orion_ns_aliases` ON `orion_ns_aliases`.`nick`=`orion_core_condata`.`nick` WHERE `orion_core_condata`.`nick`='%s' GROUP BY `orion_ns_aliases`.`nick`",mysql_quote(thread,u->nick));
	umode.fields	= mysql_num_rows(umode.res);
	umode.row		= mysql_fetch_row(umode.res);
	
	if(!umode.fields){
		/* Free Quotes-Array */
		mysql_free_quotes(thread);
		
		/* Free MySQL Result */
		mysql_free_result_myr(umode.res);
		return;
	}

	char *mode=umode.row[0];
	
	bool type=1;
	size_t i=0;
	size_t a=0;
	size_t b=0;
	for(i=0;i<dynlen(modes);i++){
		char current=modes[i];
		if(current=='+'){type=1;}
		else if(current=='-'){type=0;}
		else if(strcin(current,IRC.SupportedUserModes)){
			switch(current){
       			case 'x':
					if(stoi(umode.row[1])>0){ //there is already an account
						if(umode.row[3]!=NULL){
							domysql(thread,"UPDATE `orion_core_condata` SET `vhost`='%s' WHERE `nick`='%s'",mysql_quote(thread,_dynadd("%s.users",umode.row[3],&m->host[1])),mysql_quote(thread,u->nick));
						}
					}
            	break;
        		
				case 'r':
            		if (type) {
						if(stoi(umode.row[1])>0){ //there is already an account
							if(stoi(umode.row[8])==0){
								domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='1',`last_login`='%lu' WHERE `nick`='%s'",time(NULL),mysql_quote(thread,u->nick));
                				send_cmd(thread, p10id, "AC %s U", u->uid);
							}
						}
            		}
        			break;
			}
			if(type==0 && strcin(current,mode)){
				delmodes[a]=current;
				delmodes[a+1]='\0';
				a++;
			}
			else if(type==1 && !strcin(current,mode)){
				addmodes[b]=current;
				addmodes[b+1]='\0';
				b++;
			}
		}
	}
	dynncpy(allmode,mode,sizeof(allmode)); //copy sql saved modes to allmode
	strncat(allmode,addmodes,strlen(addmodes)); //add addmodes to allmode
	for(i=0;delmodes[i]!='\0';i++){
		strdelc(allmode,delmodes[i]); //delete delmodes[i] from allmode
	}
	
	//printf("Mode Result = %s\n",allmode);
	
	if(noop!=1){
		/* 	Update our Cache	*/
		if(streqnc(thread->c.Cdata.nick,nick)){
			bzero(&thread->c.Cdata.modes,sizeof(thread->c.Cdata.modes));
			dynncpy(thread->c.Cdata.modes,allmode,sizeof(thread->c.Cdata.modes));
		}else if(streqnc(thread->sql.Cdata.nick,nick)){
			bzero(&thread->sql.Cdata.modes,sizeof(thread->sql.Cdata.modes));
			dynncpy(thread->sql.Cdata.modes,allmode,sizeof(thread->sql.Cdata.modes));
		}
	/**			***			**/
	
		//Last Bug Found: 6.9.2005 13:11:20 UTC
		
		// > :shock MODE shock :+owghaACeW
		// < :O SVSMODE shock -xowghCeW
		// < :O SVSMODE O -
		// < :O SVSMODE O -
		// < :O SVSMODE O -
		domysql(thread,"UPDATE `orion_core_condata` SET `modes`='%s' WHERE `nick`='%s'",mysql_quote(thread,allmode),mysql_quote(thread,nick));
	}
	else if(noop==1){
		bzero(&thread->c,sizeof(thread->c));
		do_kill(thread,m->servername,nick,"your olines are nooped.");
	}
	/* Free MySQL Result */
	mysql_free_result_myr(umode.res);

	/* Free Quotes-Array */
	mysql_free_quotes(thread);
}

MOD_BOOL orion_event_clearmode(Threads *thread, size_t ac, char **av){
	if(ac<3)
		return MOD_CONT;
		
	on_mode(thread,O->nick,av[2],time(NULL),av[3][0]!='\0' ? av[3] : _dynadd("-%s",av[3]));
	domysql(thread,"UPDATE `orion_chan_users` SET `modes`='' WHERE `channel`='%s'",mysql_quote(thread,av[2]));
	return MOD_CONT;
}

//:<source server> ADDLINE [Q|Z|E|G] <mask> <setter> <time set> <duration> :<reason>
//> ABAAN GL * +*@35.35.35.35 200 1206719445 :lol
//> ABAAN GL * +*@35.35.35.35 200 1206719445 :lol
MOD_BOOL orion_event_gline(Threads *thread, size_t ac, char **av){
	Servers *server = NULL;
	Users *user = GetUser(av[0]);
	char *gline = NULL;
	
	if(user == NULL)
		user = GetUserByUID(av[0]);
	
	if(user == NULL)
		return MOD_CONT;
		
	if(!streq(av[2],"*"))
		return MOD_CONT;
		
	server = findserver(thread, servlist, user->server);
	if(!server)
		server = findserver_uid(thread, servlist, user->server);
		
	gline  = &av[3][findchar(av[3],'+')];
	
	if(av[3][0]=='+'){
		on_gline(thread,user->nick,gline,_dynadd("%s!%s@%s",user->nick,user->vident,user->vhost),(time_t)stol(av[4])+(time_t)stol(av[5]),(time_t)stol(av[5]),&_dstrjoin(' ',av,6)[1]);
	}
	//void on_ungline(Threads *thread, char *servername, char *host)
	else if(av[3][0]=='-'){
		on_ungline(thread,server->server,gline);
	}
	return MOD_CONT;
}

MOD_BOOL orion_event_version(Threads *thread, size_t ac, char **av)
{
	Users *user = NULL;
	char nick[MAX_NICK_LEN+2];
	
    if (ac < 2)
        return MOD_CONT;
	
	user = GetUserByUID(av[0]);
	if(user == NULL)
		user = GetUser(av[0]);

	bzero(&nick,sizeof(nick));
	dynncpy(nick,user->nick,sizeof(nick));
	
	version(thread,nick);
	
	/* Execute on_version event */
	_ExecuteEvent(nick,NULL,"on_version");
    return MOD_CONT;
}

//> AC B #qtest 1207606036
//> AB B #services 1206908430 +tnl 10 ABAAf:o :%rofl!*@*
MOD_BOOL orion_event_sjoin(Threads *thread, size_t ac, char **av)
{
    char outbuf[BUFSIZE];
    char users[BUFSIZE];
    char bans[BUFSIZE];
    
    char *usercheck[ARRAY_SIZE];
    char *bancheck[ARRAY_SIZE];
    char *param[ARRAY_SIZE];
    
    size_t i 		= 0;
    size_t j 		= 0;
    size_t a 		= 0;
    size_t nr_modes = 0;
    
    int usercount 	= 0;
    int bancount  	= 0;
    bool type		= 1;
    int modeparamnr = 0;
    int splicer		= 0;
    int further		= 0;
    
    /* Clear string Arrays */
    bzero(&outbuf,sizeof(outbuf));
    bzero(&users,sizeof(users));
    bzero(&bans,sizeof(bans));
    
    /* Clear ValuePointer Arrays */
    bzero(usercheck,sizeof(usercheck));
    bzero(bancheck,sizeof(bancheck));
    bzero(param,sizeof(param));
    
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_CONDATA_NO_MODES);
    
    if(ac < 4){
		dlog(thread,0,"IRCu shouldn't support \"zannels\", cauze orion isn't able to manage it");
		orion_exit(thread);
	}else if(ac == 4){
		domysql(thread,"UPDATE `orion_cs_core` SET `datecreate`='%lu' WHERE `channel` = '%s'",stol(av[3]),mysql_quote(thread,av[2]));
		if(streqnc(thread->c.chan.name,av[2]))
			thread->c.chan.datecreate=(time_t)stol(av[3]);
    	return MOD_CONT;
	}else
    
		/* Updates the Threads Channel Management */
		UpdateChannelInformation3(thread,av[2],NULL);
	
	if(av[4][0]=='+'){
		further++;
		nr_modes = dynlen(av[4]);
		
		param[modeparamnr++] = av[4];
		for(i=0;i<nr_modes;i++){
			if(av[4][i]=='+'){ type=1; splicer++; }
			else if(av[4][i]=='-'){ type=0; splicer++; }
			
			if(av[4][i]==IRC.LimitCMode){
				//printf("limit = %s, paramnr=%d\n",av[5+modeparamnr-splicer],5+modeparamnr-splicer);
				param[modeparamnr]=av[5+modeparamnr-splicer];
				further++;
				modeparamnr++;
			}
			else if(av[4][i]==IRC.KeyCMode){
				param[modeparamnr]=av[5+modeparamnr-splicer];
				further++;
				modeparamnr++;
			}
		}
		param[modeparamnr++]=NULL;
		//printf("UndernetIRCu sending mode: %s\n",_djoin(' ',param));
        on_mode(thread,findserver_uid(thread, servlist, av[0])->server,av[2],(time_t)stol(av[3]),_djoin(' ',param)); //channel mode
	}
    
    dynncpy(users,av[4+further],sizeof(users));
    if(ac-1>=5+further)
   		dynncpy(bans,&_dstrjoin(' ',av,5+further)[1],sizeof(bans));
   		
   	//printf("Users = %s\n",users);
   	//printf("Bans = %s\n",bans);
   	
   	usercount = strsplit(users,usercheck,',');
   	for(i=0;i<usercount;i++){
		//printf("usercheck[i] = %s\n",usercheck[i]);
		if(GetUserByUID(strtok(usercheck[i],":"))!=NULL){
			a = getposition(usercheck[i],':');
			//printf("a = %d\n",a);
			if(a>=0){
				for(j=0;j<dynlen(IRC.SupportedU2ChanModes);j++){
					if(a+j<dynlen(usercheck[i])){
						//printf("usercheck[i][a+j] = %c\n",usercheck[i][a+j]);
						if(usercheck[i][a+j]==IRC.OpMode)
							straddc(outbuf,IRC.SjoinOpPrefix);
						else if(usercheck[i][a+j]==IRC.VMode)
							straddc(outbuf,IRC.SjoinVoicePrefix);
					}else break;
				}
			}
			//printf("temp outbuf = %s\n",outbuf);
			stradds(outbuf,GetUserByUID(strtok(usercheck[i],":"))->nick);
			straddc(outbuf,' ');
		}
	}
		
   	bancount = strsplit(bans,bancheck,' ');
   	for(i=0;i<bancount;i++){
		if(bancheck[i][0]!=IRC.SjoinBanPrefix){
			straddc(outbuf,IRC.SjoinBanPrefix);
			stradds(outbuf,bancheck[i]);
			straddc(outbuf,' ');
		}else{
			stradds(outbuf,bancheck[i]);
			straddc(outbuf,' ');
		}
	}//AZ BURST #coder-com 949217470 ACAAB:o,ACAAD :%*!*another@*.ban.com
    
    //printf("outbuf = %s\n",outbuf);
    sjoin(thread, av[2], (time_t)stol(av[3]), outbuf, 0);
    return MOD_CONT;
}

PROTOCOL_BOOL orion_cmd_notice(Threads *thread, char *source, char *dest, char *buf)
{
    Users *u;
    Users *ud;

    if (!buf) {
        return CMD_ERROR;
    }

    u = GetUser(dest);
    ud = GetUser(source);

    send_cmd(thread,(ud ? ud->uid : p10id), "O %s :%s", (u ? u->uid : dest), buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_privmsg(Threads *thread, char *source, char *dest, char *buf)
{
    Users *u;
    Users *ud;

    if (!buf) {
        return CMD_ERROR;
    }

    u = GetUser(dest);
    ud = GetUser(source);

    send_cmd(thread,(ud ? ud->uid : p10id), "P %s :%s", (u ? u->uid : dest),
             buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_serv_notice(Threads *thread, char *source, char *dest, char *msg)
{
    Servers *u;
    Users *ud;

    u = findserver(thread, servlist, dest);
    if(!u)
		u = findserver_uid(thread, servlist, dest);
		
    ud = GetUser(source);

    send_cmd(thread,(ud ? ud->uid : p10id), "O $%s :%s", (u ? u->sid : dest), msg);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_serv_privmsg(Threads *thread, char *source, char *dest, char *msg)
{
    Servers *u;
    Users *ud;

    u = findserver(thread, servlist, dest);
    if(!u)
		u = findserver_uid(thread, servlist, dest);
		
    ud = GetUser(source);

    send_cmd(thread,(ud ? ud->uid : p10id), "P $%s :%s", (u ? u->sid : dest), msg);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_global(Threads *thread, char *source, char *buf)
{
    Servers *server = NULL;

    if (!buf) {
        return CMD_ERROR;
    }

    if (source)
		server = findserver(thread, servlist, source);
    
    if(!server)
		server = findserver_uid(thread, servlist, source);

    send_cmd(thread,(source ? (server ? server->sid : p10id) : p10id), "WA :%s", buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_wallops(Threads *thread, char *source, char *fmt)
{
    Servers *server = NULL;

    if (!fmt) {
        return CMD_ERROR;
    }

    if (source)
		server = findserver(thread, servlist, source);
    
    if(!server)
		server = findserver_uid(thread, servlist, source);
      
    send_cmd(thread,(source ? (server ? server->sid : p10id) : p10id), "WA :%s", fmt);
    return CMD_SUPPORTED;
}

/* GLOBOPS - to handle old WALLOPS */
PROTOCOL_BOOL orion_cmd_global_legacy(Threads *thread, char *source, char *fmt)
{
    Servers *server = NULL;

	server = findserver(thread, servlist, source);
    if(!server)
		server = findserver_uid(thread, servlist, source);

    send_cmd(thread,(server ? server->sid : p10id), "WA :%s", fmt);
    return CMD_SUPPORTED;
}

MOD_BOOL orion_event_account(Threads *thread, size_t ac, char **av)
{
    Access access;
    Servers *s;
    int res;
    char *cookie;
    char *account;
    char *password;

    if (!strcmp(av[1], "C")) {
        cookie = av[ac-3];
        account = ac[av-2];
        password = av[ac-1];

        s = findserver(thread, servlist, av[0]);
        if (!s)
            s = findserver_uid(thread, servlist, av[0]);

		access = sql_access(thread,account);
        if (access.authnick[0]=='\0') {
             send_cmd(thread, p10id, "AC %s D %s", av[0], cookie);
             return MOD_CONT;
        }

        res = orion_check_password(password, access.password);

        if (res == 1) {
            send_cmd(thread, p10id, "AC %s A %s %lu", av[0], cookie, time(NULL));
        } else {
            send_cmd(thread, p10id, "AC %s D %s", av[0], cookie);
        }
    }

    return MOD_CONT;
}

// AB N shock 1 1206281889 shock M806P031.adsl.highway.telekom.at +i A[L4S] ABAAA :shock
MOD_BOOL orion_event_nick(Threads *thread, size_t ac, char **av)
{
    Users *user;
    Servers *s;
    char oldnick[MAX_NICK_LEN];

	bzero(&oldnick,sizeof(oldnick));
	if (ac > 4) {
		struct in_addr ia;
		char *realname, *ip, *nick;
        char *ident, *host, *modes;
        char *uid = "";
        char *account = NULL;
        char *fakehost = NULL;
        char *sethost = NULL;
        char *timestamp = "";
        char *vhost = "";
        char *vident = NULL;
        char ipstr[17];
        char ipstr_new[17];
        
        bzero(&ipstr,sizeof(ipstr));
        bzero(&ipstr_new,sizeof(ipstr_new));
        
        s = findserver(thread, servlist, av[0]);
        if (!s)
            s = findserver_uid(thread, servlist, av[0]);
            
        nick = av[2];
        ident = av[5];
        host = av[6];

        if (av[7][0]=='+') {
			int cnt = 8;
			int c = 1;
			char *uaccount = "";
			char *accb = NULL, *acc = NULL;
			modes = av[7];
			while (*modes) {
				switch (*modes) {
					case 'r':
						uaccount = av[cnt++];
                        for (acc = strtok_r(uaccount, ":", &accb);
                             acc;
                             acc = strtok_r(NULL, ":", &accb)) {
                            if (c == 1)
                                account = acc;
                            else if (c == 2)
                                timestamp = acc;
                            c++;
                        }
                        c = 1;
                        break;
                  	case 'h':
                    	sethost = av[cnt++];
                    	//printf("SETHOST = %s\n",sethost);
                    	break;
                  	case 'f':
                    	fakehost = av[cnt++];
                    	break;
                  	default:
                    	break;
              	}
              	modes++;
          	}
          	modes = av[7];
          	
        	realname = &_dstrjoin(' ',av,cnt+2)[1];
        	uid = av[cnt+1];
        	ip = av[cnt];
        	
        	//printf("realname = %s\n",realname);
        	//printf("uid = %s\n",uid);
        	//printf("ip = %s\n",ip);
        	
        } else {
			modes = "";
			realname = &_dstrjoin(' ',av,9)[1];
			uid = av[8];
			ip = av[7];
		}

        user = GetUser(nick);
        if(user == NULL)
			user = addUser(nick,s->server);
			
        dynncpy(user->uid,uid,sizeof(user->uid));
        dynncpy(user->ip,ip,sizeof(user->ip));
        dynncpy(user->host,host,sizeof(user->host));
        dynncpy(user->ident,ident,sizeof(user->ident));
        dynncpy(user->vident,ident,sizeof(user->vident));
        dynncpy(user->vhost,host,sizeof(user->vhost));
        user->faketype = 0;
        
        ia.s_addr = base64toIP(ip);
        dynncpy(ipstr,inet_ntoa(ia),sizeof(ipstr));
        
		char *ip_tok[10];
		strsplit(ipstr,ip_tok,'.');

		sprintf(ipstr_new,"%s.%s.%s.%s",ip_tok[3],ip_tok[2],ip_tok[1],ip_tok[0]);
		//printf("before if() - user->nick = %s\n",user->nick);
		//printf("before if() - user->ident = %s\n",user->ident);
		//printf("before if() - user->vhost = %s\n",user->vhost);

       	if (user) {
         	if (fakehost || sethost) {
           		vhost = "";
           		
           		if (sethost) {
					int h = 1;
               		char *uhb, *uh = NULL;
               		vident = NULL;
               		
               		for (uh = strtok_r(sethost, "@", &uhb);
                    	uh;
                    	uh = strtok_r(NULL, "@", &uhb))
					{
                   		if (h == 1)
                       		vident = uh;
                   		else if (h == 2)
                       		vhost = uh;
                   		h++;
               		}
               		h = 1;
               		//printf("vident = %s\n",vident);
               		bzero(&user->vident,sizeof(user->vident));
               		dynncpy(user->vident,vident,sizeof(user->vident));
           		}
           		//printf("vhost = %s\n",vhost);
           		dynncpy(user->vhost,sethost ? vhost : fakehost,sizeof(user->vhost));
         	}
       	}
       	
		//printf("after if() - vident = %s\n",vident);
		//printf("after if() - user->vident = %s\n",user->vident);
		//printf("after if() - user->ident = %s\n",user->ident);
       	
       	if(account!=NULL){
			domysql(thread,
				"INSERT INTO `orion_ns_aliases` (`nick`,`display`,`active`,`time_linked`,`last_login`,`last_used`,`last_realname`,`last_usermask`,`expire`) VALUES "
				"('%s','%s','1','%lu','%lu','%lu','%s','%s','1') ON DUPLICATE KEY UPDATE "
				"`active`=			VALUES(`active`),"
				"`last_login`=		VALUES(`last_login`),"
				"`last_used`=		VALUES(`last_used`),"
				"`last_realname`=	VALUES(`last_realname`),"
				"`last_usermask`=	VALUES(`last_usermask`)"
				,mysql_quote(thread,nick),mysql_quote(thread,account),time(NULL),time(NULL),time(NULL),mysql_quote(thread,realname),mysql_quote(thread,user->vhost)
			);
		}
       	
       	/* Insert our user */
       	cnick(thread, nick, vident!=NULL ? user->vident : user->ident, user->ident , modes, (time_t)stol(av[4]), host, user->vhost, s->server, realname, ipstr_new, 0);
       	
	} else {
        user = GetUserByUID(av[0]);
        dynncpy(oldnick,user->nick,sizeof(oldnick));
        chgnick(thread, oldnick, av[2]);
    }
    return MOD_CONT;
}

//AB T #services shock!shock@62-47-129-148.adsl.highway.telekom.at 1206908430 1206908955 :bin kurz schwimmen
MOD_BOOL orion_event_topic(Threads *thread, size_t ac, char **av)
{
	Servers *s;
    s = findserver_uid(thread, servlist, av[0]);
    if (!s)
        s = findserver(thread, servlist, av[0]);
        
    if(s!=NULL){
		//server link - server topic
		//topic(thread,strtok(av[3],"!"),av[2],stol(av[5]),&_dstrjoin(' ',av,6)[1]);
	}else{
		//topic from a nick
		on_topic(thread,GetUserByUID(av[0])->nick,av[2],(time_t)stol(av[5]),&_dstrjoin(' ',av,6)[1]);
	}
    return MOD_CONT;
}

PROTOCOL_BOOL orion_cmd_ungline(Threads *thread, char *unsetter, char *host, char *user, time_t time_in)
{
    send_cmd(thread, p10id, "GL * -*@%s", host);
    return CMD_SUPPORTED;
}

/* FORMAT ? */
PROTOCOL_BOOL orion_cmd_gline(Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason)
{
    Users *ud;

    ud = GetUser(O->nick);

    send_cmd(thread, (ud ? ud->uid : O->nick), "GL * +*@%s %ld %ld :%s", host,
             (expires -  time(NULL)), time(NULL),
             reason);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_services_join(Threads *thread, char *user, char *channel, time_t chantime, int fake)
{
    Users *ud;
	char output[BUFSIZE];
	bzero(&output,sizeof(output));
	
	/* Updates the Threads Channel Management */
    ud = GetUser(user);
	UpdateChannelInformation3(thread,channel,NULL);

	if(IRC.HasOwner) 			straddnc(output,IRC.SjoinOwnerPrefix,sizeof(output));
	if(IRC.HasProtect)			straddnc(output,IRC.SjoinProtectPrefix,sizeof(output));
								straddnc(output,IRC.SjoinOpPrefix,sizeof(output));
	if(IRC.SjoinHasIgnoreChar)	straddnc(output,IRC.SjoinIgnoredChar,sizeof(output));
								straddns(output,user,sizeof(output));
							
	//printf("orion_cmd_services_join (channel = %s),(date = %lu),(output = %s)\n",channel,thread->c.chan.datecreate ? thread->c.chan.datecreate : chantime,output);	
    sjoin(thread,channel,thread->c.chan.datecreate ? thread->c.chan.datecreate : chantime,output,fake);

	if(DuringServerlink){
        send_cmd(thread, p10id, "B %s %ld %s:o", channel,
                 thread->c.chan.datecreate ? thread->c.chan.datecreate : chantime, (ud ? ud->uid : user));
    } else {
         send_cmd(thread, (ud ? ud->uid : user), "J %s %ld", channel,
                  thread->c.chan.datecreate ? thread->c.chan.datecreate : chantime);
         send_cmd(thread, p10id, "M %s +o %s", channel, (ud ? ud->uid : user));
	}
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_join(Threads *thread, char *user, char *channel, time_t chantime)
{
    Users *ud;

    ud = GetUser(user);

	send_cmd(thread, (ud ? ud->uid : user), "J %s %ld", channel, chantime);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_topic(Threads *thread, char *whosets, char *chan, char *whosetit,
                      char *topic, time_t when)
{
    Users *u;
    u = GetUser(whosets);

    send_cmd(thread, (u ? u->uid : whosets), "T %s %s %ld %ld :%s", chan, _dynadd("%s!%s@%s",u->nick,u->ident,u->host),
             when, time(NULL), topic);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svskill(Threads *thread, char *source, char *user, char *buf)
{
    Users *ud;
    Users *ud2;

    if (!buf) {
        return CMD_ERROR;
    }

    if (!source || !user) {
        return CMD_ERROR;
    }

    ud = GetUser(source);
    ud2 = GetUser(user);

    send_cmd(thread, (ud ? ud->uid : p10id), "D %s :%s", (ud2 ? ud2->uid : user), buf);
    return CMD_SUPPORTED;
}

/* PASS */
PROTOCOL_BOOL orion_cmd_pass(Threads *thread, char *pass)
{
    send_cmd(thread, NULL, "PASS :%s", pass);
    return CMD_SUPPORTED;
}

/* SERVER name hop descript */
PROTOCOL_BOOL orion_cmd_server(Threads *thread, char *servname, int hop, char *descript)
{
    char newid[3];
    Servers *s;

    if (p10inuse == 1) {
		int num = stoi(m->Numeric)+p10servcnt;
		inttobase64(newid, num, 2);
		
		s = findserver(thread, servlist, servname);
		if(s)
			dynncpy(s->sid,newid,2);
      
		p10servcnt++;
    }

    send_cmd(thread, ((p10inuse == 1) ? p10id : NULL), "%s %s %d %lu %lu J10 %s]]] +s :%s", ((p10inuse == 1) ? "S" : "SERVER"),
               servname, hop, m->starttime, time(NULL), ((p10inuse == 1) ? newid : p10id), descript);

    if (p10inuse == 1) {
      send_cmd(thread, newid, "EB");
      send_cmd(thread, newid, "EA");
    }

    p10inuse = 1;
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_nick(Threads *thread, char *nick, char *ident, char *host, char *name, char *modes)
{
    char nicknumbuf[6];
    Users *user = NULL;
    send_cmd(thread, p10id, "N %s 1 %ld %s %s %s B]AAAB %sAA%c :%s", nick,
             time(NULL), ident, host, modes, p10id, (p10nickcnt + 'A'),
             name);
    snprintf(nicknumbuf, 6, "%sAA%c", p10id, (p10nickcnt + 'A'));
    
	user = GetUser(nick);
	if(user == NULL)
		user = addUser(nick,m->servername);
	
	dynncpy(user->ident,ident,sizeof(user->ident));
	dynncpy(user->host,host,sizeof(user->host));
	dynncpy(user->vident,ident,sizeof(user->vident));
	dynncpy(user->vhost,host,sizeof(user->vhost));
	dynncpy(user->uid,nicknumbuf,sizeof(user->uid));
				
    p10nickcnt++;
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_part(Threads *thread, char *nick, char *chan, char *buf)
{
    Users *u;

    u = GetUser(nick);

    if (buf) {
        send_cmd(thread, (u ? u->uid : nick), "L %s :%s", chan, buf);
    } else {
        send_cmd(thread, (u ? u->uid : nick), "L %s", chan);
    }
    return CMD_SUPPORTED;
}

/* FORMAT ? */
MOD_BOOL orion_event_ping(Threads *thread, size_t ac, char **av)
{

    if (ac < 1)
        return MOD_CONT;
    orion_cmd_pong(thread, p10id, av[3]);
    return MOD_CONT;
}

MOD_BOOL orion_event_kill(Threads *thread, size_t ac, char **av)
{
    if (ac < 2)
        return MOD_CONT;

    _kill(thread, av[1]);
    return MOD_CONT;
}
//> ABAAe J #services 1206895941
MOD_BOOL orion_event_join(Threads *thread, size_t ac, char **av)
{
    Users *u;

    u = GetUserByUID(av[0]);

    if (ac < 3) {
        return MOD_CONT;
    }

    _join(thread, 0, (u ? u->nick : av[0]), _dstrjoin(',',av,2), (time_t)stol(av[ac-1]));
    return MOD_CONT;
}

MOD_BOOL orion_event_create(Threads *thread, size_t ac, char **av)
{
    char nick[BUFSIZE];
    Users *u;
    bzero(&nick,sizeof(nick));

    u = GetUserByUID(av[0]);

    strcat(nick, "@");
    strcat(nick, u->nick);

    sjoin(thread,av[2],(time_t)stol(av[3]),nick, 0);
    return MOD_CONT;
}

MOD_BOOL orion_event_part(Threads *thread, size_t ac, char **av)
{
    Users *u;

    u = GetUserByUID(av[0]);

    if (ac < 3)
        return MOD_CONT;
        
    on_part(thread, u->nick, av[2]);
    return MOD_CONT;
}

MOD_BOOL orion_event_privmsg(Threads *thread, size_t ac, char **av)
{
    Users *u = NULL;
    Users *id = NULL;
    char buf[BUFSIZE];
	char *privmsg[ARRAY_SIZE];
	
	bzero(&buf,sizeof(buf));
	bzero(privmsg,sizeof(privmsg));

    u = GetUserByUID(av[0]);
    id = GetUserByUID(av[2]);

    if (ac < 3)
        return MOD_CONT;
        
    sprintf(buf,":%s PRIVMSG %s %s",(u ? u->nick : av[0]),(id ? id->nick : av[2]),_dstrjoin(' ',av,3));
    //printf("MSG: %s\n",buf);
    strsplit(buf,privmsg,' ');
    
    on_privmsg(thread, privmsg);
    return MOD_CONT;
}

MOD_BOOL orion_event_quit(Threads *thread, size_t ac, char **av)
{
    Users *u = NULL;

    u = GetUserByUID(av[0]);
    if (ac < 2)
        return MOD_CONT;
        
    on_quit(thread, (u ? u->nick : av[0]));
    return MOD_CONT;
}

PROTOCOL_BOOL orion_cmd_mode(Threads *thread, char *source, char *dest, time_t time_in, char *buf)
{
    Users *u = NULL;

    u = GetUser(source);

    if (!buf) {
        return CMD_ERROR;
    }
    //printf("orion_cmd_mode - source = %s, dest = %s, buf = %s, time = %lu\n",(u ? u->uid : (source ? source : p10id)), dest, buf, time_in);
    send_cmd(thread, (u ? u->uid : (p10id ? p10id : source)), "M %s %s", dest, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_kick(Threads *thread, char *source, char *chan, char *user, char *buf)
{
    Users *ud;
    Users *tgt;
    ud = GetUser(source);
    tgt = GetUser(user);

    if (buf) {
        send_cmd(thread, (ud ? ud->uid : source), "K %s %s :%s", chan, (tgt ? tgt->uid : user), buf);
    } else {
        send_cmd(thread, (ud ? ud->uid : source), "K %s %s", chan, (tgt ? tgt->uid : user));
    }
    return CMD_SUPPORTED;
}

/* QUIT */
PROTOCOL_BOOL orion_cmd_quit(Threads *thread, char *source, char *buf)
{
    Users *ud;
    ud = GetUser(source);

    if (buf) {
        send_cmd(thread, (ud ? ud->uid : source), "QUIT :%s", buf);
    } else {
        send_cmd(thread, (ud ? ud->uid : source), "QUIT");
    }
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_pong(Threads *thread, char *servname, char *who)
{
    char *t, *s;
    time_t ts, tsnow, value;
    Servers *sv;

    if (!servname || !who) {
        return CMD_ERROR;
    }

    sv = findserver(thread, servlist, servname);
    if (!sv)
		sv = findserver_uid(thread, servlist, servname);

    t = myStrGetToken(who, '!', 1);
    tsnow = time(NULL);
    if (t) {

        s = myStrGetToken(t, '.', 0);
        ts = strtol(s, NULL, 10);
        value = tsnow - ts; 

        send_cmd(thread, (sv ? sv->sid : p10id), "Z %s %lu %ld %ld %s", (sv ? sv->sid : p10id), ts, tsnow, value, militime_float(NULL));

		sfree(t);
		sfree(s);
    } else /* cmd_pong from functions like the mysql bits */
        send_cmd(thread, (sv ? sv->sid : p10id), "Z %s %ld 0 %ld %s", (sv ? sv->sid : p10id), tsnow, tsnow, militime_float(NULL));

    return CMD_SUPPORTED;
}

/* INVITE */
PROTOCOL_BOOL orion_cmd_invite(Threads *thread, char *source, char *chan, char *nick)
{
    Users *ud;
    ud = GetUser(source);
    
    if (!source || !chan || !nick)
        return CMD_ERROR;

    send_cmd(thread, (ud ? ud->uid : source), "I %s %s", nick, chan);
    return CMD_SUPPORTED;
}

/* Kick */
MOD_BOOL orion_event_kick(Threads *thread, size_t ac, char **av)
{
    Users *sender, *target;
    sender = GetUserByUID(av[0]);
    target = GetUserByUID(av[3]);
    
    if (ac < 4 || sender==NULL || target==NULL)
        return MOD_CONT;

    on_kick(thread, sender->nick, target->nick, av[2]);
    return MOD_CONT;
}

/* SQUIT */
PROTOCOL_BOOL orion_cmd_squit(Threads *thread, char *servname, char *message)
{
    Servers *s = NULL;

    if (!servname || !message)
        return CMD_ERROR;

    s = findserver(thread, servlist, servname);
    if (!s)
      s = findserver_uid(thread, servlist, servname);

    if (!s)
      return CMD_ERROR;

    send_cmd(thread, p10id, "SQ %s :%s", s->sid, message);
    return CMD_SUPPORTED;
}

//> ABAAB M #rofl +l 10
//> ABAAe M #services -o ABAAe
MOD_BOOL orion_event_mode(Threads *thread, size_t ac, char **av)
{
    Users *u;
    Servers *s;
    char *sender;
    char *param[ARRAY_SIZE];
    int modeparamnr = 0;
    int splicer		= 0;
    bool type		= 1;
    
    size_t nr_modes	= 0;
    size_t i		= 0;
    
    u = GetUserByUID(av[0]);

    if (!u) {
		s = findserver_uid(thread, servlist, av[0]);
		
		if(!s)
			s = findserver(thread, servlist, av[0]);
		
		if(!u && !s)
        	sender = av[0];
        else
        	sender = s->server;
    } else {
        sender = u->nick;
    }

    if (ac < 3)
        return MOD_CONT;

	//on_mode(thread,sender,av[2], time(NULL),_dstrjoin(' ',av,3));
    if (*av[2] == '#' || *av[2] == '&') {
		nr_modes = dynlen(av[3]);
		
		param[modeparamnr++] = av[3];
		for(i=0;i<nr_modes;i++){
			if(av[3][i]=='+'){ type=1; splicer++; }
			else if(av[3][i]=='-'){ type=0; splicer++; }
			
			else if(av[3][i]==IRC.OpMode){
				//printf("found opmode, for user: %s (paramter = %d, id param = %s)\n",GetUserByUID(av[4+i-splicer])->nick,4+i-splicer,av[4+i-splicer]);
				param[modeparamnr++]=GetUserByUID(av[4+i-splicer])->nick;
			}
			else if(av[3][i]==IRC.VMode){
				param[modeparamnr++]=GetUserByUID(av[4+i-splicer])->nick;
			}
			
			else if(av[3][i]==IRC.LimitCMode){
				param[modeparamnr++]=av[4+i-splicer];
			}
			else if(av[3][i]==IRC.KeyCMode){
				param[modeparamnr++]=av[4+i-splicer];
			}
			else if(av[3][i]==IRC.BanMode){
				param[modeparamnr++]=av[4+i-splicer];
			}
		}
		param[modeparamnr++]=NULL;
		//printf("ircu sending mode: %s\n",_djoin(' ',param));
        on_mode(thread,sender,av[2], time(NULL),_djoin(' ',param)); //channel mode
    } else {
        orion_set_umode(thread, ac, av); //umode
    }
    return MOD_CONT;
}

PROTOCOL_BOOL orion_cmd_svsnick(Threads *thread, char *source, char *oldnick, char *newnick, time_t when)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_chg_nick(Threads *thread, char *oldnick, char *newnick)
{
    Users *ud;

    if (!oldnick || !newnick) {
        return CMD_ERROR;
    }

    ud = GetUser(oldnick);

    send_cmd(thread, (ud ? ud->uid : oldnick), "N %s", newnick);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svsjoin(Threads *thread, char *source, char *nick, char *chan, time_t date)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svspart(Threads *thread, char *source, char *nick, char *chan)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_swhois(Threads *thread, char *source, char *who, char *mask)
{
    return CMD_NOT_SUPPORTED;
}

/* Y */
MOD_BOOL orion_event_error(Threads *thread, size_t ac, char **av)
{
    if (ac >= 1) {
        if (m->debug) {
            dlog(thread,0,"debug: %s", &_dstrjoin(' ',av,1)[1]);
        }
    }
    return MOD_CONT;
}

MOD_BOOL orion_event_squit(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
    
    on_squit(thread,av[2]);
    //on_squit_server(thread, av[2]);
    return MOD_CONT;
}

PROTOCOL_BOOL orion_cmd_kill(Threads *thread, char *source, char *user, char *buf)
{
    orion_cmd_svskill(thread, source, user, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_rawsvsmode(Threads *thread, char *source, char *nick, char *modes)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svsmode(Threads *thread, char *source, char *nick, char *modes)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svs2mode(Threads *thread, char *source, char *nick, char *modes)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_rawmode(Threads *thread, char *source, char *dest, time_t time_in, char *buf)
{
	Users *u = NULL;
	u = GetUser(source);
	
    if (!buf) {
        return CMD_ERROR;
    }
	//printf("orion_cmd_rawmode - source = %s, dest = %s, buf = %s, time = %lu\n",(u ? u->uid : (source ? source : p10id)), dest, buf, time_in);
    send_cmd(thread, (u ? u->uid : (source ? source : p10id)), "M %s %s", dest, buf);
    return CMD_SUPPORTED;
}

/* PROTOCTL */
PROTOCOL_BOOL orion_cmd_protoctl(Threads *thread)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_chghost(Threads *thread, char *source, char *nick, char *vhost)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_chgident(Threads *thread, char *source, char *nick, char *vIdent)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_sqline(Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason)
{
    /* not supported */
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_unsqline(Threads *thread, char *servername, char *nick, char *snick)
{
    /* not supported */
    return CMD_NOT_SUPPORTED;
}

/* SVSO */
PROTOCOL_BOOL orion_cmd_svso(Threads *thread, char *source, char *nick, char *flag)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* SVSHOLD - set */
PROTOCOL_BOOL orion_cmd_svshold(Threads *thread, char *nick)
{
    /* Not supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* SVSHOLD - release */
PROTOCOL_BOOL orion_cmd_release_svshold(Threads *thread, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* UNSGLINE */
PROTOCOL_BOOL orion_cmd_unsgline(Threads *thread, char *mask)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* SGLINE */
PROTOCOL_BOOL orion_cmd_sgline(Threads *thread, char *mask, char *reason)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* SZLINE */
PROTOCOL_BOOL orion_cmd_szline(Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* UNSZLINE */
PROTOCOL_BOOL orion_cmd_unszline(Threads *thread, char *servername, char *mask, char *snick, time_t in_time)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_unban(Threads *thread, char *name, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svsmode_chan(Threads *thread, char *name, char *mode, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_sajoin(Threads *thread, char *source, char *nick, char *chan, time_t date)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_sapart(Threads *thread, char *source, char *nick, char *chan)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_jupe(Threads *thread, char *jserver, char *who, char *reason)
{
    char rbuf[256];

    snprintf(rbuf, sizeof(rbuf), "Juped by %s%s%s", who,
             reason ? ": " : "", reason ? reason : "");

    orion_cmd_squit(thread, jserver, rbuf);
    orion_cmd_server(thread, jserver, 2, rbuf);
    new_server(thread, me_server, jserver, rbuf, SERVER_JUPED, NULL);
    return CMD_SUPPORTED;
}

/* *INDENT-ON* */
MOD_BOOL orion_event_fakehost(Threads *thread, size_t ac, char **av)
{
    Users *ud;

    ud = GetUserByUID(av[0]);

    on_chghost(thread, ud->nick, av[2]);
    return MOD_CONT;
}

/*  ************************************************************************************************  */
/**
 * Here are all functions which tell us and all modules that a serverlink occured
 * And it tells us if the server is currently linking or finished linking (important for sockscontrol)
 **/

/* This adds Servers to the server tree */
MOD_BOOL orion_event_server(Threads *thread, size_t ac, char **av)
{
/*
{7S} *** SERVER
1 <name of new server>
2 <hops>
3 <boot TS>
4 <link TS>
5 <protocol>
6 <numeric of new server><max client numeric>
7 <flags>
-1 <description of new server>
*/
    Servers *s;
    char uplinknum[5] = "";
    *uplinknum = '\0';
    strcpy(uplinknum, av[5]);
    uplinknum[2] = '\0';
    
    if (streq(av[1], "1")) {														//Received from out uplink
		if(uplink[0]=='\0')
			dynncpy(uplink,av[1],sizeof(uplink));
			
		/* we tell all modules that we are linking to our uplink */
		/* Sockscontrol stops processing here */
        on_link(thread,m->servername,av[1]);
        
        /* Here we add our uplink to the servertree */
        addServer(thread, NULL, av[1], av[2], &_dstrjoin(' ',av,8)[1], uplinknum);
    } else {																		//Any new connecting server (just add it)
		/* we request the servername to dump it to the servertree */
        s = findserver_uid(thread, servlist, av[0]);
        if(!s)
        	s = findserver(thread, servlist, av[0]);
        
        /* We add the linked server with it's uplink to the servertree */
        addServer(thread, (s ? s->server : av[0]), av[2], av[3], &_dstrjoin(' ',av,9)[1], uplinknum);
    }
    return MOD_CONT;
}

/* Called by receiving "ENDBURST" (from orion_event_eob();) - this EOB comes from our Uplink, so we should connect all bots and join our channels */
MOD_BOOL orion_event_eob(Threads *thread, size_t ac, char **av)
{
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN); // Flush our Cache
	
	/* If our uplink said he finished bursting - so we are able to finish bursting after introducing our client's too */
    if (streq(av[0],serv_uplink->sid)){

        /* We tell all modules that the serviceslink was done (reactivating sockscontrol) */
        orion_event_services_link_done(thread,ac,av);
	}else
		/* End Acknowledge */
    	send_cmd(thread, p10id, "EA");
    	
    return MOD_CONT;
}

/* Called from serviceslink(); */
PROTOCOL_BOOL orion_cmd_initialize(Threads *thread){
	if(connected == false){	/* Proof if we are already connected, we aren't so... */
		/* We start connecting to the target UnrealIRCd, providing the Event we are doing */
		orion_cmd_connect(thread,1);
		
		/* Just sleep .. */
		sleep(1);
		
		/* Tell all modules we STARTED linking and they are able to perform their settings now */
		on_serviceslink(thread);
		
		/* We identify ourselves as connected to the target */
		connected = true;
	}
    return CMD_SUPPORTED;
}

/* Called from orion_cmd_initialize(); */
PROTOCOL_BOOL orion_cmd_connect(Threads *thread, bool how)
{
	/* We send password and server introducing to our uplink */
	orion_cmd_pass(thread,m->linkpass);
	orion_cmd_server(thread, m->servername, 0, m->description);

	if(how == true)
		/* We do the primary connect, means connecting all services clients and BotServ Bots */
		_ExecuteEvent(NULL,NULL,"on_connect_primary");

    return CMD_SUPPORTED;
}

/* Just PONG on PING */
PROTOCOL_BOOL orion_cmd_ping(Threads *thread, char *servname, char *who)
{
    char senttime[100];
    time_t tsnow = 0;
    Servers *sv, *target;

    if (!servname || !who) {
        return CMD_ERROR;
    }

    sv = findserver(thread, servlist, servname);
    if (!sv)
      sv = findserver_uid(thread, servlist, servname);
      
    target = findserver(thread, servlist, who);
    if (!target)
		target = findserver_uid(thread, servlist, who);

	sprintf(senttime,"%lu%s",tsnow,militime_float(NULL));

    send_cmd(thread, (sv ? sv->sid : p10id), "G !%s %s %s", senttime, (target ? target->sid : who), senttime);
    return CMD_SUPPORTED;
}

/* This tells us when the linked server finished linking - it should immediately reply for our PING then with a PONG */
MOD_BOOL orion_event_pong(Threads *thread, size_t ac, char **av)
{
    Servers *sv;
    
	if (ac < 3)
        return MOD_CONT;
    
    /* Get the ponging server's ass */
    sv = findserver(thread, servlist, av[0]);
    if (!sv)
		sv = findserver_uid(thread, servlist, av[0]);
    
    /* If the server that linked replies, we know he finished linking */
    if(streqnc(sv->server,linkserver)){
		/* Let the server finish it's link - reactivate Sockscontrol */
		on_linkdone(thread);
	}
    return MOD_CONT;
}

/* Called by receiving "ENDBURST" (orion_event_eob();) - this EOB comes from our Uplink, so we should connect all bots and join our channels */
MOD_BOOL orion_event_services_link_done(Threads *thread, size_t ac, char **av)
{
	//printf("SERVICESLINK - DONE\n");
	/* WTF !? - hmm forgot it =/ (End Acknowledge) */
	send_cmd(thread, p10id, "EA");
	
	/* We add the services server to the servers tree (we do that when we finished linking) */
	me_server = addServer(thread,serv_uplink->server,m->servername,0,m->description,"AA");
	
	/* Let the Bots join our channels */
	_ExecuteEvent(NULL,NULL,"on_connect_secondary");

	/* We report that our linking is done now */	
	on_serviceslink_done(thread);	/* This means that linking !FOR US! is done */							//- reactivates Sockscontrol
	on_linkdone(thread);			/* This is executed whenever a serverlink completed successfully */		//- reactivates Sockscontrol

	/* say we finished bursting */
	orion_cmd_eob(thread);
	return MOD_CONT;
}

/* Announce that we finished bursting now */
PROTOCOL_BOOL orion_cmd_eob(Threads *thread)
{
    send_cmd(thread, p10id, "EB");
    return CMD_SUPPORTED;
}

/*  ************************************************************************************************  */

#define START_OF_RAW 4
/**
 * @brief raw:  To process outgoing raw messages
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param string The outgoing Raw-message
 * @param len The length of the created Raw-message
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL orion_cmd_raw(Threads *thread, char *param[]){
	PROTOCOL_BOOL analyzer;

	//:M PRIVMSG #operserv: raw :operserv svsmode M +SjFzq
	//:operserv SVSKILL M :lalala
	//:M MODE  +a
	//Tue Oct 19 22:47:14 2004 > :OperServ GLOBOPS :M used RAW command for svsmode
	//Tue Oct 19 22:47:14 2004 > :Global PRIVMSG #services :M used RAW command for :operserv

	int paramvalue = 0;

	/* SVSMODE VIA RAW */
		//:operserv SVSMODE shock +d 1
		//operserv SVSMODE shock +d 1
	if(streqnc(param[START_OF_RAW+1],"svsmode"))       analyzer = do_svsmode(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
	else if(streqnc(param[START_OF_RAW+0],"svsmode"))  analyzer = do_svsmode(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVS2MODE VIA RAW */
	//
	else if(streqnc(param[START_OF_RAW+1],"svs2mode")) analyzer = do_svs2mode(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
	else if(streqnc(param[START_OF_RAW+0],"svs2mode")) analyzer = do_svs2mode(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVSKILL VIA RAW */
		// > :shock PRIVMSG O :raw :operserv SVSKILL shock :lalala
		// < :CServe.deepirc.net SVSKILL  ::lalala
		// > :shock PRIVMSG ircdos :raw :sockscontrol SVSKILL shock :hey alter was geht bei dir
	else if(streqnc(param[START_OF_RAW+1],"svskill")){
		char *reason = _dstrjoin(' ',param,START_OF_RAW+2);
		analyzer = do_svskill(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],reason[0]==':' ? &reason[1] : reason);
	}

		// > :shock PRIVMSG O :raw SVSKILL shock :lalala
	else if(streqnc(param[START_OF_RAW+0],"svskill")){
		char *reason = _dstrjoin(' ',param,START_OF_RAW+2);
		analyzer = do_svskill(thread,m->servername,param[START_OF_RAW+1],reason[0]==':' ? &reason[1] : reason);
	}

	/* SVSNICK VIA RAW */
		// > :shock PRIVMSG O :raw :operserv SVSNICK shock shockzZ
	else if(streqnc(param[START_OF_RAW+1],"svsnick")) analyzer = do_svsnick(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
		// > :shock PRIVMSG O :raw SVSNICK shock shockzZ
	else if(streqnc(param[START_OF_RAW+0],"svsnick")) analyzer = do_svsnick(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVSJOIN VIA RAW */
		//:operserv svsjoin Nick #chan
	else if(streqnc(param[START_OF_RAW+0],"svsjoin")) analyzer = do_svsjoin(thread,0,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2],time(NULL));
	else if(streqnc(param[START_OF_RAW+1],"svsjoin")) analyzer = do_svsjoin(thread,0,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3],time(NULL));

	/* SVSPART VIA RAW */
	else if(streqnc(param[START_OF_RAW+0],"svspart")) analyzer = do_svspart(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);
	else if(streqnc(param[START_OF_RAW+1],"svspart")) analyzer = do_svspart(thread,&param[START_OF_RAW+0][1],param[2],param[3]);

	/* NORMAL IRC (RFC) RAW */
	else{
		while(param[paramvalue])
			paramvalue++;

		if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_core_condata` WHERE `nick`='%s'",mysql_quote(thread,&param[START_OF_RAW+0][1])))>0){
				on_protocol(thread,&IRC,paramvalue-START_OF_RAW,param);
				main_raw(thread,_djoin(' ',param));
				return CMD_EXECUTED;
		}
		else
			return CMD_ERROR;
	}
	return analyzer;
}

/*  ************************************************************************************************  */

bool orion_valid_nick(Threads *thread, char *nick)
{
    return true;
}

bool orion_valid_chan(Threads *thread, char *chan)
{
	return true;
}

bool orion_flood_mode_check(Threads *thread, char *value)
{
    return false;
}
