/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 210                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 18:15:28 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file InspIRCd1.1.x.c
 *   @brief This is the Module file for InspIRCd1.1.x (Server to Server) protocol.
 *
 *   It contains all functions which are neccessary to process
 *   Inputs from from InspIRCd1.1.x and outputs to InspIRCd1.1.x!
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "osocket.h"
#include "services.h"
#include "modules.h"

static bool sapart_used = 0;
static bool sajoin_used = 0;
static bool sanick_used = 0;

static bool chghost_used = 0;
static bool chgident_used = 0;
static bool chgname_used = 0;
static bool globops_used = 0;
static bool services_used = 0;
static bool chanprotect_used = 0;

/* Inbound */
MOD_BOOL orion_event_version(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_stats(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_mode(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_fmode(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_samode(Threads *thread, size_t ac, char **av);

MOD_BOOL orion_event_sanick(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_sajoin(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_sapart(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_addline(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_sjoin(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_ping(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_pong(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_ftopic(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_topic(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_squit(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_rsquit(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_quit(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_opertype(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_kick(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_join(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_setident(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_chgident(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_sethost(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_chghost(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_nick(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_server(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_privmsg(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_part(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_setname(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_fname(Threads *thread, size_t ac, char **av);

MOD_BOOL orion_event_gline(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_qline(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_zline(Threads *thread, size_t ac, char **av);
/* SERVER MODES */
MOD_BOOL orion_event_started(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_services_link(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_services_link_done(Threads *thread, size_t ac, char **av);

/* Outbound */
PROTOCOL_BOOL orion_cmd_initialize(Threads *thread);
PROTOCOL_BOOL orion_cmd_add_account(Threads *thread, char *uid, char *authnick);
PROTOCOL_BOOL orion_cmd_del_account(Threads *thread, char *uid);

PROTOCOL_BOOL orion_cmd_topic(Threads *thread, char *whosets, char *chan, char *whosetit, char *topic, time_t when);

PROTOCOL_BOOL orion_cmd_gline(Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason);
PROTOCOL_BOOL orion_cmd_ungline(Threads *thread, char *unsetter, char *host, char *user, time_t time_in);

PROTOCOL_BOOL orion_cmd_svskill(Threads *thread, char *source, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_kill(Threads *thread, char *source, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_rawsvsmode(Threads *thread, char *source, char *nick, char *modes);
PROTOCOL_BOOL orion_cmd_svsmode(Threads *thread, char *nick, char *target, char *modes);
PROTOCOL_BOOL orion_cmd_svs2mode(Threads *thread, char *nick, char *target, char *modes);
PROTOCOL_BOOL orion_cmd_nick(Threads *thread, char *nick, char *ident, char *host, char *name, char *modes);
PROTOCOL_BOOL orion_cmd_mode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);
PROTOCOL_BOOL orion_cmd_rawmode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);

PROTOCOL_BOOL orion_cmd_kick(Threads *thread, char *source, char *chan, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_notice(Threads *thread, char *source, char *dest, char *buf);
PROTOCOL_BOOL orion_cmd_privmsg(Threads *thread, char *source, char *dest, char *buf);
PROTOCOL_BOOL orion_cmd_serv_notice(Threads *thread, char *source, char *dest, char *msg);
PROTOCOL_BOOL orion_cmd_serv_privmsg(Threads *thread, char *source, char *dest, char *msg);

PROTOCOL_BOOL orion_cmd_quit(Threads *thread, char *source, char *buf);
PROTOCOL_BOOL orion_cmd_protoctl(Threads *thread);
PROTOCOL_BOOL orion_cmd_pass(Threads *thread, char *pass);
PROTOCOL_BOOL orion_cmd_server(Threads *thread, char *servname, int hop, char *descript);
PROTOCOL_BOOL orion_cmd_pong(Threads *thread, char *servname, char *who);
PROTOCOL_BOOL orion_cmd_ping(Threads *thread, char *servname, char *who);
PROTOCOL_BOOL orion_cmd_join(Threads *thread, char *user, char *channel, time_t chantime);
PROTOCOL_BOOL orion_cmd_services_join(Threads *thread, char *user, char *channel, time_t chantime, int fake);

PROTOCOL_BOOL orion_cmd_chghost(Threads *thread, char *source, char *nick, char *vhost);
PROTOCOL_BOOL orion_cmd_chgident(Threads *thread, char *source, char *nick, char *vIdent);
PROTOCOL_BOOL orion_cmd_invite(Threads *thread, char *source, char *chan, char *nick);
PROTOCOL_BOOL orion_cmd_part(Threads *thread, char *nick, char *chan, char *buf);

PROTOCOL_BOOL orion_cmd_wallops(Threads *thread, char *source, char *fmt);
PROTOCOL_BOOL orion_cmd_global(Threads *thread, char *source, char *buf);

PROTOCOL_BOOL orion_cmd_sqline(Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason);
PROTOCOL_BOOL orion_cmd_unsqline(Threads *thread, char *servername, char *nick, char *snick);

PROTOCOL_BOOL orion_cmd_squit(Threads *thread, char *servname, char *message);
PROTOCOL_BOOL orion_cmd_svso(Threads *thread, char *source, char *nick, char *flag);
PROTOCOL_BOOL orion_cmd_chg_nick(Threads *thread, char *oldnick, char *newnick);
PROTOCOL_BOOL orion_cmd_svsnick(Threads *thread, char *source, char *oldnick, char *newnick, time_t when);
PROTOCOL_BOOL orion_cmd_connect(Threads *thread, bool how);
PROTOCOL_BOOL orion_cmd_svshold(Threads *thread, char *nick);
PROTOCOL_BOOL orion_cmd_release_svshold(Threads *thread, char *nick);

PROTOCOL_BOOL orion_cmd_szline(Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason);
PROTOCOL_BOOL orion_cmd_unszline(Threads *thread, char *servername, char *mask, char *snick, time_t in_time);

PROTOCOL_BOOL orion_cmd_unban(Threads *thread, char *name, char *nick);
PROTOCOL_BOOL orion_cmd_svsmode_chan(Threads *thread, char *name, char *mode, char *nick);
PROTOCOL_BOOL orion_cmd_svsjoin(Threads *thread, char *source, char *nick, char *chan, time_t date);
PROTOCOL_BOOL orion_cmd_svspart(Threads *thread, char *source, char *nick, char *chan);
PROTOCOL_BOOL orion_cmd_sapart(Threads *thread, char *source, char *nick, char *chan);
PROTOCOL_BOOL orion_cmd_sajoin(Threads *thread, char *source, char *nick, char *chan, time_t date);

PROTOCOL_BOOL orion_cmd_swhois(Threads *thread, char *source, char *who, char *mask);
PROTOCOL_BOOL orion_cmd_eob(Threads *thread);
PROTOCOL_BOOL orion_cmd_jupe(Threads *thread, char *jserver, char *who, char *reason);
PROTOCOL_BOOL orion_cmd_global_legacy(Threads *thread, char *source, char *fmt);

PROTOCOL_BOOL orion_cmd_raw(Threads *thread, char *param[]);
bool orion_valid_nick(Threads *thread, char *nick);
bool orion_valid_chan(Threads *thread, char *chan);
bool orion_flood_mode_check(Threads *thread, char *value);

/* *INDENT-OFF* */
void CreateProtocol(Threads *thread, Protocol *protocol, void *dl_handle) {
    
	//printf("Creating Protocol ...\n");
    event_init(thread, protocol, dl_handle, "shock", "InspIRCd 1.1.x and above [version 0.1]");

    createEvent(thread,protocol, "JOIN",      orion_event_join,     1);
    createEvent(thread,protocol, "KICK",      orion_event_kick,     1);
    createEvent(thread,protocol, "MODE",      orion_event_mode,     1);
    createEvent(thread,protocol, "NICK",      orion_event_nick,     1);
    createEvent(thread,protocol, "PART",      orion_event_part,     1);
    createEvent(thread,protocol, "PING",      orion_event_ping,     1);
    createEvent(thread,protocol, "PONG",      orion_event_pong,     1);
    createEvent(thread,protocol, "PRIVMSG",   orion_event_privmsg,  1);
    createEvent(thread,protocol, "QUIT",      orion_event_quit,     1);
    createEvent(thread,protocol, "OPERQUIT",  orion_event_quit,     1);
    createEvent(thread,protocol, "OPERTYPE",  orion_event_opertype, 1);
    
    createEvent(thread,protocol, "TOPIC",     orion_event_topic,    1);
    createEvent(thread,protocol, "SAMODE",    orion_event_samode,   1);
    createEvent(thread,protocol, "SANICK",    orion_event_sanick,   1);
    createEvent(thread,protocol, "SAJOIN",    orion_event_sajoin,   1);
    createEvent(thread,protocol, "SAPART",    orion_event_sapart,   1);
    
    createEvent(thread,protocol, "ADDLINE",   orion_event_addline,  1);
    createEvent(thread,protocol, "ZLINE",     orion_event_zline,  1);
    createEvent(thread,protocol, "QLINE",     orion_event_qline,  1);
    createEvent(thread,protocol, "GLINE",     orion_event_gline,  1);
    
    createEvent(thread,protocol, "FHOST",     orion_event_chghost,  1);
    createEvent(thread,protocol, "CHGIDENT",  orion_event_chgident, 1);
    createEvent(thread,protocol, "FNAME",     orion_event_fname,    1);

    createEvent(thread,protocol, "FJOIN",     orion_event_sjoin,    1);
    createEvent(thread,protocol, "FMODE",     orion_event_fmode,    1);
    createEvent(thread,protocol, "FTOPIC",    orion_event_ftopic,   1);
    createEvent(thread,protocol, "STATS",     orion_event_stats,    1);
    createEvent(thread,protocol, "VERSION",   orion_event_version,  1);
    //createEvent(thread,protocol, "OPERTYPE",  orion_event_opertype, 1);
    
    /** Server Modes **/
    createEvent(thread,protocol, "SERVER",    orion_event_server,					1);
    createEvent(thread,protocol, "SERVER",    orion_event_server,					0);
    createEvent(thread,protocol, "SQUIT",     orion_event_squit,					1);
    createEvent(thread,protocol, "RSQUIT",    orion_event_rsquit,					1);
    createEvent(thread,protocol, "CAPAB",     orion_event_started,					0);
    createEvent(thread,protocol, "ENDBURST",  orion_event_services_link_done,		0);
    
    //createEvent(thread,protocol, "VERSION",   orion_event_version,					1);
    
   protocol->NSMode						= "+o";
   protocol->CSMode						= "+o";
   protocol->MSMode						= "+o";
   protocol->HSMode						= "+o";
   protocol->OSMode						= "+io";
   protocol->BSMode						= "+o";
   protocol->HeServMode					= "+o";
   protocol->GlobMode					= "+io";
   protocol->QMode						= "+o";
   protocol->SCMode						= "+io";
   protocol->BSBotsMode					= "+ios";
   protocol->ChannelModes				= "iklmnpstcfgjrzCGIJKLMNOPQRSTV";	/* Avail Channel Modes */
   protocol->ParametricChannelModes		= "klLfgjJI";						/* Channel Modes with an allowed Parameter (all possible) */
   protocol->MLockAbleModes				= "klLf";							/* Channel Modes provided by MLOCK */
   protocol->OnSuspend					= "+Opmntil 1";						/* Channel Modes on adding a suspend (forbid) */
   protocol->OnUnSuspend				= "-Opmntil";						/* Channel Modes on removing a suspend (forbid) */
   protocol->SupportSVS					= true;								/* Supports SVS-COMMANDS */
   protocol->SupportSVSJoin				= true;								/* Supports SVSJOIN */
   protocol->SvsNick					= true;								/* Supports SVSNICK */
   protocol->VHost						= true;								/* Supports VHosts */
   protocol->HasOwner					= true;								/* Supports ChannelOwnerMode (+q) */
   protocol->HasProtect					= true;								/* Supports ChannelProtectMode (+a) */
   protocol->OwnerMode					= 'q';								/* UserChannelMode for Owner */
   protocol->ProtectMode				= 'a';								/* UserChannelMode for Protected */
   protocol->OpMode						= 'o';								/* UserChannelMode for Op */
   protocol->HOpMode					= 'h';								/* UserChannelMode for HalfOp */
   protocol->VMode						= 'v';								/* UserChannelMode for Voice */
   protocol->BanMode					= 'b';								/* UserChannelMode for Ban */
   protocol->ExceptMode					= 'e';								/* UserChannelMode for BanException */
   protocol->SupportedUserModes			= "oinswBGHIQRSWdhrx";				/* Known UserModes */
   protocol->SupportedUChanModes		= "qaohvbe";						/* All UserChannelModes */
   protocol->SupportedU2ChanModes		= "qaohv";							/* All UserChannelModes except "Except (+e)" and Ban (+b) */
   protocol->ModeOnNickReg				= "+r";								/* Mode for Registered / Identified Nicks */
   protocol->ModeOnNickDrop				= "-r";								/* Mode for Dropping a registered Nick */
   protocol->ModeOnChannelReg			= "+r";								/* Mode for Registered Channels */
   protocol->ModeOnChannelDrop			= "-r";								/* Mode for Dropping a registered Channel */
   protocol->RegMode					= 'r';								/* Mode for Registration */
   protocol->ChanRegMode				= true;								/* Supports a Channel Registration Mode */
   protocol->NickRegMode				= true;								/* Supports a Nick Registration Mode */
   protocol->SupportSGlines				= true;								/* Supports Glines */
   protocol->SupportSQlines				= true;								/* Supports Qlines */
   protocol->SupportSZlines				= true;								/* Supports Zlines */
   protocol->SupportHalfOp				= true;								/* Supports HalfOpMode */
   protocol->SupportBanMode				= true;								/* Supports BanMode */
   protocol->SupportExceptMode			= false;							/* Supports ExceptMode */
   protocol->SupportProtectUmode		= false;							/* Supports a Protection UMode */
   protocol->ProtectUmode				= "Sq";								/* Mode for Protecting a Nickname */
   protocol->SupportInvisibleUMode		= true;								/* Supports InvisibleUMode */
   protocol->InvisibleUMode				= "Q";								/* Mode for Invisibility */
   protocol->SupportChanSQlines			= false;							/* Supports Chan SQlines */
   protocol->SupportVIdents				= true;								/* Supports VIdent's */
   protocol->VHostOnNickChange			= true;								/* Supports setting VHosts on NickChanges */
   protocol->SupportRealnameChanges		= true;								/* Supports RealnameChanges */
   protocol->SupportAdminOnlyChanMode	= true;								/* Supports an AdminOnly ChannelMode */
   protocol->SupportOperOnlyChanMode	= true;								/* Supports an OperOnly ChannelMode */
   protocol->DoesGlobalChangeAfterSVS	= true;								/*  */
   protocol->DoesGlobalChangeAfterSA	= true;								/*  */
   protocol->DoesQuitAfterKill			= true;								/* Sends a Quit-Msg on Kill Execution */
   protocol->AdminOnlyCMode				= "A";								/* AdminOnly ChannelMode */
   protocol->OperOnlyCMode				= "O";								/* OperOnly ChannelMode */
   protocol->DefaultMLOCK				= "+ntr";							/* Default MLOCK Modes */
   protocol->VhostMode					= 'x';								/* Vhost Mode (is set on identify) */
   protocol->SupportFloodMode			= true;								/* Supports a FloodMode */
   protocol->SupportLinkMode			= true;								/* Supports a ChannelLinkMode */
   protocol->FloodCMode					= 'f';								/* ChannelFloodMode setting */
   protocol->LinkCMode					= 'L';								/* ChannelMode for linking Channels */
   protocol->KeyCMode					= 'k';								/* ChannelKeyMode setting */
   protocol->LimitCMode					= 'l';								/* ChannelMode for limit */
   protocol->InviteCMode				= 'i';								/* ChannelMode for inviteonly */
   protocol->SupportTokens				= false;							/* Supports Tokens */
   protocol->SupportSjoin				= true;								/* Supports SJOIN */
   protocol->SupportSjoinBanprefix		= false;							/* Supports a BanPrefix for SJOIN */
   protocol->SupportSjoinExceptprefix	= false;							/* Supports an ExceptPrefix for SJOIN */
   protocol->SjoinOwnerPrefix			= '~';								/* OwnerPrefix for SJOIN */
   protocol->SjoinProtectPrefix			= '&';								/* ProtectionPrefix for SJOIN */
   protocol->SjoinOpPrefix				= '@';								/* OpPrefix for SJOIN */
   protocol->SjoinHOpPrefix				= '%';								/* HalfOpPrefix for SJOIN */
   protocol->SjoinVoicePrefix			= '+';								/* VoicePrefix for SJOIN */
   protocol->SjoinBanPrefix				= '\0';								/* BanPrefix for SJOIN */
   protocol->SjoinExceptPrefix			= '\0';								/* ExceptPrefix for SJOIN */
   protocol->SjoinHasIgnoreChar			= true;								/* SJOIN has an ignored Mode (it will be skipped) */
   protocol->SjoinIgnoredChar			= ',';								/* Mode to be ignored from SJOIN */
   protocol->SupportHelperUMode			= true;								/* Supports Helper UMODE */
   protocol->SupportSyncState			= false;							/* Supports SyncState */
   protocol->p10protocol				= false;							/* Is p10 protocol */
   protocol->p10new						= false;							/* Is newp10 protocol */
   
	protocol->orion_cmd_initialize		= orion_cmd_initialize;
	protocol->orion_cmd_topic			= orion_cmd_topic;
	protocol->orion_cmd_add_account		= orion_cmd_add_account;
	protocol->orion_cmd_del_account		= orion_cmd_del_account;
	protocol->orion_cmd_kill			= orion_cmd_kill;
	protocol->orion_cmd_svskill			= orion_cmd_svskill;
	protocol->orion_cmd_rawsvsmode		= orion_cmd_rawsvsmode;
	protocol->orion_cmd_svsmode			= orion_cmd_svsmode;
	protocol->orion_cmd_svs2mode		= orion_cmd_svs2mode;
	protocol->orion_cmd_nick			= orion_cmd_nick;
	protocol->orion_cmd_mode			= orion_cmd_mode;
	protocol->orion_cmd_rawmode			= orion_cmd_rawmode;
	protocol->orion_cmd_kick			= orion_cmd_kick;
	protocol->orion_cmd_notice			= orion_cmd_notice;
	protocol->orion_cmd_privmsg			= orion_cmd_privmsg;
	protocol->orion_cmd_serv_notice		= orion_cmd_serv_notice;
	protocol->orion_cmd_serv_privmsg	= orion_cmd_serv_privmsg;
	
	protocol->orion_cmd_quit			= orion_cmd_quit;
	protocol->orion_cmd_protoctl		= orion_cmd_protoctl;
	protocol->orion_cmd_pass			= orion_cmd_pass;
	protocol->orion_cmd_server			= orion_cmd_server;
	protocol->orion_cmd_pong			= orion_cmd_pong;
	protocol->orion_cmd_ping			= orion_cmd_ping;
	protocol->orion_cmd_join			= orion_cmd_join;
	protocol->orion_cmd_services_join	= orion_cmd_services_join;
	protocol->orion_cmd_chghost			= orion_cmd_chghost;
	protocol->orion_cmd_chgident		= orion_cmd_chgident;
	protocol->orion_cmd_invite			= orion_cmd_invite;
	protocol->orion_cmd_part			= orion_cmd_part;
	
	protocol->orion_cmd_wallops			= orion_cmd_wallops;
	protocol->orion_cmd_global			= orion_cmd_global;
	
	protocol->orion_cmd_squit			= orion_cmd_squit;
	//protocol->orion_cmd_svso =orion_cmd_svso
	protocol->orion_cmd_chg_nick		= orion_cmd_chg_nick;
	protocol->orion_cmd_svsnick			= orion_cmd_svsnick;
	protocol->orion_cmd_connect			= orion_cmd_connect;
	/*protocol->orion_cmd_svshold =orion_cmd_svshold
	protocol->orion_cmd_release_svshold =orion_cmd_release_svshold*/
	
	protocol->orion_cmd_gline			= orion_cmd_gline;
	protocol->orion_cmd_ungline			= orion_cmd_ungline;
	
	protocol->orion_cmd_szline			= orion_cmd_szline;
	protocol->orion_cmd_unszline		= orion_cmd_unszline;
	
	protocol->orion_cmd_sqline			= orion_cmd_sqline;
	protocol->orion_cmd_unsqline		= orion_cmd_unsqline;
	
	protocol->orion_cmd_unban			= orion_cmd_unban;
	protocol->orion_cmd_svsmode_chan	= orion_cmd_svsmode_chan;
	protocol->orion_cmd_svsjoin			= orion_cmd_svsjoin;
	protocol->orion_cmd_svspart			= orion_cmd_svspart;
	
	protocol->orion_cmd_sajoin			= orion_cmd_sajoin;
	protocol->orion_cmd_sapart			= orion_cmd_sapart;
	
	protocol->orion_cmd_swhois			= orion_cmd_swhois;
	protocol->orion_cmd_eob				= orion_cmd_eob;
	protocol->orion_cmd_jupe			= orion_cmd_jupe;
	protocol->orion_cmd_global_legacy	= orion_cmd_global_legacy;
	protocol->orion_cmd_raw				= orion_cmd_raw;
	protocol->orion_valid_nick			= orion_valid_nick;
	protocol->orion_valid_chan			= orion_valid_chan;
	protocol->orion_flood_mode_check	= orion_flood_mode_check;
	
	dynncpy(global_target,m->network,sizeof(global_target));
	printf(" [*] Created InspIRCd Protocol (version: 0.1)...\n");
	return;
}

#ifdef _WIN32
void DeleteProtocol(Threads *thread, size_t ac, char **av)
{
    return MOD_CONT;
}
#endif

MOD_BOOL orion_event_version(Threads *thread, size_t ac, char **av)
{
    if (ac < 2)
        return MOD_CONT;
	
	version(thread,&av[0][1]);
	
	/* Execute version event */
	_ExecuteEvent(&av[0][1],NULL,"on_version");
    return MOD_CONT;
}

MOD_BOOL orion_event_stats(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
	
	on_stats(thread,m->servername,&av[0][1],av[2]);
    return MOD_CONT;
}

MOD_BOOL orion_event_mode(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
	
	if(DuringServerlink)
	/* This is REALLY IMPORTANT - it avoid's Net Desyncs on linking, if modes happen
		before finishing the synchronisation process */
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN);
		
	on_mode(thread,&av[0][1],av[2],time(NULL),_dstrjoin(' ',av,3));
    return MOD_CONT;
}

MOD_BOOL orion_event_fmode(Threads *thread, size_t ac, char **av)
{
    /* :source FMODE #test +nt */
    if (ac < 3)
        return MOD_CONT;
    
	if(DuringServerlink)
	/* This is REALLY IMPORTANT - it avoid's Net Desyncs on linking, if modes happen
		before finishing the synchronisation process */
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN);
		
	on_mode(thread,&av[0][1],av[2],(time_t)stol(av[3]),_dstrjoin(' ',av,4));
    return MOD_CONT;
}

MOD_BOOL orion_event_samode(Threads *thread, size_t ac, char **av)
{
    /* :source SAMODE targets modes */
    if (ac < 4)
        return MOD_CONT;
        
    if(av[2][0]=='#'){
		if(DuringServerlink)
		/* This is REALLY IMPORTANT - it avoid's Net Desyncs on linking, if modes happen
			before finishing the synchronisation process */
			MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN);
		
    	on_mode(thread,&av[0][1],av[2],time(NULL),_dstrjoin(' ',av,3));
	}
    return MOD_CONT;
}

MOD_BOOL orion_event_sanick(Threads *thread, size_t ac, char **av)
{
    /* :source SANICK old new */
    if (ac < 4)
        return MOD_CONT;
	
    chgnick(thread,av[2],av[3]);
    return MOD_CONT;
}

//:<source server> SVSJOIN <nick> <channel>
MOD_BOOL orion_event_sajoin(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;
    
    _join(thread,0,av[2],av[3],time(NULL));
    return MOD_CONT;
}

//:<source server> SAPART <nick> <channel>
MOD_BOOL orion_event_sapart(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
    
    on_part(thread,av[2],av[3]);
    return MOD_CONT;
}
//:<source server> ADDLINE [Q|Z|E|G] <mask> <setter> <time set> <duration> :<reason>
MOD_BOOL orion_event_addline(Threads *thread, size_t ac, char **av){

	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN); // Flush our Cache
	
	if(av[2][0]=='G'){
		on_gline(thread,&av[0][1],av[3],av[4],(time_t)stol(av[6]),(time_t)stol(av[5]),_dstrjoin(' ',av,7));
	}
	else if(av[2][0]=='Z'){
		on_szline(thread,&av[0][1],av[3],av[4],(time_t)stol(av[6]),(time_t)stol(av[5]),_dstrjoin(' ',av,7));
	}
	else if(av[2][0]=='Q'){
		on_sqline(thread,av[4],av[3],(time_t)stol(av[6]),(time_t)stol(av[5]),_dstrjoin(' ',av,7));
	}
	return MOD_CONT;
}

//:fudge.chatspike.net FJOIN #staff 1133630884 :@%,Brain @,Craig %,Dune +,Azhrarn ,w00t ,Elf
MOD_BOOL orion_event_sjoin(Threads *thread, size_t ac, char **av)
{
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_CONDATA_NO_MODES);
	
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,av[2],NULL);
	
	char *buf = _dstrjoin(' ',av,4);
	//printf("BUF : \"%s\"\n",buf);
	
    sjoin(thread,av[2],(time_t)stol(av[3]),buf[0] == ':' ? &buf[1] : buf,0);
    return MOD_CONT;
}

MOD_BOOL orion_event_ftopic(Threads *thread, size_t ac, char **av)
{
//> TOPIC #services shock 1194899530 :ROFL (serverlink topic)
//> :shock TOPIC #p2pworld shock 1194899600 :bamm (normal)
    /* :source FTOPIC channel ts setby :topic */
    
    if (ac < 6)
        return MOD_CONT;
        
    topic(thread,av[4],av[2],(time_t)stol(av[3]),&_dstrjoin(' ',av,5)[1]);
    return MOD_CONT;
}

//> TOPIC #services shock 1194899530 :ROFL (serverlink topic)
//> :shock TOPIC #p2pworld shock 1194899600 :bamm (normal)

/* :setter TOPIC #channel :topic */
MOD_BOOL orion_event_topic(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;
        
    on_topic(thread,&av[0][1],av[2],time(NULL),&_dstrjoin(' ',av,3)[1]);
    return MOD_CONT;
}

MOD_BOOL orion_event_squit(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
        
    on_squit(thread,av[2]);
    //on_squit_server(thread,av[2]);
    return MOD_CONT;
}

MOD_BOOL orion_event_rsquit(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
        
    //on_squit_server(thread,av[2]);
    on_squit(thread,av[2]);
    return MOD_CONT;
}

MOD_BOOL orion_event_quit(Threads *thread, size_t ac, char **av)
{
    if (ac < 2)
        return MOD_CONT;
        
    on_quit(thread,&av[0][1]);
    return MOD_CONT;
}

MOD_BOOL orion_event_opertype(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
        
    svsmode(thread,m->nick,&av[0][1],"+o");
    return MOD_CONT;
}

MOD_BOOL orion_event_kick(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;
        
    on_kick(thread,&av[0][1],av[3],av[2]);
    return MOD_CONT;
}


MOD_BOOL orion_event_join(Threads *thread, size_t ac, char **av)
{
	if (ac < 3)
        return MOD_CONT;
    
    _join(thread,0,&av[0][1],av[2],(time_t)stol(av[3]));
    return MOD_CONT;
}

MOD_BOOL orion_event_setident(Threads *thread, size_t ac, char **av)
{
	if(ac < 3)
		return MOD_CONT;
		
	on_setident(thread,&av[0][1],av[2]);
    return MOD_CONT;
}

MOD_BOOL orion_event_chgident(Threads *thread, size_t ac, char **av)
{
	if(ac < 4)
		return MOD_CONT;
		
 	on_chgident(thread,av[2],av[3][0]==':' ? &av[3][1] : av[3]);
    return MOD_CONT;
}

MOD_BOOL orion_event_sethost(Threads *thread, size_t ac, char **av)
{
	if(ac < 3)
		return MOD_CONT;
		
	on_sethost(thread,&av[0][1],av[2][0]==':' ? &av[2][1] : av[2]);
    return MOD_CONT;
}

MOD_BOOL orion_event_chghost(Threads *thread, size_t ac, char **av)
{
	if(ac < 4)
		return MOD_CONT;
		
 	on_chghost(thread,av[2],av[3][0]==':' ? &av[3][1] : av[3]);
    return MOD_CONT;
}

MOD_BOOL orion_event_fname(Threads *thread, size_t ac, char **av)
{
	if(ac < 3)
		return MOD_CONT;
		
    on_chgname(thread,&av[0][1],_dstrjoin(' ',av,2));
    return MOD_CONT;
}

//Inspire :test.chatspike.net NICK 1133992412 Brain synapse.brainbox.winbot.co.uk netadmin.chatspike.net ~brain +xwsioS 10.0.0.2 :Craig Edwards
//Unreal  :NICK tunix25 3 1089515018 atte python.netsplit.de mafia.de.eu.deepirc.net 0 +x pA2A90BE0.dialin.deepirc.net :Your IRC network is currently listed at irc.netspl


//> :glue.de.eu.deepirc.net NICK 1213106147 shock 62-47-155-204.adsl.highway.telekom.at of.the.world.at ruler +irs 62.47.155.204 :shock
MOD_BOOL orion_event_nick(Threads *thread, size_t ac, char **av)
{
	
	if(ac > 3){
		cnick(thread,
				av[3],	// nick
				NULL,	// vident
				av[6],	// ident
				av[7],  // modes
				(time_t)stol(av[2]),  // time
				av[4],	// realhost
				av[5],	// virtual host
				&av[0][1],  // server
				&_dstrjoin(' ',av,9)[1],	// realname
				av[8],
				0
			);
    } else {
        chgnick(thread,&av[0][1],av[2]);
    }
    return MOD_CONT;
}

MOD_BOOL orion_event_privmsg(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;
        
    on_privmsg(thread,av);
    return MOD_CONT;
}

MOD_BOOL orion_event_part(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
        
    on_part(thread,&av[0][1],av[2]);
    return MOD_CONT;
}

MOD_BOOL orion_event_gline(Threads *thread, size_t ac, char **av)
{
    if (ac < 2)
        return MOD_CONT;
        
	time_t duration = 0;
    if (ac < 3)
        return MOD_CONT;
    
    if(ac == 3)
    	on_ungline(thread,&av[0][1],av[2]);
/*
:<source nickname> GLINE <user@host> <duration> :<reason>
:<source nickname> GLINE <user@host>
*/
    else if(ac >= 5){
		if(check_expire(av[3]))
			duration = expire(av[3]);
		else
			duration = (time_t)stol(av[3]);
			
		on_gline(thread,&av[0][1],av[2],&av[0][1],time(NULL)+duration,time(NULL),&_dstrjoin(' ',av,4)[1]);
	}
    return MOD_CONT;
}

MOD_BOOL orion_event_qline(Threads *thread, size_t ac, char **av)
{
	time_t duration = 0;
    if (ac < 3)
        return MOD_CONT;
        
    if(ac == 3)
    	on_unsqline(thread,NULL,&av[0][1],av[2]);
    	
    else if(ac >= 5){
		if(check_expire(av[3]))
			duration = expire(av[3]);
		else
			duration = (time_t)stol(av[3]);
			
		on_sqline(thread,&av[0][1],av[2],time(NULL)+duration,time(NULL),&_dstrjoin(' ',av,4)[1]);
	}
    return MOD_CONT;
}

MOD_BOOL orion_event_zline(Threads *thread, size_t ac, char **av)
{
	time_t duration = 0;
    if (ac < 3)
        return MOD_CONT;
        
    if(ac == 3)
    	on_unszline(thread,&av[0][1],av[2]);
    	
    else if(ac >= 5){
		if(check_expire(av[3]))
			duration = expire(av[3]);
		else
			duration = (time_t)stol(av[3]);
			
		on_szline(thread,&av[0][1],av[2],&av[0][1],time(NULL)+duration,time(NULL),&_dstrjoin(' ',av,4)[1]);
	}
    return MOD_CONT;
}

PROTOCOL_BOOL orion_cmd_add_account(Threads *thread, char *uid, char *authnick)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_del_account(Threads *thread, char *uid)
{
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_topic(Threads *thread, char *whosets, char *chan, char *whosetit,
                      char *topic, time_t when)
{
    send_cmd(thread,m->servername, "FTOPIC %s %ld %s :%s", chan, when, whosetit, topic);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_gline(Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason)
{
    send_cmd(thread,m->servername, "ADDLINE G %s %s %ld %ld :%s", host, who ? who : _dynadd("%s!%s@%s",m->nick,m->ident,m->vhost), when, expires, reason);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_ungline(Threads *thread, char *unsetter, char *host, char *user, time_t time_in)
{
    send_cmd(thread,m->servername, "GLINE %s", host);
    return CMD_SUPPORTED;
}

/* SQLINE */
PROTOCOL_BOOL orion_cmd_sqline(Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason)
{
    send_cmd(thread,m->servername, "ADDLINE Q %s %s %ld %ld :%s", mask, sender ? sender : _dynadd("%s!%s@%s",m->nick,m->ident,m->vhost), time_in, expire, reason);
    return CMD_SUPPORTED;
}

/* UNSQLINE */
PROTOCOL_BOOL orion_cmd_unsqline(Threads *thread, char *servername, char *nick, char *snick)
{
    if (!nick) {
        return CMD_ERROR;
    }
    send_cmd(thread,m->servername, "QLINE %s", nick);
    return CMD_SUPPORTED;
}

/* SZLINE */
PROTOCOL_BOOL orion_cmd_szline(Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason)
{
    send_cmd(thread,m->servername, "ADDLINE Z %s %s %ld %ld :%s", mask, snick, when, expire, reason);
    return CMD_SUPPORTED;
}

/* UNSZLINE */
PROTOCOL_BOOL orion_cmd_unszline(Threads *thread, char *servername, char *mask, char *snick, time_t in_time)
{
    send_cmd(thread,servername ? servername : m->servername, "ZLINE %s", mask);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svskill(Threads *thread, char *source, char *user, char *buf)
{
    if (!buf) {
        return CMD_ERROR;
    }

    if (!source || !user) {
        return CMD_ERROR;
    }

    send_cmd(thread,source, "KILL %s :%s", user, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_kill(Threads *thread, char *source, char *user, char *buf)
{
    orion_cmd_svskill(thread,source, user, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_rawsvsmode(Threads *thread, char *source, char *nick, char *modes)
{
    send_cmd(thread,source, "SVSMODE %s %s", nick, modes);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svsmode(Threads *thread, char *source, char *nick, char *modes)
{
    send_cmd(thread,source, "SVSMODE %s %s", nick, modes);
	//svsmode(thread,source,nick,modes);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svs2mode(Threads *thread, char *source, char *nick, char *modes)
{
    send_cmd(thread,source, "SVSMODE %s %s", nick, modes);
	//svsmode(thread,source,nick,modes);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_nick(Threads *thread, char *nick, char *ident, char *host, char *name, char *modes)
{
    // :test.chatspike.net NICK 1133519355 Brain synapse.brainbox.winbot.co.uk netadmin.chatspike.net ~brain +xwsioS 10.0.0.2 :Craig Edwards
    send_cmd(thread,m->servername, "NICK %ld %s %s %s %s %s 0.0.0.0 :%s",time(NULL),nick,host,host,ident,modes,name);
    send_cmd(thread,m->servername, "OPERTYPE Service");
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_mode(Threads *thread, char *source, char *dest, time_t time_in, char *buf)
{
    send_cmd(thread,source, "FMODE %s %ld %s", dest, time_in, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_rawmode(Threads *thread, char *source, char *dest, time_t time_in, char *buf)
{
    if (!buf) {
        return CMD_ERROR;
    }
    
    send_cmd(thread,source ? source : O->nick, "FMODE %s %ld %s", dest, time_in, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_kick(Threads *thread, char *source, char *chan, char *user, char *buf)
{
    if (buf) {
        send_cmd(thread,source, "KICK %s %s :%s", chan, user, buf);
    } else {
        send_cmd(thread,source, "KICK %s %s :%s", chan, user, user);
    }
    return CMD_SUPPORTED;
}


PROTOCOL_BOOL orion_cmd_notice(Threads *thread, char *source, char *dest, char *buf)
{
    if (!buf) {
        return CMD_ERROR;
    }

    send_cmd(thread,source, "NOTICE %s :%s", dest, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_privmsg(Threads *thread, char *source, char *dest, char *buf)
{
    if (!buf) {
        return CMD_ERROR;
    }

    send_cmd(thread,source, "PRIVMSG %s :%s", dest, buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_serv_notice(Threads *thread, char *source, char *dest, char *msg)
{
    send_cmd(thread,source, "NOTICE $%s :%s", dest, msg);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_serv_privmsg(Threads *thread, char *source, char *dest, char *msg)
{
    send_cmd(thread,source, "PRIVMSG $%s :%s", dest, msg);
    return CMD_SUPPORTED;
}

/* QUIT */
PROTOCOL_BOOL orion_cmd_quit(Threads *thread, char *source, char *buf)
{
    if (buf) {
        send_cmd(thread,source, "QUIT :%s", buf);
    } else {
        send_cmd(thread,source, "QUIT :Exiting");
    }
    return CMD_SUPPORTED;
}

/* PROTOCTL */
PROTOCOL_BOOL orion_cmd_protoctl(Threads *thread)
{
    return CMD_NOT_SUPPORTED;
}

static char currentpass[1024];

/* PASS */
PROTOCOL_BOOL orion_cmd_pass(Threads *thread, char *pass)
{
    strncpy(currentpass,pass,MAX_PASSWORD_LEN);
    return CMD_SUPPORTED;
}

/* SERVER services-dev.chatspike.net password 0 :Description here */
PROTOCOL_BOOL orion_cmd_server(Threads *thread, char *servname, int hop, char *descript)
{
    send_cmd(thread,NULL, "SERVER %s %s %d :%s", servname, currentpass, hop, descript);
    return CMD_SUPPORTED;
}

/* PONG */
PROTOCOL_BOOL orion_cmd_pong(Threads *thread, char *servname, char *who)
{
    send_cmd(thread,servname, "PONG %s", servname);
    return CMD_SUPPORTED;
}

/* PING */
PROTOCOL_BOOL orion_cmd_ping(Threads *thread, char *servname, char *who)
{
    send_cmd(thread,servname, "PING %s", who);
    return CMD_SUPPORTED;
}

/* JOIN */
/* Althought inspircd 3.2 does not need the timestamp others do so
   we get it in the common function call */
PROTOCOL_BOOL orion_cmd_services_join(Threads *thread, char *user, char *channel, time_t chantime, int fake)
{
	char output[BUFSIZE];
	bzero(&output,sizeof(output));
	
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,channel,NULL);

	if(IRC.HasOwner) 			straddnc(output,IRC.SjoinOwnerPrefix,sizeof(output));
	if(IRC.HasProtect)			straddnc(output,IRC.SjoinProtectPrefix,sizeof(output));
								straddnc(output,IRC.SjoinOpPrefix,sizeof(output));
	if(IRC.SjoinHasIgnoreChar)	straddnc(output,IRC.SjoinIgnoredChar,sizeof(output));
								straddns(output,user,sizeof(output));
								
    sjoin(thread,channel,thread->c.chan.datecreate ? thread->c.chan.datecreate : chantime,output,fake);
    main_raw(thread,":%s FJOIN %s %ld :%s",m->servername,channel,thread->c.chan.datecreate ? thread->c.chan.datecreate : chantime,output);
    return CMD_SUPPORTED;
}
PROTOCOL_BOOL orion_cmd_join(Threads *thread, char *user, char *channel, time_t chantime)
{
	if (!user || !channel) 
        return CMD_ERROR;
		
    send_cmd(thread,user, "JOIN %s %ld", channel,chantime ? chantime : time(NULL));
    return CMD_SUPPORTED;
}

/* CHGHOST */
PROTOCOL_BOOL orion_cmd_chghost(Threads *thread, char *source, char *nick, char *vhost)
{
    if (!nick || !vhost) {
        return CMD_ERROR;
 
    }
    
	if(chghost_used)	send_cmd(thread,source ? source : m->servername,"CHGHOST %s :%s", nick, vhost);
    else				do_global(thread,NULL,"  *  ERROR: CHGHOST not loaded from InspIRCd, can't use CHGHOST");
    return CMD_SUPPORTED;
}

/* CHGIDENT */
PROTOCOL_BOOL orion_cmd_chgident(Threads *thread, char *source, char *nick, char *vIdent)
{
    if (!nick || !vIdent) {
        return CMD_ERROR;
    }
    if(chgident_used)	send_cmd(thread,source ? source : m->servername, "CHGIDENT %s %s", nick, vIdent);
    else				do_global(thread,NULL,"  *  ERROR: CHGIDENT not loaded from InspIRCd, can't use CHGIDENT");
    return CMD_SUPPORTED;
}

/* INVITE */
PROTOCOL_BOOL orion_cmd_invite(Threads *thread, char *source, char *chan, char *nick)
{
    if (!source || !chan || !nick) {
        return CMD_ERROR;
    }

    send_cmd(thread,source, "INVITE %s %s", nick, chan);
    return CMD_SUPPORTED;
}

/* PART */
PROTOCOL_BOOL orion_cmd_part(Threads *thread, char *nick, char *chan, char *buf)
{
    if (!nick || !chan) {
        return CMD_ERROR;
    }

    if (buf) {
        send_cmd(thread,nick, "PART %s :%s", chan, buf);
    } else {
        send_cmd(thread,nick, "PART %s :Leaving", chan);
    }
    return CMD_SUPPORTED;
}

/* GLOBOPS */
PROTOCOL_BOOL orion_cmd_global(Threads *thread, char *source, char *buf)
{
    if (!buf) {
        return CMD_ERROR;
    }

    if(globops_used)	send_cmd(thread,source ? source : m->servername, "GLOBOPS :%s", buf);
    else				send_cmd(thread,source ? source : m->servername, "WALLOPS :%s", buf);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_wallops(Threads *thread, char *source, char *fmt)
{
    send_cmd(thread,source ? source : O->nick, "WALLOPS :%s", fmt);
    return CMD_SUPPORTED;
}

/* SQUIT */
PROTOCOL_BOOL orion_cmd_squit(Threads *thread, char *servname, char *message)
{
    if (!servname || !message) {
        return CMD_ERROR;
    }

    send_cmd(thread,m->servername, "SQUIT %s :%s", servname, message);
    return CMD_SUPPORTED;
}

/* SVSO */
PROTOCOL_BOOL orion_cmd_svso(Threads *thread, char *source, char *nick, char *flag)
{
    return CMD_NOT_SUPPORTED;
}

/* NICK <newnick>  */
PROTOCOL_BOOL orion_cmd_chg_nick(Threads *thread, char *oldnick, char *newnick)
{
    if (!oldnick || !newnick) {
        return CMD_ERROR;
    }

    send_cmd(thread,oldnick, "NICK %s", newnick);
    return CMD_SUPPORTED;
}

/* SVSNICK */
PROTOCOL_BOOL orion_cmd_svsnick(Threads *thread, char *source, char *oldnick, char *newnick, time_t when)
{
    if (!oldnick || !newnick) {
        return CMD_ERROR;
    }
    
    if(sanick_used) send_cmd(thread,source ? source : m->servername, "SANICK %s %s", oldnick, newnick);
    else 			send_cmd(thread,source ? source : m->servername, "SVSNICK %s %s %ld", oldnick, newnick, when);
    return CMD_SUPPORTED;
}

/* SVSHOLD - set */
PROTOCOL_BOOL orion_cmd_svshold(Threads *thread, char *nick)
{
    /* Not supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* SVSHOLD - release */
PROTOCOL_BOOL orion_cmd_release_svshold(Threads *thread, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* UNSGLINE */
PROTOCOL_BOOL orion_cmd_unsgline(Threads *thread, char *mask)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* SGLINE */
PROTOCOL_BOOL orion_cmd_sgline(Threads *thread, char *mask, char *reason)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_unban(Threads *thread, char *name, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

/* SVSMODE channel modes */
PROTOCOL_BOOL orion_cmd_svsmode_chan(Threads *thread, char *name, char *mode, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svsjoin(Threads *thread, char *source, char *nick, char *chan, time_t date)
{
    if(sajoin_used) orion_cmd_sajoin(thread,source, nick, chan, date);
    else			send_cmd(thread,source ,"SVSJOIN %s %s", nick, chan);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_svspart(Threads *thread, char *source, char *nick, char *chan)
{
    if(sapart_used) orion_cmd_sapart(thread,source, nick, chan);
    else			send_cmd(thread,source ,"SVSPART %s %s", nick, chan);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_sajoin(Threads *thread, char *source, char *nick, char *chan, time_t date)
{
    send_cmd(thread,source ,"SAJOIN %s %s", nick, chan);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_sapart(Threads *thread, char *source, char *nick, char *chan)
{
    send_cmd(thread,source, "SAPART %s %s", nick, chan);
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_swhois(Threads *thread, char *source, char *who, char *mask)
{
    return CMD_NOT_SUPPORTED;	
}

PROTOCOL_BOOL orion_cmd_eob(Threads *thread)
{
    send_cmd(thread,NULL, "ENDBURST");
    return CMD_SUPPORTED;
}

PROTOCOL_BOOL orion_cmd_jupe(Threads *thread, char *jserver, char *who, char *reason)
{
    char rbuf[256];

    snprintf(rbuf, sizeof(rbuf), "Juped by %s%s%s", who,
             reason ? ": " : "", reason ? reason : "");

    send_cmd(thread,m->servername, "SERVER %s * %d :%s",jserver, 0/*serverid*/, rbuf);
	new_server(thread, me_server, jserver, rbuf, SERVER_JUPED, NULL);
    return CMD_SUPPORTED;
}

/* GLOBOPS - to handle old WALLOPS */
PROTOCOL_BOOL orion_cmd_global_legacy(Threads *thread, char *source, char *fmt)
{
    send_cmd(thread,source ? source : O->nick, "GLOBOPS :%s", fmt);
    return CMD_SUPPORTED;
}

/*  ************************************************************************************************  */
/**
 * Here are all functions which tell us and all modules that a serverlink occured
 * And it tells us if the server is currently linking or finished linking (important for sockscontrol)
 **/

/* This adds Servers to the server tree */
MOD_BOOL orion_event_server(Threads *thread, size_t ac, char **av)
{
	if(ac < 4)
		return MOD_CONT;
	
	if(digitcol(av[3])){
		//SERVER test.chatspike.net password 0 :ChatSpike InspIRCd test server
		dynncpy(uplink,av[1],sizeof(uplink));
		
		/* Execute event that we are linking and "av[1]" is our uplink */
		on_link(thread,m->servername,av[1]);
		
		/* set our uplink */
		addServer(thread, NULL, av[1], av[3], &_dstrjoin(' ',av,4)[1], NULL);
	}
	else{
		//:test.chatspike.net SERVER test2.chatspike.net * 1 :Second server
		
		/* Introduce that server to the servertree */
   		addServer(thread, &av[0][1], av[2], av[4], &_dstrjoin(' ',av,5)[1], NULL);
	}
    return MOD_CONT;
}

/* Called from serviceslink(); */
PROTOCOL_BOOL orion_cmd_initialize(Threads *thread){
	/* No Init */
    return CMD_NOT_SUPPORTED;
}

/* Called from orion_event_started(); */
PROTOCOL_BOOL orion_cmd_connect(Threads *thread, bool how)
{
	/* We send password and server introducing to our uplink */
	orion_cmd_pass(thread,m->linkpass);
	orion_cmd_server(thread, m->servername, 0, m->description);
	
	if(how == true){
		/* We start bursting here */
		main_raw(thread,"BURST %ld",time(NULL));
	
		/* We tell all inspircd servers our Version */
		send_cmd(thread,m->servername, "VERSION :[%s - Project: %s] version %s\n", MAINPROJECT,PROJECTNAME,VERSION);
		
		/* We do the primary connect, means connecting all services clients and BotServ Bots */
		_ExecuteEvent(NULL,NULL,"on_connect_primary");
	}
    return CMD_SUPPORTED;
}

/* ON CAPAB - we aren't connected now (we are, but we don't know about it yet) */
/* Here we are able to look if inspircd has loaded all modules we need */
MOD_BOOL orion_event_started(Threads *thread, size_t ac, char **av)
{
	if(ac < 2)
		return MOD_CONT;
		
	/* Search for modules we need to be loaded by inspircd */
	if(streqnc(av[1],"modules")){
		char infobuffer[6000];
		char *module[50];
		size_t i = 0;
		int a = 0;
		bzero(&infobuffer,sizeof(infobuffer));
		bzero(module,sizeof(module));
		
		dynncpy(infobuffer,av[2],6000);
		i = strsplit(infobuffer,module,',');
		for(a=0;a<i;a++){
			if(streqnc(module[a],"m_services.so"))			services_used = 1;
			else if(streqnc(module[a],"m_chanprotect.so"))	chanprotect_used = 1;
			
			else if(streqnc(module[a],"m_chghost.so"))		chghost_used = 1;
			else if(streqnc(module[a],"m_chgident.so"))		chgident_used = 1;
			else if(streqnc(module[a],"m_chgname.so"))		chgname_used = 1;
			else if(streqnc(module[a],"m_globops.so"))		globops_used = 1;
			
			else if(streqnc(module[a],"m_sajoin.so"))		sajoin_used = 1;
			else if(streqnc(module[a],"m_sanick.so"))		sanick_used = 1;
			else if(streqnc(module[a],"m_sapart.so"))		sapart_used = 1;
		}
	}
		
	/* We check if we are able to run with inspircd's loaded modules */
	else if(streqnc(av[1],"end")){
		if(services_used == 0){
			dlog(thread,0,"  *  ERROR: InspIRCd does not use \"m_services.so\" - correct that!\n");
			orion_exit(thread);
		}
		if(chanprotect_used == 0){
			dlog(thread,0,"  *  ERROR: InspIRCd does not use \"m_chanprotect.so\" - correct that!\n");
			orion_exit(thread);
		}
		
		if(chghost_used == 0 || chgident_used == 0){
			IRC.VHost = 0;
			dlog(thread,0,"  *  ERROR: InspIRCd does not use \"m_chgident.so\" or \"m_chghost.so\" - (HostServ cannot be used)\n");
		}
		
		/* We start connecting the server, and execute the event that we are linking */
		orion_cmd_connect(thread,1);
		
		/* We tell all modules that WE are linking now */
		on_serviceslink(thread);
	}
	return MOD_CONT;
}

/* Called by receiving "ENDBURST" - this EOB comes from our Uplink, so we should connect all bots and join our channels */
MOD_BOOL orion_event_services_link_done(Threads *thread, size_t ac, char **av)
{
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN); // Flush our Cache

	/* We add the services server to the servers tree (we do that when we finished linking) */
	me_server = addServer(thread, serv_uplink->server, m->servername, "1", m->description, NULL);
	
	/* Let the Bots join our channels */
	_ExecuteEvent(NULL,NULL,"on_connect_secondary");

	/* We report that our linking is done now */	
	on_serviceslink_done(thread);	/* This means that linking !FOR US! is done */							//- reactivates Sockscontrol
	on_linkdone(thread);			/* This is executed whenever a serverlink completed successfully */		//- reactivates Sockscontrol

	/* say we finished bursting */
	orion_cmd_eob(thread);
	return MOD_CONT;
}

/* Just PONG on PING */
MOD_BOOL orion_event_ping(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
        
    orion_cmd_pong(thread, m->servername, &av[0][1]);
    return MOD_CONT;
}

/* This tells us when the linked server finished linking - it should immediately reply for our PING then with a PONG */
MOD_BOOL orion_event_pong(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
        
    /* If the server that linked replies, we know he finished linking */
    if(streqnc(&av[0][1],linkserver)){
		/* Let the server finish it's link - reactivate Sockscontrol */
		on_linkdone(thread);
	}
    return MOD_CONT;
}

/*  ************************************************************************************************  */

#define START_OF_RAW 4
/**
 * @brief raw:  To process outgoing raw messages
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param string The outgoing Raw-message
 * @param len The length of the created Raw-message
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL orion_cmd_raw(Threads *thread, char *param[]){
	PROTOCOL_BOOL analyzer;

	//:M PRIVMSG #operserv: raw :operserv svsmode M +SjFzq
	//:operserv SVSKILL M :lalala
	//:M MODE  +a
	//Tue Oct 19 22:47:14 2004 > :OperServ GLOBOPS :M used RAW command for svsmode
	//Tue Oct 19 22:47:14 2004 > :Global PRIVMSG #services :M used RAW command for :operserv

	int paramvalue = 0;

	/* SVSMODE VIA RAW */
		//:operserv SVSMODE shock +d 1
		//operserv SVSMODE shock +d 1
	if(streqnc(param[START_OF_RAW+1],"svsmode"))       analyzer = do_svsmode(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
	else if(streqnc(param[START_OF_RAW+0],"svsmode"))  analyzer = do_svsmode(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVS2MODE VIA RAW */
	//
	else if(streqnc(param[START_OF_RAW+1],"svs2mode")) analyzer = do_svs2mode(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
	else if(streqnc(param[START_OF_RAW+0],"svs2mode")) analyzer = do_svs2mode(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVSKILL VIA RAW */
		// > :shock PRIVMSG O :raw :operserv SVSKILL shock :lalala
		// < :CServe.deepirc.net SVSKILL  ::lalala
		// > :shock PRIVMSG ircdos :raw :sockscontrol SVSKILL shock :hey alter was geht bei dir
	else if(streqnc(param[START_OF_RAW+1],"svskill")){
		char *reason = _dstrjoin(' ',param,START_OF_RAW+2);
		analyzer = do_svskill(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],reason[0]==':' ? &reason[1] : reason);
	}

		// > :shock PRIVMSG O :raw SVSKILL shock :lalala
	else if(streqnc(param[START_OF_RAW+0],"svskill")){
		char *reason = _dstrjoin(' ',param,START_OF_RAW+2);
		analyzer = do_svskill(thread,m->servername,param[START_OF_RAW+1],reason[0]==':' ? &reason[1] : reason);
	}

	/* SVSNICK VIA RAW */
		// > :shock PRIVMSG O :raw :operserv SVSNICK shock shockzZ
	else if(streqnc(param[START_OF_RAW+1],"svsnick")) analyzer = do_svsnick(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
		// > :shock PRIVMSG O :raw SVSNICK shock shockzZ
	else if(streqnc(param[START_OF_RAW+0],"svsnick")) analyzer = do_svsnick(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVSJOIN VIA RAW */
		//:operserv svsjoin Nick #chan
	else if(streqnc(param[START_OF_RAW+0],"svsjoin")) analyzer = do_svsjoin(thread,0,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2],time(NULL));
	else if(streqnc(param[START_OF_RAW+1],"svsjoin")) analyzer = do_svsjoin(thread,0,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3],time(NULL));

	/* SVSPART VIA RAW */
	else if(streqnc(param[START_OF_RAW+0],"svspart")) analyzer = do_svspart(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);
	else if(streqnc(param[START_OF_RAW+1],"svspart")) analyzer = do_svspart(thread,&param[START_OF_RAW+0][1],param[2],param[3]);

	/* NORMAL IRC (RFC) RAW */
	else{
		while(param[paramvalue])
			paramvalue++;

		if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_core_condata` WHERE `nick`='%s'",mysql_quote(thread,&param[START_OF_RAW+0][1])))>0){
				on_protocol(thread,&IRC,paramvalue-START_OF_RAW,param);
				main_raw(thread,_djoin(' ',param));
				return CMD_EXECUTED;
		}
		else
			return CMD_ERROR;
	}
	return analyzer;
}

/*  ************************************************************************************************  */

/* Copied from Anope */
bool orion_valid_nick(Threads *thread, char *nick){
    return true;
}

bool orion_valid_chan(Threads *thread, char *chan){
    return true;
}

bool orion_flood_mode_check(Threads *thread, char *value){
    char *dp, *end;

    if (value && *value != ':'
        && (strtoul((*value == '*' ? value + 1 : value), &dp, 10) > 0)
        && (*dp == ':') && (*(++dp) != 0) && (strtoul(dp, &end, 10) > 0)
        && (*end == 0)) {
        return true;
    } else {
        return false;
    }
}
