/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 211                   $:	Rivision of last commit
 *      $Author:: shock              $:	Author of last commit
 *      $Date:: 2009-02-08 22:00:59 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file InspIRCd1.2.x.c
 *   @brief This is the Module file for InspIRCd1.2.x (Server to Server) protocol.
 *
 *   It contains all functions which are neccessary to process
 *   Inputs from from InspIRCd1.2.x and outputs to InspIRCd1.2.x!
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "osocket.h"
#include "services.h"
#include "modules.h"

char *p10id = "00A";

static bool sapart_used = 0;
static bool sajoin_used = 0;
static bool sanick_used = 0;

static bool chghost_used = 0;
static bool chgident_used = 0;
static bool chgname_used = 0;
static bool globops_used = 0;
static bool services_used = 0;
static bool chanprotect_used = 0;

/* Inbound */
MOD_BOOL orion_event_version(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_stats(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_mode(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_fmode(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_samode(Threads *thread, size_t ac, char **av);

MOD_BOOL orion_event_sanick(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_sajoin(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_sapart(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_addline(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_delline(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_sjoin(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_ping(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_pong(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_ftopic(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_topic(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_squit(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_rsquit(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_quit(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_opertype(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_kick(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_join(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_setident(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_chgident(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_sethost(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_chghost(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_nick(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_uid(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_server(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_privmsg(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_part(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_setname(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_fname(Threads *thread, size_t ac, char **av);

/* SERVER MODES */
MOD_BOOL orion_event_started(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_services_link(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_services_link_done(Threads *thread, size_t ac, char **av);
MOD_BOOL orion_event_eob(Threads *thread, size_t ac, char **av);


/* Outbound */
PROTOCOL_BOOL orion_cmd_initialize(Threads *thread);
PROTOCOL_BOOL orion_cmd_add_account(Threads *thread, char *uid, char *authnick);
PROTOCOL_BOOL orion_cmd_del_account(Threads *thread, char *uid);

PROTOCOL_BOOL orion_cmd_topic(Threads *thread, char *whosets, char *chan, char *whosetit, char *topic, time_t when);

PROTOCOL_BOOL orion_cmd_gline(Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason);
PROTOCOL_BOOL orion_cmd_ungline(Threads *thread, char *unsetter, char *host, char *user, time_t time_in);

PROTOCOL_BOOL orion_cmd_svskill(Threads *thread, char *source, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_kill(Threads *thread, char *source, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_rawsvsmode(Threads *thread, char *source, char *nick, char *modes);
PROTOCOL_BOOL orion_cmd_svsmode(Threads *thread, char *nick, char *target, char *modes);
PROTOCOL_BOOL orion_cmd_svs2mode(Threads *thread, char *nick, char *target, char *modes);
PROTOCOL_BOOL orion_cmd_nick(Threads *thread, char *nick, char *ident, char *host, char *name, char *modes);
PROTOCOL_BOOL orion_cmd_mode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);
PROTOCOL_BOOL orion_cmd_rawmode(Threads *thread, char *source, char *dest, time_t time_in, char *buf);

PROTOCOL_BOOL orion_cmd_kick(Threads *thread, char *source, char *chan, char *user, char *buf);
PROTOCOL_BOOL orion_cmd_notice(Threads *thread, char *source, char *dest, char *buf);
PROTOCOL_BOOL orion_cmd_privmsg(Threads *thread, char *source, char *dest, char *buf);
PROTOCOL_BOOL orion_cmd_serv_notice(Threads *thread, char *source, char *dest, char *msg);
PROTOCOL_BOOL orion_cmd_serv_privmsg(Threads *thread, char *source, char *dest, char *msg);

PROTOCOL_BOOL orion_cmd_quit(Threads *thread, char *source, char *buf);
PROTOCOL_BOOL orion_cmd_protoctl(Threads *thread);
PROTOCOL_BOOL orion_cmd_pass(Threads *thread, char *pass);
PROTOCOL_BOOL orion_cmd_server(Threads *thread, char *servname, int hop, char *descript);
PROTOCOL_BOOL orion_cmd_pong(Threads *thread, char *servname, char *who);
PROTOCOL_BOOL orion_cmd_ping(Threads *thread, char *servname, char *who);
PROTOCOL_BOOL orion_cmd_join(Threads *thread, char *user, char *channel, time_t chantime);
PROTOCOL_BOOL orion_cmd_services_join(Threads *thread, char *user, char *channel, time_t chantime, int fake);

PROTOCOL_BOOL orion_cmd_chghost(Threads *thread, char *source, char *nick, char *vhost);
PROTOCOL_BOOL orion_cmd_chgident(Threads *thread, char *source, char *nick, char *vIdent);
PROTOCOL_BOOL orion_cmd_invite(Threads *thread, char *source, char *chan, char *nick);
PROTOCOL_BOOL orion_cmd_part(Threads *thread, char *nick, char *chan, char *buf);

PROTOCOL_BOOL orion_cmd_wallops(Threads *thread, char *source, char *fmt);
PROTOCOL_BOOL orion_cmd_global(Threads *thread, char *source, char *buf);

PROTOCOL_BOOL orion_cmd_sqline(Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason);
PROTOCOL_BOOL orion_cmd_unsqline(Threads *thread, char *servername, char *nick, char *snick);

PROTOCOL_BOOL orion_cmd_squit(Threads *thread, char *servname, char *message);
PROTOCOL_BOOL orion_cmd_svso(Threads *thread, char *source, char *nick, char *flag);
PROTOCOL_BOOL orion_cmd_chg_nick(Threads *thread, char *oldnick, char *newnick);
PROTOCOL_BOOL orion_cmd_svsnick(Threads *thread, char *source, char *oldnick, char *newnick, time_t when);
PROTOCOL_BOOL orion_cmd_connect(Threads *thread, bool how);
PROTOCOL_BOOL orion_cmd_svshold(Threads *thread, char *nick);
PROTOCOL_BOOL orion_cmd_release_svshold(Threads *thread, char *nick);

PROTOCOL_BOOL orion_cmd_szline(Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason);
PROTOCOL_BOOL orion_cmd_unszline(Threads *thread, char *servername, char *mask, char *snick, time_t in_time);

PROTOCOL_BOOL orion_cmd_unban(Threads *thread, char *name, char *nick);
PROTOCOL_BOOL orion_cmd_svsmode_chan(Threads *thread, char *name, char *mode, char *nick);
PROTOCOL_BOOL orion_cmd_svsjoin(Threads *thread, char *source, char *nick, char *chan, time_t date);
PROTOCOL_BOOL orion_cmd_svspart(Threads *thread, char *source, char *nick, char *chan);
PROTOCOL_BOOL orion_cmd_sapart(Threads *thread, char *source, char *nick, char *chan);
PROTOCOL_BOOL orion_cmd_sajoin(Threads *thread, char *source, char *nick, char *chan, time_t date);

PROTOCOL_BOOL orion_cmd_swhois(Threads *thread, char *source, char *who, char *mask);
PROTOCOL_BOOL orion_cmd_eob(Threads *thread);
PROTOCOL_BOOL orion_cmd_jupe(Threads *thread, char *jserver, char *who, char *reason);
PROTOCOL_BOOL orion_cmd_global_legacy(Threads *thread, char *source, char *fmt);

PROTOCOL_BOOL orion_cmd_raw(Threads *thread, char *param[]);
bool orion_valid_nick(Threads *thread, char *nick);
bool orion_valid_chan(Threads *thread, char *chan);
bool orion_flood_mode_check(Threads *thread, char *value);

/* *INDENT-OFF* */
void CreateProtocol(Threads *thread, Protocol *protocol, void *dl_handle) {
    
	//printf("Creating Protocol ...\n");
    event_init(thread, protocol, dl_handle, "shock", "InspIRCd 1.2.x and above [version 0.1]");

    createEvent(thread,protocol, "JOIN",      orion_event_join,     1);
    createEvent(thread,protocol, "KICK",      orion_event_kick,     1);
    createEvent(thread,protocol, "MODE",      orion_event_mode,     1);
    createEvent(thread,protocol, "NICK",      orion_event_nick,     1);
    createEvent(thread,protocol, "UID",       orion_event_uid,      1);

    createEvent(thread,protocol, "PART",      orion_event_part,     1);
    createEvent(thread,protocol, "PING",      orion_event_ping,     1);
    createEvent(thread,protocol, "PONG",      orion_event_pong,     1);
    createEvent(thread,protocol, "PRIVMSG",   orion_event_privmsg,  1);
    createEvent(thread,protocol, "QUIT",      orion_event_quit,     1);
    createEvent(thread,protocol, "OPERQUIT",  orion_event_quit,     1);
    createEvent(thread,protocol, "OPERTYPE",  orion_event_opertype, 1);
    
    createEvent(thread,protocol, "TOPIC",     orion_event_topic,    1);
    createEvent(thread,protocol, "SAMODE",    orion_event_samode,   1);
    createEvent(thread,protocol, "SANICK",    orion_event_sanick,   1);
    createEvent(thread,protocol, "SAJOIN",    orion_event_sajoin,   1);
    createEvent(thread,protocol, "SAPART",    orion_event_sapart,   1);
    
    createEvent(thread,protocol, "ADDLINE",   orion_event_addline,  1);
    createEvent(thread,protocol, "DELLINE",   orion_event_delline,  1);
    
    createEvent(thread,protocol, "FHOST",     orion_event_chghost,  1);
    createEvent(thread,protocol, "CHGIDENT",  orion_event_chgident, 1);
    createEvent(thread,protocol, "FNAME",     orion_event_fname,    1);

    createEvent(thread,protocol, "FJOIN",     orion_event_sjoin,    1);
    createEvent(thread,protocol, "FMODE",     orion_event_fmode,    1);
    createEvent(thread,protocol, "FTOPIC",    orion_event_ftopic,   1);
    createEvent(thread,protocol, "STATS",     orion_event_stats,    1);
    createEvent(thread,protocol, "VERSION",   orion_event_version,  1);
    
    /** Server Modes **/
    createEvent(thread,protocol, "SERVER",    orion_event_server,					1);
    createEvent(thread,protocol, "SERVER",    orion_event_server,					0);
    createEvent(thread,protocol, "SQUIT",     orion_event_squit,					1);
    createEvent(thread,protocol, "RSQUIT",    orion_event_rsquit,					1);
    createEvent(thread,protocol, "CAPAB",     orion_event_started,					0);
    createEvent(thread,protocol, "ENDBURST",  orion_event_eob,						1);
    
   protocol->NSMode						= "+o";
   protocol->CSMode						= "+o";
   protocol->MSMode						= "+o";
   protocol->HSMode						= "+o";
   protocol->OSMode						= "+io";
   protocol->BSMode						= "+o";
   protocol->HeServMode					= "+o";
   protocol->GlobMode					= "+io";
   protocol->QMode						= "+o";
   protocol->SCMode						= "+io";
   protocol->BSBotsMode					= "+ios";
   protocol->ChannelModes				= "iklmnpstcfgjrzCGIJKLMNOPQRSTV";	/* Avail Channel Modes */
   protocol->ParametricChannelModes		= "klLf";							/* Channel Modes with an allowed Parameter (removed: "gjJI" -> may cauze DB troubles) */
   protocol->MLockAbleModes				= "klLf";							/* Channel Modes provided by MLOCK */
   protocol->OnSuspend					= "+Opmntil 1";						/* Channel Modes on adding a suspend (forbid) */
   protocol->OnUnSuspend				= "-Opmntil";						/* Channel Modes on removing a suspend (forbid) */
   protocol->SupportSVS					= true;								/* Supports SVS-COMMANDS */
   protocol->SupportSVSJoin				= true;								/* Supports SVSJOIN */
   protocol->SvsNick					= true;								/* Supports SVSNICK */
   protocol->VHost						= true;								/* Supports VHosts */
   protocol->HasOwner					= true;								/* Supports ChannelOwnerMode (+q) */
   protocol->HasProtect					= true;								/* Supports ChannelProtectMode (+a) */
   protocol->OwnerMode					= 'q';								/* UserChannelMode for Owner */
   protocol->ProtectMode				= 'a';								/* UserChannelMode for Protected */
   protocol->OpMode						= 'o';								/* UserChannelMode for Op */
   protocol->HOpMode					= 'h';								/* UserChannelMode for HalfOp */
   protocol->VMode						= 'v';								/* UserChannelMode for Voice */
   protocol->BanMode					= 'b';								/* UserChannelMode for Ban */
   protocol->ExceptMode					= 'e';								/* UserChannelMode for BanException */
   protocol->SupportedUserModes			= "oinswBGHIQRSWdhrxk";				/* Known UserModes */
   protocol->SupportedUChanModes		= "qaohvbe";						/* All UserChannelModes */
   protocol->SupportedU2ChanModes		= "qaohv";							/* All UserChannelModes except "Except (+e)" and Ban (+b) */
   protocol->ModeOnNickReg				= "+r";								/* Mode for Registered / Identified Nicks */
   protocol->ModeOnNickDrop				= "-r";								/* Mode for Dropping a registered Nick */
   protocol->ModeOnChannelReg			= "+r";								/* Mode for Registered Channels */
   protocol->ModeOnChannelDrop			= "-r";								/* Mode for Dropping a registered Channel */
   protocol->RegMode					= 'r';								/* Mode for Registration */
   protocol->ChanRegMode				= true;								/* Supports a Channel Registration Mode */
   protocol->NickRegMode				= true;								/* Supports a Nick Registration Mode */
   protocol->SupportSGlines				= true;								/* Supports Glines */
   protocol->SupportSQlines				= true;								/* Supports Qlines */
   protocol->SupportSZlines				= true;								/* Supports Zlines */
   protocol->SupportHalfOp				= true;								/* Supports HalfOpMode */
   protocol->SupportBanMode				= true;								/* Supports BanMode */
   protocol->SupportExceptMode			= false;							/* Supports ExceptMode */
   protocol->SupportProtectUmode		= false;							/* Supports a Protection UMode */
   protocol->ProtectUmode				= "Sq";								/* Mode for Protecting a Nickname */
   protocol->SupportInvisibleUMode		= true;								/* Supports InvisibleUMode */
   protocol->InvisibleUMode				= "Q";								/* Mode for Invisibility */
   protocol->SupportChanSQlines			= false;							/* Supports Chan SQlines */
   protocol->SupportVIdents				= true;								/* Supports VIdent's */
   protocol->VHostOnNickChange			= true;								/* Supports setting VHosts on NickChanges */
   protocol->SupportRealnameChanges		= true;								/* Supports RealnameChanges */
   protocol->SupportAdminOnlyChanMode	= true;								/* Supports an AdminOnly ChannelMode */
   protocol->SupportOperOnlyChanMode	= true;								/* Supports an OperOnly ChannelMode */
   protocol->DoesGlobalChangeAfterSVS	= true;								/*  */
   protocol->DoesGlobalChangeAfterSA	= true;								/*  */
   protocol->DoesQuitAfterKill			= true;								/* Sends a Quit-Msg on Kill Execution */
   protocol->AdminOnlyCMode				= "A";								/* AdminOnly ChannelMode */
   protocol->OperOnlyCMode				= "O";								/* OperOnly ChannelMode */
   protocol->DefaultMLOCK				= "+ntr";							/* Default MLOCK Modes */
   protocol->VhostMode					= 'x';								/* Vhost Mode (is set on identify) */
   protocol->SupportFloodMode			= true;								/* Supports a FloodMode */
   protocol->SupportLinkMode			= true;								/* Supports a ChannelLinkMode */
   protocol->FloodCMode					= 'f';								/* ChannelFloodMode setting */
   protocol->LinkCMode					= 'L';								/* ChannelMode for linking Channels */
   protocol->KeyCMode					= 'k';								/* ChannelKeyMode setting */
   protocol->LimitCMode					= 'l';								/* ChannelMode for limit */
   protocol->InviteCMode				= 'i';								/* ChannelMode for inviteonly */
   protocol->SupportTokens				= false;							/* Supports Tokens */
   protocol->SupportSjoin				= true;								/* Supports SJOIN */
   protocol->SupportSjoinBanprefix		= false;							/* Supports a BanPrefix for SJOIN */
   protocol->SupportSjoinExceptprefix	= false;							/* Supports an ExceptPrefix for SJOIN */
   protocol->SjoinOwnerPrefix			= '~';								/* OwnerPrefix for SJOIN */
   protocol->SjoinProtectPrefix			= '&';								/* ProtectionPrefix for SJOIN */
   protocol->SjoinOpPrefix				= '@';								/* OpPrefix for SJOIN */
   protocol->SjoinHOpPrefix				= '%';								/* HalfOpPrefix for SJOIN */
   protocol->SjoinVoicePrefix			= '+';								/* VoicePrefix for SJOIN */
   protocol->SjoinBanPrefix				= '\0';								/* BanPrefix for SJOIN */
   protocol->SjoinExceptPrefix			= '\0';								/* ExceptPrefix for SJOIN */
   protocol->SjoinHasIgnoreChar			= true;								/* SJOIN has an ignored Mode (it will be skipped) */
   protocol->SjoinIgnoredChar			= ',';								/* Mode to be ignored from SJOIN */
   protocol->SupportHelperUMode			= true;								/* Supports Helper UMODE */
   protocol->SupportSyncState			= false;							/* Supports SyncState */
   protocol->p10protocol				= false;							/* Is p10 protocol */
   protocol->p10new						= true;								/* Is newp10 protocol */
   
	protocol->orion_cmd_initialize		= orion_cmd_initialize;
	protocol->orion_cmd_topic			= orion_cmd_topic;
	protocol->orion_cmd_add_account		= orion_cmd_add_account;
	protocol->orion_cmd_del_account		= orion_cmd_del_account;
	protocol->orion_cmd_kill			= orion_cmd_kill;
	protocol->orion_cmd_svskill			= orion_cmd_svskill;
	protocol->orion_cmd_rawsvsmode		= orion_cmd_rawsvsmode;
	protocol->orion_cmd_svsmode			= orion_cmd_svsmode;
	protocol->orion_cmd_svs2mode		= orion_cmd_svs2mode;
	protocol->orion_cmd_nick			= orion_cmd_nick;
	protocol->orion_cmd_mode			= orion_cmd_mode;
	protocol->orion_cmd_rawmode			= orion_cmd_rawmode;
	protocol->orion_cmd_kick			= orion_cmd_kick;
	protocol->orion_cmd_notice			= orion_cmd_notice;
	protocol->orion_cmd_privmsg			= orion_cmd_privmsg;
	protocol->orion_cmd_serv_notice		= orion_cmd_serv_notice;
	protocol->orion_cmd_serv_privmsg	= orion_cmd_serv_privmsg;
	
	protocol->orion_cmd_quit			= orion_cmd_quit;
	protocol->orion_cmd_protoctl		= orion_cmd_protoctl;
	protocol->orion_cmd_pass			= orion_cmd_pass;
	protocol->orion_cmd_server			= orion_cmd_server;
	protocol->orion_cmd_pong			= orion_cmd_pong;
	protocol->orion_cmd_ping			= orion_cmd_ping;
	protocol->orion_cmd_join			= orion_cmd_join;
	protocol->orion_cmd_services_join	= orion_cmd_services_join;
	protocol->orion_cmd_chghost			= orion_cmd_chghost;
	protocol->orion_cmd_chgident		= orion_cmd_chgident;
	protocol->orion_cmd_invite			= orion_cmd_invite;
	protocol->orion_cmd_part			= orion_cmd_part;
	
	protocol->orion_cmd_wallops			= orion_cmd_wallops;
	protocol->orion_cmd_global			= orion_cmd_global;
	
	protocol->orion_cmd_squit			= orion_cmd_squit;
	//protocol->orion_cmd_svso =orion_cmd_svso
	protocol->orion_cmd_chg_nick		= orion_cmd_chg_nick;
	protocol->orion_cmd_svsnick			= orion_cmd_svsnick;
	protocol->orion_cmd_connect			= orion_cmd_connect;
	/*protocol->orion_cmd_svshold =orion_cmd_svshold
	protocol->orion_cmd_release_svshold =orion_cmd_release_svshold*/
	
	protocol->orion_cmd_gline			= orion_cmd_gline;
	protocol->orion_cmd_ungline			= orion_cmd_ungline;
	
	protocol->orion_cmd_szline			= orion_cmd_szline;
	protocol->orion_cmd_unszline		= orion_cmd_unszline;
	
	protocol->orion_cmd_sqline			= orion_cmd_sqline;
	protocol->orion_cmd_unsqline		= orion_cmd_unsqline;
	
	protocol->orion_cmd_unban			= orion_cmd_unban;
	protocol->orion_cmd_svsmode_chan	= orion_cmd_svsmode_chan;
	protocol->orion_cmd_svsjoin			= orion_cmd_svsjoin;
	protocol->orion_cmd_svspart			= orion_cmd_svspart;
	
	protocol->orion_cmd_sajoin			= orion_cmd_sajoin;
	protocol->orion_cmd_sapart			= orion_cmd_sapart;
	
	protocol->orion_cmd_swhois			= orion_cmd_swhois;
	protocol->orion_cmd_eob				= orion_cmd_eob;
	protocol->orion_cmd_jupe			= orion_cmd_jupe;
	protocol->orion_cmd_global_legacy	= orion_cmd_global_legacy;
	protocol->orion_cmd_raw				= orion_cmd_raw;
	protocol->orion_valid_nick			= orion_valid_nick;
	protocol->orion_valid_chan			= orion_valid_chan;
	protocol->orion_flood_mode_check	= orion_flood_mode_check;
	
	dynncpy(global_target,m->network,sizeof(global_target));
	printf(" [*] Created InspIRCd Protocol (version: 0.1)...\n");
	return;
}

#ifdef _WIN32
void DeleteProtocol(Threads *thread, size_t ac, char **av)
{
    return MOD_CONT;
}
#endif

int p10nickcnt = 1;
int p10servcnt = 1;
int p10inuse = 0;

//
MOD_BOOL orion_event_version(Threads *thread, size_t ac, char **av)
{
    if (ac < 2)
        return MOD_CONT;
	
	Users *u = NULL;
	u = GetUserByUID(&av[0][1]);
	
	if(!u)
		u = GetUser(&av[0][1]);
	
	if(u){
		version(thread,u->nick);
		
		/* Execute version Event */
		_ExecuteEvent(u->nick,NULL,"on_version");
	}
    return MOD_CONT;
}


MOD_BOOL orion_event_stats(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
	
	Users *u = NULL;
	u = GetUserByUID(&av[0][1]);
	
	if(u){
		if(streqnc("u", av[2])){
			time_t t      = time(NULL);
			time_t uptime = t-m->starttime;
		
			send_cmd(thread,p10id, "PUSH %s :%s 242 :Services up: %d day%s, %02d:%02d:%02d", u->nick, m->servername, uptime / 86400,(uptime / 86400 == 1) ? "" : "s", (uptime / 3600) % 24, (uptime / 60) % 60, uptime % 60);
			send_cmd(thread,p10id, "PUSH %s :%s 219 u End of /STATS report", u->nick, m->servername);
		}
	}
    return MOD_CONT;
}

//
MOD_BOOL orion_event_mode(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
          
    Users *u = NULL;
    Servers *s = NULL;
    char *sender;
    char *param[ARRAY_SIZE];
    //char modes[50];
    size_t nr_modes		= 0;
    size_t i			= 0;
    int modeparamnr 	= 0;
    int splicer			= 0;
    bool type			= 1;
    u = GetUserByUID(&av[0][1]);

    if (!u) {
		s = findserver_uid(thread, servlist, &av[0][1]);
		
		if(!s)
			s = findserver(thread, servlist, &av[0][1]);
		
		if(!u && !s)
        	sender = &av[0][1];
        else
        	sender = s->server;
    } else {
        sender = u->nick;
    }

    if (ac < 3)
        return MOD_CONT;

	//on_mode(thread,sender,av[2], time(NULL),_dstrjoin(' ',av,3));
    if (av[2][0] == '#') {
		nr_modes = dynlen(av[3]);
		
		param[modeparamnr++] = av[3];
		for(i=0;i<nr_modes;i++){
			if(av[3][i]=='+'){ type=1; splicer++; }
			else if(av[3][i]=='-'){ type=0; splicer++; }
			
			else if(av[3][i]==IRC.OpMode){
				//printf("found opmode, for user: %s (paramter = %d, id param = %s)\n",GetUserByUID(av[4+i-splicer])->nick,4+i-splicer,av[4+i-splicer]);
				param[modeparamnr++]=GetUserByUID(av[4+i-splicer])->nick;
			}
			else if(av[3][i]==IRC.HOpMode){
				param[modeparamnr++]=GetUserByUID(av[4+i-splicer])->nick;
			}
			else if(av[3][i]==IRC.VMode){
				param[modeparamnr++]=GetUserByUID(av[4+i-splicer])->nick;
			}
			else if(av[3][i]==IRC.OwnerMode){
				param[modeparamnr++]=GetUserByUID(av[4+i-splicer])->nick;
			}
			else if(av[3][i]==IRC.ProtectMode){
				param[modeparamnr++]=GetUserByUID(av[4+i-splicer])->nick;
			}
			
			else if(av[3][i]==IRC.LimitCMode){
				param[modeparamnr++]=av[4+i-splicer];
			}
			else if(av[3][i]==IRC.KeyCMode){
				param[modeparamnr++]=av[4+i-splicer];
			}
			else if(av[3][i]==IRC.BanMode){
				param[modeparamnr++]=av[4+i-splicer];
			}
			else if(av[3][i]==IRC.ExceptMode){
				param[modeparamnr++]=av[4+i-splicer];
			}
		}
		param[modeparamnr++]=NULL;
		
		//printf("nefarious sending mode: %s\n",_djoin(' ',param));
		
		if(DuringServerlink)
		/* This is REALLY IMPORTANT - it avoid's Net Desyncs on linking, if modes happen
			before finishing the synchronisation process */
			MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN);
			
        on_mode(thread,sender,av[2], time(NULL),_djoin(' ',param)); //channel mode
    } else {
		//:<sid/uuid> MODE <target> <modes and parameters>
		on_mode(thread,sender,GetUserByUID(av[2])->nick, time(NULL),av[3]); //user mode
    }
    return MOD_CONT;
}

//
MOD_BOOL orion_event_fmode(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
          
    Users *u = NULL;
    Servers *s = NULL;
    char *sender;
    char *param[ARRAY_SIZE];

    size_t i			= 0;
    size_t nr_modes		= 0;
    int splicer			= 0;
    int modeparamnr 	= 0;
    bool type			= 1;
    u = GetUserByUID(&av[0][1]);

    if (!u) {
		s = findserver_uid(thread, servlist, &av[0][1]);
		
		if(!s)
			s = findserver(thread, servlist, &av[0][1]);
		
		if(!u && !s)
        	sender = &av[0][1];
        else
        	sender = s->server;
    } else {
        sender = u->nick;
    }

    if (ac < 3)
        return MOD_CONT;

	//on_mode(thread,sender,av[2], time(NULL),_dstrjoin(' ',av,3));
	
//	> :748AAAAAD FMODE #services 1213642863 +lk 15 roflwasgeht
    if (av[2][0] == '#') {
		nr_modes = dynlen(av[4]);
		
		param[modeparamnr++] = av[4];
		for(i=0;i<nr_modes;i++){
			if(av[4][i]=='+'){ type=1; splicer++; }
			else if(av[4][i]=='-'){ type=0; splicer++; }
			
			else if(av[4][i]==IRC.OpMode){
				//printf("found opmode, for user: %s (paramter = %d, id param = %s)\n",GetUserByUID(av[4+i-splicer])->nick,4+i-splicer,av[4+i-splicer]);
				param[modeparamnr++]=GetUserByUID(av[5+i-splicer])->nick;
			}
			else if(av[4][i]==IRC.HOpMode){
				param[modeparamnr++]=GetUserByUID(av[5+i-splicer])->nick;
			}
			else if(av[4][i]==IRC.VMode){
				param[modeparamnr++]=GetUserByUID(av[5+i-splicer])->nick;
			}
			else if(av[4][i]==IRC.OwnerMode){
				param[modeparamnr++]=GetUserByUID(av[5+i-splicer])->nick;
			}
			else if(av[4][i]==IRC.ProtectMode){
				param[modeparamnr++]=GetUserByUID(av[5+i-splicer])->nick;
			}
			
			else if(av[4][i]==IRC.LimitCMode){
				param[modeparamnr++]=av[5+i-splicer];
			}
			else if(av[4][i]==IRC.KeyCMode){
				param[modeparamnr++]=av[5+i-splicer];
			}
			else if(av[4][i]==IRC.BanMode){
				param[modeparamnr++]=av[5+i-splicer];
			}
			else if(av[4][i]==IRC.ExceptMode){
				param[modeparamnr++]=av[5+i-splicer];
			}
		}
		param[modeparamnr++]=NULL;
		//printf("nefarious sending mode: %s\n",_djoin(' ',param));
		
		if(DuringServerlink)
		/* This is REALLY IMPORTANT - it avoid's Net Desyncs on linking, if modes happen
			before finishing the synchronisation process */
			MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN);
		
        on_mode(thread,sender,av[2], time(NULL),_djoin(' ',param)); //channel mode
    }
    return MOD_CONT;
}

MOD_BOOL orion_event_samode(Threads *thread, size_t ac, char **av)
{
    /* :source SAMODE targets modes */
    if (ac < 4)
        return MOD_CONT;
        
    if(av[2][0]=='#'){
		if(DuringServerlink)
		/* This is REALLY IMPORTANT - it avoid's Net Desyncs on linking, if modes happen
			before finishing the synchronisation process */
			MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN);
		
    	on_mode(thread,&av[0][1],av[2], time(NULL),_dstrjoin(' ',av,3));
	}
    return MOD_CONT;
}

MOD_BOOL orion_event_sanick(Threads *thread, size_t ac, char **av)
{
    /* :source SANICK old new */
    if (ac < 4)
        return MOD_CONT;
	
    chgnick(thread,av[2],av[3]);
    return MOD_CONT;
}

//:<source server> SVSJOIN <nick> <channel>
MOD_BOOL orion_event_sajoin(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;
    
    _join(thread,0,av[2],av[3],(time_t)time(NULL));
    return MOD_CONT;
}

//:<source server> SAPART <nick> <channel>
MOD_BOOL orion_event_sapart(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
    
    on_part(thread,av[2],av[3]);
    return MOD_CONT;
}

//
//:<source server> ADDLINE [Q|Z|E|G] <mask> <setter> <time set> <duration> :<reason>
MOD_BOOL orion_event_addline(Threads *thread, size_t ac, char **av){
	
	Users *u = NULL;
	Servers *s = NULL;
	
	u = GetUserByUID(&av[0][1]);
	if(!u)
		s = findserver_uid(thread, servlist, &av[0][1]);
	
	if(av[2][0]=='G'){
		on_gline(thread,u ? u->nick : s->server,av[3],av[4],(time_t)stol(av[6]),(time_t)stol(av[5]),_dstrjoin(' ',av,7));
	}
	else if(av[2][0]=='Z'){
		on_szline(thread,u ? u->nick : s->server,av[3],av[4],(time_t)stol(av[6]),(time_t)stol(av[5]),_dstrjoin(' ',av,7));
	}
	else if(av[2][0]=='Q'){
		on_sqline(thread,av[4],av[3],(time_t)stol(av[6]),(time_t)stol(av[5]),_dstrjoin(' ',av,7));
	}
	return MOD_CONT;
}

//
//:497 FJOIN #staff 1133630884 +ikl mooc0w 10 :oh,497AAAAAB o,497AAAAQZ h,497AAART0 ,497AAAZZH v,497AAAZZ6 ,497AAYU9B ,497AAB67L
//> :748 FJOIN #services 1213650199 +r :,748AAAAAD
MOD_BOOL orion_event_sjoin(Threads *thread, size_t ac, char **av)
{
	
    char outbuf[BUFSIZE];
    char usermodes[10/*UserID+IgnoredChar*/+MAX_USER_MODES];
    char *param[ARRAY_SIZE];
    
    size_t i			= 0;
    size_t j			= 0;
    size_t a			= 0;
    size_t nr_modes		= 0;
    int use_modes		= 0;
    bool type			= 1;
    int modeparamnr 	= 0;
    int splicer			= 0;
    int further			= 0;
    
    bzero(&outbuf,sizeof(outbuf));
    bzero(param,sizeof(param));
    
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_CONDATA_NO_MODES);
    
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,av[2],NULL);
	
	if(av[4][0]=='+'){
		further++;
		nr_modes = dynlen(av[4]);
		
		param[modeparamnr++] = av[4];
		for(i=0;i<nr_modes;i++){
			if(av[4][i]=='+'){ type=1; splicer++; }
			else if(av[4][i]=='-'){ type=0; splicer++; }
			
			if(av[4][i]==IRC.LimitCMode){
				param[modeparamnr]=av[5+modeparamnr-splicer];
				further++;
				modeparamnr++;
			}
			else if(av[4][i]==IRC.KeyCMode){
				param[modeparamnr]=av[5+modeparamnr-splicer];
				further++;
				modeparamnr++;
			}
			else if(av[4][i]==IRC.LinkCMode){
				param[modeparamnr]=av[5+modeparamnr-splicer];
				further++;
				modeparamnr++;
			}
			else if(av[4][i]==IRC.FloodCMode){
				param[modeparamnr]=av[5+modeparamnr-splicer];
				further++;
				modeparamnr++;
			}
		}
		param[modeparamnr++]=NULL;
		//printf("nefarious sending mode: %s\n",_djoin(' ',param));
        on_mode(thread,findserver_uid(thread, servlist, av[0])->server,av[2],(time_t)stol(av[3]),_djoin(' ',param)); //channel mode
	}
	
	if(further>1){
		for(i=4+further;i<ac;i++){
			if(av[i][0]==':')
				break;
		}
	}else
		i = 5;
		
   	use_modes = i;
   	
	//:497 FJOIN #staff 1133630884 +ikl mooc0w 10 :oh,497AAAAAB o,497AAAAQZ h,497AAART0 ,497AAAZZH v,497AAAZZ6 ,497AAYU9B ,497AAB67L
   	for(i=use_modes;i<ac;i++){
		if(av[i][0]==':') av[i]=&av[i][1];

		bzero(&usermodes,sizeof(usermodes));
		dynncpy(usermodes,av[i],sizeof(usermodes));
		char *modes = strtok(usermodes,",");
		char *user  = strtok(NULL ,",");
		
		if(user==NULL){
			user=modes;
			modes=NULL;
		}
		
		Users *dbuser = GetUserByUID(user);
		if(dbuser!=NULL){
			if(modes!=NULL){
				a = dynlen(modes);
				for(j=0;j<dynlen(IRC.SupportedU2ChanModes);j++){
					if(j<a){
						if(modes[j]==IRC.OpMode)
							straddc(outbuf,IRC.SjoinOpPrefix);
						else if(modes[j]==IRC.VMode)
							straddc(outbuf,IRC.SjoinVoicePrefix);
						else if(modes[j]==IRC.HOpMode)
							straddc(outbuf,IRC.SjoinHOpPrefix);
						else if(modes[j]==IRC.OwnerMode)
							straddc(outbuf,IRC.SjoinOwnerPrefix);
						else if(modes[j]==IRC.ProtectMode)
							straddc(outbuf,IRC.SjoinProtectPrefix);
					}else break;
				}
			}
			straddc(outbuf,',');
			stradds(outbuf,dbuser->nick);
			straddc(outbuf,' ');
		}
	}
    
    sjoin(thread, av[2], (time_t)stol(av[3]), outbuf, 0);
    return MOD_CONT;
}

//
//> :748 FTOPIC #services 1213653529 shock!bla@shock.deepirc.net :lol bor 1
MOD_BOOL orion_event_ftopic(Threads *thread, size_t ac, char **av)
{
    /* :source FTOPIC channel ts setby :topic */
    
    if (ac < 6)
        return MOD_CONT;
        
    topic(thread,av[4],av[2],stol(av[3]),&_dstrjoin(' ',av,5)[1]);
    return MOD_CONT;
}

/* :setter TOPIC #channel :topic */
//
MOD_BOOL orion_event_topic(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;

    Users *u = NULL;
    Servers *s = NULL;
    u = GetUserByUID(&av[0][1]);
    if(!u)
    	s = findserver_uid(thread, servlist, &av[0][1]);
    	
    on_topic(thread,u ? u->nick : s->server,av[2],(time_t)time(NULL),&_dstrjoin(' ',av,3)[1]);
    return MOD_CONT;
}
//
MOD_BOOL orion_event_squit(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;

	on_squit(thread,av[2]);
    return MOD_CONT;
}

MOD_BOOL orion_event_rsquit(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
        
	on_squit(thread,av[2]);
    return MOD_CONT;
}
//
MOD_BOOL orion_event_quit(Threads *thread, size_t ac, char **av)
{
    if (ac < 2)
        return MOD_CONT;
        
    
    Users *u = NULL;
    u = GetUserByUID(&av[0][1]);
    
    if(!u)
    	u = GetUser(&av[0][1]);
    
    on_quit(thread,u->nick);
    return MOD_CONT;
}

//
MOD_BOOL orion_event_opertype(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
    
    Users *u = NULL;
    u = GetUserByUID(&av[0][1]);
    
    if(!u)
    	u = GetUser(&av[0][1]);
    	
    svsmode(thread,p10id,u->nick,"+o");
    return MOD_CONT;
}

//
MOD_BOOL orion_event_kick(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;
        
    Users *u = NULL;
    Users *utarget = NULL;
    Servers *s = NULL;
    u = GetUserByUID(&av[0][1]);
    if(!u)
    	s = findserver_uid(thread, servlist, &av[0][1]);
    
    utarget = GetUserByUID(av[3]);
    	
        
    on_kick(thread,u ? u->nick : s->server,utarget->nick,av[2]);
    return MOD_CONT;
}

//
MOD_BOOL orion_event_join(Threads *thread, size_t ac, char **av)
{
	if (ac < 3)
        return MOD_CONT;
    
    Users *u = NULL;
    u = GetUserByUID(&av[0][1]);
    if(!u)
    	u = GetUser(&av[0][1]);
    	
    _join(thread,0,u->nick,av[2],(time_t)stol(av[3]));
    return MOD_CONT;
}
//
MOD_BOOL orion_event_setident(Threads *thread, size_t ac, char **av)
{
	if(ac < 3)
		return MOD_CONT;
		
    Users *u = NULL;
    u = GetUserByUID(&av[0][1]);
    if(!u)
    	u = GetUser(&av[0][1]);
    		
	on_setident(thread,u->nick,av[2][0]==':' ? &av[2][1] : av[2]);
    return MOD_CONT;
}
//
MOD_BOOL orion_event_chgident(Threads *thread, size_t ac, char **av)
{
	if(ac < 4)
		return MOD_CONT;
		
    Users *u = NULL;
    u = GetUserByUID(av[2]);
    if(!u)
    	u = GetUser(av[2]);
		
 	on_chgident(thread,u->nick,av[3][0]==':' ? &av[3][1] : av[3]);
    return MOD_CONT;
}
//
MOD_BOOL orion_event_sethost(Threads *thread, size_t ac, char **av)
{
	if(ac < 3)
		return MOD_CONT;
		
    Users *u = NULL;
    u = GetUserByUID(&av[0][1]);
    if(!u)
    	u = GetUser(&av[0][1]);
    	
	on_sethost(thread,u->nick,av[2][0]==':' ? &av[2][1] : av[2]);
    return MOD_CONT;
}

//
//> :748AAAAAD FHOST netadmin.omega.org.za
MOD_BOOL orion_event_chghost(Threads *thread, size_t ac, char **av)
{
	if(ac < 3)
		return MOD_CONT;
		
    Users *u = NULL;
    u = GetUserByUID(&av[0][1]);
    if(!u)
    	u = GetUser(&av[0][1]);
		
 	on_chghost(thread,u->nick,av[2]);
    return MOD_CONT;
}
//
MOD_BOOL orion_event_fname(Threads *thread, size_t ac, char **av)
{
	if(ac < 3)
		return MOD_CONT;

    Users *u = NULL;
    u = GetUserByUID(av[2]);
    if(!u)
    	u = GetUser(av[2]);
		
    on_chgname(thread,u->nick,_dstrjoin(' ',av,2));
    return MOD_CONT;
}

//
MOD_BOOL orion_event_nick(Threads *thread, size_t ac, char **av)
{
    char oldnick[MAX_NICK_LEN];
	bzero(&oldnick,sizeof(oldnick));
	
    Users *user = GetUserByUID(&av[0][1]);
    dynncpy(oldnick,user->nick,sizeof(oldnick));
    chgnick(thread, oldnick, av[2]);
    return MOD_CONT;
}

//
MOD_BOOL orion_event_uid(Threads *thread, size_t ac, char **av)
{
	
	//:497 UID 497AAAAAB 1188302517 w00t 127.0.0.1 127.0.0.1 w00t +s 127.0.0.1 :Robin Burchell
    Users *user = NULL;
    Servers *s = NULL;
	
	if (ac > 3) {

        s = findserver(thread, servlist, &av[0][1]);
        if (!s)
            s = findserver_uid(thread, servlist, &av[0][1]);
            
        user = GetUser(av[4]);
        if(user == NULL)
			user = addUser(av[4],s->server);
			
        dynncpy(user->uid,av[2],sizeof(user->uid));
        dynncpy(user->ip,av[9],sizeof(user->ip));
        dynncpy(user->host,av[5],sizeof(user->host));
        dynncpy(user->ident,av[7],sizeof(user->ident));
        dynncpy(user->vident,av[7],sizeof(user->vident));        
        dynncpy(user->vhost,av[6],sizeof(user->vhost));
        user->faketype = 0;
        
        user->time = atol(av[3]);
			
		cnick(thread,
				av[4],	// nick
				NULL,	// vident
				av[7],	// ident
				av[8],  // modes
				(time_t)stol(av[3]),  // time
				av[5],	// realhost
				av[6],	// virtual host
				s->server,  // server
				&_dstrjoin(' ',av,11)[1],	// realname
				av[9],
				0
			);
	}
    return MOD_CONT;
}

//
MOD_BOOL orion_event_privmsg(Threads *thread, size_t ac, char **av)
{
    if (ac < 4)
        return MOD_CONT;

    Users *u = NULL;
    Users *id = NULL;
    char buf[BUFSIZE];
	char *privmsg[ARRAY_SIZE];
	
	bzero(&buf,sizeof(buf));
	bzero(privmsg,sizeof(privmsg));

    u = GetUserByUID(&av[0][1]);
    id = GetUserByUID(av[2]);

    if (ac < 3)
        return MOD_CONT;
        
    sprintf(buf,":%s PRIVMSG %s %s",(u ? u->nick : av[0]),(id ? id->nick : av[2]),_dstrjoin(' ',av,3));
    printf("MSG: %s\n",buf);
    strsplit(buf,privmsg,' ');
    
    on_privmsg(thread, privmsg);
    return MOD_CONT;
}
//
MOD_BOOL orion_event_part(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
    Users *u = NULL;
    u = GetUserByUID(&av[0][1]);
    if(!u)
    	u = GetUser(&av[0][1]);
    if(u)
    	on_part(thread,u->nick,av[2]);
    return MOD_CONT;
}
//:<source server> DELLINE [Q|Z|E|G] <mask>
MOD_BOOL orion_event_delline(Threads *thread, size_t ac, char **av){
	
    if (ac < 4)
        return MOD_CONT;
        
	Users *u = NULL;
	Servers *s = NULL;
	
	u = GetUserByUID(&av[0][1]);
	if(!u)
		s = findserver_uid(thread, servlist, &av[0][1]);
	
	if(av[2][0]=='G'){
		on_ungline(thread,u ? u->nick : s->server,av[3]);
	}
	else if(av[2][0]=='Z'){
		on_unszline(thread,u ? u->nick : s->server,av[3]);
	}
	else if(av[2][0]=='Q'){
		on_unsqline(thread,u ? u->nick : s->server,u ? u->nick : s->server,av[3]);
	}
	return MOD_CONT;
}

//
PROTOCOL_BOOL orion_cmd_add_account(Threads *thread, char *uid, char *authnick)
{
    return CMD_NOT_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_del_account(Threads *thread, char *uid)
{
    return CMD_NOT_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_topic(Threads *thread, char *whosets, char *chan, char *whosetit,
                      char *topic, time_t when)
{
    send_cmd(thread,p10id, "FTOPIC %s %ld %s :%s", chan, time(NULL), whosetit, topic);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_gline(Threads *thread, char *user, char *host, char *who, time_t when, time_t expires, char *reason)
{
    send_cmd(thread,p10id, "ADDLINE G %s %s %ld %ld :%s", host, who ? who : _dynadd("%s!%s@%s",m->nick,m->ident,m->vhost),  when,  expires, reason);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_ungline(Threads *thread, char *unsetter, char *host, char *user, time_t time_in)
{
    send_cmd(thread,p10id, "DELLINE G %s", host);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_sqline(Threads *thread, char *sender, char *mask, time_t time_in, time_t expire, char *reason)
{
    send_cmd(thread,p10id, "ADDLINE Q %s %s %ld %ld :%s", mask, sender ? sender : _dynadd("%s!%s@%s",m->nick,m->ident,m->vhost), time_in, expire, reason);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_unsqline(Threads *thread, char *servername, char *nick, char *snick)
{
    if (!nick) {
        return CMD_ERROR;
    }
    send_cmd(thread,p10id, "DELLINE Q %s", nick);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_szline(Threads *thread, char *servername, char *mask, char *snick, time_t when, time_t expire, char *reason)
{
    send_cmd(thread,p10id, "ADDLINE Z %s %s %ld %ld :%s", mask, snick, when, expire, reason);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_unszline(Threads *thread, char *servername, char *mask, char *snick, time_t in_time)
{
    send_cmd(thread,p10id, "DELLINE Z %s", mask);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_svskill(Threads *thread, char *source, char *user, char *buf)
{
    if (!buf) {
        return CMD_ERROR;
    }

    if (!source || !user) {
        return CMD_ERROR;
    }
    
    Users *ud = NULL;
    ud = GetUser(source);

    send_cmd(thread,ud ? ud->uid : p10id, "KILL %s :%s", user, buf);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_kill(Threads *thread, char *source, char *user, char *buf)
{
    orion_cmd_svskill(thread,source, user, buf);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_rawsvsmode(Threads *thread, char *source, char *nick, char *modes)
{
    Users *ud = NULL;
    ud = GetUser(source);

    send_cmd(thread,ud ? ud->uid : GetUser(O->nick)->uid, "SVSMODE %s %s", nick, modes);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_svsmode(Threads *thread, char *source, char *nick, char *modes)
{
    Users *u = NULL;
    u = GetUser(source);
    
    Users *ud = NULL;
    ud = GetUser(source);

	if(ud)
    	send_cmd(thread,u ? u->uid : GetUser(O->nick)->uid, "SVSMODE %s %s", ud->uid, modes);
	//svsmode(thread,source,nick,modes);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_svs2mode(Threads *thread, char *source, char *nick, char *modes)
{
    Users *u = NULL;
    u = GetUser(source);
    
    Users *ud = NULL;
    ud = GetUser(source);

	if(ud)
    	send_cmd(thread,u ? u->uid : GetUser(O->nick)->uid, "SVSMODE %s %s", ud->uid, modes);
	//svsmode(thread,source,nick,modes);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_nick(Threads *thread, char *nick, char *ident, char *host, char *name, char *modes)
{
    // :test.chatspike.net NICK 1133519355 Brain synapse.brainbox.winbot.co.uk netadmin.chatspike.net ~brain +xwsioS 10.0.0.2 :Craig Edwards
    
	char nicknumbuf[50];
	bzero(&nicknumbuf,sizeof(nicknumbuf));
	
	straddns(nicknumbuf,p10id,sizeof(nicknumbuf));
	straddns(nicknumbuf,GetNewID(GetUserLastUID(p10id)),sizeof(nicknumbuf));
	
	send_cmd(thread,p10id, "UID %s %ld %s %s %s %s %s 0.0.0.0 %ld :%s",nicknumbuf, time(NULL),nick,host,host,ident,modes, time(NULL),name);
    send_cmd(thread,p10id, "OPERTYPE Service");
    
	Users *user = GetUser(nick);
	if(user == NULL)
		user = addUser(nick,m->servername);
		
	user->time=time(NULL);
	
	dynncpy(user->ident,ident,sizeof(user->ident));
	dynncpy(user->ip,"0.0.0.0",sizeof(user->ip));
	dynncpy(user->host,host,sizeof(user->host));
	dynncpy(user->vident,ident,sizeof(user->vident));
	dynncpy(user->vhost,host,sizeof(user->vhost));
	dynncpy(user->uid,nicknumbuf,sizeof(user->uid));
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_mode(Threads *thread, char *source, char *dest, time_t time_in, char *buf)
{
    if (!buf) {
        return CMD_ERROR;
    }
    
    Users *ud = NULL;
    ud = GetUser(source);

    send_cmd(thread,ud ? ud->uid : p10id, "FMODE %s %ld %s", dest, time_in, buf);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_rawmode(Threads *thread, char *source, char *dest, time_t time_in, char *buf)
{
    if (!buf) {
        return CMD_ERROR;
    }
    
    Users *ud = NULL;
    ud = GetUser(source);

    send_cmd(thread,ud ? ud->uid : p10id, "FMODE %s %ld %s", dest, time_in, buf);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_kick(Threads *thread, char *source, char *chan, char *user, char *buf)
{
    Users *ud = NULL;
    ud = GetUser(source);
    
    Users *taru = NULL;
    taru = GetUser(user);
    
    if (!taru) {
        return CMD_ERROR;
    }

    if (buf) {
        send_cmd(thread,ud ? ud->uid : p10id, "KICK %s %s :%s", chan, taru ? taru->uid : user, buf);
    } else {
        send_cmd(thread,ud ? ud->uid : p10id, "KICK %s %s :%s", chan, taru ? taru->uid : user, user);
    }
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_notice(Threads *thread, char *source, char *dest, char *buf)
{
    if (!buf || !dest) {
        return CMD_ERROR;
    }

    Users *u = NULL;
    u = GetUser(source);

    Users *ud = NULL;
    ud = GetUser(dest);
    
    if(ud)
    	send_cmd(thread,(u ? u->uid : p10id), "NOTICE %s :%s",  ud->uid, buf);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_privmsg(Threads *thread, char *source, char *dest, char *buf)
{
    if (!buf || !dest) {
        return CMD_ERROR;
    }

    Users *u = NULL;
    u = GetUser(source);
    
    Users *ud = NULL;
    ud = GetUser(dest);
    
    if(ud)
    	send_cmd(thread,(u ? u->uid : p10id), "PRIVMSG %s :%s", ud->uid, buf);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_serv_notice(Threads *thread, char *source, char *dest, char *msg)
{
    if (!msg || !dest) {
        return CMD_ERROR;
    }
    
    Users *u = NULL;
    u = GetUser(source);
    
    send_cmd(thread,(u ? u->uid : p10id), "NOTICE $%s :%s", dest, msg);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_serv_privmsg(Threads *thread, char *source, char *dest, char *msg)
{
    if (!msg || !dest) {
        return CMD_ERROR;
    }
    
    Users *u = NULL;
    u = GetUser(source);
    
    send_cmd(thread,(u ? u->uid : p10id), "PRIVMSG $%s :%s", dest, msg);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_quit(Threads *thread, char *source, char *buf)
{
    Users *u = NULL;
    u = GetUser(source);
    
    if (buf) {
        send_cmd(thread,u ? u->uid : source, "QUIT :%s", buf);
    } else {
        send_cmd(thread,u ? u->uid : source, "QUIT :Exiting");
    }
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_protoctl(Threads *thread)
{
    return CMD_NOT_SUPPORTED;
}

static char currentpass[1024];

//
PROTOCOL_BOOL orion_cmd_pass(Threads *thread, char *pass)
{
    strncpy(currentpass,pass,MAX_PASSWORD_LEN);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_server(Threads *thread, char *servname, int hop, char *descript)
{
    char newid[4];
    Servers *s;

    if (p10inuse == 1) {
		int num = stoi(m->Numeric)+p10servcnt;
		inttobase64(newid, num, 3);
		
		s = findserver(thread, servlist, servname);
		if(s)
			dynncpy(s->sid,newid,3);
      
		p10servcnt++;
    }

	send_cmd(thread, ((p10inuse == 1) ? p10id : NULL), "SERVER %s %s %d %s :%s", servname, p10inuse ? "*" : currentpass, 0, ((p10inuse == 1) ? newid : p10id), descript);

    if (p10inuse == 1) {
      send_cmd(thread, newid, "ENDBURST");
    }

    p10inuse = 1;
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_pong(Threads *thread, char *servname, char *who)
{
    send_cmd(thread,p10id, "PONG %s", servname);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_ping(Threads *thread, char *servname, char *who)
{
    send_cmd(thread,p10id, "PING %s", who);
    return CMD_SUPPORTED;
}

/* JOIN */
/* Althought inspircd 3.2 does not need the timestamp others do so
   we get it in the common function call */
PROTOCOL_BOOL orion_cmd_services_join(Threads *thread, char *user, char *channel, time_t chantime, int fake)
{
    Users *ud;
	char output[BUFSIZE];
	bzero(&output,sizeof(output));
	
	/* Updates the Threads Channel Management */
    ud = GetUser(user);
	UpdateChannelInformation3(thread,channel,NULL);

	if(IRC.HasOwner) 			straddnc(output,IRC.SjoinOwnerPrefix,sizeof(output));
	if(IRC.HasProtect)			straddnc(output,IRC.SjoinProtectPrefix,sizeof(output));
								straddnc(output,IRC.SjoinOpPrefix,sizeof(output));
	if(IRC.SjoinHasIgnoreChar)	straddnc(output,IRC.SjoinIgnoredChar,sizeof(output));
								straddns(output,user,sizeof(output));
							
	//printf("orion_cmd_services_join (channel = %s),(date = %lu),(output = %s)\n",channel,thread->c.chan.datecreate ? thread->c.chan.datecreate : chantime,output);	
    sjoin(thread,channel,thread->c.chan.datecreate ? thread->c.chan.datecreate : chantime,output,fake);

    send_cmd(thread, p10id, "FJOIN %s %ld + :oaq,%s", channel,
                thread->c.chan.datecreate ? thread->c.chan.datecreate : chantime, (ud ? ud->uid : user));
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_join(Threads *thread, char *user, char *channel, time_t chantime)
{
	if (!user || !channel) 
        return CMD_ERROR;
		
	Users *u = NULL;
	u = GetUser(user);
		
	if(u)
    	send_cmd(thread,u ? u->uid : user, "JOIN %s %ld", channel,chantime ? chantime : time(NULL));
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_chghost(Threads *thread, char *source, char *nick, char *vhost)
{
    if (!nick || !vhost) {
        return CMD_ERROR;
 
    }
    
	Users *u = NULL;
	u = GetUser(source);
	
	Users *ud = NULL;
	ud = GetUser(nick);
    
    if(ud){
		if(chghost_used)	send_cmd(thread,u ? u->uid : p10id,"CHGHOST %s :%s", ud->uid, vhost);
	    else				do_global(thread,NULL,"  *  ERROR: CHGHOST not loaded from InspIRCd, can't use CHGHOST");
	}
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_chgident(Threads *thread, char *source, char *nick, char *vIdent)
{
    if (!nick || !vIdent) {
        return CMD_ERROR;
    }
    
	Users *u = NULL;
	u = GetUser(source);
	
	Users *ud = NULL;
	ud = GetUser(nick);
    
    if(ud){
    	if(chgident_used)	send_cmd(thread,u ? u->uid : p10id, "CHGIDENT %s %s", ud->uid, vIdent);
    	else				do_global(thread,NULL,"  *  ERROR: CHGIDENT not loaded from InspIRCd, can't use CHGIDENT");
	}
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_invite(Threads *thread, char *source, char *chan, char *nick)
{
    if (!source || !chan || !nick) {
        return CMD_ERROR;
    }
    
	Users *u = NULL;
	u = GetUser(source);
	
	Users *ud = NULL;
	ud = GetUser(nick);
    
    if(ud)
    	send_cmd(thread,u ? u->uid : p10id, "INVITE %s %s", ud->uid, chan);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_part(Threads *thread, char *nick, char *chan, char *buf)
{
    if (!nick || !chan) {
        return CMD_ERROR;
    }

	Users *u = NULL;
	u = GetUser(nick);

	if (buf) {
	    send_cmd(thread,u ? u->uid : nick, "PART %s :%s", chan, buf);
	} else {
	    send_cmd(thread,u ? u->uid : nick, "PART %s :Leaving", chan);
	}
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_global(Threads *thread, char *source, char *buf)
{
    if (!buf) {
        return CMD_ERROR;
    }

	char *sender = NULL;
	Users *u = NULL;
	Servers *s = NULL;
	u = GetUser(source);
	if(!u){
		s = findserver(thread, servlist, source);
		if(s)
			sender = s->sid;
		else
			sender = p10id;
	}
	else
		sender = u->uid;

    if(globops_used)	send_cmd(thread,sender, "GLOBOPS :%s", buf);
    else				send_cmd(thread,sender, "WALLOPS :%s", buf);
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_wallops(Threads *thread, char *source, char *fmt)
{
	char *sender = NULL;
	Users *u = NULL;
	Servers *s = NULL;
	u = GetUser(source);
	if(!u){
		s = findserver(thread, servlist, source);
		if(s)
			sender = s->sid;
		else
			sender = p10id;
	}
	else
		sender = u->uid;

    send_cmd(thread,sender, "WALLOPS :%s", fmt);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_squit(Threads *thread, char *servname, char *message)
{
    if (!servname || !message) {
        return CMD_ERROR;
    }
    
    send_cmd(thread,m->servername, "SQUIT %s :%s", servname, message);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_svso(Threads *thread, char *source, char *nick, char *flag)
{
    return CMD_NOT_SUPPORTED;
}

/* NICK <newnick>  */
PROTOCOL_BOOL orion_cmd_chg_nick(Threads *thread, char *oldnick, char *newnick)
{
    if (!oldnick || !newnick) {
        return CMD_ERROR;
    }
    
    Users *ouser = GetUser(oldnick);
	if(ouser)
    	send_cmd(thread,ouser->uid, "NICK %s %ld", newnick, ouser->time);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_svsnick(Threads *thread, char *source, char *oldnick, char *newnick, time_t when)
{
    if (!oldnick || !newnick) {
        return CMD_ERROR;
    }
    
    Users *ouser = GetUser(oldnick);
    
    if(ouser){
		ouser->time = when;
    	send_cmd(thread,p10id, "SVSNICK %s %s %ld", ouser->uid, newnick, when);
	}
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_svshold(Threads *thread, char *nick)
{
    /* Not supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_release_svshold(Threads *thread, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_unsgline(Threads *thread, char *mask)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_sgline(Threads *thread, char *mask, char *reason)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_unban(Threads *thread, char *name, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_svsmode_chan(Threads *thread, char *name, char *mode, char *nick)
{
    /* Not Supported by this IRCD */
    return CMD_NOT_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_svsjoin(Threads *thread, char *source, char *nick, char *chan, time_t date)
{
    Users *sender = GetUser(source);
    Users *target = GetUser(nick);
    
    if(sender && target){
	    if(sajoin_used) orion_cmd_sajoin(thread,sender->uid, target->uid, chan, date);
	    else			send_cmd(thread,sender->uid ,"SVSJOIN %s %s", target->uid, chan);
	}
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_svspart(Threads *thread, char *source, char *nick, char *chan)
{
    Users *sender = GetUser(source);
    Users *target = GetUser(nick);
    
    if(sender && target){
	    if(sapart_used) orion_cmd_sapart(thread,sender->nick, target->nick, chan);
	    else			send_cmd(thread,sender->uid ,"SVSPART %s %s", target->uid, chan);
	}
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_sajoin(Threads *thread, char *source, char *nick, char *chan, time_t date)
{
    Users *sender = GetUser(source);
    Users *target = GetUser(nick);
    
    if(sender && target)
    	send_cmd(thread,sender->uid ,"SAJOIN %s %s", target->uid, chan);
    	
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_sapart(Threads *thread, char *source, char *nick, char *chan)
{
    Users *sender = GetUser(source);
    Users *target = GetUser(nick);
    
    if(sender && target)
      	send_cmd(thread,sender->uid,"SAPART %s %s", target->uid, chan);
      	
    return CMD_SUPPORTED;
}
//
PROTOCOL_BOOL orion_cmd_swhois(Threads *thread, char *source, char *who, char *mask)
{
    return CMD_NOT_SUPPORTED;	
}

//
PROTOCOL_BOOL orion_cmd_eob(Threads *thread)
{
    send_cmd(thread,p10id, "ENDBURST");
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_jupe(Threads *thread, char *jserver, char *who, char *reason)
{
    char rbuf[256];

    snprintf(rbuf, sizeof(rbuf), "Juped by %s%s%s", who,
             reason ? ": " : "", reason ? reason : "");

    orion_cmd_squit(thread, jserver, rbuf);
    orion_cmd_server(thread, jserver, 0, rbuf);
    new_server(thread, me_server, jserver, rbuf, SERVER_JUPED, NULL);
    return CMD_SUPPORTED;
}

//
PROTOCOL_BOOL orion_cmd_global_legacy(Threads *thread, char *source, char *fmt)
{
	char *sender = NULL;
	Users *u = NULL;
	Servers *s = NULL;
	u = GetUser(source);
	if(!u){
		s = findserver(thread, servlist, source);
		if(s)
			sender = s->sid;
		else
			sender = p10id;
	}
	else
		sender = u->uid;

    send_cmd(thread,sender, "GLOBOPS :%s", fmt);
    return CMD_SUPPORTED;
}

/*  ************************************************************************************************  */
/**
 * Here are all functions which tell us and all modules that a serverlink occured
 * And it tells us if the server is currently linking or finished linking (important for sockscontrol)
 **/

/* This adds Servers to the server tree */
MOD_BOOL orion_event_server(Threads *thread, size_t ac, char **av)
{
	//> :748 BURST 1213637519
	if(ac < 4)
		return MOD_CONT;

	if(digitcol(av[3])){
		//> SERVER glue.de.eu.deepirc.net Qserver 0 748 :Waddle World
		
		/* set our uplink */
		addServer(thread, NULL, av[1], av[3], &_dstrjoin(' ',av,5)[1], av[4]);
		
		/* Save our uplink */
		if(uplink[0]=='\0')
			dynncpy(uplink,av[1],sizeof(uplink));
		
		//printf("WE ADDED UPLINK = %s (server), %s (sid)\n",uplink,av[4]);
		/* Inform all modules that we are linking to our uplink */
		on_link(thread,m->servername,av[1]);
	}
	else{
		//:<local server id> SERVER <remote server> * <distance> <id> :<description>
		
		/* Save our uplink */
		if(uplink[0]=='\0')
			dynncpy(uplink,av[2],sizeof(uplink));
		
		/* Introduce that server to the servertree */
		addServer(thread, findserver_uid(thread, servlist, &av[0][1])->server, av[2], av[4], &_dstrjoin(' ',av,6)[1], av[5]);
	}
    return MOD_CONT;
}

/* Called from serviceslink(); */
PROTOCOL_BOOL orion_cmd_initialize(Threads *thread){
	/* No Init */
    return CMD_NOT_SUPPORTED;
}

/* Called when a server finishes bursting */
MOD_BOOL orion_event_eob(Threads *thread, size_t ac, char **av)
{
	MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN); // Flush our Cache
	
	/* If EOB came from our uplink */
    if (streq(&av[0][1],serv_uplink->sid)){
        /* We tell all modules that the serviceslink was done (reactivating sockscontrol) */
        orion_event_services_link_done(thread,ac,av);
        
	}
	/* If it wasn't our uplink who finished bursting, we finish "linkserver" and mark it's link as "done" */
	else
		/* Tell all modules that "linkserver" finished linking and bursting */
		on_linkdone(thread);
    	
    return MOD_CONT;
}

/* Called from orion_event_started(); */
PROTOCOL_BOOL orion_cmd_connect(Threads *thread, bool how)
{
	/* We send password and server introducing to our uplink */
	orion_cmd_pass(thread,m->linkpass);
	orion_cmd_server(thread, m->servername, 0, m->description);
	
	if(how == true){
		/* We start bursting here */
		main_raw(thread,"BURST %ld",time(NULL));
	
		/* We tell all inspircd servers our Version */
		send_cmd(thread,m->servername, "VERSION :[%s - Project: %s] version %s\n", MAINPROJECT,PROJECTNAME,VERSION);
		
		/* We do the primary connect, means connecting all services clients and BotServ Bots */
		_ExecuteEvent(NULL,NULL,"on_connect_primary");
	}
    return CMD_SUPPORTED;
}

/* ON CAPAB - we aren't connected now (we are, but we don't know about it yet) */
/* Here we are able to look if inspircd has loaded all modules we need */
MOD_BOOL orion_event_started(Threads *thread, size_t ac, char **av)
{
	if(ac < 2)
		return MOD_CONT;

	/* Search for modules we need to be loaded by inspircd */
	if(streqnc(av[1],"modules")){
		char infobuffer[6000];
		char *module[50];
		size_t 	i = 0;
		int 	a = 0;
		bzero(&infobuffer,sizeof(infobuffer));
		bzero(module,sizeof(module));
		
		dynncpy(infobuffer,av[2],6000);
		i = strsplit(infobuffer,module,',');
		for(a=0;a<i;a++){
			if(streqnc(module[a],"m_services_account.so"))	services_used = 1;
			else if(streqnc(module[a],"m_chanprotect.so"))	chanprotect_used = 1;
			
			else if(streqnc(module[a],"m_chghost.so"))		chghost_used = 1;
			else if(streqnc(module[a],"m_chgident.so"))		chgident_used = 1;
			else if(streqnc(module[a],"m_chgname.so"))		chgname_used = 1;
			else if(streqnc(module[a],"m_globops.so"))		globops_used = 1;
			
			else if(streqnc(module[a],"m_sajoin.so"))		sajoin_used = 1;
			else if(streqnc(module[a],"m_sanick.so"))		sanick_used = 1;
			else if(streqnc(module[a],"m_sapart.so"))		sapart_used = 1;
		}
	}

	/* We check if we are able to run with inspircd's loaded modules */
	else if(streqnc(av[1],"end")){
		if(services_used == 0){
			dlog(thread,0,"  *  ERROR: InspIRCd does not use \"m_services_account.so\" - correct that!\n");
			orion_exit(thread);
		}
		if(chanprotect_used == 0){
			dlog(thread,0,"  *  ERROR: InspIRCd does not use \"m_chanprotect.so\" - correct that!\n");
			orion_exit(thread);
		}
		
		if(chghost_used == 0 || chgident_used == 0){
			IRC.VHost = 0;
			//unload_modules(thread,"hs");
			dlog(thread,0,"  *  ERROR: InspIRCd does not use \"m_chgident.so\" or \"m_chghost.so\" - (HostServ cannot be used)\n");
		}
		
		/* We start connecting the server, and execute the event that we are linking */
		orion_cmd_connect(thread,1);
		
		/* We tell all modules that WE are linking now */
		on_serviceslink(thread);
	}
	return MOD_CONT;
}

/* Called by receiving "ENDBURST" (from orion_event_eob();) - this EOB comes from our Uplink, so we should connect all bots and join our channels */
MOD_BOOL orion_event_services_link_done(Threads *thread, size_t ac, char **av)
{
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_SJOIN); // Flush our Cache
	
	/* We add the services server to the servers tree (we do that when we finished linking) */
    me_server = addServer(thread,serv_uplink->server,m->servername,0,m->description,p10id);
	
	/* Let the Bots join our channels */
	_ExecuteEvent(NULL,NULL,"on_connect_secondary");

	/* We report that our linking is done now */	
	on_serviceslink_done(thread);	/* This means that linking !FOR US! is done */							//- reactivates Sockscontrol
	on_linkdone(thread);			/* This is executed whenever a serverlink completed successfully */		//- reactivates Sockscontrol

	/* say we finished bursting */
	orion_cmd_eob(thread);
	return MOD_CONT;
}

/* Just PONG on PING */
MOD_BOOL orion_event_ping(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
        
    orion_cmd_pong(thread, p10id, &av[0][1]);
    return MOD_CONT;
}

/* This tells us when the linked server finished linking - it should immediately reply for our PING then with a PONG */
MOD_BOOL orion_event_pong(Threads *thread, size_t ac, char **av)
{
    if (ac < 3)
        return MOD_CONT;
        
    Servers *s = NULL;
    
    /* Here we get re repling servers ass */
    s = findserver(thread, servlist, &av[0][1]);
    if(!s)
    	s = findserver_uid(thread, servlist, &av[0][1]);
    	
    /* If the server replied is the one that linked, we finish it's link */
    //-> sockscontrol reactivated
    if(streqnc(s->server,linkserver)){
		on_linkdone(thread);
	}
    return MOD_CONT;
}

/*  ************************************************************************************************  */

#define START_OF_RAW 4
/**
 * @brief raw:  To process outgoing raw messages
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param string The outgoing Raw-message
 * @param len The length of the created Raw-message
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL orion_cmd_raw(Threads *thread, char *param[]){
	PROTOCOL_BOOL analyzer;

	//:M PRIVMSG #operserv: raw :operserv svsmode M +SjFzq
	//:operserv SVSKILL M :lalala
	//:M MODE  +a
	//Tue Oct 19 22:47:14 2004 > :OperServ GLOBOPS :M used RAW command for svsmode
	//Tue Oct 19 22:47:14 2004 > :Global PRIVMSG #services :M used RAW command for :operserv

	int paramvalue = 0;

	/* SVSMODE VIA RAW */
		//:operserv SVSMODE shock +d 1
		//operserv SVSMODE shock +d 1
	if(streqnc(param[START_OF_RAW+1],"svsmode"))       analyzer = do_svsmode(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
	else if(streqnc(param[START_OF_RAW+0],"svsmode"))  analyzer = do_svsmode(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVS2MODE VIA RAW */
	//
	else if(streqnc(param[START_OF_RAW+1],"svs2mode")) analyzer = do_svs2mode(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
	else if(streqnc(param[START_OF_RAW+0],"svs2mode")) analyzer = do_svs2mode(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVSKILL VIA RAW */
		// > :shock PRIVMSG O :raw :operserv SVSKILL shock :lalala
		// < :CServe.deepirc.net SVSKILL  ::lalala
		// > :shock PRIVMSG ircdos :raw :sockscontrol SVSKILL shock :hey alter was geht bei dir
	else if(streqnc(param[START_OF_RAW+1],"svskill")){
		char *reason = _dstrjoin(' ',param,START_OF_RAW+2);
		analyzer = do_svskill(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],reason[0]==':' ? &reason[1] : reason);
	}

		// > :shock PRIVMSG O :raw SVSKILL shock :lalala
	else if(streqnc(param[START_OF_RAW+0],"svskill")){
		char *reason = _dstrjoin(' ',param,START_OF_RAW+2);
		analyzer = do_svskill(thread,m->servername,param[START_OF_RAW+1],reason[0]==':' ? &reason[1] : reason);
	}

	/* SVSNICK VIA RAW */
		// > :shock PRIVMSG O :raw :operserv SVSNICK shock shockzZ
	else if(streqnc(param[START_OF_RAW+1],"svsnick")) analyzer = do_svsnick(thread,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3]);
		// > :shock PRIVMSG O :raw SVSNICK shock shockzZ
	else if(streqnc(param[START_OF_RAW+0],"svsnick")) analyzer = do_svsnick(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);

	/* SVSJOIN VIA RAW */
		//:operserv svsjoin Nick #chan
	else if(streqnc(param[START_OF_RAW+0],"svsjoin")) analyzer = do_svsjoin(thread,0,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2],time(NULL));
	else if(streqnc(param[START_OF_RAW+1],"svsjoin")) analyzer = do_svsjoin(thread,0,&param[START_OF_RAW+0][1],param[START_OF_RAW+2],param[START_OF_RAW+3],time(NULL));

	/* SVSPART VIA RAW */
	else if(streqnc(param[START_OF_RAW+0],"svspart")) analyzer = do_svspart(thread,m->servername,param[START_OF_RAW+1],param[START_OF_RAW+2]);
	else if(streqnc(param[START_OF_RAW+1],"svspart")) analyzer = do_svspart(thread,&param[START_OF_RAW+0][1],param[2],param[3]);

	/* NORMAL IRC (RFC) RAW */
	else{
		while(param[paramvalue])
			paramvalue++;

		if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_core_condata` WHERE `nick`='%s'",mysql_quote(thread,&param[START_OF_RAW+0][1])))>0){
				on_protocol(thread,&IRC,paramvalue-START_OF_RAW,param);
				main_raw(thread,_djoin(' ',param));
				return CMD_EXECUTED;
		}
		else
			return CMD_ERROR;
	}
	return analyzer;
}

/*  ************************************************************************************************  */

/* Copied from Anope */
bool orion_valid_nick(Threads *thread, char *nick)
{
    return 1;
}

bool orion_valid_chan(Threads *thread, char *chan){
    if (strchr(chan, ':')) {
      return false;
    }
    return true;
}

bool orion_flood_mode_check(Threads *thread, char *value)
{
    char *dp, *end;

    if (value && *value != ':'
        && (strtoul((*value == '*' ? value + 1 : value), &dp, 10) > 0)
        && (*dp == ':') && (*(++dp) != 0) && (strtoul(dp, &end, 10) > 0)
        && (*end == 0)) {
        return true;
    } else {
        return false;
    }
}
