/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 207                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:32:05 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file quit.c
 *   @brief This is the Module file for QUIT (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the QUIT command.
 *
 *   The QUIT File is used to process QUITS/SQUITS sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Quit is used to handle Quit's and SQUIT's, those executed by us and by others as well.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief do_quit:  Sends out a QUIT (depending on the Loaded Protocol) - using QUIT
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The ServicesNick which should QUIT from the Network
 * @param message The QuitMessage which we want to provide to the public
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_quit(Threads *thread, char *snick, char *message, ...){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	
	/* Let'z check out inputs - are they valid? */
	if(snick==NULL || message==NULL)
		return CMD_ERROR;
		
	if(GetUser(snick)==NULL)
		return CMD_ERROR;
		
	/* Deklarate Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);

	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_quit(thread,snick,tmp);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;

	/* Command supported .. we start analyzing it */
	quit(thread,snick);
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief on_quit:  Receives a QUIT (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that sent us a QUIT
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_quit(Threads *thread, char *nick){
	//printf("if in on_quit();\n");
	if(nick == NULL)
		return CMD_ERROR;
		
	if(GetUser(nick)==NULL)
		return CMD_ERROR;
			
	quit(thread,nick);
	//printf("ja doie bersteht man das ja noch leiwand ^^\n");
	/************ SEND on_quit to the eventhandler ***********/
	_ExecuteEvent(nick,NULL,"on_quit");
	/*********************************************************/
	//printf("passt beim event auzt die gosch gflogen\n");
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief quit:  Processes the X-QUIT-Statements for deleting Information from SQL (NO OUTPUT!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that sent us a QUIT (The Nick we will delete from the Database)
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL quit(Threads *thread, char *nick){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	
	/* Global variables */
	deepSQL channels;
	char chan_update[(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	char chan_delete[(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	char chan_bot   [(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	int i=0;

	/* Initialise Variables to (NULL) */
	bzero(&channels,sizeof(channels));
	bzero(&chan_bot,sizeof(chan_bot));
	bzero(&chan_update,sizeof(chan_update));
	bzero(&chan_delete,sizeof(chan_delete));
	
	//printf("int quit(); nach bzero und vor domysql();\n");
	domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='0', `last_used`='%ld' WHERE `nick`='%s' AND `linked`='0'",time(NULL),mysql_quote(thread,nick));	
	domysql(thread,"DELETE `orion_core_condata`,`orion_chan_users`,`orion_ns_aliases` FROM `orion_core_condata` LEFT OUTER JOIN `orion_chan_users` ON `orion_chan_users`.`nick`=`orion_core_condata`.`nick` LEFT OUTER JOIN `orion_ns_aliases` ON (`orion_ns_aliases`.`nick`=`orion_core_condata`.`nick` AND `orion_ns_aliases`.`linked`=1) WHERE `orion_core_condata`.`nick`='%s'",mysql_quote(thread,nick));
	/** CASCADE (MySQL - InnoDB) does the other statements, because of referencing - chanusers, chanop, ... **/
	//printf("vor DelUserNick();\n");
	DelUserNick(nick);
	//printf("nach DelUserNick();\n");
	
	/* 	Update our Cache	*/
#ifndef USE_THREADS
	if(streqnc(thread->c.Cdata.nick,nick)){
		bzero(&thread->c.Cdata,sizeof(thread->c.Cdata));
		thread->c.aliases.active = 0;
		
		unknown_access(thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
	}else if(streqnc(thread->sql.Cdata.nick,nick)){
		bzero(&thread->sql.Cdata,sizeof(thread->sql.Cdata));
		thread->sql.aliases.active = 0;
		
		unknown_access(thread->c.chan.access.ownlevel,sizeof(thread->c.chan.access.ownlevel));
	}
	//printf("thread cache umgeschrieben\n");
#endif
	/**			***			**/
	
	//printf("vor doSelectMySQL();\n");
	channels.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`serv`,`orion_cs_core`.`channel`,`orion_cs_core`.`botnick`, SUM(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`) as `botinchan`, COUNT(`orion_chan_users`.`nick`) as `count_users`,`orion_cs_core`.`settings` FROM `orion_cs_core` LEFT JOIN `orion_chan_users` ON (`orion_cs_core`.`channel` = `orion_chan_users`.`channel`) WHERE `orion_cs_core`.`datecreate`!='' GROUP BY `orion_cs_core`.`channel`");
	channels.fields = mysql_num_rows(channels.res);
	//printf("vor for();\n");
	for(i=0;i<channels.fields;i++){
		channels.row = mysql_fetch_row(channels.res);
		int settings = stoi(channels.row[5]);

		if(
			(
				( ~settings&CS_INHABIT_BIT && stoi(channels.row[4])==1 && stoi(channels.row[3])==1 ) ||
				( settings&CS_INHABIT_BIT && stoi(channels.row[4])==2 && stoi(channels.row[3])==1)
			) ||
			stoi(channels.row[4])==0
		){
			if(streqnc(channels.row[1],thread->c.chan.name))
				thread->c.chan.countusers--;
			
			if(channels.row[0][0]!='\0'){	// channel empty, but registered by a bot (Q/L/ChanServ/etc...)
			
				straddnc(chan_update,toascii(39),sizeof(chan_update));
				straddns(chan_update,mysql_quote(thread,channels.row[1]),sizeof(chan_update));
				straddnc(chan_update,toascii(39),sizeof(chan_update));
				straddnc(chan_update,',',sizeof(chan_update));
			
				if(channels.row[2]!=NULL && stoi(channels.row[3])==1 && bs->loaded){
					analyzer = orion_WRCtranslate_cmd_part(thread,channels.row[2],channels.row[1],NULL);
					
					straddnc(chan_bot,toascii(39),sizeof(chan_bot));
					straddns(chan_bot,mysql_quote(thread,channels.row[1]),sizeof(chan_bot));
					straddnc(chan_bot,toascii(39),sizeof(chan_bot));
					straddnc(chan_bot,',',sizeof(chan_bot));
					
				}
	
			}else{
				straddnc(chan_delete,toascii(39),sizeof(chan_delete));
				straddns(chan_delete,mysql_quote(thread,channels.row[1]),sizeof(chan_delete));
				straddnc(chan_delete,toascii(39),sizeof(chan_delete));
				straddnc(chan_delete,',',sizeof(chan_delete));
			}
		}
	}
	
	/** Update MySQL (Update channels which are empty & registered, Delete all bans (cauze channel is empty), 
		Delete and the Bot (it was parted in the loop), Delete unregistered Channels that are empty **/
		
	//printf("Chan Update = %s\n",chan_update);
	//printf("Chan delete = %s\n",chan_delete);
	//printf("Chan Bot = %s\n",chan_bot);
		
	if(chan_update[0]!='\0'){
		chan_update[dynlen(chan_update)-1]='\0';
		
		domysql(thread,"UPDATE `orion_cs_core` SET "
			"`datecreate`='',`mode`='',`link`='',`limit`='',`flood`='',`key`=''"
			" WHERE `channel` IN(%s)",chan_update);
		
		domysql(thread,"DELETE `orion_chan_bans`,`orion_chan_expbans` FROM `orion_chan_bans` LEFT OUTER JOIN `orion_chan_expbans` ON `orion_chan_expbans`.`channel`=`orion_chan_bans`.`channel` WHERE `orion_chan_bans`.`channel` IN(%s)",chan_update);
	}
	
	if(chan_delete[0]!='\0'){
		chan_delete[dynlen(chan_delete)-1]='\0';
		domysql(thread,"DELETE FROM `orion_cs_core` WHERE `channel`='%s'",chan_delete);
	}
	
	if(chan_bot[0]!='\0'){
		chan_bot[dynlen(chan_bot)-1]='\0';
		domysql(thread,"DELETE `orion_chan_users` FROM `orion_chan_users` LEFT OUTER JOIN `orion_cs_core` ON (`orion_cs_core`.`botnick`=`orion_chan_users`.`nick` AND `orion_cs_core`.`channel`=`orion_chan_users`.`channel`) WHERE `orion_cs_core`.`botnick` IS NOT NULL AND `orion_chan_users`.`channel` IN(%s)",chan_bot);
	}
	//printf("vor mysql_free_result_myr();\n");
	mysql_free_result_myr(channels.res); //free result

	/* 
	InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
	NOTE: mode system changed, old tables not neccessary anylonger...
	*/
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
