/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 204                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-03 20:33:44 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file services.c
 *   @brief This is the Module file contains miscellous functions.
 *
 *   It contains all functions which are used
 *   anywhere in the project, to manage single users/channels
 *   or the whole Network. (deopall --> defcon)
 */
/*  ************************************************************************************************  */

#include "sysconf.h"	//ifdef USE_THREADS && THREAD_SAFE
#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

#include <time.h>

char encoding[] =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"; //62

char figures[] =
	"0123456789"; //10

/**
 * @brief check_expire:  Transforms a time string "1M2d1y" to an integer value + time(NULL)!
 *
 * @param expirestring The timestring we want to check
 * @return int 1 if it's a valid string and 0 if not
 **/
bool check_expire(char *expirestring){
	size_t i = 0;
	int b = 0;
	int awaitingchar = 0;
	
	if(dynlen(expirestring)<=1)
		return 0;
		
	if(isdigit(expirestring[0])>0){
		while(expirestring[i]!='\0'){
			//printf("i = %d; expirestring[i] = %c; isdigit(expirestring[i]) = %d\n",i,expirestring[i],isdigit(expirestring[i]));
			if(strcin(expirestring[i],"yMwdhms") && b!=i){
				b=i+1;
				awaitingchar = 0;
			}
			else if(isdigit(expirestring[i])>0){
				awaitingchar = 1;
			}
			else
				return 0;
			i++;
		}
		if(awaitingchar)
			return 0;
		else
			return 1;
	}
	return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief expire:  Transforms a time string "1M2d1y" to an integer value + time(NULL)!
 *
 * @param time_in The timestring to transform
 * @return long seconds The time in seconds (time(NULL) + transformed timestring)
 **/
time_t expire(char *time_in){
	size_t length=strlen(time_in);
	char expire[30][100];
	char revstring[300];
	time_t seconds	=	0;
	int count		=	0;
	int i			=	0;
	
	if(time_in==NULL) return 0;

	bzero(&expire[count],sizeof(expire[count]));
	
	for(i=0;i<length;i++){
		if(strcin(time_in[i],"yMwdhms")){
			straddnc(expire[count],time_in[i],sizeof(expire[count]));
			count++;
			bzero(&expire[count],sizeof(expire[count]));
		}else{
			if(isdigit(time_in[i])>0){
				straddnc(expire[count],time_in[i],sizeof(expire[count]));
			}
		}
	}
	
	for(i=0;i<count;i++){
		//expire[i] == value
		bzero(&revstring,sizeof(revstring));
		reverse(revstring,expire[i]);

		if(revstring[0]=='y'){
			substr(revstring,0);
			reverse(expire[i],revstring);
			seconds+=(time_t)stol(expire[i])*365*24*60*60;
		}
		if(revstring[0]=='M'){
			substr(revstring,0);
			reverse(expire[i],revstring);
			seconds+=(time_t)stol(expire[i])*30*24*60*60;
		}
		if(revstring[0]=='w'){
			substr(revstring,0);
			reverse(expire[i],revstring);
			seconds+=(time_t)stol(expire[i])*7*24*60*60;
		}
		if(revstring[0]=='d'){
			substr(revstring,0);
			reverse(expire[i],revstring);
			seconds+=(time_t)stol(expire[i])*24*60*60;
		}
		if(revstring[0]=='h'){
			substr(revstring,0);
			reverse(expire[i],revstring);
			seconds+=(time_t)stol(expire[i])*60*60;
		}
		if(revstring[0]=='m'){
			substr(revstring,0);
			reverse(expire[i],revstring);
			seconds+=(time_t)stol(expire[i])*60;
		}
		if(revstring[0]=='s'){
			reverse(expire[i],revstring);
			seconds+=(time_t)stol(expire[i]);
		}
		
	}
	return seconds;
}

/*  ************************************************************************************************  */

/**
 * @brief randomstring:  Creates a random string
 *
 * @param array The buffer for the random chars
 * @param stellen The number of chars to randomly create
 * @return void
 **/
void randomstring(char *array, size_t stellen){
	size_t i=0;
	while(i<stellen){
		int a=rand()%sizeof(encoding);
		array[i]=encoding[a];
		i++;
	}
	array[i++]='\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief randomnumber:  Creates a random number
 *
 * @param array The buffer for the random chars
 * @param stellen The number of figures to randomly create
 * @return void
 **/
void randomnumber(char *array, size_t stellen){
	size_t i=0;

	while(i<stellen){
		array[i]=figures[rand()%10];
		i++;
	}
	array[i++]='\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief get_localtime:  Transforms the current local time to a time-string
 *
 * @param time_in The time in seconds since 1.1.1970 (UNIX-timestamp)
 * @return char* time1 Returns a time string transformed from the current time
 **/
char *get_localtime(time_t time_in){
	static char time1[200];
	struct tm *time_get;
	const time_t timer = (const time_t)time_in;
	bzero(&time1,sizeof(time1));
	/*Jetzt struct tm-Format mit asctime() */
	time_get=localtime(&timer);
	straddns(time1,asctime(time_get),sizeof(time1));
	time1[strlen(time1)-1]='\0';
	strclean(time1);
	return time1;
}

/*  ************************************************************************************************  */

/**
 * @brief deopall:  A given service clears all ops in a given channel
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param service The service name that should be used to deop all in target channel
 * @param channel The channel for which target service should deop all users
 * @return int
 **/
bool deopall(Threads *thread, char *service, char *channel){
	/* Global variables */
	Channel c;
	deepSQL deops;

	int 	j=0, pos=0;
	int 	i=0;
	char nicks[BUFFER];
	char mode[4];

	/* Initialise Variables to (NULL) */
	bzero(&deops,sizeof(deops));
	bzero(&c,sizeof(c));
	
	c = sql_chan(thread,channel);

	if(streqnc(service,O->nick))	deops.res = doSelectMySQL(thread,"SELECT `nick` FROM `orion_chan_users` WHERE `channel`='%s' AND `modes` REGEXP '(o)'",mysql_quote(thread,channel));
	else 							deops.res = doSelectMySQL(thread,"SELECT `nick` FROM `orion_chan_users` WHERE `channel`='%s' AND `faketype`=0 AND `modes` REGEXP '(o)'",mysql_quote(thread,channel));
	deops.fields = mysql_num_rows(deops.res);

	for(j=0;j<deops.fields;j++){
		deops.row = mysql_fetch_row(deops.res);
		
		if(i==0){
			bzero(&mode,sizeof(mode));
			bzero(&nicks,sizeof(nicks));
		}
		if(i<3){
			if((!streqnc(deops.row[0],"q") && !streqnc(deops.row[0],"l")) || streqnc(service,O->nick)){
				straddnc(mode,'o',sizeof(mode));
				straddns(nicks,deops.row[0],sizeof(nicks));
				straddnc(nicks,' ',sizeof(nicks));
				i++;
			}
		}
		if(i==3 || pos == (deops.fields-1)){
			//straddnc(mode,'\0',sizeof(mode));
			do_mode(thread,service,channel,c.datecreate,_dynadd("-%s %s",mode,nicks));
			i=0;
		}
		pos++;
	}
	mysql_free_result_myr(deops.res);

	if(deops.fields>0)	return 1;
	else				return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief dehopall:  A given service clears all halfops in a given channel
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param service The service name that should be used to dehalfop all in target channel
 * @param channel The channel for which target service should dehalfop all users
 * @return bool
 **/
bool dehopall(Threads *thread, char *service, char *channel){
	/* Global variables */
	Channel c;
	deepSQL dehops;

	int 	j=0, pos=0;
	int 	i=0;
	char nicks[BUFFER];
	char mode[4];

	/* Initialise Variables to (NULL) */
	bzero(&dehops,sizeof(dehops));
	bzero(&c,sizeof(c));
	
	c = sql_chan(thread,channel);

	dehops.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_chan_users` WHERE `channel`='%s' AND `modes` REGEXP '(h)'",mysql_quote(thread,channel));
	dehops.fields = mysql_num_rows(dehops.res);

	for(j=0;j<dehops.fields;j++){
		dehops.row = mysql_fetch_row(dehops.res);
		
		if(i==0){
			bzero(&mode,sizeof(mode));
			bzero(&nicks,sizeof(nicks));
		}
		if(i<3){
			if((!streqnc(dehops.row[0],"q") && !streqnc(dehops.row[0],"l")) || streqnc(service,O->nick)){
				straddnc(mode,'h',sizeof(mode));
				straddns(nicks,dehops.row[0],sizeof(nicks));
				straddnc(nicks,' ',sizeof(nicks));
				i++;
			}
		}
		if(i==3 || pos == (dehops.fields -1)){
			//straddnc(mode,'\0',sizeof(mode));
			do_mode(thread,service,channel,c.datecreate,_dynadd("-%s %s",mode,nicks));
			i=0;
		}
		pos++;
	}
	mysql_free_result_myr(dehops.res);

	if(dehops.fields >0)	return 1;
	else					return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief devoiceall:  A given service clears all voices in a given channel
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param service The service name that should be used to devoice all in target channel
 * @param channel The channel for which target service should devoice all users
 * @return bool
 **/
bool devoiceall(Threads *thread, char *service, char *channel){
	/* Global variables */
	Channel c;
	deepSQL devoice;

	int 	j=0, pos=0;
	int 	i=0;
	char nicks[BUFFER];
	char mode[4];

	/* Initialise Variables to (NULL) */
	bzero(&devoice,sizeof(devoice));
	bzero(&c,sizeof(c));
	
	c = sql_chan(thread,channel);

	devoice.res     = doSelectMySQL(thread,"SELECT `nick` FROM `orion_chan_users` WHERE `channel`='%s' AND `modes` REGEXP '(v)'",mysql_quote(thread,channel));
	devoice.fields  = mysql_num_rows(devoice.res);

	for(j=0;j<devoice.fields;j++){
		devoice.row = mysql_fetch_row(devoice.res);
		
		if(i==0){
			bzero(&mode,sizeof(mode));
			bzero(&nicks,sizeof(nicks));
		}
		if(i<3){
			if((!streqnc(devoice.row[0],"q") && !streqnc(devoice.row[0],"l")) || streqnc(service,O->nick)){
				straddnc(mode,'v',sizeof(mode));
				straddns(nicks,devoice.row[0],sizeof(nicks));
				straddnc(nicks,' ',sizeof(nicks));
				i++;
			}
		}
		if(i==3 || pos == (devoice.fields-1)){
			//straddnc(mode,'\0',sizeof(mode));
			do_mode(thread,service,channel,c.datecreate,_dynadd("-%s %s",mode,nicks));
			i=0;
		}
		pos++;
	}
	mysql_free_result_myr(devoice.res);

	if(devoice.fields>0)	return 1;
	else					return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief unbanall:  A given service clears all bans for a given channel
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param service The service name that should be used to clear all bans for target channel
 * @param channel The channel for which target service should clear all bans
 * @return bool
 **/
bool unbanall(Threads *thread, char *service, char *channel){
	/* Global variables */
	Channel c;
	deepSQL bans;

	int 	j=0, pos=0;
	int 	i=0;
	char nicks[BUFFER];
	char mode[4];

	/* Initialise Variables to (NULL) */
	bzero(&bans,sizeof(bans));
	bzero(&c,sizeof(c));
	
	c = sql_chan(thread,channel);
			
	bans.res    = doSelectMySQL(thread,"SELECT `ban` FROM `orion_chan_bans` WHERE `channel`='%s'",mysql_quote(thread,channel));
	bans.fields = mysql_num_rows(bans.res);

	for(j=0;j<bans.fields;j++){
		bans.row = mysql_fetch_row(bans.res);
		
		if(i==0){
			bzero(&mode,sizeof(mode));
			bzero(&nicks,sizeof(nicks));
		}
		if(i<3){
			straddnc(mode,IRC.BanMode,sizeof(mode));
			straddns(nicks,bans.row[0],sizeof(nicks));
			straddnc(nicks,' ',sizeof(nicks));
			i++;
		}
		if(i==3 || pos >= (bans.fields-1)){
			//do_privmsg(Q->nick,m->channel,"MODE STRING: %s",mode);
			//do_privmsg(Q->nick,m->channel,"MODE STRING: %s",nicks);
			do_mode(thread,service,channel,c.datecreate,_dynadd("-%s %s",mode,nicks));
			i=0;
		}
		pos++;
	}
	mysql_free_result_myr(bans.res);

	if(bans.fields>0)	return 1;
	else				return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief defconlev:  Changes the current defcon level (Security-Level)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param defconlevel The defconlevel to set
 * @return void
 **/
void defconlev(Threads *thread, int defconlevel){
	/* Global variables */
	deepSQL channels;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&channels,sizeof(channels));

	if(defconlevel==5){
		if(defconlevel>defcon && (defcon==1 || defcon==2)){
			channels.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`channel`,`mode`,`datecreate` FROM `orion_cs_core` LEFT JOIN `orion_chan_users` ON (`orion_chan_users`.`channel`=`orion_cs_core`.`channel`) WHERE `orion_chan_users`.`channel` IS NOT NULL GROUP BY `orion_chan_users`.`channel`");
			channels.fields = mysql_num_rows(channels.res);
			
			for(i=0;i<channels.fields;i++){
				channels.row = mysql_fetch_row(channels.res);
				
				do_rawmode(thread,O->nick,channels.row[0],(time_t)stol(channels.row[2]),_dynadd("-%s",m->DefConChanModes));
				//printf("Channel %s - mode to set %s\n",channels.row[0],channels.row[1]);
				do_mode(thread,O->nick,channels.row[0],stol(channels.row[2]),_dynadd("+%s",channels.row[1]));
			}
			mysql_free_result_myr(channels.res);
		}/*else if(defconlevel>defcon && defcon==4)
			readonly=0;*/
		
		defcon		= 5;
		readonly	= 0;
		if(m->GlobalOnDefcon){
			if(m->DefConOffMessage[0]!='\0'){
				char message[BUFFER];
				bzero(&message,sizeof(message));
				
				snprintf(message,sizeof(message),"(Broadcast) %s",m->DefConOffMessage);
				do_server_notice(thread,message);
			}
			else
				do_server_notice(thread,"(Broadcast) server entered Defcon 5, reset to defaults.");
		}
		
        _ExecuteEvent(NULL,NULL,"socks_start");
	}else if(defconlevel==4){
		if(defconlevel>defcon && (defcon==1 || defcon==2)){

			channels.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`channel`,`mode`,`datecreate` FROM `orion_cs_core` LEFT JOIN `orion_chan_users` ON (`orion_chan_users`.`channel`=`orion_cs_core`.`channel`) WHERE `orion_chan_users`.`channel` IS NOT NULL GROUP BY `orion_chan_users`.`channel`");
			channels.fields = mysql_num_rows(channels.res);
			
			for(i=0;i<channels.fields;i++){
				channels.row = mysql_fetch_row(channels.res);
				
				do_rawmode(thread,O->nick,channels.row[0],stol(channels.row[2]),_dynadd("-%s",m->DefConChanModes));
				do_mode(thread,O->nick,channels.row[0],stol(channels.row[2]),_dynadd("+%s",channels.row[1]));
			}
			mysql_free_result_myr(channels.res);
		}
		defcon		= 4;
		readonly	= 1;
 		if(m->GlobalOnDefcon)
			do_server_notice(thread,"(Broadcast) server entered Defcon 4, services entered read-only mode.");

        _ExecuteEvent(NULL,NULL,"socks_start");
	}else if(defconlevel==3){
		if(defconlevel>defcon && (defcon==1 || defcon==2)){
			channels.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`channel`,`mode`,`datecreate` FROM `orion_cs_core` LEFT JOIN `orion_chan_users` ON (`orion_chan_users`.`channel`=`orion_cs_core`.`channel`) WHERE `orion_chan_users`.`channel` IS NOT NULL GROUP BY `orion_chan_users`.`channel`");
			channels.fields = mysql_num_rows(channels.res);
			
			for(i=0;i<channels.fields;i++){
				channels.row = mysql_fetch_row(channels.res);
				
				do_rawmode(thread,O->nick,channels.row[0],stol(channels.row[2]),_dynadd("-%s",m->DefConChanModes));
				do_mode(thread,O->nick,channels.row[0],stol(channels.row[2]),_dynadd("+%s",channels.row[1]));
			}
			mysql_free_result_myr(channels.res);
		}
		defcon		= 3;
		readonly	= 1;
		if(m->GlobalOnDefcon)
			do_server_notice(thread,"(Broadcast) server entered Defcon 3, services ignore usercommands.");
			
        _ExecuteEvent(NULL,NULL,"socks_start");
	}else if(defconlevel==2){
		if(defcon<2){
			channels.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`channel`,`mode`,`datecreate` FROM `orion_cs_core` LEFT JOIN `orion_chan_users` ON (`orion_chan_users`.`channel`=`orion_cs_core`.`channel`) WHERE `orion_chan_users`.`channel` IS NOT NULL GROUP BY `orion_chan_users`.`channel`");
			channels.fields = mysql_num_rows(channels.res);
			
			for(i=0;i<channels.fields;i++){
				channels.row = mysql_fetch_row(channels.res);
				
				do_rawmode(thread,O->nick,channels.row[0],stol(channels.row[2]),_dynadd("+%s",m->DefConChanModes));
			}
			mysql_free_result_myr(channels.res);
		}
		defcon		= 2;
		readonly	= 1;
		if(m->GlobalOnDefcon)
			do_server_notice(thread,"(Broadcast) server entered Defcon 2, locking channels.");
			
        _ExecuteEvent(NULL,NULL,"socks_start");
	}else if(defconlevel==1){
		if(defcon>2){
			channels.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`channel`,`mode`,`datecreate` FROM `orion_cs_core` LEFT JOIN `orion_chan_users` ON (`orion_chan_users`.`channel`=`orion_cs_core`.`channel`) WHERE `orion_chan_users`.`channel` IS NOT NULL GROUP BY `orion_chan_users`.`channel`");
			channels.fields = mysql_num_rows(channels.res);
			
			for(i=0;i<channels.fields;i++){
				channels.row = mysql_fetch_row(channels.res);
				
				do_rawmode(thread,O->nick,channels.row[0],stol(channels.row[2]),_dynadd("+%s",m->DefConChanModes));
			}
			mysql_free_result_myr(channels.res);
		}
		defcon		= 1;
		readonly	= 1;
		if(m->GlobalOnDefcon)
			do_server_notice(thread,"(Broadcast) server entered Defcon 1, won't accept connections anymore.");
			
		_ExecuteEvent(NULL,NULL,"socks_stop");
	}
	else return;
}

/*  ************************************************************************************************  */

/**
 * @brief do_mlock:  Sets some mlock Modes for a Channel
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The ServicesNick which sets the Mlock
 * @param channel The Channel for which it should be set
 * @param date The Channel's creation date
 * @param modes The Channel's modes
 * @param buf The Parametric Buffer
 * @return bool 1 on Success, 0 on Error
 **/
bool do_mlock(Threads *thread, char *snick, char *channel, time_t date, char *modes, char *buf){
	char infobuffer[BUFSIZE];
	char *param[ARRAY_SIZE];
	char positive[35];
	char negative[35];
	
	int 	nr	= 0;
	int 	max	= 0;
	size_t 	i	= 0;
	bool 	type= 0;

	
	bzero(&infobuffer,sizeof(infobuffer));
	bzero(param,sizeof(param));
	bzero(&positive,sizeof(positive));
	bzero(&negative,sizeof(negative));

	dynncpy(infobuffer,buf,sizeof(infobuffer));
	max = strsplit(infobuffer,param,' ');
	
    for(i=0;i<strlen(modes) && nr<max;i++){
		char current=modes[i];
		printf("i = %d\n",(int)i);
		printf("mode = %c\n",current);
		           		
        if(current=='+') type=1;
		else if(current=='-') type=0;
        else if(findchar(modes,current)>0){
            if(type==1){
		        if(strcin(current,IRC.ParametricChannelModes)){
					if(!strcin(current,positive)){
						straddnc(positive,current,sizeof(positive));
					}
									
            		if(current==IRC.FloodCMode){
						/* Update Memory, for caching */
						bool flood_mode_valid = orion_WRCtranslate_flood_mode_check(thread,param[nr]);
#ifndef USE_THREADS
						if(flood_mode_valid == true){
							//bzero(&thread->c.chan.mlock_flood,sizeof(thread->c.chan.mlock_flood));
							dynncpy(thread->c.chan.mlock_flood,param[nr],sizeof(thread->c.chan.mlock_flood));
						}else
							strdelc(positive,current);
#endif
						
						/* Update SQL as Live Update */
						if(flood_mode_valid == true)
		           			domysql(thread,"UPDATE `orion_cs_core` SET `mlock_flood`='%s' WHERE `channel`='%s'",mysql_quote(thread,param[nr]),mysql_quote(thread,channel));
					}else if(current==IRC.KeyCMode){
						/* Update Memory, for caching */
#ifndef USE_THREADS
						//bzero(&thread->c.chan.mlock_key,sizeof(thread->c.chan.mlock_key));
						dynncpy(thread->c.chan.mlock_key,param[nr],sizeof(thread->c.chan.mlock_key));
#endif
						
						/* Update SQL as Live Update */
            			domysql(thread,"UPDATE `orion_cs_core` SET `mlock_key`='%s' WHERE `channel`='%s'",mysql_quote(thread,param[nr]),mysql_quote(thread,channel));
					}else if(current==IRC.LimitCMode){
						/* Update Memory, for caching */
#ifndef USE_THREADS
						thread->c.chan.mlock_limit = stol(param[nr]);
#endif
						
						/* Update SQL as Live Update */
		           		domysql(thread,"UPDATE `orion_cs_core` SET `mlock_limit`='%ld' WHERE `channel`='%s'",stol(param[nr]),mysql_quote(thread,channel));
					}else if(current==IRC.LinkCMode){
						/* Update Memory, for caching */
#ifndef USE_THREADS
						//bzero(&thread->c.chan.mlock_redirect,sizeof(thread->c.chan.mlock_redirect));
						dynncpy(thread->c.chan.mlock_redirect,param[nr],sizeof(thread->c.chan.mlock_redirect));
#endif
						
						/* Update SQL as Live Update */
		           		domysql(thread,"UPDATE `orion_cs_core` SET `mlock_redirect`='%s' WHERE `channel`='%s'",mysql_quote(thread,param[nr]),mysql_quote(thread,channel));
					}
					nr++;
            	}else{
            		if(!strcin(current,positive)){
						straddnc(positive,current,sizeof(positive));
		           		//domysql(thread,"UPDATE `orion_cs_core` SET `mlock_positive`='%s' WHERE `channel`='%s'",mysql_quote(thread,c.mlock_positive),mysql_quote(thread,c.name));
            		}
				}
		    }else if(type==0){
            	if(strcin(current,IRC.ParametricChannelModes)){
            		if(!strcin(current,negative)){
						straddnc(negative,current,sizeof(negative));
					}
            		//if(current=='f') domysql(thread,"UPDATE `orion_cs_core` SET `mlock_negative`='%s' WHERE `channel`='%s'",mysql_quote(thread,c.mlock_negative),mysql_quote(thread,c.name));
		           	//else if(current=='k') domysql(thread,"UPDATE `orion_cs_core` SET `mlock_negative`='%s' WHERE `channel`='%s'",mysql_quote(thread,c.mlock_negative),mysql_quote(thread,c.name));
            		//else if(current=='l') domysql(thread,"UPDATE `orion_cs_core` SET `mlock_negative`='%s' WHERE `channel`='%s'",mysql_quote(thread,c.mlock_negative),mysql_quote(thread,c.name));
				}else{
            		if(!strcin(current,negative) && !strcin(current,positive)){
						straddnc(negative,current,sizeof(negative));
		           		//domysql(thread,"UPDATE `orion_cs_core` SET `mlock_negative`='%s' WHERE `channel`='%s'",mysql_quote(thread,c.mlock_negative),mysql_quote(thread,c.name));
            		}
            	}
		    }
		}else
			return 0;
    }
	sort(negative);
	sort(positive);
	
	//bzero(&thread->c.chan.mlock_negative,sizeof(thread->c.chan.mlock_negative));
	//bzero(&thread->c.chan.mlock_positive,sizeof(thread->c.chan.mlock_positive));
	dynncpy(thread->c.chan.mlock_negative,negative,sizeof(thread->c.chan.mlock_negative));
	dynncpy(thread->c.chan.mlock_positive,positive,sizeof(thread->c.chan.mlock_positive));
	
	domysql(thread,"UPDATE `orion_cs_core` SET `mlock_negative`='%s',`mlock_positive`='%s' WHERE `channel`='%s'",mysql_quote(thread,negative),mysql_quote(thread,positive),mysql_quote(thread,channel));

	if(modes[0]!='\0'){
		if(nr>0)	do_mode(thread,snick,channel,date,_dynadd("%s %s",modes,buf));
		else		do_mode(thread,snick,channel,date,modes);
	}
	return 1;
}

/*  ************************************************************************************************  */

/**
 * @brief get_last_function:  Tool for the sql-log (backtrace support for sql)
 *
 * @param ret The returned backtrace lookup (function lookup)
 * @param steps The steps to go for the backtrace log
 * @return void
 **/
void get_last_function(char *ret, int steps){
#ifdef HAVE_BACKTRACE
  #ifdef USE_THREADS
	my_pthread_mutex_lock(&backtrace_lock);
  #endif

	char *filename;
	char *splitbracket[5];
	char *splitfunction[5];
	void *array[steps+1];
	size_t size;
	char **strings;

	size = backtrace(array, steps+1);
	strings = (char **)backtrace_symbols(array, size);

	strsplit(strings[steps],splitbracket,'(');

	// filename
	filename = strrchr(splitbracket[0],'/') + 1;

	// function
	strsplit(splitbracket[1],splitfunction,'+');

	sprintf(ret,"[%s->%s] ",filename,splitfunction[0]);
/*	for (i = 1; i < size; i++) {
		dlog(0,"Backtrace FAKE TEST(%d): %s\n", i, strings[i]);
	}*/
	free(strings);
  #ifdef USE_THREADS
	my_pthread_mutex_unlock(&backtrace_lock);
  #endif
#endif
}

/*  ************************************************************************************************  */

/**
 * @brief GetLevelvalueByIndex:  To calculate Levels between Q and ChanServ (for alist and movechan)
 *
 * @param leveltable The leveltable, where our levels are stored
 * @param index The Index for what we are searching
 * @return LevelChanging The levelstruct, for target index
 **/
LevelChanging GetLevelvalueByIndex(LevelChanging *leveltable, char *index){
	LevelChanging levels;
	bzero(&levels,sizeof(levels));

	levels.index[0] = '\0';
	levels.type = 0;
	levels.CSlevel[0] = '\0';
	levels.XOPlevel = 0;
	levels.Qlevel[0] = '\0';

	int i = 0;
	for(i=0;i<MAX_CHANNEL_LEVELS;i++){
		if(streq(index,leveltable[i].index)) return leveltable[i];
	}
	return levels;
}

/*  ************************************************************************************************  */

/**
 * @brief FillLevel:  Puts new Levels (for calculation) to our level list
 *
 * @param level The struct where we want to store that level information
 * @param index The Index for what we are able to search
 * @param type The Type - used for the direction (1 ChanServ->Q, 2 Q->ChanServ, 3 ChanServ->Q (Founder, Successor), 4 Q->ChanServ (Owner, Master)
 * @param CSlevel The ChanServ Level that has to be used to execute a command depening on that Channel-Level
 * @param XOPlevel The ChanServ-XOP Level that has to be used to execute a command depening on that Channel-Level
 * @param Qlevel The Q-Level that has to be used to execute the command in the List
 * @return void
 **/
void FillLevel(LevelChanging *level, char *index, int type, char *CSlevel, int XOPlevel, char *Qlevel){
	level->index[0] = '\0';
	level->type = 0;
	level->CSlevel[0] = '\0';
	level->XOPlevel = 0;
	level->Qlevel[0] = '\0';

	dynncpy(level->index,index,sizeof(level->index));
	level->type = type;
	dynncpy(level->CSlevel,CSlevel,sizeof(level->CSlevel));
	level->XOPlevel = XOPlevel;
	dynncpy(level->Qlevel,Qlevel,sizeof(level->Qlevel));
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief NSCheckSettings:  Returns the NickServ Settings, for adding to the Database
 *
 * @return int
 **/
int NSCheckSettings(void){
	int settings = 0;
	
	if(m->NSDefHideEmail && ~settings&NS_SET_HIDE_EMAIL)
		settings |= NS_SET_HIDE_EMAIL;
	if(m->NSDefHideUsermask && ~settings&NS_SET_HIDE_MASK)
		settings |= NS_SET_HIDE_MASK;
	if(m->NSDefHideQuit && ~settings&NS_SET_HIDE_QUIT)
		settings |= NS_SET_HIDE_QUIT;
	if(m->NSDefHideStatus && ~settings&NS_SET_HIDE_STATUS)
		settings |= NS_SET_HIDE_STATUS;
	if(m->NSDefKill && ~settings&NS_SET_KILLPROTECT)
		settings |= NS_SET_KILLPROTECT;
	if(m->NSDefKillQuick && ~settings&NS_SET_KILL_QUICK)
		settings |= NS_SET_KILL_QUICK;
	if(m->NSDefKillImmed && ~settings&NS_SET_KILL_IMMED)
		settings |= NS_SET_KILL_IMMED;
	if(m->NSDefMsg && ~settings&NS_SET_MSG)
		settings |= NS_SET_MSG;
	if(m->NSDefSecure && ~settings&NS_SET_SECURE)
		settings |= NS_SET_SECURE;

	return settings;
}

/*  ************************************************************************************************  */

/**
 * @brief BSCheckSettings:  Returns the BotServ-Channel Settings, for adding to the Database
 *
 * @return int
 **/
int BSCheckSettings(void){
	int settings = 0;
	
	if(m->BSDefDontKickOps && ~settings&BS_SET_DONTKICKOPS_BIT)
		settings |= BS_SET_DONTKICKOPS_BIT;
	if(m->BSDefDontKickVoices && ~settings&BS_SET_DONTKICKVOICES_BIT)
		settings |= BS_SET_DONTKICKVOICES_BIT;
	if(m->BSDefGreet && ~settings&BS_SET_GREET_BIT)
		settings |= BS_SET_GREET_BIT;
	if(m->BSDefFantasy && ~settings&BS_SET_FANTASY_BIT)
		settings |= BS_SET_FANTASY_BIT;
	if(m->BSDefSymbiosis && ~settings&BS_SET_SYMBIOSIS_BIT)
		settings |= BS_SET_SYMBIOSIS_BIT;

	return settings;
}

/*  ************************************************************************************************  */

/**
 * @brief CSCheckBot:  Check for Bot, If none - Set ChanServ as Bot (Symbioses setting proofed)
 *
 * @param newbot pointer to the variable where we like to store the new bot
 * @param bot The pointer to the variable where the bot is stored
 * @param set The channels botserv settings
 * @param size The size of "newbot"
 * @return void
 **/
void CSCheckBot(char *newbot, char *bot, long set, size_t size){
	bzero(newbot,size);
	
	if(bot[0]!='\0' && bs->loaded && set&BS_SET_SYMBIOSIS_BIT)
		dynncpy(newbot,bot,size);
	else
		dynncpy(newbot,cs->nick,size);
	return;
}


/*  ************************************************************************************************  */

/**
 * @brief QCheckBot:  Check for Bot, If none - Set Q as Bot (Symbioses setting proofed)
 *
 * @param newbot pointer to the variable where we like to store the new bot
 * @param bot The pointer to the variable where the bot is stored
 * @param set The channels botserv settings
 * @param size The size of "newbot"
 * @return void
 **/
void QCheckBot(char *newbot, char *bot, long set, size_t size){
	bzero(newbot,size);
	
	if(bot[0]!='\0' && bs->loaded && set&BS_SET_SYMBIOSIS_BIT)
		dynncpy(newbot,bot,size);
	else
		dynncpy(newbot,Q->nick,size);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief CSCheckBotInPlace:  Check for Bot, If none - ChanServ is returned (Symbioses setting proofed)
 *
 * @param bot The pointer to the variable where the bot is stored
 * @param set The channels botserv settings
 * @return char* Returns the ChannelBot (if none - ChanServ)
 **/
char *CSCheckBotInPlace(char *bot, long set){
	if(bot==NULL || bot[0]=='\0' || !bs->loaded || ~set&BS_SET_SYMBIOSIS_BIT)
		return cs->nick;
	else
		return bot;
}

/*  ************************************************************************************************  */

/**
 * @brief !CheckBotInPlace:  Check for Bot, If none - ! is returned (Symbioses setting proofed)
 *
 * @param bot The pointer to the variable where the bot is stored
 * @param set The channels botserv settings
 * @return char* Returns the ChannelBot (if none - ChanServ)
 **/
char *QCheckBotInPlace(char *bot, long set){
	if(bot==NULL || bot[0]=='\0' || !bs->loaded || ~set&BS_SET_SYMBIOSIS_BIT)
		return Q->nick;
	else
		return bot;
}

/*  ************************************************************************************************  */

/**
 * @brief CSCheckSettings:  Returns the ChanServ-Channel Settings, for adding to the Database
 *
 * @return int Returns the channels default settings
 **/
int CSCheckSettings(void){
	int settings = 0;
	
	if(m->CSDefNone)
		return settings;
	
	if(m->CSDefKeepTopic && ~settings&CS_KEEPTOPIC_BIT)
		settings |= CS_KEEPTOPIC_BIT;
		
	if(m->CSDefOpNotice && ~settings&CS_OPNOTICE_BIT)
		settings |= CS_OPNOTICE_BIT;
		
	if(m->CSDefPeace && ~settings&CS_PEACE_BIT)
		settings |= CS_PEACE_BIT;
		
	if(m->CSDefPrivate && ~settings&CS_PRIVATE_BIT)
		settings |= CS_PRIVATE_BIT;
		
	if(m->CSDefRestricted && ~settings&CS_RESTRICTED_BIT)
		settings |= CS_RESTRICTED_BIT;
		
	if(m->CSDefSecure && ~settings&CS_SECURE_BIT)
		settings |= CS_SECURE_BIT;
		
	if(m->CSDefSecureOps && ~settings&CS_SECUREOPS_BIT)
		settings |= CS_SECUREOPS_BIT;
		
	if(m->CSDefSecureFounder && ~settings&CS_SECUREFOUNDER_BIT)
		settings |= CS_SECUREFOUNDER_BIT;
		
	if(m->CSDefSignKick && ~settings&CS_SIGNKICK_1_BIT)
		settings |= CS_SIGNKICK_1_BIT;

	if(m->CSDefSignKickLevel && settings&CS_SIGNKICK_2_BIT)
		settings |= CS_SIGNKICK_2_BIT;
		
	if(m->CSDefTopicLock && ~settings&CS_TOPICLOCK_BIT)
		settings |= CS_TOPICLOCK_BIT;
		
	if(m->CSDefXOP && ~settings&CS_XOP_BIT)
		settings |= CS_XOP_BIT;
							
	return settings;
}

/*  ************************************************************************************************  */

/**
 * @brief CSAddBanType:  Returns the Bantype-Settings for the Database (calculated from Input 0-3)
 *
 * @param settings The Bantype-Setting (ChanServ channel settings)
 * @return int Returns the new ChanServ channel settings (with added bantype)
 **/
int CSAddBanType(int settings){
	if(m->CSDefBantype == 0){
		if(settings&CS_BANTYPE_1_BIT) settings &= ~CS_BANTYPE_1_BIT;
		if(settings&CS_BANTYPE_2_BIT) settings &= ~CS_BANTYPE_2_BIT;
	}else if(m->CSDefBantype == 1){
		if(~settings&CS_BANTYPE_1_BIT) settings |= CS_BANTYPE_1_BIT;
		if(settings&CS_BANTYPE_2_BIT) settings &= ~CS_BANTYPE_2_BIT;
	}else if(m->CSDefBantype == 2){
		if(settings&CS_BANTYPE_1_BIT) settings &= ~CS_BANTYPE_1_BIT;
		if(~settings&CS_BANTYPE_2_BIT) settings |= CS_BANTYPE_2_BIT;
	}else if(m->CSDefBantype == 3){
		if(~settings&CS_BANTYPE_1_BIT) settings |= CS_BANTYPE_1_BIT;
		if(~settings&CS_BANTYPE_2_BIT) settings |= CS_BANTYPE_2_BIT;
	}
	return settings;
}

/*  ************************************************************************************************  */

/**
 * @brief CSAnalyseBantype:  Returns the Bantype-Settings depending on the Settings read from the Database
 *
 * @param settings The Database-Bantype-Setting (ChanServ channel settings)
 * @return int Returns The Bantype (0-3) or -1 if there was no setting (Q-channel)
 **/
int CSAnalyseBantype(int settings){
	if(~settings&CS_BANTYPE_1_BIT && ~settings&CS_BANTYPE_2_BIT) return 0;
	else if(settings&CS_BANTYPE_1_BIT && ~settings&CS_BANTYPE_2_BIT) return 1;
	else if(~settings&CS_BANTYPE_1_BIT && settings&CS_BANTYPE_2_BIT) return 2;
	else if(settings&CS_BANTYPE_1_BIT && settings&CS_BANTYPE_2_BIT) return 3;
	
	return -1;
}

/*  ************************************************************************************************  */

/**
 * @brief CSAnalyseSignKick:  Returns the Signkick-Settings depending on the Settings read from the Database
 *
 * @param settings The Database-Signkick-Setting (ChanServ channel settings)
 * @return int Returns The Signkick-level (off,on,level) or -1 if there was no setting (Q-channel)
 **/
int CSAnalyseSignKick(int settings){
	if(~settings&CS_SIGNKICK_1_BIT && ~settings&CS_SIGNKICK_2_BIT) return 0;
	else if(settings&CS_SIGNKICK_1_BIT && ~settings&CS_SIGNKICK_2_BIT) return 1;
	else if(settings&CS_SIGNKICK_1_BIT && settings&CS_SIGNKICK_2_BIT) return 2;
	
	return -1;
}

/*  ************************************************************************************************  */

/**
 * @brief BuildBanMask:  Builds the Banmask depending on the Channels BanSetting
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param Cdata The ConnectionData for target user (the user we want to ban)
 * @param type The Bantype which should be performed (0-3) returned from "CSAnalyseBantype(c->settings)"
 * @return char* Returns the Banmask
 **/
char *BuildBanMask(Threads *thread, Condata *Cdata, int type){
	static char banmask[300];
	bzero(&banmask,sizeof(banmask));

	if(type == 0)
		dynncpy(banmask,_dynadd("*!%s@%s",Cdata->nick,Cdata->vhost),sizeof(banmask));

	else if(type == 1)
		dynncpy(banmask,_dynadd("*!*%s@%s",Cdata->nick,Cdata->vhost),sizeof(banmask));

	else if(type == 2)
		dynncpy(banmask,_dynadd("*!*@%s",Cdata->vhost),sizeof(banmask));

	else if(type == 3){
		char host[MAX_HOST_LEN];
		char usedhost[MAX_HOST_LEN];
		char *domain[5];
		size_t parts = 0;
				
		bzero(&host,sizeof(host));
		bzero(&usedhost,sizeof(usedhost));
		bzero(domain,sizeof(domain));
				
		dynncpy(host,Cdata->vhost,sizeof(host));
		parts = strsplit(host,domain,'.');
				
		if(parts>=2){
			snprintf(usedhost,sizeof(usedhost),"*.%s.%s",domain[parts-2],domain[parts-1]);
			dynncpy(banmask,_dynadd("*!*%s@%s",Cdata->nick,usedhost),sizeof(banmask));
		}
		else dynncpy(banmask,_dynadd("*!*%s@*.%s",Cdata->nick,domain[parts-1]),sizeof(banmask));
				
	}
	else
		dynncpy(banmask,_dynadd("*!*@%s",Cdata->vhost),sizeof(banmask));
		
	return banmask;
}

/*  ************************************************************************************************  */

/**
 * @brief unknown_access:  Sets a users channel level to unknown on that channel
 *
 * @param access The Access level to overwrite
 * @param len The size of the access level
 * @return void
 **/
void unknown_access(char *access, size_t len){
	bzero(access,len);
	straddns(access,ACCESS_UNKNOWN_STRING,sizeof(access));
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief UpdateChannelInformation:  Updates Information for target channel (only if necessary!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channels for which we like to update the Information
 * @param target The targetnick Name for which's level we like to get (could be NULL, if we only need the channel)
 * @return void
 **/
void UpdateChannelInformation(Threads *thread, char *channel, char *target){
	if(target==NULL){
		if(!streqnc(channel,thread->c.chan.name) || !streqnc(thread->sql.access.authnick,thread->c.chan.access.ownauthnick) || (THREAD_SAFE)){
			//printf("retrieving channel information for channel: %s\n",channel);
			thread->c = sql_thread_data(thread,channel,SQL_THREAD,thread->sql.access.authnick);
		}
	}
	else if(!streqnc(target,thread->c.aliases.nick) || !streqnc(channel,thread->c.chan.name) || !streqnc(thread->sql.access.authnick,thread->c.chan.access.ownauthnick) || (THREAD_SAFE)){
		thread->c = sql_thread_data(thread,channel,SQL_THREAD_AAC,target,thread->sql.access.authnick);
		if(!thread->sql.aliases.active){
			if(streqnc(thread->c.chan.serv,"chanserv"))
				unknown_access(thread->c.chan.access.ownlevel,sizeof(thread->c.chan.access.ownlevel));
			else
				bzero(&thread->c.chan.access.ownlevel,sizeof(thread->c.chan.access.ownlevel));
		}
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief UpdateChannelInformation2:  Updates Information for target channel (only if necessary!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channels for which we like to update the Information
 * @param target The targetnick Name for which's level we like to get (could be NULL, if we only need the channel)
 * @return void
 **/
void UpdateChannelInformation2(Threads *thread, char *channel, char *target){
	if(!streqnc(target,thread->c.aliases.nick) || !streqnc(channel,thread->c.chan.name) || (THREAD_SAFE)){
		thread->c = sql_thread_data(thread,channel,SQL_THREAD_AAC2,target);
		if(!thread->c.aliases.active){
			if(streqnc(thread->c.chan.serv,"chanserv"))
				unknown_access(thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
			else
				bzero(&thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
		}
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief UpdateChannelInformation3:  Updates Information for target channel (only if necessary!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channels for which we like to update the Information
 * @param target The targetnick Name for which's level we like to get (could be NULL, if we only need the channel)
 * @return void
 **/
void UpdateChannelInformation3(Threads *thread, char *channel, char *target){
	if(target==NULL){
		if(!streqnc(channel,thread->c.chan.name) || (THREAD_SAFE))
			thread->c = sql_thread_data(thread,channel,SQL_THREAD_CCLB);
	}
	else if(!streqnc(target,thread->c.aliases.nick) || !streqnc(channel,thread->c.chan.name) || !streqnc(thread->sql.access.authnick,thread->c.chan.access.ownauthnick) || (THREAD_SAFE)){
		thread->c = sql_thread_data(thread,channel,SQL_THREAD_AAC,target,thread->sql.access.authnick);
		if(!thread->sql.aliases.active){
			if(streqnc(thread->c.chan.serv,"chanserv"))
				unknown_access(thread->c.chan.access.ownlevel,sizeof(thread->c.chan.access.ownlevel));
			else
				bzero(&thread->c.chan.access.ownlevel,sizeof(thread->c.chan.access.ownlevel));
		}
	}
	return;
}

/*  ************************************************************************************************  */
