/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 194                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-16 18:33:19 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file sjoin.c
 *   @brief This is the Module file for SJOIN (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the SJOIN command.
 *
 *   The SJOIN File is used to process SJOINS sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Sjoin is used to handle the SJOIN Protocol -> less traffic)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

#define SJOIN_CACHING 0

/*
> SJOIN 0 #idlrpg :Shadow-666
> SJOIN 0 #Leiche :@~Leiche
> SJOIN 0 #cstrike-dev :@~HLServ
> SJOIN 0 #novell :%XSERVER %NWCONFIG %MONITOR %INETCFG %CRON %ARCSERV
> SJOIN 1179920273 #DeepIRC-Games :shock @~GameServ
> SJOIN 0 #DeepIRC-Games-Debug +sntO :@~GameServ

> :klebestreifen.dev.hub.deepirc.net SJOIN 1193066856 #ircops :shock
> :klebestreifen.dev.hub.deepirc.net SJOIN 1193066856 #services :shock
> :klebestreifen.dev.hub.deepirc.net SJOIN 1193066856 #deepirc :shock
> :klebestreifen.dev.hub.deepirc.net SJOIN 1193076220 #help :@shock
> :klebestreifen.dev.hub.deepirc.net SJOIN 1193076220 #technical-team :@shock
> :klebestreifen.dev.hub.deepirc.net SJOIN 1193076220 #deepQ :@shock
> :klebestreifen.dev.hub.deepirc.net SJOIN 1193076220 #p2pworld :@shock
> :klebestreifen.dev.hub.deepirc.net SJOIN 1193076220 #deepirc-games :@shock
> :klebestreifen.dev.hub.deepirc.net SJOIN 1193076220 #orion :@shock
> :klebestreifen.dev.hub.deepirc.net SJOIN 1193076220 #wald :@shock
> :klebestreifen.dev.hub.deepirc.net SJOIN 1193076220 #dev-team :@shock
> :klebestreifen.dev.hub.deepirc.net SJOIN 1193076220 #deep-coderz :@shock
*/

/*
q = *
a = ~
o = @
h = %
v = +
b = & ban
e = " exception
*/

/**
 * @brief sjoin:  Handles Incoming & Outgoing SJOIN'S
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The Channel itself (for which we handle modes and clients)
 * @param date The Channels creation date
 * @param buf The SJOIN buf (holds data about clients and Modes)
 * @param fake The Faketype, used to Avoid the Eventhandler in case of an outgoing SJOIN (in that case >0)
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL sjoin(Threads *thread, char *channel, time_t date, char *buf, int fake){
	deepSQL MySQL;
	Stays *stay = NULL; /* Manages Bot, ChanServ / Q Stay in Channel for a while, after certain actions */

	char *nicks      [ARRAY_SIZE];
	char nbuf        [(MAX_MODE_PARAMS_IRCD*MAX_NICK_LEN+MAX_MODE_PARAMS_IRCD*3)*2];
	char sender		 [MAX_NICK_LEN];
	char modes		 [200];
	char search		 [70];
	bool proof			= 1;
	size_t ucmodes_count= 0;
	size_t i			= 0;
	size_t j			= 0;
	bool useradd		= 1;
	int user			= 0;
	int users_in_chan   = thread->c.chan.countusers; //GetUsersInChannel(thread->c.chan.name); //Users in Channel before sjoin was managed
	/**     Bot Modes    **/
	bool csjoined		= 0;
	bool HasBot			= 1;
	/**        ***       **/
	UserChanModes ucmodes	[MAX_MODE_PARAMS_IRCD];
	char selection			[(MAX_MODE_PARAMS_IRCD*MAX_NICK_LEN+MAX_MODE_PARAMS_IRCD*3)*2];
	
	/* Flush Inserts - Since on serverlink or not the NICK connect Event comes before SJOIN! */
	
	//BZERO
	bzero(&MySQL,sizeof(MySQL));
	bzero(&nbuf,sizeof(nbuf));
	dynncpy(nbuf,buf,sizeof(nbuf));
	
	dlog(thread,3,"buf = %s\n",buf);
	dlog(thread,3,"nbuf = %s\n",nbuf);
	
	strsplit(nbuf,nicks,' ');

	if(thread->c.chan.name[0]=='\0'){
		domysql(thread,"INSERT IGNORE INTO `orion_cs_core` (`datecreate`,`channel`) VALUES ('%ld','%s')",date ? date : time(NULL),mysql_quote(thread,channel));
		thread->c.chan.datecreate=date;
	}else{
		if(thread->c.chan.datecreate==0 || date<thread->c.chan.datecreate){
			//printf("incoming date is lower = %ld (date), %ld (thread->c.chan.datecreate)\n",date,thread->c.chan.datecreate);
			domysql(thread,"UPDATE `orion_cs_core` SET `datecreate`='%ld' WHERE `channel` = '%s'",date,mysql_quote(thread,channel));
			thread->c.chan.datecreate=date;
		}
		
		if(!fake)
			proof=1;
		else
			proof=0;
	}
	
	dlog(thread,4,"proof = %d\n",proof);
	
	if(thread->c.chan.bot[0]=='\0' || !bs->loaded)
		HasBot = 0;
	
	for(i=0;dynlen(nicks[i])>0;i++){
		dlog(thread,3,"added nick = %s\n",nicks[i]);
		char mode[MAX_MODES_LEN];
		bzero(&mode,sizeof(mode));
		
		
/*
buf = *~@BotServ
nbuf = *~@BotServ
proof = 0
added nick = *~@BotServ
USERADD
Nick: BotServ - Mode: qao
qry: INSERT INTO `orion_chan_users` (`nick`,`channel`,`modes`,`faketype`) VALUES ('BotServ','#services','','2') ON DUPLICATE KEY UPDATE `modes`=VALUES(`modes`)
sputs-buf: :CServe.deepirc.net SJOIN 1232014496 #services :*~@BotServ
*/
		useradd=1;
		for(j=0;j<dynlen(nicks[i]);j++){
			if(nicks[i][j]==IRC.SjoinOwnerPrefix && IRC.HasOwner)
				straddnc(mode,IRC.OwnerMode,sizeof(mode));
			else if(nicks[i][j]==IRC.SjoinProtectPrefix && IRC.HasProtect)
				straddnc(mode,IRC.ProtectMode,sizeof(mode));
			else if(nicks[i][j]==IRC.SjoinOpPrefix)
				straddnc(mode,IRC.OpMode,sizeof(mode));
			else if(nicks[i][j]==IRC.SjoinHOpPrefix && IRC.SupportHalfOp)
				straddnc(mode,IRC.HOpMode,sizeof(mode));
			else if(nicks[i][j]==IRC.SjoinVoicePrefix)
				straddnc(mode,IRC.VMode,sizeof(mode));
				
			else if(IRC.SupportSjoinBanprefix && nicks[i][j]==IRC.SjoinBanPrefix){
				domysql(thread,"REPLACE `orion_chan_bans` (`ban`,`channel`) VALUES ('%s','%s')",mysql_quote(thread,&nicks[i][j+1]),mysql_quote(thread,channel));
				useradd=0;
			}
			else if(IRC.SupportSjoinExceptprefix && nicks[i][j]==IRC.SjoinExceptPrefix){
				domysql(thread,"REPLACE `orion_chan_expbans` (`exception`,`channel`) VALUES ('%s','%s')",mysql_quote(thread,&nicks[i][j+1]),mysql_quote(thread,channel));
				useradd=0;
			}
			else if(IRC.SjoinHasIgnoreChar && nicks[i][j]==IRC.SjoinIgnoredChar){
				/* ignored */
			}
			else	
				break;
	    }
	    dlog(thread,3,"USERADD\n");
	    
	    if(useradd && GetUser(&nicks[i][j])!=NULL){
			dlog(thread,3,"Nick: %s - Mode: %s\n",&nicks[i][j],mode);
			thread->c.chan.countusers++;
			if(proof){
				dynncpy(ucmodes[ucmodes_count].nick,&nicks[i][j],sizeof(ucmodes[ucmodes_count].nick));
				dynncpy(ucmodes[ucmodes_count].mode,mode,sizeof(ucmodes[ucmodes_count].mode));
				
				bzero(&ucmodes[ucmodes_count].authnick,sizeof(ucmodes[ucmodes_count].authnick));
				bzero(&ucmodes[ucmodes_count].access,sizeof(ucmodes[ucmodes_count].access));
				bzero(&ucmodes[ucmodes_count].greet,sizeof(ucmodes[ucmodes_count].greet));
				/* For Akick */
				bzero(&ucmodes[ucmodes_count].banmask,sizeof(ucmodes[ucmodes_count].banmask));
				bzero(&ucmodes[ucmodes_count].reason,sizeof(ucmodes[ucmodes_count].reason));
				ucmodes[ucmodes_count].getakick = 0;
				/* EO Akick  */
				ucmodes[ucmodes_count].active = 0;
				ucmodes[ucmodes_count].qlevel = 0;
				ucmodes[ucmodes_count].olevel = 0;
				ucmodes[ucmodes_count].llevel = 0;
				ucmodes[ucmodes_count].slevel = 0;
				ucmodes[ucmodes_count].rights = 0;
				ucmodes_count				++;
			}
			else{
				dlog(thread,3,"MODE: %s \n",mode);
				sort(mode);
				dlog(thread,3,"MODE after sort: %s \n",mode);
				
				/* Add the user it's new channel */
				addUserChannel(GetUser(&nicks[i][j]), channel);
				
				if(DuringServerlink && SJOIN_CACHING) doBufferedInsert(thread,1,SQL_BUFFER_SJOIN,"INSERT INTO `orion_chan_users` (`nick`,`channel`,`modes`,`faketype`) VALUES ('%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `modes`=VALUES(`modes`)",mysql_quote(thread,&nicks[i][j]),mysql_quote(thread,channel),mysql_quote(thread,mode),fake);
				else domysql(thread,"INSERT INTO `orion_chan_users` (`nick`,`channel`,`modes`,`faketype`) VALUES ('%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `modes`=VALUES(`modes`)",mysql_quote(thread,&nicks[i][j]),mysql_quote(thread,channel),mysql_quote(thread,mode),fake);
				
				if(HasBot && ((~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers>=m->BSMinUsers) || (thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers>=(m->BSMinUsers+1))) && fake!=1 && getStayChannel(thread,thread->c.chan.name)==NULL && !thread->c.chan.botinchan){ /* CHANGED */
					//do_join(thread,1,thread->c.bot,thread->c.name,thread->c.datecreate);
		        	//do_mode(thread,thread->c.bot,thread->c.name,thread->c.datecreate,"+oaq %s %s %s",thread->c.bot,thread->c.bot,thread->c.bot);
		        	do_services_join(thread,thread->c.chan.bot,thread->c.chan.name,time(NULL),1);
		        	thread->c.chan.botinchan=1;
				}
			}
		}
    }
    
    if(proof){
		bzero(&selection,sizeof(selection));
		for(i=0;i<ucmodes_count;i++){
			if(i>0) straddnc(selection,',',sizeof(selection));
			straddnc(selection,toascii(39),sizeof(selection));
			straddns(selection,mysql_quote(thread,ucmodes[i].nick),sizeof(selection));
			straddnc(selection,toascii(39),sizeof(selection));
		}
	    
		if(selection[0]!='\0'){
			dlog(thread,4,"SELECTION = %s\n",selection);
			selection[strlen(selection)]='\0';
			dlog(thread,4,"SELECTION = %s\n",selection);
				
			MySQL.res = doSelectMySQL(thread, 
				" SELECT "
				"   `orion_core_condata`.`nick`,CONCAT(`orion_core_condata`.`nick`,'!',`orion_core_condata`.`ident`,'@',`orion_core_condata`.`vhost`) as `hostmask`, "
				"   IF((CASE `orion_cs_core`.`serv` "
				"      WHEN 'ChanServ' THEN `orion_cs_access`.`level`>=IF(`orion_cs_core`.`cs_nokick`=-1000,`orion_cs_core`.`cs_autoop`,`orion_cs_core`.`cs_nokick`) "
				"      WHEN 'Q' THEN `orion_cs_access`.`level` RLIKE '(n|m|o|h|p)' "
				"      END "
				"      AND `orion_ns_aliases`.`active`)=1,1,0)"
				"	AS `allowed_user`, "
				"   `orion_cs_akick`.`mask`, `orion_cs_akick`.`reason`,`orion_ns_aliases`.`display`,`orion_ns_aliases`.`active`,`orion_core_condata`.`ident`,`orion_core_condata`.`vhost`, "
				"   `orion_core_condata`.`rhost`,`orion_cs_access`.`level`,"
				"	IF(`orion_ns_aliases`.`active`=1,`orion_ns_core`.`qlevel`,0) AS `orion_qlevel`,"
				"	IF(`orion_ns_aliases`.`active`=1,`orion_ns_core`.`olevel`,0) AS `orion_olevel`,"
				"	IF(`orion_ns_aliases`.`active`=1,`orion_ns_core`.`llevel`,0) AS `orion_llevel`,"
				"	IF(`orion_ns_aliases`.`active`=1,`orion_ns_core`.`slevel`,0) AS `orion_slevel`,"
				"	IF(`orion_ns_aliases`.`active`=1,`orion_ns_core`.`rights`,0) AS `orion_rights`,"
				"	`orion_ns_core`.`greet`"
				" FROM "
				"   `orion_core_condata` "  
				"     LEFT JOIN `orion_cs_core` ON `orion_cs_core`.`channel`='%s' " // CHANNEL
				"     LEFT OUTER JOIN `orion_cs_akick` ON (CONCAT(`orion_core_condata`.`nick`,'!',`orion_core_condata`.`ident`,'@',`orion_core_condata`.`vhost`) "
				"       LIKE REPLACE(`mask`,'*','%') AND `orion_cs_akick`.`channel`=`orion_cs_core`.`channel`) "
				"     LEFT JOIN `orion_ns_aliases` "
				"       LEFT OUTER JOIN `orion_ns_core` "
				"       ON (`orion_ns_core`.`authnick`=`orion_ns_aliases`.`display`) "
				"       LEFT OUTER JOIN `orion_cs_access` "
				"       ON (`orion_cs_access`.`authnick`=`orion_ns_aliases`.`display` AND `channel`='%s') " // CHANNEL
				"     ON `orion_core_condata`.`nick`=`orion_ns_aliases`.`nick` "
				" WHERE "
				"   `orion_core_condata`.`nick` IN(%s) "
				" GROUP BY "
				"   `orion_core_condata`.`nick` "
				" ORDER BY "
				"   FIELD(`orion_core_condata`.`nick`,%s)",mysql_quote(thread,channel),mysql_quote(thread,channel),selection,selection
			);	
					
			MySQL.fields = mysql_num_rows(MySQL.res);
			
			for(i=0;i<MySQL.fields;i++){
				MySQL.row = mysql_fetch_row(MySQL.res);
				
				user = getModeNick(ucmodes,ucmodes_count,MySQL.row[0]);
				if(user != -1){
					ucmodes[user].getakick=(stoi(MySQL.row[15])>is_services_admin && stoi(MySQL.row[6])==1) ? 0 : abs(stoi(MySQL.row[2])-1);
					dynncpy(ucmodes[user].banmask,MySQL.row[3],sizeof(ucmodes[user].banmask));
					dynncpy(ucmodes[user].reason,MySQL.row[4],sizeof(ucmodes[user].reason));
					
					dynncpy(ucmodes[user].authnick,MySQL.row[5],sizeof(ucmodes[user].authnick));
					ucmodes[user].active=stoi(MySQL.row[6]);
					dynncpy(ucmodes[user].ident,MySQL.row[7],sizeof(ucmodes[user].ident));
					dynncpy(ucmodes[user].vhost,MySQL.row[8],sizeof(ucmodes[user].vhost));
					dynncpy(ucmodes[user].rhost,MySQL.row[9],sizeof(ucmodes[user].rhost));
					dynncpy(ucmodes[user].access,MySQL.row[10],sizeof(ucmodes[user].access));
					ucmodes[user].qlevel=stoi(MySQL.row[11]);
					ucmodes[user].olevel=stoi(MySQL.row[12]);
					ucmodes[user].llevel=stoi(MySQL.row[13]);
					ucmodes[user].slevel=stoi(MySQL.row[14]);
					ucmodes[user].rights=stoi(MySQL.row[15]);
					dynncpy(ucmodes[user].greet,MySQL.row[16],sizeof(ucmodes[user].greet));
				}
			}
			mysql_free_result_myr(MySQL.res);
		}

		/* We have to part ChanServ if we joined it before */
		stay = getStayChannel(thread, thread->c.chan.name);

		if(stay != NULL){
			if(thread->c.chan.countusers > /*m->BSMinUsers+*/1 && stay->csjoin)
				do_part(thread, cs->nick, stay->channel,NULL);
		}

		if(thread->c.chan.countusers > 1/*m->BSMinUsers*/)
			delStayChannel(thread,thread->c.chan.name);
		/***************************************************/

		if(ucmodes_count && HasBot && ((~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers>=m->BSMinUsers) || (thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers>=(m->BSMinUsers+1))) && fake!=1 && getStayChannel(thread,thread->c.chan.name)==NULL && !thread->c.chan.botinchan){ /* CHANGED */
			do_services_join(thread,thread->c.chan.bot,thread->c.chan.name,time(NULL),1);
			thread->c.chan.botinchan=1;
		}
				
		if(!HasBot || (~thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT && HasBot)){
			if(streqnc(thread->c.chan.serv,"chanserv"))
				dynncpy(sender,cs->nick,sizeof(sender));
			else if(streqnc(thread->c.chan.serv,"q"))
				dynncpy(sender,Q->nick,sizeof(sender));
		}else if(thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT && HasBot)
			dynncpy(sender,thread->c.chan.bot,sizeof(sender));
			
		for(user=0;user<ucmodes_count;user++){
			
			/* Manages the Akick */
			if(ucmodes[user].getakick && ucmodes[user].banmask[0]!='\0'){
				/* Manages Bots and ChanServ / Q to stay in Channel for a while */
				thread->c.chan.countusers--;

				/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
				if(thread->c.chan.settings&CS_INHABIT_BIT){
					Users 		*my_chanserv 		= GetUser(cs->nick);
					Uchannels 	*my_chanserv_channel= NULL;

					if(my_chanserv!=NULL)
						my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

					if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
						do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
						do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
					}
				}

				/* ChanServ isn't inhabitant, so if there is only one user or less in the channel (the one we want kick now)
				 we have to join the channel to make the ban "permanent" */
				else if(!thread->c.chan.botinchan && !HasBot){
					if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=1 && streqnc(thread->c.chan.serv,"chanserv") && !csjoined && getStayChannel(thread,thread->c.chan.name)==NULL){ /* CHANGED */
						do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
						do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
						csjoined=1;
						addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
					}
					//else if(!csjoined) csjoined=3;
				}

				/* If ChanServ isn't inhabitant in that channel and user count in channel is lower
				 or equal to the minimum number of users for a bot in that channel we add a Stay for a certain time
				 and leave the channel by the time the saytime is reached unless another user joins the channel */
				else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=m->BSMinUsers)
					addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
				/****************************************************************/
				
				do_mode(thread,sender,channel,thread->c.chan.datecreate,"+b %s",ucmodes[user].banmask);
				orion_WRCtranslate_cmd_kick(thread,sender,channel,ucmodes[user].nick,_dynadd("Akicked: %s",ucmodes[user].reason));
				continue;
			}
			/* ***************** */
			
			if(streqnc(thread->c.chan.serv,"chanserv") && stoi(ucmodes[user].access)>0)
				domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,channel));
			else if(streqnc(thread->c.chan.serv,"q") && strcor(ucmodes[user].access,"ahmnoptv"))
				domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),mysql_quote(thread,channel));	

	/* ChanServ on sjoin */
			if(streqnc(thread->c.chan.serv,"chanserv") && cs->loaded){
				/* Set unknown State */
				if(!ucmodes[user].active) unknown_access(ucmodes[user].access,sizeof(ucmodes[user].access));
				
				/* Get Condata Info */
				Condata Cdata;
				bzero(&Cdata,sizeof(Cdata));
				dynncpy(Cdata.nick,ucmodes[user].nick,sizeof(Cdata.nick));
				dynncpy(Cdata.ident,ucmodes[user].ident,sizeof(Cdata.ident));
				dynncpy(Cdata.vhost,ucmodes[user].vhost,sizeof(Cdata.vhost));
				dynncpy(Cdata.rhost,ucmodes[user].rhost,sizeof(Cdata.rhost));
				
				if((thread->c.chan.settings&CS_SUSPEND_BIT || thread->c.chan.settings&CS_FORBID_BIT) && ucmodes[user].rights<is_services_admin){
					/* Manages Bots and ChanServ / Q to stay in Channel for a while */
					thread->c.chan.countusers--;

					/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
					if(thread->c.chan.settings&CS_INHABIT_BIT){
						Users 		*my_chanserv 		= GetUser(cs->nick);
						Uchannels 	*my_chanserv_channel= NULL;

						if(my_chanserv!=NULL)
							my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

						if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
							do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
						}
					}
					/* ChanServ isn't inhabitant, so if there is only one user or less in the channel (the one we want kick now)
					 we have to join the channel to make the ban "permanent" */
					else if(!thread->c.chan.botinchan && !HasBot){
						if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=1 && streqnc(thread->c.chan.serv,"chanserv") && !csjoined && getStayChannel(thread,thread->c.chan.name)==NULL){ /* CHANGED */
							do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
							csjoined=1;
							addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
						}
						//else if(!csjoined) csjoined=3;
					}
					
					/* If ChanServ isn't inhabitant in that channel and user count in channel is lower
					 or equal to the minimum number of users for a bot in that channel we add a Stay for a certain time
					 and leave the channel by the time the saytime is reached unless another user joins the channel */
					else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=m->BSMinUsers)
						addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
					/****************************************************************/
					
					/* Do modes */
					do_mode(thread,sender,thread->c.chan.name,thread->c.chan.datecreate,IRC.OnSuspend);
					do_mode(thread,sender,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&Cdata,CSAnalyseBantype(thread->c.chan.settings)));
					
					if(thread->c.chan.settings&CS_SUSPEND_BIT)		orion_WRCtranslate_cmd_kick(thread,sender,thread->c.chan.name,ucmodes[user].nick,_dynadd("suspended [%s]",thread->c.chan.reason));
					else if(thread->c.chan.settings&CS_FORBID_BIT)	orion_WRCtranslate_cmd_kick(thread,sender,thread->c.chan.name,ucmodes[user].nick,thread->c.chan.reason[0]=='\0' ? "Forbidden" : thread->c.chan.reason);
					continue;
				}
				
				if(stoi(ucmodes[user].access)<=0 && (thread->c.chan.settings&CS_RESTRICTED_BIT) && ucmodes[user].rights<is_services_oper){
					/* Manages Bots and ChanServ / Q to stay in Channel for a while */
					thread->c.chan.countusers--;
					
					/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
					if(thread->c.chan.settings&CS_INHABIT_BIT){
						Users 		*my_chanserv 		= GetUser(cs->nick);
						Uchannels 	*my_chanserv_channel= NULL;

						if(my_chanserv!=NULL)
							my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

						if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
							do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
						}
					}

					/* ChanServ isn't inhabitant, so if there is only one user or less in the channel (the one we want kick now)
					 we have to join the channel to make the ban "permanent" */
					else if(!thread->c.chan.botinchan && !HasBot){
						if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=1 && streqnc(thread->c.chan.serv,"chanserv") && !csjoined && getStayChannel(thread,thread->c.chan.name)==NULL){
							do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
							csjoined=1;
							addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
						}
						//else if(!csjoined) csjoined=3;
					}
					
					/* If ChanServ isn't inhabitant in that channel and user count in channel is lower
					 or equal to the minimum number of users for a bot in that channel we add a Stay for a certain time
					 and leave the channel by the time the saytime is reached unless another user joins the channel */
					else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=m->BSMinUsers)
						addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
					/****************************************************************/
					
					/* Do modes */
					do_mode(thread,sender,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&Cdata,CSAnalyseBantype(thread->c.chan.settings)));
					orion_WRCtranslate_cmd_kick(thread,sender,thread->c.chan.name,ucmodes[user].nick,"You are not permitted to be on this channel.");
					continue;
				}
				
				if(stoi(ucmodes[user].access)<=thread->c.chan.levels.nojoin && ucmodes[user].rights<is_services_oper){
					
					dlog(thread,4,"Cdata.nick = %s\n",Cdata.nick);
					dlog(thread,4,"Cdata.ident = %s\n",Cdata.ident);
					dlog(thread,4,"Cdata.vhost = %s\n",Cdata.vhost);
					dlog(thread,4,"Cdata.rhost = %s\n",Cdata.rhost);
					dlog(thread,4,"CSAnalyseBantype(thread->c.settings) = %d\n",CSAnalyseBantype(thread->c.chan.settings));
					dlog(thread,4,"BanMask = %s\n",BuildBanMask(thread,&Cdata,CSAnalyseBantype(thread->c.chan.settings)));
					
					/* Manages Bots and ChanServ / Q to stay in Channel for a while */
					thread->c.chan.countusers--;

					/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
					if(thread->c.chan.settings&CS_INHABIT_BIT){
						Users 		*my_chanserv 		= GetUser(cs->nick);
						Uchannels 	*my_chanserv_channel= NULL;

						if(my_chanserv!=NULL)
							my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

						if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
							do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
						}
					}

					/* ChanServ isn't inhabitant, so if there is only one user or less in the channel (the one we want kick now)
					 we have to join the channel to make the ban "permanent" */
					else if(!thread->c.chan.botinchan && !HasBot){
						if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=1 && streqnc(thread->c.chan.serv,"chanserv") && !csjoined && getStayChannel(thread,thread->c.chan.name)==NULL){
							do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
							csjoined=1;
							addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
						}
						//else if(!csjoined) csjoined=3;
					}
					
					/* If ChanServ isn't inhabitant in that channel and user count in channel is lower
					 or equal to the minimum number of users for a bot in that channel we add a Stay for a certain time
					 and leave the channel by the time the saytime is reached unless another user joins the channel */
					else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=m->BSMinUsers) /* CHANGED */
						addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
					/****************************************************************/

					do_mode(thread,sender,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&Cdata,CSAnalyseBantype(thread->c.chan.settings)));
					orion_WRCtranslate_cmd_kick(thread,sender,thread->c.chan.name,ucmodes[user].nick,"matched NOJOIN levelsetting");
					continue;
				}
				
				if(stoi(ucmodes[user].access)>IDENTIFIED_NO_ACCESS)
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%d' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),mysql_quote(thread,thread->c.chan.name));
				
				if(stoi(ucmodes[user].access)>=ACCESS_FOUNDER){
					do_raw2mode(thread,sender,thread->c.chan.name,"+qao",ucmodes[user].nick,thread->c.chan.datecreate);
					if(IRC.HasOwner && !strcin(IRC.OwnerMode,ucmodes[user].mode))		straddns(ucmodes[user].mode,"q",sizeof(ucmodes[user].mode));
					if(IRC.HasProtect && !strcin(IRC.ProtectMode,ucmodes[user].mode))	straddns(ucmodes[user].mode,"a",sizeof(ucmodes[user].mode));
					if(!strcin(IRC.OpMode,ucmodes[user].mode))							straddns(ucmodes[user].mode,"o",sizeof(ucmodes[user].mode));
				
				}else if(streqnc(ucmodes[user].nick,thread->c.chan.founder) && ucmodes[user].active){
					do_raw2mode(thread,sender,thread->c.chan.name,"+qao",ucmodes[user].nick,thread->c.chan.datecreate);
					if(IRC.HasOwner && !strcin(IRC.OwnerMode,ucmodes[user].mode))		straddns(ucmodes[user].mode,"q",sizeof(ucmodes[user].mode));
					if(IRC.HasProtect && !strcin(IRC.ProtectMode,ucmodes[user].mode))	straddns(ucmodes[user].mode,"a",sizeof(ucmodes[user].mode));
					if(!strcin(IRC.OpMode,ucmodes[user].mode))							straddns(ucmodes[user].mode,"o",sizeof(ucmodes[user].mode));
				
				}else if(stoi(ucmodes[user].access)>=thread->c.chan.levels.autoop){
					if(IRC.HasProtect && stoi(ucmodes[user].access)>=thread->c.chan.levels.autoprotect && !strcin(IRC.ProtectMode,ucmodes[user].mode) && !strcin(IRC.OpMode,ucmodes[user].mode)){
						do_raw2mode(thread,sender,thread->c.chan.name,"+ao",ucmodes[user].nick,thread->c.chan.datecreate);
						straddns(ucmodes[user].mode,"ao",sizeof(ucmodes[user].mode));
					}else if(!strcin(IRC.OpMode,ucmodes[user].mode)){
						do_raw2mode(thread,sender,thread->c.chan.name,"+o",ucmodes[user].nick,thread->c.chan.datecreate);
						straddns(ucmodes[user].mode,"o",sizeof(ucmodes[user].mode));
					}
						
				}else if(IRC.SupportHalfOp && stoi(ucmodes[user].access)>=thread->c.chan.levels.autohalfop){
					if(IRC.HasProtect && stoi(ucmodes[user].access)>=thread->c.chan.levels.autoprotect && !strcin(IRC.ProtectMode,ucmodes[user].mode) && !strcin(IRC.HOpMode,ucmodes[user].mode)){
						do_raw2mode(thread,sender,thread->c.chan.name,"+ah",ucmodes[user].nick,thread->c.chan.datecreate);
						straddns(ucmodes[user].mode,"ah",sizeof(ucmodes[user].mode));
					}else if(!strcin(IRC.HOpMode,ucmodes[user].mode)){
						do_raw2mode(thread,sender,thread->c.chan.name,"+h",ucmodes[user].nick,thread->c.chan.datecreate);
						straddns(ucmodes[user].mode,"h",sizeof(ucmodes[user].mode));
					}
						
				}else if(stoi(ucmodes[user].access)>=thread->c.chan.levels.autovoice && !strcin(IRC.VMode,ucmodes[user].mode)){
					do_raw2mode(thread,sender,thread->c.chan.name,"+v",ucmodes[user].nick,thread->c.chan.datecreate);
					straddns(ucmodes[user].mode,"v",sizeof(ucmodes[user].mode));
				}
				
				if(HasBot && thread->c.chan.botserv.set&BS_SET_SYMBIOSIS_BIT && stoi(ucmodes[user].access)>=thread->c.chan.levels.greet && ucmodes[user].greet[0]!='\0')
					do_privmsg(thread,sender,thread->c.chan.name,"[%s] %s",ucmodes[user].nick,ucmodes[user].greet);
				
				if(thread->c.chan.fields && thread->c.chan.welcome[0]!='\0')
					msend(thread,0,sender,ucmodes[user].nick,"[%s] %s",thread->c.chan.name,thread->c.chan.welcome);
			}
			
	/* Q on sjoin */
			else if(streqnc(thread->c.chan.serv,"q") && Q->loaded){
				if(!ucmodes[user].active) bzero(&ucmodes[user].access,sizeof(ucmodes[user].access));
				
				if((thread->c.chan.settings&CS_SUSPEND_BIT || thread->c.chan.settings&CS_FORBID_BIT) && ucmodes[user].qlevel<100){
					/* Manages Bots and ChanServ / Q to stay in Channel for a while */
					thread->c.chan.countusers--;

					/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
					if(thread->c.chan.settings&CS_INHABIT_BIT){
						Users 		*my_chanserv 		= GetUser(cs->nick);
						Uchannels 	*my_chanserv_channel= NULL;

						if(my_chanserv!=NULL)
							my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

						if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
							do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
						}
					}
					
					/* ChanServ isn't inhabitant, so if there is only one user or less in the channel (the one we want kick now)
					 we have to join the channel to make the ban "permanent" */
					else if(!thread->c.chan.botinchan && !HasBot){
						if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=1 && streqnc(thread->c.chan.serv,"chanserv") && !csjoined && getStayChannel(thread,thread->c.chan.name)==NULL){
							do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
							csjoined=1;
							addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
						}
						//else if(!csjoined) csjoined=3;
					}
					
					/* If ChanServ isn't inhabitant in that channel and user count in channel is lower
					 or equal to the minimum number of users for a bot in that channel we add a Stay for a certain time
					 and leave the channel by the time the saytime is reached unless another user joins the channel */
					else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=m->BSMinUsers) /* CHANGED */
						addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
					/****************************************************************/
					
					do_mode(thread,sender,thread->c.chan.name,thread->c.chan.datecreate,IRC.OnSuspend);
					do_mode(thread,sender,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",ucmodes[user].vhost);

					if(thread->c.chan.settings&CS_SUSPEND_BIT)		orion_WRCtranslate_cmd_kick(thread,sender,thread->c.chan.name,ucmodes[user].nick,_dynadd("suspended [%s]",thread->c.chan.reason));
					else if(thread->c.chan.settings&CS_FORBID_BIT)	orion_WRCtranslate_cmd_kick(thread,sender,thread->c.chan.name,ucmodes[user].nick,thread->c.chan.reason[0]=='\0' ? "Forbidden" : thread->c.chan.reason);
					continue;
				}
				
				if(strcor("b",ucmodes[user].access)){
					/* Manages Bots and ChanServ / Q to stay in Channel for a while */
					thread->c.chan.countusers--;

					/* if ChanServ should be a channel inhabitant we look if he is already in (maybe it's happining on link and we aren't in yet ^^) */
					if(thread->c.chan.settings&CS_INHABIT_BIT){
						Users 		*my_chanserv 		= GetUser(cs->nick);
						Uchannels 	*my_chanserv_channel= NULL;

						if(my_chanserv!=NULL)
							my_chanserv_channel = GetUserChannel(my_chanserv,thread->c.chan.name);

						if(my_chanserv_channel==NULL || !my_chanserv_channel->isin_chan){
							do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
						}
					}

					/* ChanServ isn't inhabitant, so if there is only one user or less in the channel (the one we want kick now)
					 we have to join the channel to make the ban "permanent" */
					else if(!thread->c.chan.botinchan && !HasBot){
						if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=1 && streqnc(thread->c.chan.serv,"chanserv") && !csjoined && getStayChannel(thread,thread->c.chan.name)==NULL){
							do_join(thread,2,cs->nick,thread->c.chan.name,thread->c.chan.datecreate);
							do_mode(thread,cs->nick,thread->c.chan.name,thread->c.chan.datecreate,"+oaq %s %s %s",cs->nick,cs->nick,cs->nick);
							csjoined=1;
							addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
						}
						//else if(!csjoined) csjoined=3;
					}
					
					/* If ChanServ isn't inhabitant in that channel and user count in channel is lower
					 or equal to the minimum number of users for a bot in that channel we add a Stay for a certain time
					 and leave the channel by the time the saytime is reached unless another user joins the channel */
					else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers<=m->BSMinUsers) /* CHANGED */
						addStayChannel(thread,thread->c.chan.name,sender,thread->c.chan.botinchan,csjoined);
					/****************************************************************/
					
					do_mode(thread,sender,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",ucmodes[user].vhost);
					orion_WRCtranslate_cmd_kick(thread,sender,thread->c.chan.name,ucmodes[user].nick,_dynadd("%s-banned",Q->nick));
					continue;
					
				}else if(strcor("a",ucmodes[user].access)){
					if(strcor("o",ucmodes[user].access)){
						if(IRC.HasProtect && strcor("p",ucmodes[user].access) && !strcor(_dynadd("%c%c",IRC.OpMode,IRC.ProtectMode),ucmodes[user].mode)){
							do_raw2mode(thread,sender,thread->c.chan.name,"+ao",ucmodes[user].nick,thread->c.chan.datecreate);
							straddns(ucmodes[user].mode,"ao",sizeof(ucmodes[user].mode));
						}
						else if(!strcin(IRC.OpMode,ucmodes[user].mode)){
							do_raw2mode(thread,sender,thread->c.chan.name,"+o",ucmodes[user].nick,thread->c.chan.datecreate);
							straddns(ucmodes[user].mode,"o",sizeof(ucmodes[user].mode));
						}
					}else if(IRC.SupportHalfOp && strcor("h",ucmodes[user].access)){
						if(IRC.HasProtect && strcor("p",ucmodes[user].access) && !strcor(_dynadd("%c%c",IRC.HOpMode,IRC.ProtectMode),ucmodes[user].mode)){
							do_raw2mode(thread,sender,thread->c.chan.name,"+ah",ucmodes[user].nick,thread->c.chan.datecreate);
							straddns(ucmodes[user].mode,"ah",sizeof(ucmodes[user].mode));
						}
						else if(!strcin(IRC.HOpMode,ucmodes[user].mode)){
							do_raw2mode(thread,sender,thread->c.chan.name,"+h",ucmodes[user].nick,thread->c.chan.datecreate);
							straddns(ucmodes[user].mode,"h",sizeof(ucmodes[user].mode));
						}
					}else if(strcor("v",ucmodes[user].access) && !strcin(IRC.VMode,ucmodes[user].mode)){
						do_raw2mode(thread,sender,thread->c.chan.name,"+v",ucmodes[user].nick,thread->c.chan.datecreate);
						straddns(ucmodes[user].mode,"v",sizeof(ucmodes[user].mode));
					}
				}
				
				if(strcor("w",thread->c.chan.flags) && thread->c.chan.welcome[0]!='\0')
					msend(thread,0,sender,ucmodes[user].nick,"[%s] %s",thread->c.chan.name,thread->c.chan.welcome);
				if(strcor("c",thread->c.chan.flags) && thread->c.chan.alimit!=0)
					addLimit(thread,thread->c.chan.name,thread->c.chan.datecreate,strcor("c",thread->c.chan.flags),thread->c.chan.serv,thread->c.chan.alimit);
				
				if(strcor("o",thread->c.chan.flags) && !strcor("o",ucmodes[user].mode)){
					do_raw2mode(thread,sender,thread->c.chan.name,"+o",ucmodes[user].nick,thread->c.chan.datecreate);
					straddns(ucmodes[user].mode,"o",sizeof(ucmodes[user].mode));
				}
				else if(strcor("v",thread->c.chan.flags) && !strcor("v",ucmodes[user].mode)){
					do_raw2mode(thread,sender,thread->c.chan.name,"+v",ucmodes[user].nick,thread->c.chan.datecreate);
					straddns(ucmodes[user].mode,"v",sizeof(ucmodes[user].mode));
				}
			}
			
/* MLOCK FORCE */
		j=0;
		if(~thread->c.chan.settings&CS_INHABIT_BIT && users_in_chan == 0 && thread->c.chan.serv[0]!='\0' && (thread->c.chan.mlock_positive[0]!='\0' || thread->c.chan.mlock_negative[0]!='\0')){
			bzero(&modes,sizeof(modes));
			bzero(&search,sizeof(search));

			straddnc(search,'+',sizeof(search));
			if(!strcin(IRC.RegMode,thread->c.chan.mlock_positive) && IRC.ChanRegMode) straddnc(search,IRC.RegMode,sizeof(search));
			straddns(search,thread->c.chan.mlock_positive,sizeof(search));
			sort(search);
			straddnc(search,'-',sizeof(search));
			sort(thread->c.chan.mlock_negative);
			straddns(search,thread->c.chan.mlock_negative,sizeof(search));

			dynncpy(modes,search,sizeof(modes));

			while(search[j]!='\0'){
				if(search[j]==IRC.LimitCMode){
					straddnc(modes,' ',sizeof(modes));
					straddns(modes,_dynadd("%ld",thread->c.chan.mlock_limit),sizeof(modes));
				}
				else if(search[j]==IRC.FloodCMode){
					straddnc(modes,' ',sizeof(modes));
					straddns(modes,thread->c.chan.mlock_flood,sizeof(modes));
				}
				else if(search[j]==IRC.KeyCMode){
					straddnc(modes,' ',sizeof(modes));
					straddns(modes,thread->c.chan.mlock_key,sizeof(modes));
				}
				j++;
			}

			if(modes[0]!='\0') do_mode(thread,sender,thread->c.chan.name,thread->c.chan.datecreate == 0 ? time(NULL) : thread->c.chan.datecreate,"%s",modes);
		}
		else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.serv[0]!='\0' && users_in_chan == 0 && IRC.ChanRegMode)
			do_mode(thread, sender, thread->c.chan.name, thread->c.chan.datecreate == 0 ? time(NULL) : thread->c.chan.datecreate, IRC.ModeOnChannelReg);
		else if(~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.serv[0]!='\0' && users_in_chan == 0 && ~thread->c.chan.settings&CS_SUSPEND_BIT && thread->c.chan.settings&CS_KEEPTOPIC_BIT && thread->c.chan.last_topic[0]!='\0')
			do_topic(thread, sender, thread->c.chan.last_topic_setter[0]=='\0' ? sender : thread->c.chan.last_topic_setter, thread->c.chan.name, time(NULL), thread->c.chan.last_topic);
			
/* MLOCK FORCE END */
			
			if(strcor("o",ucmodes[user].mode)){
				
				//printf("stoi(ucmodes[user].access) = %d\n",stoi(ucmodes[user].access));
				//printf("thread->c.chan.levels.autodeop = %d\n",thread->c.chan.levels.autodeop);
				
				if(thread->c.chan.fields && strcor("b",thread->c.chan.flags) && streqnc(thread->c.chan.serv,"q")){
					if(!strcor("o",ucmodes[user].access) && !strcor("o",thread->c.chan.flags)){
						do_raw2mode(thread,sender,channel,"-ho",ucmodes[user].nick,thread->c.chan.datecreate);
						strdelc(ucmodes[user].mode,IRC.OpMode);
						if(IRC.SupportHalfOp) 
							strdelc(ucmodes[user].mode,IRC.HOpMode);
					}
					
					else if(!strcor("h",ucmodes[user].access) && !strcor("o",thread->c.chan.flags) && IRC.SupportHalfOp){
						do_raw2mode(thread,sender,channel,"-h",ucmodes[user].nick,thread->c.chan.datecreate);
						strdelc(ucmodes[user].mode,IRC.HOpMode);
					}
					
					else if(!strcor("v",ucmodes[user].access) && !strcor("v",thread->c.chan.flags)){
						do_raw2mode(thread,sender,channel,"-v",ucmodes[user].nick,thread->c.chan.datecreate);
						strdelc(ucmodes[user].mode,IRC.VMode);
					}
					
				}
				else if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && streqnc(thread->c.chan.serv,"chanserv")){
					if(stoi(ucmodes[user].access)<thread->c.chan.levels.opdeop || stoi(ucmodes[user].access)<=IDENTIFIED_NO_ACCESS){
						do_raw2mode(thread,sender,channel,"-qao",ucmodes[user].nick,thread->c.chan.datecreate);
						strdelc(ucmodes[user].mode,IRC.OpMode);
						if(IRC.HasProtect) 
							strdelc(ucmodes[user].mode,IRC.ProtectMode);
						if(IRC.HasOwner) 
							strdelc(ucmodes[user].mode,IRC.OwnerMode);	
					}
				}/* AUTODEOP LEVEL CHECK*/
				else if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && stoi(ucmodes[user].access)<thread->c.chan.levels.autodeop && streqnc(thread->c.chan.serv,"chanserv")){
					do_raw2mode(thread,sender,channel,"-qao",ucmodes[user].nick,thread->c.chan.datecreate);
					strdelc(ucmodes[user].mode,IRC.OpMode);
					if(IRC.HasProtect) 
						strdelc(ucmodes[user].mode,IRC.ProtectMode);
					if(IRC.HasOwner) 
						strdelc(ucmodes[user].mode,IRC.OwnerMode);								
				}
			}
									
			if(IRC.HasOwner && strcin(IRC.OwnerMode,ucmodes[user].mode)){
									
				if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && streqnc(thread->c.chan.serv,"chanserv")){
					if(stoi(ucmodes[user].access)<thread->c.chan.levels.opdeop){
						do_raw2mode(thread,sender,channel,"-q",ucmodes[user].nick,thread->c.chan.datecreate);
						strdelc(ucmodes[user].mode,IRC.OwnerMode);
					}
				}
			}
			
			if(IRC.HasProtect && strcin(IRC.ProtectMode,ucmodes[user].mode)){
									
				if(thread->c.chan.fields && thread->c.chan.settings&CS_SECUREOPS_BIT && streqnc(thread->c.chan.serv,"chanserv")){
					if(stoi(ucmodes[user].access)<thread->c.chan.levels.protect){
						do_raw2mode(thread,sender,channel,"-a",ucmodes[user].nick,thread->c.chan.datecreate);
						strdelc(ucmodes[user].mode,IRC.ProtectMode);
					}
				}
			}
			
			sort(ucmodes[user].mode);
			
			/* Add the user it's new channel */
			addUserChannel(GetUser(ucmodes[user].nick), channel);
				
			if(DuringServerlink && SJOIN_CACHING) doBufferedInsert(thread,1,SQL_BUFFER_SJOIN,"INSERT INTO `orion_chan_users` (`nick`,`channel`,`modes`,`faketype`) VALUES ('%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `modes`=VALUES(`modes`)",mysql_quote(thread,ucmodes[user].nick),mysql_quote(thread,channel),mysql_quote(thread,ucmodes[user].mode),fake);
			else domysql(thread,"INSERT INTO `orion_chan_users` (`nick`,`channel`,`modes`,`faketype`) VALUES ('%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `modes`=VALUES(`modes`)",mysql_quote(thread,ucmodes[user].nick),mysql_quote(thread,channel),mysql_quote(thread,ucmodes[user].mode),fake);
		}
	}
    return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
