/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file slink.c
 *   @brief This is the Module file for SLINK (Server Side) linking process.
 *
 *   It contains all functions which are neccessary to process OUR
 *   Serverlinks!
 *
 *   The SLINK File is used to process Serverlinks between
 *   us and our servers. (Protocol-Process-File -> PPF)
 *   (Netinfo - Information received after linking is done - execute an Event and manage Glines.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief on_serviceslink:  Called when a Serverlink occurs (Handles Sockscontrol and some other stuff)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_serviceslink(Threads *thread){
	/* We tell all modules we are currently linking - Sockscontrol stops process while linking */
	_ExecuteEvent(NULL,NULL,"on_protoctl");
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief on_serviceslink_done:  Called when a Serverlink finished (Handles Sockscontrol and some other stuff)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_serviceslink_done(Threads *thread){

	/* Global variables */
	deepSQL gline;
	deepSQL sqline;
	deepSQL szline;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&gline,sizeof(gline));
	bzero(&sqline,sizeof(sqline));
	bzero(&szline,sizeof(szline));

	/* Call netinfo event, THIS MEANS WE FINISHED LINKING - FOR ALL MODULES */
	/* This is HIGHLY IMPORTANT FOR SOCKSCONTROL (continues processing again)!! */
	_ExecuteEvent(NULL,NULL,"on_netinfo");
	
	domysql(thread,"DELETE `orion_os_szlines`,`orion_os_sqlines`,`orion_core_glines` FROM `orion_os_szlines` LEFT OUTER JOIN `orion_os_sqlines` ON (`orion_os_sqlines`.`duration`=`orion_os_szlines`.`duration`) LEFT OUTER JOIN `orion_core_glines` ON (`orion_core_glines`.`duration`=`orion_os_szlines`.`duration`) WHERE `orion_os_szlines`.`duration`<=%ld",time(NULL));
	
/* GLINES */
	gline.res    = doSelectMySQL(thread,"SELECT `gline`,`duration`,`addedby`,`reason` FROM `orion_core_glines` WHERE `rng`='0'");
	gline.fields = mysql_num_rows(gline.res);

	for(i=0;i<gline.fields;i++){
		gline.row = mysql_fetch_row(gline.res);
		
		if(streqnc(gline.row[2],Q->nick) && !streq(gline.row[0],"*") && !streq(gline.row[0],"*@*")){
			if(time(NULL)>=stol(gline.row[1]))
				//Time now >= Expiretime .. so we should expire NOW!
				do_ungline(thread,m->servername,gline.row[0],_dynadd("%s!%s@%s",Q->nick,Q->ident,Q->host),time(NULL));
			else
				do_gline(thread,m->servername,gline.row[0],_dynadd("%s!%s@%s",Q->nick,Q->ident,Q->host),stol(gline.row[1]),time(NULL),gline.row[3]);
		}
		else if(streqnc(gline.row[2],O->nick) && !streq(gline.row[0],"*") && !streq(gline.row[0],"*@*")){
			if(time(NULL)>=stol(gline.row[1]))
				//Time now >= Expiretime .. so we should expire NOW!
				do_ungline(thread,m->servername,gline.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),time(NULL));
			else
				do_gline(thread,m->servername,gline.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),stol(gline.row[1]),time(NULL),gline.row[3]);
		}
		else if(streqnc(gline.row[2],SC->nick)){
			if(time(NULL)>=stol(gline.row[1]))
				//Time now >= Expiretime .. so we should expire NOW!
				do_ungline(thread,m->servername,gline.row[0],_dynadd("%s!%s@%s",SC->nick,SC->ident,SC->host),time(NULL));
			else
				do_gline(thread,m->servername,gline.row[0],_dynadd("%s!%s@%s",SC->nick,SC->ident,SC->host),stol(gline.row[1]),time(NULL),gline.row[3]);
		}
		else if(streqnc(gline.row[2],m->nick)){
			if(time(NULL)>=stol(gline.row[1]))
				//Time now >= Expiretime .. so we should expire NOW!
				do_ungline(thread,m->servername,gline.row[0],_dynadd("%s!%s@%s",m->nick,m->ident,m->host),time(NULL));
			else
				do_gline(thread,m->servername,gline.row[0],_dynadd("%s!%s@%s",m->nick,m->ident,m->host),stol(gline.row[1]),time(NULL),gline.row[3]);
		}
	}
	mysql_free_result_myr(gline.res);
	mysql_free_quotes(thread);
	
/* SQLINE */
	sqline.res    = doSelectMySQL(thread,"SELECT `qline`,`duration`,`addedby`,`reason` FROM `orion_os_sqlines`");
	sqline.fields = mysql_num_rows(sqline.res);

	for(i=0;i<sqline.fields;i++){
		sqline.row = mysql_fetch_row(sqline.res);
			//do_sqline(thread,m->servername,Q->nick,0,(long)time(NULL),"nick used by orion");
		if(streqnc(sqline.row[2],m->servername)){
			if(time(NULL)>=stol(sqline.row[1]))
				//Time now >= Expiretime .. so we should expire NOW!
				do_unsqline(thread,m->servername,m->servername,sqline.row[0]);
			else
				do_sqline(thread,m->servername,sqline.row[0],stol(sqline.row[1]),time(NULL),sqline.row[3]);
		}
		else if(streqnc(sqline.row[2],O->nick)){
			if(time(NULL)>=stol(sqline.row[1]))
				do_unsqline(thread,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),m->servername,sqline.row[0]);
			else
				do_sqline(thread,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),sqline.row[0],stol(sqline.row[1]),time(NULL),sqline.row[3]);
		}
		else if(streqnc(sqline.row[2],SC->nick)){
			if(time(NULL)>=stol(sqline.row[1]))
				do_unsqline(thread,_dynadd("%s!%s@%s",SC->nick,SC->ident,SC->host),m->servername,sqline.row[0]);
			else
				do_sqline(thread,_dynadd("%s!%s@%s",SC->nick,SC->ident,SC->host),sqline.row[0],stol(sqline.row[1]),time(NULL),sqline.row[3]);
		}
		else if(streqnc(sqline.row[2],m->nick)){
			if(time(NULL)>=stol(sqline.row[1]))
				do_unsqline(thread,_dynadd("%s!%s@%s",m->nick,m->ident,m->host),m->servername,sqline.row[0]);
			else
				do_sqline(thread,_dynadd("%s!%s@%s",m->nick,m->ident,m->host),sqline.row[0],stol(sqline.row[1]),time(NULL),sqline.row[3]);
		}
	}
	mysql_free_result_myr(sqline.res);
	mysql_free_quotes(thread);
	
/* SZLINE */
	szline.res    = doSelectMySQL(thread,"SELECT `zline`,`duration`,`addedby`,`reason` FROM `orion_os_szlines`");
	szline.fields = mysql_num_rows(szline.res);

	for(i=0;i<szline.fields;i++){
		szline.row = mysql_fetch_row(szline.res);
			//do_szline(thread,m->servername,Q->nick,0,(long)time(NULL),"nick used by orion");
		if(streqnc(szline.row[2],m->servername)){
			if(time(NULL)>=stol(szline.row[1]))
				do_unszline(thread,m->servername,szline.row[0],m->servername,time(NULL));
			else
				do_szline(thread,m->servername,szline.row[0],szline.row[2],stol(szline.row[1]),time(NULL),szline.row[3]);
		}
		else if(streqnc(szline.row[2],O->nick)){
			if(time(NULL)>=stol(szline.row[1]))
				do_unszline(thread,m->servername,szline.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),time(NULL));
			else
				do_szline(thread,m->servername,szline.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),stol(szline.row[1]),time(NULL),szline.row[3]);
		}
		else if(streqnc(szline.row[2],SC->nick)){
			if(time(NULL)>=stol(szline.row[1]))
				do_unszline(thread,m->servername,szline.row[0],_dynadd("%s!%s@%s",SC->nick,SC->ident,SC->host),time(NULL));
			else
				do_szline(thread,m->servername,szline.row[0],_dynadd("%s!%s@%s",SC->nick,SC->ident,SC->host),stol(szline.row[1]),time(NULL),szline.row[3]);
		}
		else if(streqnc(szline.row[2],m->nick)){
			if(time(NULL)>=stol(szline.row[1]))
				do_unszline(thread,m->servername,szline.row[0],_dynadd("%s!%s@%s",m->nick,m->ident,m->host),time(NULL));
			else
				do_szline(thread,m->servername,szline.row[0],_dynadd("%s!%s@%s",m->nick,m->ident,m->host),stol(szline.row[1]),time(NULL),szline.row[3]);
		}
	}
	mysql_free_result_myr(szline.res);
	mysql_free_quotes(thread);

	dlog(thread,2,"fnetinfo(); done\n");
	netinfo = true;
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
