/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 210                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 18:15:28 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file string.c
 *   @brief This is the Module file for string manipulations.
 *
 *   It contains all functions which are neccessary to
 *   manipulate strings, pointer arrays.
 *
 *   main string functions, they all accept '0' or NULL values as parameter.
 */
/*  ************************************************************************************************  */

#include "global.h"
#include "services.h"
#include <math.h>

/**
 * @brief CleanSocketString:  Cleans a string (escapes %s %d %f %c)
 *
 * @param string The string that should be escaped
 * @param strsize The size fo "string"
 * @return void
 **/
void CleanSocketString(char *string, size_t strsize){
	char temp[BUFSIZE+2];
	size_t b=0, i=0;
	
    //strclean(string);
	
	while(string[i]!='\0'){
		if(string[i]=='%' && string[i+1]=='s'){
			temp[b]='\0';
			straddns(temp,"%%s",sizeof(temp));
			b+=3;
			i+=2;
			// %%s %s
        }else if(string[i]=='%' && string[i+1]=='d'){
			temp[b]='\0';
			straddns(temp,"%%d",sizeof(temp));
			b+=3;
			i+=2;
			// %%s %s
        }else if(string[i]=='%' && string[i+1]=='f'){
			temp[b]='\0';
			straddns(temp,"%%f",sizeof(temp));
			b+=3;
			i+=2;
			// %%s %s
        }else if(string[i]=='%' && string[i+1]=='c'){
			temp[b]='\0';
			straddns(temp,"%%c",sizeof(temp));
			b+=3;
			i+=2;
			// %%s %s
        }else{
            temp[b]=string[i];
            b++;i++;
        }
	}
	temp[b]='\0';
	dynncpy(string,temp,strsize);
	strclean(string);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief strclean:  Deletes all double Spaces in a string
 *
 * @param receive The string that should be cleaned
 * @return void
 **/
void strclean(char *receive){
	size_t i=0, b=0;
	if(receive==NULL) return;

	while(receive[i]!='\0'){
		if(receive[i]==' ' && receive[i+1]==' '){ i++; continue; }
		if(receive[i]==' ' && receive[i+1]=='\n'){ i++; continue; }
		if(receive[i]=='\n' && receive[i+1]!='\0'){ i++; continue; }
		if(receive[i]=='\r' && receive[i+1]=='\n'){ i++; continue; }
		
		else if(receive[i]=='\n' && receive[i+1]=='\0'){ receive[b]='\0'; break; }
		else{
			receive[b]=receive[i];
			i++;b++;
		}
	}
	receive[b]='\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief c_lowercase:  Lowers a whole string
 *
 * @param string The string that should be lowered
 * @return void
 **/
void c_lowercase(char *string){
	size_t i=0;
	if(string==NULL) return;

	while(string[i]!='\0'){
		string[i]=tolower(string[i]);
		i++;
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief lc:  Lowers a whole string
 *
 * @param str The string that should be lowered
 * @return char* Returns the a pointer to a static - where the lowered string is saved
 **/
#ifdef USE_THREADS
char* lc(Threads *thread, char *str){
	size_t i=0;
	if(str==NULL) return NULL;

	bzero(&thread->lc,sizeof(thread->lc));
	while(str[i]!='\0'){
		thread->lc[i] = tolower(str[i]);
		i++;
	}
	thread->lc[i]='\0';
	return thread->lc;
}
#else
char* lc(char *str){
	static char buf[BUFFER];
	size_t i=0;
	if(str==NULL) return NULL;

	bzero(buf,sizeof(buf));
	while(str[i]!='\0'){
		buf[i] = tolower(str[i]);
		i++;
	}
	buf[i]='\0';
	return buf;
}
#endif

/*  ************************************************************************************************  */

/**
 * @brief uc:  Uppers a whole string
 *
 * @param str The string that should be uppered
 * @return char* Returns the a pointer to a static - where the uppered string is saved
 **/
#ifdef USE_THREADS
char* uc(Threads *thread, char *str){
	size_t i=0;
	if(str==NULL) return NULL;

	bzero(&thread->uc,sizeof(thread->uc));
	while(str[i]!='\0'){
		thread->uc[i] = toupper(str[i]);
		i++;
	}
	thread->uc[i]='\0';
	return thread->uc;
}
#else
char* uc(char *str){
	static char buf[BUFFER];
	size_t i=0;
	if(str==NULL) return NULL;

	bzero(&buf,sizeof(buf));
	while(str[i]!='\0'){
		buf[i] = toupper(str[i]);
		i++;
	}
	buf[i]='\0';
	return buf;
}
#endif

/*  ************************************************************************************************  */

/**
 * @brief strrepc:  Replaces all occurances of one character with another
 *
 * @param search_string The string that should be searched for the character to be replaced
 * @param nower The character for which should be searched, to replace it with "newchar"
 * @param newchar The newchar which should be written where "nower" was found
 * @return void
 **/
void strrepc(char *search_string, char nower, char newchar){
	size_t i=0;
	if(search_string==NULL || nower=='\0' || newchar=='\0') return;

	while(search_string[i]!='\0'){
		if(search_string[i]==nower) search_string[i]=newchar;
		i++;
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief strrepcof:  Replaces the first occurance of one character with another
 *
 * @param search_string The string that should be searched for the character to be replaced
 * @param nower The character for which should be searched, to replace it with "newchar"
 * @param newchar The newchar which should be written where "nower" was found
 * @return void
 **/
void strrepcof(char *search_string, char nower, char newchar){
	size_t i=0;
	if(search_string==NULL || nower=='\0' || newchar=='\0') return;

	while(search_string[i]!='\0'){
		if(search_string[i]==nower){
			search_string[i]=newchar;
			break;
		}
		i++;
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief straddc:  Adds a character to the end of a string
 *
 * @param string The string where the character should be added to
 * @param addchar The character that should be added
 * @return void
 **/
void straddc(char *string, char addchar){
	size_t i=0;

	if(addchar=='\0' || string==NULL) return;
	while(1){
		if(string[i]=='\0'){
			string[i]=addchar;
			string[i+1]='\0';
			break;
		}
		i++;
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief stradds:  Adds a string to the end of a string
 *
 * @param string The string where another string should be added to
 * @param addstring The string that should be added to "string"
 * @return void
 **/
void stradds(char *string, char *addstring){
	size_t i=0, a=0;
	if(addstring==NULL || string==NULL) return;

	while(1){
		if(string[i]=='\0'){
			while(addstring[a]!='\0'){
				string[i+a]=addstring[a];
				a++;
			}
			string[i+a]=addstring[a];
			break;
		}
		i++;
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief straddnc:  Adds a character to the end of a string (n is a maxsize)
 *
 * @param string The string where the character should be added to
 * @param addchar The character that should be added
 * @param strsize The sizeof() string - to avoid Buffer overflows
 * @return void
 **/
void straddnc(char *string, char addchar, size_t strsize){
	size_t i=0;

	if(addchar=='\0' || string==NULL) return;
	while(1){
		if(string[i]=='\0'){
			if(i+1<strsize){
				string[i]=addchar;
				string[i+1]='\0';
			}
			break;
		}
		i++;
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief straddns:  Adds a string to the end of a string (n is a maxsize)
 *
 * @param string The string where another string should be added to
 * @param addstring The string that should be added to "string"
 * @param strsize The sizeof() string - to avoid Buffer overflows
 * @return void
 **/
void straddns(char *string, char *addstring, size_t strsize){
	size_t i=0, a=0;
	if(addstring==NULL || string==NULL) return;

	while(1){
		if(string[i]=='\0'){
			while(addstring[a]!='\0'){
				if(i+a<strsize){
					string[i+a]=addstring[a];
					a++;
				}else break;
			}
			
			if(i+a<strsize)
				string[i+a]=addstring[a];
			else string[i+a-1]='\0';
			
			break;
		}
		i++;
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief straddsni:  Adds characters from addstring to string but only if they aren't in string now
 *
 * @param string The string where another string should be added to
 * @param addstring The string that should be added to "string"
 * @return void
 **/
void straddsni(char *string, char *addstring){
	char notin[300];
	size_t i=0, a=0, b=0;
	int matched = 0;
	
	bzero(&notin,sizeof(notin));
	if(addstring==NULL || string==NULL) return;

	while(addstring[i]!='\0'){
		while(string[a]!='\0'){
			if(string[a] == addstring[i]){
				matched = 1;
				break;
			}
			a++;
		}
		if(matched==0){
			notin[b]=addstring[i];
			b++;
		}
		matched = 0;
		i++;
	}
	notin[b]='\0';
	
	stradds(string,notin);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief straddnsni:  Adds characters from addstring to string but only if they aren't in string now
 *
 * @param string The string where another string should be added to
 * @param addstring The string that should be added to "string"
 * @param strsize The sizeof() string - to avoid Buffer overflows
 * @return void
 **/
void straddnsni(char *string, char *addstring, size_t strsize){
	char notin[300];
	size_t i=0, a=0, b=0;
	int matched = 0;
	
	bzero(&notin,sizeof(notin));
	if(addstring==NULL || string==NULL) return;

	while(addstring[i]!='\0'){
		while(string[a]!='\0'){
			if(string[a] == addstring[i]){
				matched = 1;
				break;
			}
			a++;
		}
		if(matched==0){
			notin[b]=addstring[i];
			b++;
		}
		matched = 0;
		i++;
	}
	notin[b]='\0';
	
	if(dynlen(notin)+dynlen(string)+1 < strsize)
		straddns(string,notin,strsize);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief strsplit:  Splits a string into a pointer-array at the occurance of a character
 *
 * @param search_string The string that we want to split up
 * @param param The pointer array that points to the terms after splitting
 * @param repchar The character at which's occurance the string should be split
 * @return size_t Returns the number of terms in which "search_string" was split
 **/
size_t strsplit(char *search_string, char *param[], char repchar){
	size_t ip	= 0;  // ip zeigt auf Wort in param
	size_t i	= 0;
	if(search_string==NULL || repchar=='\0'){ param[0]=NULL; return 0; }

	else{
		param[ip++]=search_string;
		while(search_string[i]!='\0'){
			if(search_string[i]==repchar && i!=0){
				search_string[i]='\0';
				param[ip++]=&search_string[i+1];
			}
			i++;
		}
		param[ip]=NULL;
		return ip;
	}
}

/*  ************************************************************************************************  */

/**
 * @brief substr:  Overwrites the n-th character in a string
 *
 * @param string The string where the character should be overwritten
 * @param position The position of the character which should be overwritten
 * @return void
 **/
void substr(char *string, size_t position){
	size_t i=0;
	size_t b=0;
	if(string==NULL) return;

	while(i<dynlen(string)){
		if(i==position){ i++; continue; }
		string[b]=string[i];
		b++; i++;
	}
	string[b]='\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief strdelc:  Deletes all occurances of a charakter out of a string
 *
 * @param search_string The string where the character should be deleted
 * @param delchar The character which should be deleted
 * @return void
 **/
void strdelc(char *search_string, char delchar){
	size_t i=0, b=0;
	if(search_string==NULL || delchar=='\0') return;

	while(search_string[i]!='\0'){
		if(search_string[i]==delchar){ i++; continue; }
		search_string[b]=search_string[i];
		i++; b++;
	}
	search_string[b]='\0';
	return;
}


/*  ************************************************************************************************  */

/**
 * @brief strdelcof:  Deletes the furst occurance of a charakter out of a string
 *
 * @param search_string The string where the character should be deleted
 * @param delchar The character which should be deleted
 * @return void
 **/
void strdelcof(char *search_string, char delchar){
	size_t i=0,b=0;
	int k=0;
	if(search_string==NULL || delchar=='\0') return;

	while(search_string[i]!='\0'){
		if(!k && search_string[i]==delchar){ i++; k=1; continue; }
		search_string[b]=search_string[i];
		i++; b++;
	}
	search_string[b]='\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief strdels:  Deletes all occurances of a string-A out of a string-B
 *
 * @param search_string The string where the character should be deleted
 * @param search The mode for deleting (n = normal plain, w = wildmask)
 * @param delchar The string which should be deleted from "search_string"
 * @return void
 **/
void strdels(char *search_string, char search, char *delchar){
	size_t i=0,c=0,b=0;
	int d=0;

	if(search_string==NULL || delchar==NULL || search=='\0') return;

	while(search_string[i]!='\0'){
		if(search_string[i]==delchar[0]){
			for(c=0;delchar[c]!='\0';c++){
				if(search_string[i+c]!=delchar[c]){
					if(search=='n') d=0;
					break;
				}
				if(search=='n') d=1;
			}
			if(search=='n'){ if(d) i+=c; }
			else if(search=='w'){ i+=c; }
		}
		search_string[b]=search_string[i];
		i++;b++;
	}
	search_string[b]='\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief strins:  Proofes the occurance of string-B in string-A (case sensivity)
 *
 * @param string_to_search The string where the other string is searched for
 * @param search_string The string for which should be searched in "string_to_search"
 * @return bool Returns 1 if string-B is in string-A, otherwise 0
 **/
bool strins(char *string_to_search, char *search_string){
	size_t i=0,b=0;
	if(search_string==NULL || string_to_search==NULL) return 0;

	while(string_to_search[i]!='\0'){
		if(string_to_search[i]==search_string[0]){
			for(b=0;search_string[b]!='\0' && string_to_search[i+b]!='\0';b++){
				if(string_to_search[i+b]==search_string[b]){
					continue;
				}else
					return 0;
			}
			if(string_to_search[i+b]!='\0') return 1;
		}
		i++;
	}
	return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief strinsnc:  Proofes the occurance of string-B in string-A (case UNsensivity)
 *
 * @param string_to_search The string where the other string is searched for
 * @param search_string The string for which should be searched in "string_to_search"
 * @return bool Returns 1 if string-B is in string-A, otherwise 0
 **/
bool strinsnc(char *string_to_search, char *search_string){
	size_t i=0,b=0;
	if(search_string==NULL || string_to_search==NULL) return 0;

	while(string_to_search[i]!='\0'){
		if(tolower(string_to_search[i])==tolower(search_string[0])){
			for(b=0;search_string[b]!='\0' && string_to_search[i+b]!='\0';b++){
				if(tolower(string_to_search[i+b])==tolower(search_string[b])){
					continue;
				}else
					return 0;
			}
			if(string_to_search[i+b]!='\0') return 1;
		}
		i++;
	}
	return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief streq:  Proofes the occurance of string-B in string-A (case sensivity)
 *
 * @param str1 The string which should be compared with "str2"
 * @param str2 The string which should be compared with "str1"
 * @return bool Returns 1 if string-B is string-A, otherwise 0
 **/
bool streq(char *str1, char *str2){
	size_t a=0;
	if(str1!=NULL && str2!=NULL){
		while(str1[a]==str2[a]){
			if(str1[a]=='\0' && str2[a]=='\0') return 1;
			a++;
		}
		return 0;
	}else return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief streqnc:  Proofes the occurance of string-B in string-A (case UNsensivity)
 *
 * @param str1 The string which should be compared with "str2"
 * @param str2 The string which should be compared with "str1"
 * @return bool Returns 1 if string-B is string-A, otherwise 0
 **/
bool streqnc(char *str1, char *str2){
	size_t a=0;
	if(str1!=NULL && str2!=NULL){
		while(tolower(str1[a])==tolower(str2[a])){
			if(str1[a]=='\0' && str2[a]=='\0') return 1;
			a++;
		}
		return 0;
	}else return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief strcor:  Proofes if any character of "chr" is in "string" (case sensivity)
 *
 * @param string The string in which should be searched
 * @param chr The list of character in which one should match a character in "string"
 * @return bool Returns 1 if a character matched and 0 if not
 **/
bool strcor(char *string, char *chr){
	size_t b=0, i=0;
	if(string==NULL || chr==NULL) return 0;

	while(string[b]!='\0'){
		for(i=0;chr[i]!='\0';i++){
			if(string[b]==chr[i]) return 1;
		}
		b++;
	}
	return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief stricor:  Proofes if any character of "chr" is in "string" (case unsensivity)
 *
 * @param string The string in which should be searched
 * @param chr The list of character in which one should match a character in "string"
 * @return bool Returns 1 if a character matched and 0 if not
 **/
bool stricor(char *string, char *chr){
	size_t b=0, i=0;
	if(string==NULL || chr==NULL) return 0;

	while(string[b]!='\0'){
		for(i=0;chr[i]!='\0';i++){
			if(tolower(string[b])==tolower(chr[i])){
				return 1;
			}
		}
		b++;
	}
	return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief strcin:  Proofes if a given character is in "str" (case sensivity)
 *
 * @param str The string in which should be searched
 * @param chr The character which should occur in "str"
 * @return bool Returns 1 if the character occured and 0 if not
 **/
bool strcin(char chr, char *str){
	size_t i=0;
	if(str==NULL || chr=='\0') return 0;

	for(i=0;str[i]!='\0';i++){
		if(str[i]==chr) return 1;
	}
	return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief stricin:  Proofes if a given character is in "str" (case unsensivity)
 *
 * @param str The string in which should be searched
 * @param chr The character which should occur in "str"
 * @return bool Returns 1 if the character occured and 0 if not
 **/
bool stricin(char chr, char *str){
	size_t i=0;
	if(str==NULL || chr=='\0') return 0;

	for(i=0;str[i]!='\0';i++){
		if(tolower(str[i])==tolower(chr)) return 1;
	}
	return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief strcsea:  Proofes if a given mask (reg expression) is in "string" (case sensivity)
 *
 * @param string The string to search for the regular expression
 * @param search_in The regular expression
 * @return bool Returns 1 if the (reg expression) matched and 0 if not
 **/
bool strcsea(char *string,char *search_in){
	char search[200];
	int or[20];
	int a=0,b=0,c=0,i=0,d=0;
	char *array[20];
	if(string==NULL || search_in==NULL) return 0;

	bzero(search,sizeof(search));

	dyncpy(search,search_in);
	a=strsplit(search,array,'&');
	if(a!=0){
		for(i=0;i<a;i++){
			char value[50];
			for(b=0;array[i][b]!='\0';b++){
				if(array[i][b]=='|') continue;
				value[c]=array[i][b];
				c++;
			}
			or[d]=strcor(string,value);
			d++;
		}
		b=0;c=0;
		while(b<d){
			if(b==0){ c=or[b]; b++; continue; }
			c=c&or[b];
			b++;
		}
		return c;
	}
	return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief lower:  Lowers a given character
 *
 * @param value The character that should be lowered
 * @return char (char)tolower(value) Returns the lowered character
 **/
char lower(char value){
     return (char)tolower(value);
}

/*  ************************************************************************************************  */

/**
 * @brief strunshift:  Adds a string to the beginning of another string
 *
 * @param addchar The string that should be added at the beginning of "string"
 * @param string The string to which "addchar" should be added at the beginning
 * @param strsize The size of "string" to avoid an overflow
 * @return void
 **/
void strunshift(char *addchar,char *string, size_t strsize){
	size_t i=0;

	if(addchar==NULL || string==NULL) return;
	else if(addchar[0]=='\0' || string[0]=='\0') return;
	
	else if(dynlen(string)+dynlen(addchar)+1 < strsize){
		string[dynlen(string)+dynlen(addchar)+1]='\0';

		for(i=dynlen(string);i>0;i--){
			string[i+dynlen(addchar)-1]=string[i-1];
		}

		for(i=0;i<dynlen(addchar);i++){
			string[i]=addchar[i];
		}
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief digitcol:  Proofes if there are only digits in the given string
 *
 * @param string The string to which should be proofed
 * @return bool Returns 1 if the string consists only of digits and 0 if not
 **/
bool digitcol(char *string){
	int a=1,b=0;
	size_t i=0;
	
	if(string==NULL) return 0;

	while(string[i]!='\0'){
		if(isdigit(string[i])>0) b=1;
		else b=0;
		a=a&b;
		i++;
	}
	return (bool)a;
}

/*  ************************************************************************************************  */

/**
 * @brief alphacol:  Proofes if there are only characters in the given string
 *
 * @param string The string to which should be proofed
 * @return bool Returns 1 if the string consists only of characters and 0 if not
 **/
bool alphacol(char *string){
	int a=1,b=0;
	size_t i=0;
	if(string==NULL) return 0;

	while(string[i]!='\0'){
		if(isalpha(string[i])>0) b=1;
		else b=0;
		a=a&b;
		i++;
	}
	return (bool)a;
}

/*  ************************************************************************************************  */

/**
 * @brief sstrdup:  Duplicates the given string
 *
 * @param s The string which we like to duplicate
 * @return char* Returns the duplicated string (pointer from the beginning)
 **/
char *sstrdup(const char *s)
{
    char *t = strdup(s);
    if (!t)
        abort();
    return t;
}

/*  ************************************************************************************************  */

/**
 * @brief findchar:  Proofes a string for the occurances of a character
 *
 * @param mode_string The string that should be searched for the given character
 * @param findchar The character which we are searching for
 * @return int Returns a number of occurances for "findchar"
 **/
int findchar(char *mode_string, char findchar){
	size_t i=0;
	int b=0;
	
	if(mode_string==NULL || findchar=='\0') return b;

	while(mode_string[i]!='\0'){
        if(mode_string[i]==findchar) b++;
        i++;
	}
	return b;
}

/*  ************************************************************************************************  */

/**
 * @brief getposition:  Proofes a string for the occurance of a character
 *
 * @param mode_string The string that should be searched for the given character
 * @param findchar The character which we are searching for
 * @return int Returns the first position of target character
 **/
int getposition(char *mode_string, char findchar){
	int i=0;
	if(mode_string==NULL || findchar=='\0') return -1;

	while(mode_string[i]!='\0'){
        if(mode_string[i]==findchar) return i;
        i++;
	}
	return -2;
}

/*  ************************************************************************************************  */

/**
 * @brief getoccurance_x:  Returns the place of the Xth occurance of a character
 *
 * @param mode_string The string that should be searched for the given character
 * @param findchar The character which we are searching for
 * @param x The number of occurances (which occurance do we want?)
 * @return int Returns the first position of target character
 **/
int getoccurance_x(char *mode_string, char findchar, int x){
	int i=0;
	int a=0;
	if(mode_string==NULL || findchar=='\0') return -1;

	while(mode_string[i]!='\0'){
        if(mode_string[i]==findchar){
			a++;
			if(a==x){
				return i;
			}
		}
        i++;
	}
	return -2;
}

/*  ************************************************************************************************  */

/**
 * @brief strmatch:  Deletes all occurances of a string-A out of a string-B
 *
 * @param string_to_search The string which should be searched for a match of "search_string"
 * @param search_string The string which should match anywhere with any part of "string_to_search"
 * @param stringmatch The matched part of the two strings
 * @param strmatchsize The sizeof() stringmatch (maximum length)
 * @return void
 **/
void strmatch(char *string_to_search, char *search_string, char *stringmatch, size_t strmatchsize){
	size_t i=0,b=0;
	if(search_string==NULL || string_to_search==NULL || stringmatch==NULL) return;

	while(string_to_search[i]!='\0'){
		if(string_to_search[i]==search_string[0]){
			for(b=0;search_string[b]!='\0' && string_to_search[i+b]!='\0';b++){
				if(string_to_search[i+b]==search_string[b]){
					straddnc(stringmatch,search_string[b],strmatchsize);
					continue;
				}else{
					if(dynlen(stringmatch)+1 < strmatchsize)
						stringmatch[dynlen(stringmatch)+1]='\0';
					else
						stringmatch[dynlen(stringmatch)]='\0';
					return;
				}
			}
		}
		i++;
	}
	
	if(dynlen(stringmatch)+1 < strmatchsize)
		stringmatch[dynlen(stringmatch)+1]='\0';
	else
		stringmatch[dynlen(stringmatch)]='\0';
		
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief strstart:  Proofes if instring is starting with parameter
 *
 * @param instring The string which we like to proof
 * @param parameter The string with which instring has to start
 * @return bool
 **/
bool str_start(char *instring, char *parameter){
	size_t i      = 0;
	size_t length = dynlen(parameter);
	
	if(instring == NULL || parameter == NULL) return false;
	while(i<length){
		if(m->BSCaseSensitive){	
			if(instring[i] == parameter[i])
				i++;
			else break;
		}else{
			if(tolower(instring[i])==tolower(parameter[i]))
				i++;
			else break;
		}
	}
	if ( i == length ) return true;
	return false;
}

/*  ************************************************************************************************  */

/**
 * @brief strend:  Proofes if instring is ending with parameter
 *
 * @param instring The string which we like to proof
 * @param parameter The string with which instring has to end
 * @return bool
 **/
bool str_end(char *instring, char *parameter){
	size_t end_parameter = dynlen(parameter);
	size_t end_instring  = dynlen(instring);
	
	if(instring == NULL || parameter == NULL) return false;
	while(end_parameter>0){
		if(m->BSCaseSensitive){	
			if(instring[end_instring] == parameter[end_parameter]){
				end_instring--;
				end_parameter--;
			}else break;
		}else{
			if(tolower(instring[end_instring])==tolower(parameter[end_parameter])){
				end_instring--;
				end_parameter--;
			}else break;
		}
	}
	if ( end_parameter == 0 ) return true;
	return false;
}

/*  ************************************************************************************************  */

/**
 * @brief str_is_ip: Proofes if a string is an ip address
 *
 * @param str String to check
 * @return The IP, if one found. 0 if none.
 **/
uint32 str_is_ip(char *str){
    int i;
    int octets[4] = { -1, -1, -1, -1 };
    char *s = str;
    uint32 ip;

    for (i = 0; i < 4; i++) {
        octets[i] = strtol(s, &s, 10);
        /* Bail out if the octet is invalid or wrongly terminated */
        if ((octets[i] < 0) || (octets[i] > 255)
            || ((i < 3) && (*s != '.')))
            return 0;
        if (i < 3)
            s++;
    }

    /* Fill the IP - the dirty way */
    ip = octets[3];
    ip += octets[2] * 256;
    ip += octets[1] * 65536;
    ip += octets[0] * 16777216;

    return ip;
}

/*  ************************************************************************************************  */

/**
 * @brief maxValue:  Selects the greatest value out of the list and returns it
 *
 * @param a A variable list of integers
 * @return int32 Returns the greatest value from the given list
 **/
int maxValue(int a, ...) {
	int maxValue = 0;
	int i = a;
	va_list liste;
	va_start(liste, a);
	while (i != 0)
	{
		if(i>=maxValue) maxValue=i;
		i = va_arg(liste, int);
	}
	va_end(liste);
	return maxValue;
}

/*  ************************************************************************************************  */

/**
 * @brief is_valid_chars:  Lowers a given character
 *
 * @param text The string which should be searched
 * @param allowed The string in which the allowed characters are given
 * @return bool Returns 1 if all characters of "text" are in allowed and 0 if one or more are not in allowed
 **/
bool is_valid_chars(char *text, char *allowed){
    //iS.aLwAyS.tH3r3.wH3n.YoU.cAlL - 0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-.
	size_t i = 0;
	size_t length = dynlen(text);
	for(i = 0; i < length; i++){
        if(!strcin(text[i],allowed)) return 1;
	}
    return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief sort:  Sorts a string (alphabethically)
 *
 * @param string The string you want to sort (the string is overwritten by the result)
 * @return void
 **/
void sort(char *string){
	char sorted[60];
	int b = 0, k = 32768;
	
	int i = 0, j = 0;
	int len_a = 0, len_b = 0;
	
	bzero(&sorted,sizeof(sorted));
    len_b = len_a = dynlen(string);

	for(i = 0; i < len_a; i++){
    	for(j = 0; j < len_b; j++){
    		if(k>(int)string[j]){
    			k=(int)string[j];
    			b=j;
            }
    	}
    	substr(string,b);
    	straddnc(sorted,(char)k,sizeof(sorted));
    	// Correct Values
    	len_b-=1;
    	k = 32768;
	}
	dyncpy(string,sorted);
    return;
}

/*  ************************************************************************************************  */

/**
 * @brief stoi:  Converts a String to an integer (like atoi();)
 *         Difference to atoi: if convert==NULL || convert=='0' - 0 is returned! (no segfault)
 *
 * @param convert The string we want to convert
 * @return int Returns the integer value of "convert"
 **/
int stoi(char* convert){
	if(convert==NULL || convert[0]=='\0') return 0;
	else return atoi(convert);
}

/*  ************************************************************************************************  */

/**
 * @brief stol:  Converts a String to a long (like atol();)
 *         Defference to atol: if convert==NULL || convert=='0' - 0 is returned! (no segfault)
 *
 * @param convert The string we want to convert
 * @return long Returns the long value of "convert"
 **/
long stol(char* convert){
	if(convert==NULL || convert[0]=='\0') return 0;
	else return atol(convert);
}

/*  ************************ The Following functions where taken from Anope ************************  */

/**
 * @brief myStrGetToken: Get the token
 *
 * @param str String to search in
 * @param dilim Character to search for
 * @param token_number the token number
 * @return char* Returns the token
 */
char *myStrGetToken(const char *str, const char dilim, int token_number)
{
#ifdef USE_THREADS
	my_pthread_mutex_lock(&substring_lock);
#endif
	int len, idx, counter = 0, start_pos = 0;
	char *substring = NULL;
 	if (!str) {
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&substring_lock);
#endif
		return NULL;
	}

	len = strlen(str);
	for (idx = 0; idx <= len; idx++) {
		if ((str[idx] == dilim) || (idx == len)) {
			if (counter == token_number) {
				substring = myStrSubString(str, start_pos, idx);
				counter++;
			} else {
				start_pos = idx + 1;
				counter++;
			}
		}   
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&substring_lock);
#endif
 	return substring;
}

/*  ************************************************************************************************  */

/**
 * @brief myStrGetOnlyToken: Get the token only
 *
 * @param str String to search in
 * @param dilim Character to search for
 * @param token_number the token number
 * @return char* Returns the token
 */
char *myStrGetOnlyToken(const char *str, const char dilim,
                    	int token_number)
{
#ifdef USE_THREADS
	my_pthread_mutex_lock(&substring_lock);
#endif
	size_t len, idx, counter = 0, start_pos = 0;
	char *substring = NULL;
	if (!str) {
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&substring_lock);
#endif
		return NULL;
	}
	len = strlen(str);
	for (idx = 0; idx <= len; idx++) {
		if (str[idx] == dilim) {
			if (counter == token_number) {
				if (str[idx] == '\r')
					substring = myStrSubString(str, start_pos, idx - 1);
				else
					substring = myStrSubString(str, start_pos, idx);
				counter++;
			} else {
				start_pos = idx + 1;
				counter++;
			}
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&substring_lock);
#endif
	return substring;
}

/*  ************************************************************************************************  */

/**
 * @brief myStrGetTokenRemainder: Get the Remaining tokens
 *
 * @param str String to search in
 * @param dilim Character to search for
 * @param token_number the token number
 * @return char* Returns the remaining token
 */
char *myStrGetTokenRemainder(const char *str, const char dilim,
                             int token_number)
{
#ifdef USE_THREADS
	my_pthread_mutex_lock(&substring_lock);
#endif
	size_t len, idx, counter = 0, start_pos = 0;
	char *substring = NULL;
	if (!str) {
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&substring_lock);
#endif
		return NULL;
	}
	len = strlen(str);

	for (idx = 0; idx <= len; idx++) {
		if ((str[idx] == dilim) || (idx == len)) {
			if (counter == token_number) {
				substring = myStrSubString(str, start_pos, len);
				counter++;
			} else {
				start_pos = idx + 1;
				counter++;
			}
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&substring_lock);
#endif
	return substring;
}

/*  ************************************************************************************************  */

/**
 * @brief myStrSubString: Get the string between point A and point B
 *
 * @param src String to search in
 * @param start Point A
 * @param end Point B
 * @return char* Returns the string in between
 */
char *myStrSubString(const char *src, int start, int end)
{
#ifdef USE_THREADS
	my_pthread_mutex_lock(&substring_lock);
#endif
	char *substring = NULL;
	size_t len, idx;
	if (!src) {
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&substring_lock);
#endif
		return NULL;
	}
	len = strlen(src);
	if (((start >= 0) && (end <= len)) && (end > start)) {
		substring = (char *) malloc(sizeof(char) * ((end - start) + 1));
		for (idx = 0; idx <= end - start; idx++) {
			substring[idx] = src[start + idx];
		}
		substring[end - start] = '\0';
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&substring_lock);
#endif
	return substring;
}

/*  ************************************************************************************************  */

/**
 * @brief str_signed: Change an unsigned string to a signed string, overwriting the original
 *                 string.
 *
 * @param str input string
 * @return char* output string, same as input string.
 */
char *str_signed(unsigned char *str)
{
    char *nstr;

    nstr = (char *) str;
    while (*str) {
        *nstr = (char) *str;
        str++;
        nstr++;
    }

    return nstr;
}

/*  ************************************************************************************************  */
